@@ -1,6 +1,5 @@
 #include "AIInterface.h"
 
-#include "../util/MultiplayerCommon.h"
 #include "../network/ClientNetworking.h"
 #include "../client/AI/AIClientApp.h"
 
@@ -23,6 +22,7 @@
 
 #include <boost/timer.hpp>
 #include <boost/python/str.hpp>
+#include <boost/uuid/random_generator.hpp>
 
 
 #include <map>
@@ -33,6 +33,10 @@
 using boost::python::object;
 using boost::python::str;
 
+namespace {
+    const float DUMMY_FLOAT = 0.0f;
+}
+    
 //////////////////////////////////
 //          AI Base             //
 //////////////////////////////////
@@ -75,8 +79,8 @@ namespace AIInterface {
     { return AIClientApp::GetApp()->PlayerName(); }
 
     const std::string& PlayerName(int player_id) {
-        const std::map<int, PlayerInfo>& players = AIClientApp::GetApp()->Players();
-        std::map<int, PlayerInfo>::const_iterator it = players.find(player_id);
+        auto& players = AIClientApp::GetApp()->Players();
+        auto it = players.find(player_id);
         if (it != players.end())
             return it->second.name;
         else {
@@ -92,7 +96,7 @@ namespace AIInterface {
     { return AIClientApp::GetApp()->EmpireID(); }
 
     int PlayerEmpireID(int player_id) {
-        for (std::map<int, PlayerInfo>::value_type& entry : AIClientApp::GetApp()->Players()) {
+        for (auto& entry : AIClientApp::GetApp()->Players()) {
             if (entry.first == player_id)
                 return entry.second.empire_id;
         }
@@ -101,7 +105,7 @@ namespace AIInterface {
 
     std::vector<int>  AllEmpireIDs() {
         std::vector<int> empire_ids;
-        for (std::map<int, PlayerInfo>::value_type& entry : AIClientApp::GetApp()->Players())
+        for (auto& entry : AIClientApp::GetApp()->Players())
             empire_ids.push_back(entry.second.empire_id);
         return empire_ids;
     }
@@ -121,7 +125,7 @@ namespace AIInterface {
 
     std::vector<int> AllPlayerIDs() {
         std::vector<int> player_ids;
-        for (std::map<int, PlayerInfo>::value_type& entry : AIClientApp::GetApp()->Players())
+        for (auto& entry : AIClientApp::GetApp()->Players())
             player_ids.push_back(entry.first);
         return player_ids;
     }
@@ -130,8 +134,8 @@ namespace AIInterface {
     { return AIClientApp::GetApp()->GetPlayerClientType(player_id) == Networking::CLIENT_TYPE_AI_PLAYER; }
 
     bool PlayerIsHost(int player_id) {
-        const std::map<int, PlayerInfo>& players = AIClientApp::GetApp()->Players();
-        std::map<int, PlayerInfo>::const_iterator it = players.find(player_id);
+        auto& players = AIClientApp::GetApp()->Players();
+        auto it = players.find(player_id);
         if (it == players.end())
             return false;
         return it->second.host;
@@ -200,7 +204,7 @@ namespace AIInterface {
             // ownership to all, while remembering which planets this is done
             // to.
             universe.InhibitUniverseObjectSignals(true);
-            for (std::shared_ptr<Planet> planet : universe.Objects().FindObjects<Planet>()) {
+            for (auto& planet : universe.Objects().FindObjects<Planet>()) {
                  if (planet->Unowned()) {
                      unowned_planets.push_back(planet);
                      planet->SetOwner(player_id);
@@ -213,15 +217,22 @@ namespace AIInterface {
 
         if (pretend_to_own_unowned_planets) {
             // remove temporary ownership added above
-            for (std::shared_ptr<Planet> planet : unowned_planets)
+            for (auto& planet : unowned_planets)
                 planet->SetOwner(ALL_EMPIRES);
             universe.InhibitUniverseObjectSignals(false);
         }
     }
 
     void UpdateResourcePools() {
-        for (std::map<int, Empire*>::value_type& entry : AIClientApp::GetApp()->Empires())
-            entry.second->UpdateResourcePools();
+//        for (auto& entry : AIClientApp::GetApp()->Empires())
+//            entry.second->UpdateResourcePools();
+        int empire_id = AIClientApp::GetApp()->EmpireID();
+        Empire* empire = ::GetEmpire(empire_id);
+        if (!empire) {
+            ErrorLogger() << "UpdateResourcePools : couldn't get empire with id " << empire_id;
+            return;
+        }
+        empire->UpdateResourcePools();
     }
 
     void UpdateResearchQueue() {
@@ -245,7 +256,7 @@ namespace AIInterface {
     }
 
     int IssueFleetMoveOrder(int fleet_id, int destination_id) {
-        std::shared_ptr<const Fleet> fleet = GetFleet(fleet_id);
+        auto fleet = GetFleet(fleet_id);
         if (!fleet) {
             ErrorLogger() << "IssueFleetMoveOrder : passed an invalid fleet_id";
             return 0;
@@ -264,7 +275,7 @@ namespace AIInterface {
         if (destination_id != INVALID_OBJECT_ID && destination_id == start_id)
             DebugLogger() << "AIInterface::IssueFleetMoveOrder : pass destination system id (" << destination_id << ") that fleet is already in";
 
-        AIClientApp::GetApp()->Orders().IssueOrder(OrderPtr(new FleetMoveOrder(empire_id, fleet_id, start_id, destination_id)));
+        AIClientApp::GetApp()->Orders().IssueOrder(std::make_shared<FleetMoveOrder>(empire_id, fleet_id, start_id, destination_id));
 
         return 1;
     }
@@ -276,7 +287,7 @@ namespace AIInterface {
         }
 
         int empire_id = AIClientApp::GetApp()->EmpireID();
-        std::shared_ptr<const UniverseObject> obj = GetUniverseObject(object_id);
+        auto obj = GetUniverseObject(object_id);
 
         if (!obj) {
             ErrorLogger() << "IssueRenameOrder : passed an invalid object_id";
@@ -287,7 +298,7 @@ namespace AIInterface {
             return 0;
         }
 
-        AIClientApp::GetApp()->Orders().IssueOrder(OrderPtr(new RenameOrder(empire_id, object_id, new_name)));
+        AIClientApp::GetApp()->Orders().IssueOrder(std::make_shared<RenameOrder>(empire_id, object_id, new_name));
 
         return 1;
     }
@@ -302,7 +313,7 @@ namespace AIInterface {
 
             // make sure all objects exist and are owned just by this player
             for (int object_id : object_ids) {
-                std::shared_ptr<const UniverseObject> obj = GetUniverseObject(object_id);
+                auto obj = GetUniverseObject(object_id);
 
                 if (!obj) {
                     ErrorLogger() << "IssueScrapOrder : passed an invalid object_id";
@@ -314,7 +325,7 @@ namespace AIInterface {
                     return 0;
                 }
 
-                AIClientApp::GetApp()->Orders().IssueOrder(OrderPtr(new ScrapOrder(empire_id, object_id)));
+                AIClientApp::GetApp()->Orders().IssueOrder(std::make_shared<ScrapOrder>(empire_id, object_id));
             }
 
             return 1;
@@ -360,20 +371,19 @@ namespace AIInterface {
             return 0;
         }
 
-        std::vector<int>::const_iterator it = ship_ids.begin();
+        auto it = ship_ids.begin();
         for (++it; it != ship_ids.end(); ++it) {
-            std::shared_ptr<const Ship> ship2 = GetShip(*it);
+            auto ship2 = GetShip(*it);
             if (ship2->SystemID() != system_id) {
                 ErrorLogger() << "IssueNewFleetOrder : passed ship_ids of ships at different locations";
                 return 0;
             }
         }
 
-        int new_fleet_id = ClientApp::GetApp()->GetNewObjectID();
+        auto order = std::make_shared<NewFleetOrder>(empire_id, fleet_name, system_id, ship_ids);
+        AIClientApp::GetApp()->Orders().IssueOrder(order);
 
-        AIClientApp::GetApp()->Orders().IssueOrder(OrderPtr(new NewFleetOrder(empire_id, fleet_name, new_fleet_id, system_id, ship_ids)));
-
-        return new_fleet_id;
+        return *order->FleetIDs().begin();
     }
 
     int IssueNewFleetOrder(const std::string& fleet_name, int ship_id) {
@@ -385,7 +395,7 @@ namespace AIInterface {
     int IssueFleetTransferOrder(int ship_id, int new_fleet_id) {
         int empire_id = AIClientApp::GetApp()->EmpireID();
 
-        std::shared_ptr<const Ship> ship = GetShip(ship_id);
+        auto ship = GetShip(ship_id);
         if (!ship) {
             ErrorLogger() << "IssueFleetTransferOrder : passed an invalid ship_id " << ship_id;
             return 0;
@@ -400,7 +410,7 @@ namespace AIInterface {
             return 0;
         }
 
-        std::shared_ptr<const Fleet> fleet = GetFleet(new_fleet_id);
+        auto fleet = GetFleet(new_fleet_id);
         if (!fleet) {
             ErrorLogger() << "IssueFleetTransferOrder : passed an invalid new_fleet_id " << new_fleet_id;
             return 0;
@@ -422,7 +432,7 @@ namespace AIInterface {
 
         std::vector<int> ship_ids;
         ship_ids.push_back(ship_id);
-        AIClientApp::GetApp()->Orders().IssueOrder(OrderPtr(new FleetTransferOrder(empire_id, new_fleet_id, ship_ids)));
+        AIClientApp::GetApp()->Orders().IssueOrder(std::make_shared<FleetTransferOrder>(empire_id, new_fleet_id, ship_ids));
 
         return 1;
     }
@@ -431,14 +441,14 @@ namespace AIInterface {
         int empire_id = AIClientApp::GetApp()->EmpireID();
 
         // make sure ship_id is a ship...
-        std::shared_ptr<const Ship> ship = GetShip(ship_id);
+        auto ship = GetShip(ship_id);
         if (!ship) {
             ErrorLogger() << "IssueColonizeOrder : passed an invalid ship_id";
             return 0;
         }
 
         // get fleet of ship
-        std::shared_ptr<const Fleet> fleet = GetFleet(ship->FleetID());
+        auto fleet = GetFleet(ship->FleetID());
         if (!fleet) {
             ErrorLogger() << "IssueColonizeOrder : ship with passed ship_id has invalid fleet_id";
             return 0;
@@ -455,12 +465,12 @@ namespace AIInterface {
         }
 
         // verify that planet exists and is un-occupied.
-        std::shared_ptr<const Planet> planet = GetPlanet(planet_id);
+        auto planet = GetPlanet(planet_id);
         if (!planet) {
             ErrorLogger() << "IssueColonizeOrder : no planet with passed planet_id";
             return 0;
         }
-        if ((!planet->Unowned()) && !( planet->OwnedBy(empire_id) && planet->CurrentMeterValue(METER_POPULATION)==0)) {
+        if ((!planet->Unowned()) && !( planet->OwnedBy(empire_id) && planet->InitialMeterValue(METER_POPULATION) == 0.0f)) {
             ErrorLogger() << "IssueColonizeOrder : planet with passed planet_id "<<planet_id<<" is already owned, or colonized by own empire";
             return 0;
         }
@@ -475,7 +485,7 @@ namespace AIInterface {
             return 0;
         }
 
-        AIClientApp::GetApp()->Orders().IssueOrder(OrderPtr(new ColonizeOrder(empire_id, ship_id, planet_id)));
+        AIClientApp::GetApp()->Orders().IssueOrder(std::make_shared<ColonizeOrder>(empire_id, ship_id, planet_id));
 
         return 1;
     }
@@ -484,14 +494,14 @@ namespace AIInterface {
         int empire_id = AIClientApp::GetApp()->EmpireID();
 
         // make sure ship_id is a ship...
-        std::shared_ptr<const Ship> ship = GetShip(ship_id);
+        auto ship = GetShip(ship_id);
         if (!ship) {
             ErrorLogger() << "IssueInvadeOrder : passed an invalid ship_id";
             return 0;
         }
 
         // get fleet of ship
-        std::shared_ptr<const Fleet> fleet = GetFleet(ship->FleetID());
+        auto fleet = GetFleet(ship->FleetID());
         if (!fleet) {
             ErrorLogger() << "IssueInvadeOrder : ship with passed ship_id has invalid fleet_id";
             return 0;
@@ -508,17 +518,17 @@ namespace AIInterface {
         }
 
         // verify that planet exists and is occupied by another empire
-        std::shared_ptr<const Planet> planet = GetPlanet(planet_id);
+        auto planet = GetPlanet(planet_id);
         if (!planet) {
             ErrorLogger() << "IssueInvadeOrder : no planet with passed planet_id";
             return 0;
         }
         bool owned_by_invader = planet->OwnedBy(empire_id);
         bool unowned = planet->Unowned();
-        bool populated = planet->CurrentMeterValue(METER_POPULATION) > 0.;
+        bool populated = planet->InitialMeterValue(METER_POPULATION) > 0.0f;
         bool visible = GetUniverse().GetObjectVisibilityByEmpire(planet_id, empire_id) >= VIS_PARTIAL_VISIBILITY;
-        bool vulnerable = planet->CurrentMeterValue(METER_SHIELD) <= 0.;
-        float shields = planet->CurrentMeterValue(METER_SHIELD);
+        bool vulnerable = planet->InitialMeterValue(METER_SHIELD) <= 0.0f;
+        float shields = planet->InitialMeterValue(METER_SHIELD);
         std::string this_species = planet->SpeciesName();
         //bool being_invaded = planet->IsAboutToBeInvaded();
         bool invadable = !owned_by_invader && vulnerable && (populated || !unowned) && visible ;// && !being_invaded; a 'being_invaded' check prevents AI from invading with multiple ships at once, which is important
@@ -526,7 +536,7 @@ namespace AIInterface {
             ErrorLogger() << "IssueInvadeOrder : planet with passed planet_id " << planet_id
                           << " and species " << this_species << " is not invadable due to one or more of: owned by invader empire, "
                           << "not visible to invader empire, has shields above zero, or is already being invaded.";
-            if (!unowned) 
+            if (!unowned)
                 ErrorLogger() << "IssueInvadeOrder : planet (id " << planet_id << ") is not unowned";
             if (!visible)
                 ErrorLogger() << "IssueInvadeOrder : planet (id " << planet_id << ") is not visible";
@@ -545,7 +555,7 @@ namespace AIInterface {
             return 0;
         }
 
-        AIClientApp::GetApp()->Orders().IssueOrder(OrderPtr(new InvadeOrder(empire_id, ship_id, planet_id)));
+        AIClientApp::GetApp()->Orders().IssueOrder(std::make_shared<InvadeOrder>(empire_id, ship_id, planet_id));
 
         return 1;
     }
@@ -554,12 +564,12 @@ namespace AIInterface {
         int empire_id = AIClientApp::GetApp()->EmpireID();
 
         // make sure ship_id is a ship...
-        std::shared_ptr<const Ship> ship = GetShip(ship_id);
+        auto ship = GetShip(ship_id);
         if (!ship) {
             ErrorLogger() << "IssueBombardOrder : passed an invalid ship_id";
             return 0;
         }
-        if (ship->TotalWeaponsDamage() <= 0) {
+        if (ship->TotalWeaponsDamage() <= 0.0f) {   // this will test the current meter values. potential issue if some local change sets these to zero even though they will be nonzero on server when bombard is processed before effects application / meter update
             ErrorLogger() << "IssueBombardOrder : ship can't attack / bombard";
             return 0;
         }
@@ -570,7 +580,7 @@ namespace AIInterface {
 
 
         // verify that planet exists and is occupied by another empire
-        std::shared_ptr<const Planet> planet = GetPlanet(planet_id);
+        auto planet = GetPlanet(planet_id);
         if (!planet) {
             ErrorLogger() << "IssueBombardOrder : no planet with passed planet_id";
             return 0;
@@ -600,19 +610,15 @@ namespace AIInterface {
             return 0;
         }
 
-        AIClientApp::GetApp()->Orders().IssueOrder(OrderPtr(new BombardOrder(empire_id, ship_id, planet_id)));
+        AIClientApp::GetApp()->Orders().IssueOrder(std::make_shared<BombardOrder>(empire_id, ship_id, planet_id));
 
         return 1;
     }
 
-    int IssueDeleteFleetOrder() {
-        return 0;
-    }
-
     int IssueAggressionOrder(int object_id, bool aggressive) {
         int empire_id = AIClientApp::GetApp()->EmpireID();
 
-        std::shared_ptr<const Fleet> fleet = GetFleet(object_id);
+        auto fleet = GetFleet(object_id);
         if (!fleet) {
             ErrorLogger() << "IssueAggressionOrder : no fleet with passed id";
             return 0;
@@ -622,8 +628,8 @@ namespace AIInterface {
             return 0;
         }
 
-        AIClientApp::GetApp()->Orders().IssueOrder(OrderPtr(
-            new AggressiveOrder(empire_id, object_id, aggressive)));
+        AIClientApp::GetApp()->Orders().IssueOrder(
+            std::make_shared<AggressiveOrder>(empire_id, object_id, aggressive));
 
         return 1;
     }
@@ -641,7 +647,7 @@ namespace AIInterface {
             return 0;
         }
 
-        std::shared_ptr<UniverseObject> obj = GetUniverseObject(object_id);
+        auto obj = GetUniverseObject(object_id);
         if (!obj) {
             ErrorLogger() << "IssueGiveObjectToEmpireOrder : passed invalid object id";
             return 0;
@@ -657,7 +663,7 @@ namespace AIInterface {
             return 0;
         }
 
-        std::shared_ptr<System> system = GetSystem(obj->SystemID());
+        auto system = GetSystem(obj->SystemID());
         if (!system) {
             ErrorLogger() << "IssueGiveObjectToEmpireOrder : couldn't get system of object";
             return 0;
@@ -665,10 +671,9 @@ namespace AIInterface {
 
         // can only give to empires with something present to receive the gift
         bool recipient_has_something_here = false;
-        std::vector<std::shared_ptr<const UniverseObject>> system_objects =
-            Objects().FindObjects<const UniverseObject>(system->ObjectIDs());
-        for (std::shared_ptr<const UniverseObject> obj : system_objects) {
-            if (obj->Owner() == recipient_id) {
+        auto system_objects = Objects().FindObjects<const UniverseObject>(system->ObjectIDs());
+        for (auto& system_object : system_objects) {
+            if (system_object->Owner() == recipient_id) {
                 recipient_has_something_here = true;
                 break;
             }
@@ -678,8 +683,8 @@ namespace AIInterface {
             return 0;
         }
 
-        AIClientApp::GetApp()->Orders().IssueOrder(OrderPtr(
-            new GiveObjectToEmpireOrder(empire_id, object_id, recipient_id)));
+        AIClientApp::GetApp()->Orders().IssueOrder(
+            std::make_shared<GiveObjectToEmpireOrder>(empire_id, object_id, recipient_id));
 
         return 1;
     }
@@ -687,7 +692,7 @@ namespace AIInterface {
     int IssueChangeFocusOrder(int planet_id, const std::string& focus) {
         int empire_id = AIClientApp::GetApp()->EmpireID();
 
-        std::shared_ptr<const Planet> planet = GetPlanet(planet_id);
+        auto planet = GetPlanet(planet_id);
         if (!planet) {
             ErrorLogger() << "IssueChangeFocusOrder : no planet with passed planet_id "<<planet_id;
             return 0;
@@ -701,8 +706,8 @@ namespace AIInterface {
             return 0;
         }
 
-        AIClientApp::GetApp()->Orders().IssueOrder(OrderPtr(
-            new ChangeFocusOrder(empire_id, planet_id, focus)));
+        AIClientApp::GetApp()->Orders().IssueOrder(
+            std::make_shared<ChangeFocusOrder>(empire_id, planet_id, focus));
 
         return 1;
     }
@@ -716,8 +721,8 @@ namespace AIInterface {
 
         int empire_id = AIClientApp::GetApp()->EmpireID();
 
-        AIClientApp::GetApp()->Orders().IssueOrder(OrderPtr(
-            new ResearchQueueOrder(empire_id, tech_name, position)));
+        AIClientApp::GetApp()->Orders().IssueOrder(
+            std::make_shared<ResearchQueueOrder>(empire_id, tech_name, position));
 
         return 1;
     }
@@ -731,7 +736,7 @@ namespace AIInterface {
 
         int empire_id = AIClientApp::GetApp()->EmpireID();
 
-        AIClientApp::GetApp()->Orders().IssueOrder(OrderPtr(new ResearchQueueOrder(empire_id, tech_name)));
+        AIClientApp::GetApp()->Orders().IssueOrder(std::make_shared<ResearchQueueOrder>(empire_id, tech_name));
 
         return 1;
     }
@@ -745,8 +750,8 @@ namespace AIInterface {
             return 0;
         }
 
-        AIClientApp::GetApp()->Orders().IssueOrder(OrderPtr(
-            new ProductionQueueOrder(empire_id, ProductionQueue::ProductionItem(BT_BUILDING, item_name), 1, location_id)));
+        AIClientApp::GetApp()->Orders().IssueOrder(
+            std::make_shared<ProductionQueueOrder>(empire_id, ProductionQueue::ProductionItem(BT_BUILDING, item_name), 1, location_id));
 
         return 1;
     }
@@ -760,8 +765,8 @@ namespace AIInterface {
             return 0;
         }
 
-        AIClientApp::GetApp()->Orders().IssueOrder(OrderPtr(
-            new ProductionQueueOrder(empire_id, ProductionQueue::ProductionItem(BT_SHIP, design_id), 1, location_id)));
+        AIClientApp::GetApp()->Orders().IssueOrder(
+            std::make_shared<ProductionQueueOrder>(empire_id, ProductionQueue::ProductionItem(BT_SHIP, design_id), 1, location_id));
 
         return 1;
     }
@@ -780,8 +785,8 @@ namespace AIInterface {
             return 0;
         }
 
-        AIClientApp::GetApp()->Orders().IssueOrder(OrderPtr(
-            new ProductionQueueOrder(empire_id, queue_index, new_quantity, new_blocksize)));
+        AIClientApp::GetApp()->Orders().IssueOrder(
+            std::make_shared<ProductionQueueOrder>(empire_id, queue_index, new_quantity, new_blocksize));
 
         return 1;
     }
@@ -813,8 +818,8 @@ namespace AIInterface {
             return 0;
         }
 
-        AIClientApp::GetApp()->Orders().IssueOrder(OrderPtr(
-            new ProductionQueueOrder(empire_id, old_queue_index, new_queue_index)));
+        AIClientApp::GetApp()->Orders().IssueOrder(
+            std::make_shared<ProductionQueueOrder>(empire_id, old_queue_index, new_queue_index));
 
         return 1;
     }
@@ -829,49 +834,78 @@ namespace AIInterface {
             return 0;
         }
 
-        AIClientApp::GetApp()->Orders().IssueOrder(OrderPtr(
-            new ProductionQueueOrder(empire_id, queue_index)));
+        AIClientApp::GetApp()->Orders().IssueOrder(
+            std::make_shared<ProductionQueueOrder>(empire_id, queue_index));
+
+        return 1;
+    }
+
+    int IssuePauseProductionOrder(int queue_index, bool pause) {
+        int empire_id = AIClientApp::GetApp()->EmpireID();
+        Empire* empire = AIClientApp::GetApp()->GetEmpire(empire_id);
+
+        const ProductionQueue& queue = empire->GetProductionQueue();
+        if (queue_index < 0 || static_cast<int>(queue.size()) <= queue_index) {
+            ErrorLogger() << "IssueChangeProductionPauseOrder : passed queue_index outside range of items on queue.";
+            return 0;
+        }
+
+        AIClientApp::GetApp()->Orders().IssueOrder(
+            std::make_shared<ProductionQueueOrder>(empire_id, queue_index, pause, DUMMY_FLOAT));
+
+        return 1;
+    }
+
+    int IssueAllowStockpileProductionOrder(int queue_index, bool use_stockpile) {
+        int empire_id = AIClientApp::GetApp()->EmpireID();
+        Empire* empire = AIClientApp::GetApp()->GetEmpire(empire_id);
+
+        const ProductionQueue& queue = empire->GetProductionQueue();
+        if (queue_index < 0 || static_cast<int>(queue.size()) <= queue_index) {
+            ErrorLogger() << "IssueChangeProductionStockpileOrder : passed queue_index outside range of items on queue.";
+            return 0;
+        }
+        
+        AIClientApp::GetApp()->Orders().IssueOrder(
+            std::make_shared<ProductionQueueOrder>(empire_id, queue_index, use_stockpile, DUMMY_FLOAT, DUMMY_FLOAT));
 
         return 1;
     }
 
     int IssueCreateShipDesignOrder(const std::string& name, const std::string& description,
                                    const std::string& hull, const std::vector<std::string> parts,
-                                   const std::string& icon, const std::string& model, bool nameDescInStringTable)
+                                   const std::string& icon, const std::string& model, bool name_desc_in_stringtable)
     {
         if (name.empty() || description.empty() || hull.empty()) {
             ErrorLogger() << "IssueCreateShipDesignOrderOrder : passed an empty name, description, or hull.";
             return 0;
         }
-        if (!ShipDesign::ValidDesign(hull, parts)) {
-            ErrorLogger() << "IssueCreateShipDesignOrderOrder : pass a hull and parts that do not make a valid ShipDesign";
-            return 0;
-        }
 
         int empire_id = AIClientApp::GetApp()->EmpireID();
         int current_turn = CurrentTurn();
 
-        // create design from stuff chosen in UI
-        ShipDesign* design = new ShipDesign(name, description, current_turn, ClientApp::GetApp()->EmpireID(),
-                                            hull, parts, icon, model, nameDescInStringTable);
+        const auto uuid = boost::uuids::random_generator()();
 
-        if (!design) {
+        // create design from stuff chosen in UI
+        ShipDesign* design;
+        try {
+            design = new ShipDesign(std::invalid_argument(""), name, description, current_turn, ClientApp::GetApp()->EmpireID(),
+                                    hull, parts, icon, model, name_desc_in_stringtable, false, uuid);
+
+        } catch (const std::invalid_argument&) {
             ErrorLogger() << "IssueCreateShipDesignOrderOrder failed to create a new ShipDesign object";
             return 0;
         }
 
-        int new_design_id = AIClientApp::GetApp()->GetNewDesignID();
-        AIClientApp::GetApp()->Orders().IssueOrder(OrderPtr(new ShipDesignOrder(empire_id, new_design_id, *design)));
+        auto order = std::make_shared<ShipDesignOrder>(empire_id, *design);
+        AIClientApp::GetApp()->Orders().IssueOrder(order);
         delete design;
 
         return 1;
     }
 
     void SendPlayerChatMessage(int recipient_player_id, const std::string& message_text) {
-        if (recipient_player_id == -1)
-            AIClientApp::GetApp()->Networking().SendMessage(GlobalChatMessage(PlayerID(), message_text));
-        else
-            AIClientApp::GetApp()->Networking().SendMessage(SingleRecipientChatMessage(PlayerID(), recipient_player_id, message_text));
+        AIClientApp::GetApp()->Networking().SendMessage(PlayerChatMessage(message_text, recipient_player_id));
     }
 
     void SendDiplomaticMessage(const DiplomaticMessage& diplo_message) {
@@ -882,7 +916,7 @@ namespace AIInterface {
         int recipient_empire_id = diplo_message.RecipientEmpireID();
         int recipient_player_id = app->EmpirePlayerID(recipient_empire_id);
         if (recipient_player_id == Networking::INVALID_PLAYER_ID) return;
-        app->Networking().SendMessage(DiplomacyMessage(sender_player_id, recipient_player_id, diplo_message));
+        app->Networking().SendMessage(DiplomacyMessage(diplo_message));
     }
 
     void DoneTurn() {
