@@ -34,6 +34,8 @@
 #include <GG/Font.h>
 #include <GG/GLClientAndServerBuffer.h>
 
+#include <boost/function.hpp>
+
 
 namespace GG {
 
@@ -107,6 +109,7 @@ public:
     TextBoxBrowseInfoWnd(X w, const std::shared_ptr<Font>& font, Clr color, Clr border_color, Clr text_color,
                          Flags<TextFormat> format = FORMAT_LEFT | FORMAT_WORDBREAK,
                          unsigned int border_width = 2, unsigned int text_margin = 4);
+    void CompleteConstruction() override;
     //@}
 
     /** \name Accessors */ ///@{
@@ -148,14 +151,15 @@ private:
     virtual void InitBuffer();
     void UpdateImpl(std::size_t mode, const Wnd* target) override;
 
-    GG::GL2DVertexBuffer    m_buffer;
-    bool                    m_text_from_target;
-    std::shared_ptr<Font> m_font;
-    Clr                     m_color;
-    Clr                     m_border_color;
-    unsigned int            m_border_width;
-    X                       m_preferred_width;
-    TextControl*            m_text_control;
+    GG::GL2DVertexBuffer            m_buffer;
+    bool                            m_text_from_target;
+    std::shared_ptr<Font>           m_font;
+    Clr                             m_color;
+    Clr                             m_border_color;
+    unsigned int                    m_border_width;
+    X                               m_preferred_width;
+    std::shared_ptr<TextControl>    m_text_control;
+    unsigned int                    m_text_margin;
 };
 
 } // namespace GG
