@@ -176,11 +176,11 @@ public:
     /** \name Accessors */ ///@{
     /** Returns true iff FlagSpec contains \a flag. */
     bool contains(FlagType flag) const
-    { return find(flag) != end(); }
+    { return m_flags.count(flag); }
     /** Returns true iff \a flag is a "permanent" flag -- a flag used
         internally by the GG library, as opposed to a user-added flag. */
     bool permanent(FlagType flag) const
-    { return m_permanent.find(flag) != m_permanent.end(); }
+    { return m_permanent.count(flag); }
     /** Returns an iterator to \a flag, if flag is in the FlagSpec, or end()
         otherwise. */
     const_iterator find(FlagType flag) const
@@ -196,7 +196,7 @@ public:
         unknown flag's stringification is requested. */
     const std::string& ToString(FlagType flag) const
     {
-        typename std::map<FlagType, std::string>::const_iterator it = m_strings.find(flag);
+        auto it = m_strings.find(flag);
         if (it == m_strings.end())
             throw UnknownFlag("Could not find string corresponding to unknown flag");
         return it->second;
@@ -205,7 +205,7 @@ public:
         GG::FlagSpec::UnknownString if an unknown string is provided. */
     FlagType FromString(const std::string& str) const
     {
-        for (const typename std::map<FlagType, std::string>::value_type& string : m_strings) {
+        for (const auto& string : m_strings) {
             if (string.second == str)
                 return string.first;
         }
