@@ -36,6 +36,7 @@
 
 namespace GG
 {
+class RichTextPrivate;
 
 /** \brief A control for showing text and images.
  */
@@ -50,16 +51,16 @@ public:
     public:
         //! Creates a control from the tag (with unparsed parameters) and the content between the tags.
         //! You own the returned control.
-        virtual BlockControl* CreateFromTag(const std::string& tag,
-                                            const TAG_PARAMS& params,
-                                            const std::string& content,
-                                            const std::shared_ptr<Font>& font,
-                                            const Clr& color,
-                                            Flags<TextFormat> format) = 0;
+        virtual std::shared_ptr<BlockControl> CreateFromTag(const std::string& tag,
+                                                            const TAG_PARAMS& params,
+                                                            const std::string& content,
+                                                            const std::shared_ptr<Font>& font,
+                                                            const Clr& color,
+                                                            Flags<TextFormat> format) = 0;
     };
 
     //! The type of the object where we store control factories of tags.
-    typedef std::map<std::string, IBlockControlFactory*> BLOCK_FACTORY_MAP;
+    typedef std::map<std::string, std::shared_ptr<IBlockControlFactory>> BLOCK_FACTORY_MAP;
 
     //! The special tag that is used to represent plaintext.
     // Allows you to register a custom control for displaying plaintext.
@@ -70,6 +71,8 @@ public:
              Clr color = CLR_BLACK, Flags<TextFormat> format = FORMAT_NONE,
              Flags<WndFlag> flags = NO_WND_FLAGS);
 
+    void CompleteConstruction() override;
+
     ~RichText();
     //@}
 
@@ -92,14 +95,14 @@ public:
     void SetBlockFactoryMap(const std::shared_ptr<BLOCK_FACTORY_MAP>& block_factory_map);
 
     //! Registers a factory in the default block factory map.
-    static int RegisterDefaultBlock(const std::string& tag, IBlockControlFactory* factory);
+    static int RegisterDefaultBlock(const std::string& tag, std::shared_ptr<IBlockControlFactory>&& factory);
 
     //! Access the default block factory map.
     static std::shared_ptr<RichText::BLOCK_FACTORY_MAP>& DefaultBlockFactoryMap();
 
 private:
     friend class RichTextPrivate;
-    class RichTextPrivate* const m_self;
+    std::unique_ptr<RichTextPrivate> const m_self;
 };
 
 }
