@@ -31,6 +31,8 @@
 #include <GG/Control.h>
 #include <GG/GLClientAndServerBuffer.h>
 
+#include <boost/signals2/signal.hpp>
+
 
 namespace GG {
 
@@ -79,6 +81,7 @@ public:
     /** Ctor. */
     Scroll(Orientation orientation, Clr color, Clr interior);
     //@}
+    void CompleteConstruction() override;
 
     /** \name Accessors */ ///@{
     Pt MinUsableSize() const override;
@@ -154,9 +157,9 @@ private:
     int                     m_range_max;    ///< highest value "
     unsigned int            m_line_sz;      ///< logical units traversed in a line movement (such as a click on either end button)
     unsigned int            m_page_sz;      ///< logical units traversed for a page movement (such as a click in non-tab middle area, or PgUp/PgDn)
-    Button*                 m_tab;          ///< the button representing the tab
-    Button*                 m_incr;         ///< the increase button (line down/line right)
-    Button*                 m_decr;         ///< the decrease button (line up/line left)
+    std::shared_ptr<Button>                 m_tab;          ///< the button representing the tab
+    std::shared_ptr<Button>                 m_incr;         ///< the increase button (line down/line right)
+    std::shared_ptr<Button>                 m_decr;         ///< the decrease button (line up/line left)
     ScrollRegion            m_initial_depressed_region; ///< the part of the scrollbar originally under cursor in LButtonDown msg
     ScrollRegion            m_depressed_region;         ///< the part of the scrollbar currently being "depressed" by held-down mouse button
     bool                    m_dragging_tab;
