@@ -30,9 +30,9 @@
 using namespace GG;
 
 EventPumpState::EventPumpState() :
-    last_FPS_time(0),
-    last_frame_time(0),
-    most_recent_time(0),
+    last_FPS_time(std::chrono::high_resolution_clock::now()),
+    last_frame_time(std::chrono::high_resolution_clock::now()),
+    most_recent_time(std::chrono::high_resolution_clock::now()),
     frames(0)
 {}
 
@@ -40,28 +40,33 @@ EventPumpState::EventPumpState() :
 void EventPumpBase::LoopBody(GUI* gui, EventPumpState& state, bool do_non_rendering, bool do_rendering)
 {
     if (do_non_rendering) {
-        int time = gui->Ticks();
+        std::chrono::high_resolution_clock::time_point time = std::chrono::high_resolution_clock::now();
 
         // send an idle message, so that the gui has timely updates for triggering browse info windows, etc.
         gui->HandleGGEvent(GUI::IDLE, GGK_UNKNOWN, 0, gui->ModKeys(), gui->MousePosition(), Pt());
 
         // govern FPS speed if needed
         if (double max_FPS = gui->MaxFPS()) {
-            double min_ms_per_frame = 1000.0 / max_FPS;
-            double ms_to_wait = min_ms_per_frame - (time - state.last_frame_time);
-            if (0.0 < ms_to_wait) {
-                gui->Wait(static_cast<unsigned int>(ms_to_wait));
-                time = gui->Ticks();
+            std::chrono::microseconds min_us_per_frame = std::chrono::duration_cast<std::chrono::microseconds>(
+            std::chrono::duration<double>(1.0 / (max_FPS + 1)));
+            std::chrono::microseconds us_elapsed = std::chrono::duration_cast<std::chrono::microseconds>(
+                time - state.last_frame_time);
+            std::chrono::microseconds us_to_wait = (min_us_per_frame - us_elapsed);
+            if (std::chrono::microseconds(0) < us_to_wait) {
+                gui->Wait(us_to_wait);
+                time = std::chrono::high_resolution_clock::now();
             }
         }
         state.last_frame_time = time;
 
         // track FPS if needed
-        gui->SetDeltaT(time - state.most_recent_time);
+        gui->SetDeltaT(std::chrono::duration_cast<std::chrono::microseconds>(time - state.most_recent_time).count());
         if (gui->FPSEnabled()) {
             ++state.frames;
-            if (1000 < time - state.last_FPS_time) { // calculate FPS at most once a second
-                gui->SetFPS(state.frames / ((time - state.last_FPS_time) / 1000.0));
+            if (std::chrono::seconds(1) < time - state.last_FPS_time) { // calculate FPS at most once a second
+                double time_since_last_FPS = std::chrono::duration_cast<std::chrono::microseconds>(
+                    time - state.last_FPS_time).count() / 1000000.0;
+                gui->SetFPS(state.frames / time_since_last_FPS);
                 state.last_FPS_time = time;
                 state.frames = 0;
             }
@@ -71,6 +76,7 @@ void EventPumpBase::LoopBody(GUI* gui, EventPumpState& state, bool do_non_render
 
     if (do_rendering) {
         // do one iteration of the render loop
+        gui->PreRender();
         gui->RenderBegin();
         gui->Render();
         gui->RenderEnd();
