@@ -36,15 +36,16 @@ namespace {
 
     //! Skips over white space characters. On return, \a it will point to \a end
     //! or the next non-white space character.
-    void pass_space(std::string::const_iterator& it, const std::string::const_iterator& end)
+    void pass_space(std::string::const_iterator& it,
+                    const std::string::const_iterator& end)
     {
-        for (; it != end && isspace(*it); ++it) {
-        }
+        for (; it != end && isspace(*it); ++it) {}
     }
 
     //! Reads from \a it, expecting to find somthing of the form "key =".
     //! Returns key. Leaves \a it past the '=' or at \a end.
-    std::string read_key(std::string::const_iterator& it, const std::string::const_iterator& end)
+    std::string read_key(std::string::const_iterator& it,
+                         const std::string::const_iterator& end)
     {
         // Move past space.
         pass_space(it, end);
@@ -70,7 +71,8 @@ namespace {
     //! Read from \a it, expecting a string of the form '"escape \" with \\"'.
     //! Returns the text between the quotes.
     //! \a it will be at the first character after the second " or at \a end.
-    std::string read_quoted(std::string::const_iterator& it, const std::string::const_iterator& end)
+    std::string read_quoted(std::string::const_iterator& it,
+                            const std::string::const_iterator& end)
     {
         // Move past space.
         pass_space(it, end);
@@ -114,15 +116,16 @@ namespace {
     }
 
     //! Extracts key="value" pairs from a string to a map.
-    void ExtractParameters(const std::string& params_string, RichText::TAG_PARAMS& tag_params)
+    void ExtractParameters(const std::string& params_string,
+                           RichText::TAG_PARAMS& tag_params)
     {
         // Next key to be stored.
         std::string key;
         // Next value to be stored.
         std::string value;
 
-        std::string::const_iterator it = params_string.begin();
-        const std::string::const_iterator& end = params_string.end();
+        auto it = params_string.begin();
+        const auto& end = params_string.end();
 
         try {
             while (it != end) {
@@ -148,7 +151,6 @@ namespace {
 }
 
 namespace GG {
-
     /**
      * \brief The tag to use for text without explicit tags, or inside unknown (to the rich text system) tags.
      */
@@ -159,33 +161,16 @@ namespace GG {
      */
     class RichTextPrivate {
     public:
-        RichTextPrivate(RichText* q, const std::string& str, const std::shared_ptr<Font>& font,
-                        Clr color, Flags<TextFormat> format = FORMAT_NONE) :
-            m_owner(q),
-            m_font(font),
-            m_color(color),
-            m_format(format),
-            m_block_factory_map(RichText::DefaultBlockFactoryMap()),
-            m_padding(0)
-        {}
+        RichTextPrivate(RichText* q, const std::string& content,
+                        const std::shared_ptr<Font>& font,
+                        Clr color, Flags<TextFormat> format = FORMAT_NONE);
+
+        void CompleteConstruction();
 
         virtual ~RichTextPrivate() {}
 
         // Set the text to show. Parses it into tags and uses the factory map to turn them into blocks.
-        void SetText(const std::string& content)
-        {
-            m_blocks.clear();
-            m_owner->DeleteChildren();
-
-            // Get a set of tags registered for rich text usage.
-            std::set<std::string> known_tags = MapKeys(*m_block_factory_map);
-
-            // Parse the content into a vector of tags.
-            std::vector<RichTextTag> tags = TagParser::ParseTags(content, known_tags);
-
-            // Create blocks from the tags and populate the control with them.
-            PopulateBlocks(tags);
-        }
+        void SetText(const std::string& content);
 
         /**
           * @brief Set the whitespace around the content.
@@ -193,105 +178,178 @@ namespace GG {
           * @param pixels The number of pixels to reserve for empty space around the content.
           * @return void
           */
-        void SetPadding(int pixels)
-        {
-            if (m_padding != pixels) {
-                m_padding = pixels;
-                DoLayout();
-            }
-        }
-
-        void SizeMove(Pt ul, Pt lr)
-        {
-            Pt original_size = m_owner->Size();
-            m_owner->Control::SizeMove(ul, lr);
-
-            // Redo layout if necessary.
-            if (m_owner->Size() != original_size) {
-                DoLayout();
-            }
-        }
+        void SetPadding(int pixels);
+        void SizeMove(Pt ul, Pt lr);
 
         // Get the set of keys from a map.
-        template <typename T, typename V> std::set<T> MapKeys(const std::map<T, V>& arg_map)
-        {
-            typedef typename std::map<T, V>::value_type map_value;
-            std::set<T> keys;
-            for (const map_value& pair : arg_map) {
-                keys.insert(pair.first);
-            }
-            return keys;
-        }
+        template <typename T, typename V>
+        std::set<T> MapKeys(const std::map<T, V>& arg_map);
 
         // Set the mapping from tags to factories that should be used to generate blocks from them.
         void SetBlockFactoryMap(std::shared_ptr<RichText::BLOCK_FACTORY_MAP> block_factory_map)
-        { m_block_factory_map = block_factory_map; }
+        { m_block_factory_map = std::move(block_factory_map); }
 
     private:
+        // Easier access to m_block_factory_map
+        RichText::BLOCK_FACTORY_MAP& FactoryMap()
+        { return *m_block_factory_map; }
+
+        // Parses content into tags.
+        std::vector<RichTextTag> ParseTags(const std::string& content);
+
+        // Create blocks from tags.
+        void CreateBlocks(const std::vector<RichTextTag>& tags);
+
+        void AttachBlocks();
+
+        // Update the sizes of all blocks.
+        void DoLayout();
+
         RichText* const             m_owner;                //!< The public control.
         std::shared_ptr<Font>       m_font;                 //!< The font to use for text.
         Clr                         m_color;                //! < The color to use for text.
         Flags<TextFormat>           m_format;               //!< Text format.
         std::shared_ptr<RichText::BLOCK_FACTORY_MAP>
                                     m_block_factory_map;    //!< A map that tells us how to generate block controls from tags.
-        std::vector<BlockControl*>  m_blocks;               //!< The blocks generated from our content.
+        std::vector<std::shared_ptr<BlockControl>>
+                                    m_blocks;               //!< The blocks generated from our content.
         int                         m_padding;
 
-        // Easier access to m_block_factory_map
-        RichText::BLOCK_FACTORY_MAP& FactoryMap()
-        { return *m_block_factory_map; }
+    };
 
-        // Create blocks from tags.
-        void PopulateBlocks(const std::vector<RichTextTag>& tags)
-        {
-            // Create blocks using factories.
-            for (const RichTextTag& tag : tags) {
-                RichText::TAG_PARAMS params;
-                // Extract the parameters from params_string to the tag_params map.
-                ExtractParameters(tag.tag_params, params);
+    RichTextPrivate::RichTextPrivate(RichText* q, const std::string& content,
+                                     const std::shared_ptr<Font>& font,
+                                     Clr color, Flags<TextFormat> format /*= FORMAT_NONE*/) :
+        m_owner(q),
+        m_font(font),
+        m_color(color),
+        m_format(format),
+        m_block_factory_map(RichText::DefaultBlockFactoryMap()),
+        m_padding(0)
+    {
+        // Parse the content into a vector of tags.
+        auto tags = ParseTags(content);
 
-                BlockControl* block = FactoryMap()[tag.tag]->CreateFromTag(tag.tag, params, tag.content, m_font, m_color, m_format);
-                if (block) {
-                    m_owner->AttachChild(block);
-                    m_blocks.push_back(block);
-                }
-            }
+        // Create blocks from the tags and populate the control with them.
+        CreateBlocks(tags);
+    }
 
+    void RichTextPrivate::CompleteConstruction()
+    { AttachBlocks(); }
+
+    void RichTextPrivate::SetText(const std::string& content)
+    {
+        // Parse the content into a vector of tags.
+        auto tags = ParseTags(content);
+
+        // Create blocks from the tags and populate the control with them.
+        CreateBlocks(tags);
+        AttachBlocks();
+    }
+
+    void RichTextPrivate::SetPadding(int pixels)
+    {
+        if (m_padding != pixels) {
+            m_padding = pixels;
             DoLayout();
         }
+    }
 
-        // Update the sizes of all blocks.
-        void DoLayout() {
-            X width = m_owner->ClientWidth() - X(m_padding)*2;
-            Pt pos = Pt(X(m_padding), Y(m_padding));
+    void RichTextPrivate::SizeMove(Pt ul, Pt lr)
+    {
+        Pt original_size = m_owner->Size();
+        m_owner->Control::SizeMove(ul, lr);
 
-            // The contract between RichText and block controls is this:
-            // RichText tells them their width, and they determine their height.
-            for (BlockControl* block : m_blocks) {
-                Pt size = block->SetMaxWidth(width);
-                block->MoveTo(pos);
-                pos.y += size.y;
-            }
-
-            Pt size(m_owner->Width(), pos.y + Y(m_padding));
-            m_owner->Resize(size);
+        // Redo layout if necessary.
+        if (m_owner->Size() != original_size) {
+            DoLayout();
         }
-    };
+    }
+
+    // Get the set of keys from a map.
+    template <typename T, typename V>
+    std::set<T> RichTextPrivate::MapKeys(const std::map<T, V>& arg_map)
+    {
+        std::set<T> keys;
+        for (const auto& pair : arg_map) {
+            keys.insert(pair.first);
+        }
+        return keys;
+    }
+
+    // Parses content into tags.
+    std::vector<RichTextTag> RichTextPrivate::ParseTags(const std::string& content)
+    {
+        // Get a set of tags registered for rich text usage.
+        std::set<std::string> known_tags = MapKeys(*m_block_factory_map);
+
+        // Parse the content into a vector of tags.
+        std::vector<RichTextTag> tags = TagParser::ParseTags(content, known_tags);
+
+        return tags;
+    }
+
+    // Create blocks from tags.
+    void RichTextPrivate::CreateBlocks(const std::vector<RichTextTag>& tags)
+    {
+        m_blocks.clear();
+
+        // Create blocks using factories.
+        for (const RichTextTag& tag : tags) {
+            RichText::TAG_PARAMS params;
+            // Extract the parameters from params_string to the tag_params map.
+            ExtractParameters(tag.tag_params, params);
+
+            auto block(FactoryMap()[tag.tag]->CreateFromTag(tag.tag, params, tag.content, m_font, m_color, m_format));
+            if (block)
+                m_blocks.push_back(block);
+        }
+    }
+
+    void RichTextPrivate::AttachBlocks()
+    {
+        m_owner->DetachChildren();
+
+        for (auto& block : m_blocks)
+            m_owner->AttachChild(block);
+
+        DoLayout();
+    }
+
+    // Update the sizes of all blocks.
+    void RichTextPrivate::DoLayout()
+    {
+        X width = m_owner->ClientWidth() - X(m_padding)*2;
+        Pt pos = Pt(X(m_padding), Y(m_padding));
+
+        // The contract between RichText and block controls is this:
+        // RichText tells them their width, and they determine their height.
+        for (auto& block : m_blocks) {
+            Pt size = block->SetMaxWidth(width);
+            block->MoveTo(pos);
+            pos.y += size.y;
+        }
+
+        Pt size(m_owner->Width(), pos.y + Y(m_padding));
+        m_owner->Resize(size);
+    }
 
     /////////////////////////////////
     /// RichText public interface //
     ///////////////////////////////
     RichText::RichText(X x, Y y, X w, Y h, const std::string& str,
-                       const std::shared_ptr<Font>& font, Clr color, Flags<TextFormat> format,
-                       Flags<WndFlag> flags) :
-        Control(x, y, w, h, flags), m_self(new RichTextPrivate(this, str, font, color, format))
-    {
-        // Set text requires our members to be set.
-        // Therefore we must call it here, not in constructor of Private.
-        m_self->SetText(str);
+                       const std::shared_ptr<Font>& font, Clr color,
+                       Flags<TextFormat> format, Flags<WndFlag> flags) :
+        Control(x, y, w, h, flags),
+        m_self(new RichTextPrivate(this, str, font, color, format))
+    {}
+
+    void RichText::CompleteConstruction() {
+        Control::CompleteConstruction();
+        m_self->CompleteConstruction();
     }
 
-    RichText::~RichText() { delete m_self; }
+    RichText::~RichText() {}
 
     void RichText::SetText(const std::string& str) { m_self->SetText(str); }
 
@@ -313,10 +371,11 @@ namespace GG {
         return tag_map;
     }
 
-    int RichText::RegisterDefaultBlock(const std::string& tag, IBlockControlFactory* factory)
+    int RichText::RegisterDefaultBlock(const std::string& tag,
+                                       std::shared_ptr<IBlockControlFactory>&& factory)
     {
         Font::RegisterKnownTag(tag);
-        (*DefaultBlockFactoryMap()) [tag] = factory;
+        (*DefaultBlockFactoryMap()) [tag] = std::move(factory);
 
         // Return a dummy to enable static registration.
         return 0;
