@@ -24,9 +24,8 @@
 
 #include <GG/Texture.h>
 
-#include <GG/GUI.h>
+#include <GG/GLClientAndServerBuffer.h>
 #include <GG/Config.h>
-#include <GG/DrawUtil.h>
 #include <GG/utf8/checked.h>
 
 #include <boost/filesystem/operations.hpp>
@@ -37,7 +36,12 @@
 #include <boost/algorithm/string/case_conv.hpp>
 
 #if GG_HAVE_LIBPNG
-# include "gilext/io/png_dynamic_io.hpp"
+# if GIGI_CONFIG_USE_OLD_IMPLEMENTATION_OF_GIL_PNG_IO
+#  include "gilext/io/png_dynamic_io.hpp"
+#  include "gilext/io/png_io_v2_compat.hpp"
+# else
+#  include <boost/gil/extension/io/png.hpp>
+# endif
 #endif
 
 #include <iostream>
@@ -120,7 +124,8 @@ X Texture::DefaultWidth() const
 Y Texture::DefaultHeight() const
 { return m_default_height; }
 
-void Texture::OrthoBlit(const Pt& pt1, const Pt& pt2, const GLfloat* tex_coords/* = 0*/) const
+void Texture::OrthoBlit(const Pt& pt1, const Pt& pt2,
+                        const GLfloat* tex_coords/* = 0*/) const
 {
     if (m_opengl_id == 0)
         return;
@@ -128,8 +133,6 @@ void Texture::OrthoBlit(const Pt& pt1, const Pt& pt2, const GLfloat* tex_coords/
     if (!tex_coords) // use default texture coords when not given any others
         tex_coords = m_tex_coords;
 
-    glBindTexture(GL_TEXTURE_2D, m_opengl_id);
-
     // HACK! This code ensures that unscaled textures are reproduced exactly, even
     // though they theoretically should be even when using non-GL_NEAREST* scaling.
     bool render_scaled = (pt2.x - pt1.x) != m_default_width || (pt2.y - pt1.y) != m_default_height;
@@ -141,41 +144,42 @@ void Texture::OrthoBlit(const Pt& pt1, const Pt& pt2, const GLfloat* tex_coords/
         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
 
     // render texture
+    GL2DVertexBuffer vertex_buffer;
+    vertex_buffer.reserve(4);
+    vertex_buffer.store(pt2.x, pt1.y);
+    vertex_buffer.store(pt1.x, pt1.y);
+    vertex_buffer.store(pt2.x, pt2.y);
+    vertex_buffer.store(pt1.x, pt2.y);
 
-    GG::Pt position_data[4];
-    GLfloat texture_coordinate_data[8];
-
-    texture_coordinate_data[2*0] = tex_coords[0];
-    texture_coordinate_data[2*0 + 1] = tex_coords[1];
-    position_data[0] = pt1;
-
-    texture_coordinate_data[2*1] = tex_coords[2];
-    texture_coordinate_data[2*1 + 1] = tex_coords[1];
-    position_data[1].x = pt2.x;
-    position_data[1].y = pt1.y;
-
-    texture_coordinate_data[2*2] = tex_coords[0];
-    texture_coordinate_data[2*2 + 1] = tex_coords[3];
-    position_data[2].x = pt1.x;
-    position_data[2].y = pt2.y;
-
-    texture_coordinate_data[2*3] = tex_coords[2];
-    texture_coordinate_data[2*3 + 1] = tex_coords[3];
-    position_data[3] = pt2;
+    GLTexCoordBuffer tex_coord_buffer;
+    tex_coord_buffer.reserve(4);
+    if (tex_coords) {
+        tex_coord_buffer.store(tex_coords[2], tex_coords[1]);
+        tex_coord_buffer.store(tex_coords[0], tex_coords[1]);
+        tex_coord_buffer.store(tex_coords[2], tex_coords[3]);
+        tex_coord_buffer.store(tex_coords[0], tex_coords[3]);
+    } else {
+        tex_coord_buffer.store(1.0f, 0.0f);
+        tex_coord_buffer.store(0.0f, 0.0f);
+        tex_coord_buffer.store(1.0f, 1.0f);
+        tex_coord_buffer.store(0.0f, 1.0f);
+    }
 
+    glPushAttrib(GL_ENABLE_BIT);
     glEnable(GL_TEXTURE_2D);
-    glPushClientAttrib(GL_CLIENT_ALL_ATTRIB_BITS);
 
+    glPushClientAttrib(GL_CLIENT_ALL_ATTRIB_BITS);
     glEnableClientState(GL_VERTEX_ARRAY);
     glDisableClientState(GL_COLOR_ARRAY);
     glEnableClientState(GL_TEXTURE_COORD_ARRAY);
 
-    glVertexPointer(2, GL_INT, sizeof (GLint)*2, &position_data);
-    glTexCoordPointer(2, GL_FLOAT, 0, &texture_coordinate_data);
-    glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
+    glBindTexture(GL_TEXTURE_2D, m_opengl_id);
+    vertex_buffer.activate();
+    tex_coord_buffer.activate();
+    glDrawArrays(GL_TRIANGLE_STRIP, 0, vertex_buffer.size());
 
-    glDisableClientState(GL_VERTEX_ARRAY);
-    glDisableClientState(GL_TEXTURE_COORD_ARRAY);
+    //glDisableClientState(GL_VERTEX_ARRAY);
+    //glDisableClientState(GL_TEXTURE_COORD_ARRAY);
 
     if (need_min_filter_change)
         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, m_min_filter);
@@ -183,6 +187,8 @@ void Texture::OrthoBlit(const Pt& pt1, const Pt& pt2, const GLfloat* tex_coords/
         glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, m_mag_filter);
 
     glPopClientAttrib();
+
+    glPopAttrib();
 }
 
 void Texture::OrthoBlit(const Pt& pt) const
@@ -240,12 +246,12 @@ void Texture::Load(const boost::filesystem::path& path, bool mipmap/* = false*/)
         // formats above.
 #if GG_HAVE_LIBPNG
         if (extension == ".png")
-            gil::png_read_image(path, image);
+            gil::read_image(filename, image, gil::image_read_settings<gil::png_tag>());
         else
 #endif
 #if GG_HAVE_LIBTIFF
         if (extension == ".tif" || extension == ".tiff")
-            gil::tiff_read_image(filename, image);
+            gil::read_image(filename, image, gil::image_read_settings<gil::tiff_tag>());
         else
 #endif
             throw BadFile("Texture file \"" + filename + "\" does not have a supported file extension");
@@ -255,14 +261,14 @@ void Texture::Load(const boost::filesystem::path& path, bool mipmap/* = false*/)
 #if GG_HAVE_LIBPNG
         if (extension == ".png") {
             gil::rgba8_image_t rgba_image;
-            gil::png_read_and_convert_image(path, rgba_image);
+            gil::read_and_convert_image(filename, rgba_image, gil::image_read_settings<gil::png_tag>());
             image.move_in(rgba_image);
         }
 #endif
 #if GG_HAVE_LIBTIFF
         if (extension == ".tif" || extension == ".tiff") {
             gil::rgba8_image_t rgba_image;
-            gil::tiff_read_and_convert_image(filename, rgba_image);
+            gil::read_and_convert_image(filename, rgba_image, gil::image_read_settings<gil::tiff_tag>());
             image.move_in(rgba_image);
         }
 #endif
@@ -443,9 +449,9 @@ void Texture::InitFromRawData(X width, Y height, const unsigned char* image, GLe
             image_copy.reset(GetRawBytes());
         unsigned char* image_to_use = image_copy ? image_copy.get() : const_cast<unsigned char*>(image);
         gluBuild2DMipmaps(GL_PROXY_TEXTURE_2D, format, Value(GL_texture_width), Value(GL_texture_height), format, type, image_to_use);
-        GLint checked_format;
-        glGetTexLevelParameteriv(GL_PROXY_TEXTURE_2D, 0, GL_TEXTURE_INTERNAL_FORMAT, &checked_format);
-        if (!checked_format)
+        GLint mipmap_checked_format;
+        glGetTexLevelParameteriv(GL_PROXY_TEXTURE_2D, 0, GL_TEXTURE_INTERNAL_FORMAT, &mipmap_checked_format);
+        if (!mipmap_checked_format)
             throw InsufficientResources("Insufficient resources to create requested mipmapped OpenGL texture");
         gluBuild2DMipmaps(GL_TEXTURE_2D, format, Value(GL_texture_width), Value(GL_texture_height), format, type, image_to_use);
     } else {
