@@ -24,14 +24,14 @@
 
 #include <GG/dialogs/ColorDlg.h>
 
-#include <GG/GUI.h>
 #include <GG/DrawUtil.h>
 #include <GG/Font.h>
+#include <GG/GLClientAndServerBuffer.h>
+#include <GG/GUI.h>
 #include <GG/Layout.h>
 #include <GG/Slider.h>
 #include <GG/StyleFactory.h>
 #include <GG/WndEvent.h>
-#include <GG/GLClientAndServerBuffer.h>
 
 
 using namespace GG;
@@ -381,7 +381,6 @@ void ValuePicker::SetValueFromPt(Pt pt)
 // ColorDlg
 ////////////////////////////////////////////////
 
-// ColorDlg::ColorButton
 ColorDlg::ColorButton::ColorButton(const Clr& color) :
     Button("", nullptr, color),
     m_represented_color(CLR_BLACK)
@@ -514,7 +513,151 @@ ColorDlg::ColorDlg(X x, Y y, Clr original_color, const std::shared_ptr<Font>& fo
     m_color(dialog_color),
     m_border_color(border_color),
     m_text_color(text_color)
-{ Init(font); }
+{
+    m_current_color = m_original_color_specified ? Convert(m_original_color) : Convert(CLR_BLACK);
+    Clr color = Convert(m_current_color);
+
+    const auto& style = GetStyleFactory();
+
+    const int COLOR_BUTTON_ROWS = 4;
+    const int COLOR_BUTTON_COLS = 5;
+    if (s_custom_colors.empty()) {
+        s_custom_colors = { GG::CLR_WHITE,      GG::CLR_LIGHT_GRAY, GG::CLR_GRAY,       GG::CLR_DARK_GRAY,  GG::CLR_BLACK,
+                            GG::CLR_PINK,       GG::CLR_RED,        GG::CLR_DARK_RED,   GG::CLR_MAGENTA,    GG::CLR_PURPLE,
+                            GG::CLR_BLUE,       GG::CLR_DARK_BLUE,  GG::CLR_TEAL,       GG::CLR_CYAN,       GG::CLR_GREEN,
+                            GG::CLR_DARK_GREEN, GG::CLR_OLIVE,      GG::CLR_YELLOW,     GG::CLR_ORANGE};
+
+        for (unsigned int i = s_custom_colors.size(); i < COLOR_BUTTON_ROWS * COLOR_BUTTON_COLS; ++i) {
+            s_custom_colors.push_back(CLR_GRAY);
+        }
+    }
+
+    m_hue_saturation_picker = Wnd::Create<HueSaturationPicker>(X(10), Y(10), X(300), Y(300));
+    m_hue_saturation_picker->SetHueSaturation(m_current_color.h, m_current_color.s);
+    m_value_picker = Wnd::Create<ValuePicker>(X(320), Y(10), X(25), Y(300), m_text_color);
+    m_value_picker->SetHueSaturation(m_current_color.h, m_current_color.s);
+    m_value_picker->SetValue(m_current_color.v);
+    const int HUE_SATURATION_PICKER_SIZE = 200;
+    m_pickers_layout = Wnd::Create<Layout>(X0, Y0, X(HUE_SATURATION_PICKER_SIZE + 30), Y(HUE_SATURATION_PICKER_SIZE),
+                                           1, 2, 0, 5);
+    m_pickers_layout->SetColumnStretch(0, 1);
+    m_pickers_layout->SetMinimumColumnWidth(1, X(24));
+    m_pickers_layout->Add(m_hue_saturation_picker, 0, 0);
+    m_pickers_layout->Add(m_value_picker, 0, 1);
+
+    m_color_squares_layout = Wnd::Create<Layout>(X0, m_pickers_layout->Bottom() + 5, m_pickers_layout->Width(), Y(40),
+                                                 1, 1, 0, 4);
+    m_new_color_square = Wnd::Create<ColorDisplay>(color);
+    if (m_original_color_specified) {
+        m_new_color_square_text = style->NewTextControl(style->Translate("New"), font, m_text_color, FORMAT_RIGHT);
+        m_color_squares_layout->Add(m_new_color_square_text, 0, 0);
+        m_color_squares_layout->Add(m_new_color_square, 0, 1);
+        m_old_color_square_text = style->NewTextControl(style->Translate("Old"), font, m_text_color, FORMAT_RIGHT);
+        m_color_squares_layout->Add(m_old_color_square_text, 1, 0);
+        m_old_color_square = Wnd::Create<ColorDisplay>(m_original_color);
+        m_color_squares_layout->Add(m_old_color_square, 1, 1);
+        m_color_squares_layout->SetMinimumColumnWidth(0, X(30));
+        m_color_squares_layout->SetColumnStretch(1, 1);
+    } else {
+        m_color_squares_layout->Add(m_new_color_square, 0, 0);
+    }
+
+    m_color_buttons_layout = Wnd::Create<Layout>(X0, m_color_squares_layout->Bottom() + 5, m_pickers_layout->Width(), Y(80),
+                                                 COLOR_BUTTON_ROWS, COLOR_BUTTON_COLS, 0, 4);
+    for (int i = 0; i < COLOR_BUTTON_ROWS; ++i) {
+        for (int j = 0; j < COLOR_BUTTON_COLS; ++j) {
+            m_color_buttons.push_back(Wnd::Create<ColorButton>(m_color));
+            m_color_buttons.back()->SetRepresentedColor(s_custom_colors[i * COLOR_BUTTON_COLS + j]);
+            m_color_buttons_layout->Add(m_color_buttons.back(), i, j);
+        }
+    }
+
+    m_sliders_ok_cancel_layout = Wnd::Create<Layout>(m_pickers_layout->Right() + 5, Y0, X(150), Y((25 + 5) * 8 - 5),
+                                                     9, 3, 0, 5);
+    m_sliders_ok_cancel_layout->SetMinimumColumnWidth(0, X(15));
+    m_sliders_ok_cancel_layout->SetMinimumColumnWidth(1, X(30));
+    m_sliders_ok_cancel_layout->SetColumnStretch(2, 1);
+
+    int row = 0;
+
+    for (auto entry : {
+            std::make_tuple(static_cast<int>(color.r), 0, 255, "R:"),
+            std::make_tuple(static_cast<int>(color.g), 0, 255, "G:"),
+            std::make_tuple(static_cast<int>(color.b), 0, 255, "B:"),
+            std::make_tuple(static_cast<int>(color.a), 0, 255, "A:"),
+            std::make_tuple(static_cast<int>(m_current_color.h * 359), 0, 359, "H:"),
+            std::make_tuple(static_cast<int>(m_current_color.s * 255), 0, 255, "S:"),
+            std::make_tuple(static_cast<int>(m_current_color.v * 255), 0, 255, "V:")
+        })
+    {
+        auto color_value = std::get<0>(entry);
+        auto color_min   = std::get<1>(entry);
+        auto color_max   = std::get<2>(entry);
+        auto color_label = std::get<3>(entry);
+
+        m_slider_labels.push_back(style->NewTextControl(style->Translate(color_label), font, m_text_color, FORMAT_RIGHT));
+        m_sliders_ok_cancel_layout->Add(m_slider_labels.back(), row, 0);
+        m_slider_values.push_back(style->NewTextControl(std::to_string(color_value),
+                                                        font, m_text_color, FORMAT_LEFT));
+        m_sliders_ok_cancel_layout->Add(m_slider_values.back(), row, 1);
+        m_sliders.push_back(style->NewIntSlider(color_min, color_max, HORIZONTAL, m_color, 10));
+        m_sliders.back()->SlideTo(color_value);
+        m_sliders_ok_cancel_layout->Add(m_sliders.back(), row, 2);
+
+        ++row;
+    }
+
+    m_ok = style->NewButton(style->Translate("Ok"), font, m_color, m_text_color);
+    m_sliders_ok_cancel_layout->Add(m_ok, 7, 0, 1, 3);
+    m_cancel = style->NewButton(style->Translate("Cancel"), font, m_color, m_text_color);
+    m_sliders_ok_cancel_layout->Add(m_cancel, 8, 0, 1, 3);
+}
+
+void ColorDlg::CompleteConstruction()
+{
+    Wnd::CompleteConstruction();
+
+    auto master_layout = Wnd::Create<Layout>(X0, Y0, ClientWidth(), ClientHeight(), 3, 2, 5, 5);
+    master_layout->SetColumnStretch(0, 1.25);
+    master_layout->SetColumnStretch(1, 1);
+    master_layout->SetRowStretch(0, 1.25);
+    master_layout->SetMinimumRowHeight(1, Y(40));
+    master_layout->SetRowStretch(2, 1);
+    master_layout->Add(m_pickers_layout, 0, 0);
+    master_layout->Add(m_color_squares_layout, 1, 0);
+    master_layout->Add(m_color_buttons_layout, 2, 0);
+    master_layout->Add(m_sliders_ok_cancel_layout, 0, 1, 3, 1);
+    SetLayout(master_layout);
+
+    for (std::size_t i = 0; i < m_color_buttons.size(); ++i) {
+        m_color_buttons[i]->LeftClickedSignal.connect(
+            [this, i](){ this->ColorButtonClicked(i); });
+    }
+    m_sliders[R]->SlidSignal.connect(
+        boost::bind(&ColorDlg::RedSliderChanged, this, _1, _2, _3));
+    m_sliders[G]->SlidSignal.connect(
+        boost::bind(&ColorDlg::GreenSliderChanged, this, _1, _2, _3));
+    m_sliders[B]->SlidSignal.connect(
+        boost::bind(&ColorDlg::BlueSliderChanged, this, _1, _2, _3));
+    m_sliders[A]->SlidSignal.connect(
+        boost::bind(&ColorDlg::AlphaSliderChanged, this, _1, _2, _3));
+    m_sliders[H]->SlidSignal.connect(
+        boost::bind(&ColorDlg::HueSliderChanged, this, _1, _2, _3));
+    m_sliders[S]->SlidSignal.connect(
+        boost::bind(&ColorDlg::SaturationSliderChanged, this, _1, _2, _3));
+    m_sliders[V]->SlidSignal.connect(
+        boost::bind(&ColorDlg::ValueSliderChanged, this, _1, _2, _3));
+    m_ok->LeftClickedSignal.connect(
+        boost::bind(&ColorDlg::OkClicked, this));
+    m_cancel->LeftClickedSignal.connect(
+        boost::bind(&ColorDlg::CancelClicked, this));
+    m_hue_saturation_picker->ChangedSignal.connect(
+        boost::bind(&ValuePicker::SetHueSaturation, m_value_picker, _1, _2));
+    m_hue_saturation_picker->ChangedSignal.connect(
+        boost::bind(&ColorDlg::HueSaturationPickerChanged, this, _1, _2));
+    m_value_picker->ChangedSignal.connect(
+        boost::bind(&ColorDlg::ValuePickerChanged, this, _1));
+}
 
 bool ColorDlg::ColorWasSelected() const
 { return m_color_was_picked; }
@@ -541,173 +684,6 @@ void ColorDlg::KeyPress(Key key, std::uint32_t key_code_point, Flags<ModKey> mod
         CancelClicked();
 }
 
-void ColorDlg::Init(const std::shared_ptr<Font>& font)
-{
-    m_current_color = m_original_color_specified ? Convert(m_original_color) : Convert(CLR_BLACK);
-    Clr color = Convert(m_current_color);
-
-    std::shared_ptr<StyleFactory> style = GetStyleFactory();
-
-    const int COLOR_BUTTON_ROWS = 4;
-    const int COLOR_BUTTON_COLS = 5;
-    if (s_custom_colors.empty()) {
-        s_custom_colors = { GG::CLR_WHITE,      GG::CLR_LIGHT_GRAY, GG::CLR_GRAY,       GG::CLR_DARK_GRAY,  GG::CLR_BLACK,
-                            GG::CLR_PINK,       GG::CLR_RED,        GG::CLR_DARK_RED,   GG::CLR_MAGENTA,    GG::CLR_PURPLE,
-                            GG::CLR_BLUE,       GG::CLR_DARK_BLUE,  GG::CLR_TEAL,       GG::CLR_CYAN,       GG::CLR_GREEN,
-                            GG::CLR_DARK_GREEN, GG::CLR_OLIVE,      GG::CLR_YELLOW,     GG::CLR_ORANGE};
-
-        for (unsigned int i = s_custom_colors.size(); i < COLOR_BUTTON_ROWS * COLOR_BUTTON_COLS; ++i) {
-            s_custom_colors.push_back(CLR_GRAY);
-        }
-    }
-
-    m_hue_saturation_picker = new HueSaturationPicker(X(10), Y(10), X(300), Y(300));
-    m_hue_saturation_picker->SetHueSaturation(m_current_color.h, m_current_color.s);
-    m_value_picker = new ValuePicker(X(320), Y(10), X(25), Y(300), m_text_color);
-    m_value_picker->SetHueSaturation(m_current_color.h, m_current_color.s);
-    m_value_picker->SetValue(m_current_color.v);
-    const int HUE_SATURATION_PICKER_SIZE = 200;
-    m_pickers_layout = new Layout(X0, Y0, X(HUE_SATURATION_PICKER_SIZE + 30), Y(HUE_SATURATION_PICKER_SIZE),
-                                  1, 2, 0, 5);
-    m_pickers_layout->SetColumnStretch(0, 1);
-    m_pickers_layout->SetMinimumColumnWidth(1, X(24));
-    m_pickers_layout->Add(m_hue_saturation_picker, 0, 0);
-    m_pickers_layout->Add(m_value_picker, 0, 1);
-
-    m_color_squares_layout = new Layout(X0, m_pickers_layout->Bottom() + 5, m_pickers_layout->Width(), Y(40),
-                                        1, 1, 0, 4);
-    m_new_color_square = new ColorDisplay(color);
-    if (m_original_color_specified) {
-        m_new_color_square_text = style->NewTextControl(style->Translate("New"), font, m_text_color, FORMAT_RIGHT);
-        m_color_squares_layout->Add(m_new_color_square_text, 0, 0);
-        m_color_squares_layout->Add(m_new_color_square, 0, 1);
-        m_old_color_square_text = style->NewTextControl(style->Translate("Old"), font, m_text_color, FORMAT_RIGHT);
-        m_color_squares_layout->Add(m_old_color_square_text, 1, 0);
-        m_old_color_square = new ColorDisplay(m_original_color);
-        m_color_squares_layout->Add(m_old_color_square, 1, 1);
-        m_color_squares_layout->SetMinimumColumnWidth(0, X(30));
-        m_color_squares_layout->SetColumnStretch(1, 1);
-    } else {
-        m_color_squares_layout->Add(m_new_color_square, 0, 0);
-    }
-
-    m_color_buttons_layout = new Layout(X0, m_color_squares_layout->Bottom() + 5, m_pickers_layout->Width(), Y(80),
-                                        COLOR_BUTTON_ROWS, COLOR_BUTTON_COLS, 0, 4);
-    for (int i = 0; i < COLOR_BUTTON_ROWS; ++i) {
-        for (int j = 0; j < COLOR_BUTTON_COLS; ++j) {
-            m_color_buttons.push_back(new ColorButton(m_color));
-            m_color_buttons.back()->SetRepresentedColor(s_custom_colors[i * COLOR_BUTTON_COLS + j]);
-            m_color_buttons_layout->Add(m_color_buttons.back(), i, j);
-        }
-    }
-
-    m_sliders_ok_cancel_layout = new Layout(m_pickers_layout->Right() + 5, Y0, X(150), Y((25 + 5) * 8 - 5),
-                                            9, 3, 0, 5);
-    m_sliders_ok_cancel_layout->SetMinimumColumnWidth(0, X(15));
-    m_sliders_ok_cancel_layout->SetMinimumColumnWidth(1, X(30));
-    m_sliders_ok_cancel_layout->SetColumnStretch(2, 1);
-    m_slider_labels.push_back(style->NewTextControl(style->Translate("R:"), font, m_text_color, FORMAT_RIGHT));
-    m_sliders_ok_cancel_layout->Add(m_slider_labels.back(), 0, 0);
-    m_slider_values.push_back(style->NewTextControl(std::to_string(color.r),
-                                                    font, m_text_color, FORMAT_LEFT));
-    m_sliders_ok_cancel_layout->Add(m_slider_values.back(), 0, 1);
-    m_sliders.push_back(style->NewIntSlider(0, 255, HORIZONTAL, m_color, 10));
-    m_sliders.back()->SlideTo(color.r);
-    m_sliders_ok_cancel_layout->Add(m_sliders.back(), 0, 2);
-
-    m_slider_labels.push_back(style->NewTextControl(style->Translate("G:"), font, m_text_color, FORMAT_RIGHT));
-    m_sliders_ok_cancel_layout->Add(m_slider_labels.back(), 1, 0);
-    m_slider_values.push_back(style->NewTextControl(std::to_string(color.g),
-                                                    font, m_text_color, FORMAT_LEFT));
-    m_sliders_ok_cancel_layout->Add(m_slider_values.back(), 1, 1);
-    m_sliders.push_back(style->NewIntSlider(0, 255, HORIZONTAL, m_color, 10));
-    m_sliders.back()->SlideTo(color.g);
-    m_sliders_ok_cancel_layout->Add(m_sliders.back(), 1, 2);
-
-    m_slider_labels.push_back(style->NewTextControl(style->Translate("B:"), font, m_text_color, FORMAT_RIGHT));
-    m_sliders_ok_cancel_layout->Add(m_slider_labels.back(), 2, 0);
-    m_slider_values.push_back(style->NewTextControl(std::to_string(color.b),
-                                                    font, m_text_color, FORMAT_LEFT));
-    m_sliders_ok_cancel_layout->Add(m_slider_values.back(), 2, 1);
-    m_sliders.push_back(style->NewIntSlider(0, 255, HORIZONTAL, m_color, 10));
-    m_sliders.back()->SlideTo(color.b);
-    m_sliders_ok_cancel_layout->Add(m_sliders.back(), 2, 2);
-
-    m_slider_labels.push_back(style->NewTextControl(style->Translate("A:"), font, m_text_color, FORMAT_RIGHT));
-    m_sliders_ok_cancel_layout->Add(m_slider_labels.back(), 3, 0);
-    m_slider_values.push_back(style->NewTextControl(std::to_string(color.a),
-                                                    font, m_text_color, FORMAT_LEFT));
-    m_sliders_ok_cancel_layout->Add(m_slider_values.back(), 3, 1);
-    m_sliders.push_back(style->NewIntSlider(0, 255, HORIZONTAL, m_color, 10));
-    m_sliders.back()->SlideTo(color.a);
-    m_sliders_ok_cancel_layout->Add(m_sliders.back(), 3, 2);
-
-    m_slider_labels.push_back(style->NewTextControl(style->Translate("H:"), font, m_text_color, FORMAT_RIGHT));
-    m_sliders_ok_cancel_layout->Add(m_slider_labels.back(), 4, 0);
-    m_slider_values.push_back(style->NewTextControl(std::to_string(static_cast<int>(m_current_color.h * 359)),
-                                                    font, m_text_color, FORMAT_LEFT));
-    m_sliders_ok_cancel_layout->Add(m_slider_values.back(), 4, 1);
-    m_sliders.push_back(style->NewIntSlider(0, 359, HORIZONTAL, m_color, 10));
-    m_sliders.back()->SlideTo(static_cast<int>(m_current_color.h * 359));
-    m_sliders_ok_cancel_layout->Add(m_sliders.back(), 4, 2);
-
-    m_slider_labels.push_back(style->NewTextControl(style->Translate("S:"), font, m_text_color, FORMAT_RIGHT));
-    m_sliders_ok_cancel_layout->Add(m_slider_labels.back(), 5, 0);
-    m_slider_values.push_back(style->NewTextControl(std::to_string(static_cast<int>(m_current_color.s * 255)),
-                                                    font, m_text_color, FORMAT_LEFT));
-    m_sliders_ok_cancel_layout->Add(m_slider_values.back(), 5, 1);
-    m_sliders.push_back(style->NewIntSlider(0, 255, HORIZONTAL, m_color, 10));
-    m_sliders.back()->SlideTo(static_cast<int>(m_current_color.s * 255));
-    m_sliders_ok_cancel_layout->Add(m_sliders.back(), 5, 2);
-
-    m_slider_labels.push_back(style->NewTextControl(style->Translate("V:"), font, m_text_color, FORMAT_RIGHT));
-    m_sliders_ok_cancel_layout->Add(m_slider_labels.back(), 6, 0);
-    m_slider_values.push_back(style->NewTextControl(std::to_string(static_cast<int>(m_current_color.v * 255)),
-                                                    font, m_text_color, FORMAT_LEFT));
-    m_sliders_ok_cancel_layout->Add(m_slider_values.back(), 6, 1);
-    m_sliders.push_back(style->NewIntSlider(0, 255, HORIZONTAL, m_color, 10));
-    m_sliders.back()->SlideTo(static_cast<int>(m_current_color.v * 255));
-    m_sliders_ok_cancel_layout->Add(m_sliders.back(), 6, 2);
-
-    m_ok = style->NewButton(style->Translate("Ok"), font, m_color, m_text_color);
-    m_sliders_ok_cancel_layout->Add(m_ok, 7, 0, 1, 3);
-    m_cancel = style->NewButton(style->Translate("Cancel"), font, m_color, m_text_color);
-    m_sliders_ok_cancel_layout->Add(m_cancel, 8, 0, 1, 3);
-
-    Layout* master_layout = new Layout(X0, Y0, ClientWidth(), ClientHeight(), 3, 2, 5, 5);
-    master_layout->SetColumnStretch(0, 1.25);
-    master_layout->SetColumnStretch(1, 1);
-    master_layout->SetRowStretch(0, 1.25);
-    master_layout->SetMinimumRowHeight(1, Y(40));
-    master_layout->SetRowStretch(2, 1);
-    master_layout->Add(m_pickers_layout, 0, 0);
-    master_layout->Add(m_color_squares_layout, 1, 0);
-    master_layout->Add(m_color_buttons_layout, 2, 0);
-    master_layout->Add(m_sliders_ok_cancel_layout, 0, 1, 3, 1);
-    SetLayout(master_layout);
-
-    ConnectSignals();
-}
-
-void ColorDlg::ConnectSignals()
-{
-    for (std::size_t i = 0; i < m_color_buttons.size(); ++i) {
-        Connect(m_color_buttons[i]->LeftClickedSignal, [this, i](){ this->ColorButtonClicked(i); });
-    }
-    Connect(m_sliders[R]->SlidSignal, &ColorDlg::RedSliderChanged, this);
-    Connect(m_sliders[G]->SlidSignal, &ColorDlg::GreenSliderChanged, this);
-    Connect(m_sliders[B]->SlidSignal, &ColorDlg::BlueSliderChanged, this);
-    Connect(m_sliders[A]->SlidSignal, &ColorDlg::AlphaSliderChanged, this);
-    Connect(m_sliders[H]->SlidSignal, &ColorDlg::HueSliderChanged, this);
-    Connect(m_sliders[S]->SlidSignal, &ColorDlg::SaturationSliderChanged, this);
-    Connect(m_sliders[V]->SlidSignal, &ColorDlg::ValueSliderChanged, this);
-    Connect(m_ok->LeftClickedSignal, &ColorDlg::OkClicked, this);
-    Connect(m_cancel->LeftClickedSignal, &ColorDlg::CancelClicked, this);
-    Connect(m_hue_saturation_picker->ChangedSignal, &ValuePicker::SetHueSaturation, m_value_picker);
-    Connect(m_hue_saturation_picker->ChangedSignal, &ColorDlg::HueSaturationPickerChanged, this);
-    Connect(m_value_picker->ChangedSignal, &ColorDlg::ValuePickerChanged, this);
-}
-
 void ColorDlg::ColorChanged(HSVClr color)
 {
     m_current_color = color;
