@@ -18,11 +18,15 @@
 About::About():
     CUIWnd(UserString("ABOUT_WINDOW_TITLE"), GG::X(80), GG::Y(130), GG::X(600), GG::Y(500),
            GG::INTERACTIVE | GG::DRAGABLE | GG::MODAL)
-{
-    m_done_btn = new CUIButton(UserString("DONE"));
-    m_license = new CUIButton(UserString("LICENSE"));
-    m_vision = new CUIButton(UserString("VISION"));
-    m_info = new CUIMultiEdit(UserString("FREEORION_VISION"), GG::MULTI_WORDBREAK | GG::MULTI_READ_ONLY);
+{}
+
+void About::CompleteConstruction() {
+    CUIWnd::CompleteConstruction();
+
+    m_done_btn = Wnd::Create<CUIButton>(UserString("DONE"));
+    m_license = Wnd::Create<CUIButton>(UserString("LICENSE"));
+    m_vision = Wnd::Create<CUIButton>(UserString("VISION"));
+    m_info = GG::Wnd::Create<CUIMultiEdit>(UserString("FREEORION_VISION"), GG::MULTI_WORDBREAK | GG::MULTI_READ_ONLY);
     AttachChild(m_info);
     AttachChild(m_vision);
     AttachChild(m_license);
@@ -41,9 +45,12 @@ About::About():
     }
     fin.close();
 
-    GG::Connect(m_done_btn->LeftClickedSignal, boost::bind(&GG::Wnd::EndRun, this));
-    GG::Connect(m_license->LeftClickedSignal, &About::ShowLicense, this);
-    GG::Connect(m_vision->LeftClickedSignal, &About::ShowVision, this);
+    m_done_btn->LeftClickedSignal.connect(
+        boost::bind(&GG::Wnd::EndRun, this));
+    m_license->LeftClickedSignal.connect(
+        boost::bind(&About::ShowLicense, this));
+    m_vision->LeftClickedSignal.connect(
+        boost::bind(&About::ShowVision, this));
 }
 
 About::~About()
