@@ -12,19 +12,20 @@
 #include "../util/Directories.h"
 #include "../util/Logger.h"
 
-#include <GG/GUI.h>
 #include <GG/DrawUtil.h>
+#include <GG/GUI.h>
 
 #include <limits>
+#include <boost/algorithm/string.hpp>
 
 
 namespace {
     void PlayMinimizeSound()
-    { Sound::GetSound().PlaySound(GetOptionsDB().Get<std::string>("UI.sound.window-maximize"), true); }
+    { Sound::GetSound().PlaySound(GetOptionsDB().Get<std::string>("ui.window.maximize.sound.path"), true); }
     void PlayMaximizeSound()
-    { Sound::GetSound().PlaySound(GetOptionsDB().Get<std::string>("UI.sound.window-minimize"), true); }
+    { Sound::GetSound().PlaySound(GetOptionsDB().Get<std::string>("ui.window.minimize.sound.path"), true); }
     void PlayCloseSound()
-    { Sound::GetSound().PlaySound(GetOptionsDB().Get<std::string>("UI.sound.window-close"), true); }
+    { Sound::GetSound().PlaySound(GetOptionsDB().Get<std::string>("ui.window.close.sound.path"), true); }
 
     void AddOptions(OptionsDB& db) {
         db.AddFlag('w', "window-reset", UserStringNop("OPTIONS_DB_WINDOW_RESET"), false);
@@ -44,7 +45,10 @@ namespace {
 CUI_MinRestoreButton::CUI_MinRestoreButton() :
     GG::Button("", nullptr, ClientUI::WndInnerBorderColor()),
     m_mode(MIN_BUTTON)
-{ GG::Connect(LeftClickedSignal, &CUI_MinRestoreButton::Toggle, this); }
+{
+    LeftClickedSignal.connect(
+        boost::bind(&CUI_MinRestoreButton::Toggle, this));
+}
 
 void CUI_MinRestoreButton::Render() {
     GG::Pt ul = UpperLeft();
@@ -80,7 +84,8 @@ void CUI_MinRestoreButton::Toggle() {
 CUI_PinButton::CUI_PinButton() :
     GG::Button("", nullptr, ClientUI::WndInnerBorderColor())
 {
-    GG::Connect(LeftClickedSignal, &PlayCloseSound, -1);
+    LeftClickedSignal.connect(-1,
+        &PlayCloseSound);
     SetUnpressedGraphic(GG::SubTexture(ClientUI::GetTexture( ClientUI::ArtDir() / "icons" / "buttons" / "pin.png"   )));
     SetPressedGraphic  (GG::SubTexture(ClientUI::GetTexture( ClientUI::ArtDir() / "icons" / "buttons" / "pin.png"  )));
     SetRolloverGraphic (GG::SubTexture(ClientUI::GetTexture( ClientUI::ArtDir() / "icons" / "buttons" / "pin_mouseover.png")));
@@ -113,6 +118,23 @@ namespace {
         return true;
     }
     bool dummy = RegisterWndFlags();
+
+    std::string WindowNameFromOption(const std::string& option_name) {
+        std::string::size_type prefix_len { std::string("ui.").length() };
+
+        // Determine end of window name from start of window mode
+        std::string::size_type mode_substr_pos { option_name.find(".fullscreen", prefix_len + 1) };
+        if (mode_substr_pos == std::string::npos) {
+            mode_substr_pos = option_name.find(".windowed", prefix_len + 1);
+            if (mode_substr_pos == std::string::npos) {
+                ErrorLogger() << "Could not determine window name from option " << option_name;
+                return {};
+            }
+        }
+
+        std::string::size_type name_len = mode_substr_pos - prefix_len;
+        return option_name.substr(prefix_len, name_len);
+    }
 }
 
 const GG::Y CUIWnd::BUTTON_TOP_OFFSET(3);
@@ -136,25 +158,15 @@ CUIWnd::CUIWnd(const std::string& wnd_name,
     m_resizable(flags & GG::RESIZABLE),
     m_closable(flags & CLOSABLE),
     m_minimizable(flags & MINIMIZABLE),
-    m_minimized(false),
     m_pinable(flags & PINABLE),
-    m_pinned(false),
     m_drag_offset(-GG::X1, -GG::Y1),
-    m_mouse_in_resize_tab(false),
-    m_config_save(true),
-    m_config_name(AddWindowOptions(config_name, x, y, w, h, visible, false, false)),
-    m_close_button(nullptr),
-    m_minimize_button(nullptr),
-    m_pin_button(nullptr),
-    m_vertex_buffer(),
-    m_buffer_indices()
+    m_config_name(AddWindowOptions(config_name, x, y, w, h, visible, false, false))
 {
-    Init(wnd_name);
+    SetName(wnd_name);
     if (!m_config_name.empty()) {
         // Default position was already supplied
-        GetOptionsDB().Set<bool>("UI.windows." + m_config_name + ".initialized", true);
+        GetOptionsDB().Set<bool>("ui." + m_config_name + ".initialized", true);
     }
-    ValidatePosition();
 }
 
 CUIWnd::CUIWnd(const std::string& wnd_name, GG::Flags<GG::WndFlag> flags, const std::string& config_name, bool visible) :
@@ -162,28 +174,26 @@ CUIWnd::CUIWnd(const std::string& wnd_name, GG::Flags<GG::WndFlag> flags, const
     m_resizable(flags & GG::RESIZABLE),
     m_closable(flags & CLOSABLE),
     m_minimizable(flags & MINIMIZABLE),
-    m_minimized(false),
     m_pinable(flags & PINABLE),
-    m_pinned(false),
     m_drag_offset(-GG::X1, -GG::Y1),
-    m_mouse_in_resize_tab(false),
-    m_config_save(true),
-    m_config_name(AddWindowOptions(config_name, INVALID_POS, INVALID_POS, 1, 1, visible, false, false)),
-    m_close_button(nullptr),
-    m_minimize_button(nullptr),
-    m_pin_button(nullptr),
-    m_vertex_buffer(),
-    m_buffer_indices()
-{ Init(wnd_name); }
+    m_config_name(AddWindowOptions(config_name, INVALID_POS, INVALID_POS, 1, 1, visible, false, false))
+{ SetName(wnd_name); }
 
-void CUIWnd::Init(const std::string& wnd_name) {
-    SetName(wnd_name);
+void CUIWnd::CompleteConstruction() {
+    GG::Wnd::CompleteConstruction();
+    Init();
+    ValidatePosition();
+    SetDefaultedOptions();
+}
+
+void CUIWnd::Init() {
     InitButtons();
     SetChildClippingMode(ClipToClientAndWindowSeparately);
 
     if (!m_config_name.empty()) {
         LoadOptions();
-        GG::Connect(HumanClientApp::GetApp()->FullscreenSwitchSignal, boost::bind(&CUIWnd::LoadOptions, this));
+        HumanClientApp::GetApp()->FullscreenSwitchSignal.connect(
+            boost::bind(&CUIWnd::LoadOptions, this));
     }
 
     // User-dragable windows recalculate their position only when told to (e.g.
@@ -191,29 +201,32 @@ void CUIWnd::Init(const std::string& wnd_name) {
     // Non-user-dragable windows are given the chance to position themselves on
     // every resize event.
     if (Dragable() || m_resizable)
-        GG::Connect(HumanClientApp::GetApp()->RepositionWindowsSignal, &CUIWnd::ResetDefaultPosition, this);
+        HumanClientApp::GetApp()->RepositionWindowsSignal.connect(
+            boost::bind(&CUIWnd::ResetDefaultPosition, this));
     else
-        GG::Connect(HumanClientApp::GetApp()->WindowResizedSignal, boost::bind(&CUIWnd::ResetDefaultPosition, this));
-
-    // call to CUIWnd::MinimizedWidth() because MinimizedWidth is virtual
-    SetMinSize(GG::Pt(CUIWnd::MinimizedSize().x, TopBorder() + INNER_BORDER_ANGLE_OFFSET + BORDER_BOTTOM + 50));
+        HumanClientApp::GetApp()->WindowResizedSignal.connect(
+            boost::bind(&CUIWnd::ResetDefaultPosition, this));
 }
 
 void CUIWnd::InitSizeMove(const GG::Pt& ul, const GG::Pt& lr) {
     OptionsDB& db = GetOptionsDB();
 
     if (!m_config_name.empty()) {
-        if (db.OptionExists("UI.windows." + m_config_name + ".initialized")) {
-            std::string windowed = ""; // empty string in fullscreen mode, appends -windowed in windowed mode
-            if (!db.Get<bool>("fullscreen"))
-                windowed = "-windowed";
+        std::string option_prefix = "ui." + m_config_name;
+        if (db.OptionExists(option_prefix + ".initialized")) {
+            std::string window_mode = db.Get<bool>("video.fullscreen.enabled") ?
+                                      ".fullscreen" : ".windowed";
             // If the window has already had its default position specified
             // (either in the ctor or a previous call to this function), apply
             // this position to the window.
-            if (db.Get<bool>("UI.windows." + m_config_name + ".initialized") ||
-                db.Get<int>("UI.windows." + m_config_name + ".left" + windowed) == INVALID_X)
-            { SizeMove(ul, lr); }
-            db.Set<bool>("UI.windows."+m_config_name+".initialized", true);
+            if (db.Get<bool>(option_prefix + ".initialized") ||
+                db.Get<int>(option_prefix + window_mode + ".left") == INVALID_X)
+            {
+                SetDefaultedOptions();
+                SizeMove(ul, lr);
+                SaveDefaultedOptions();
+            }
+            db.Set<bool>(option_prefix + ".initialized", true);
         } else {
             ErrorLogger() << "CUIWnd::InitSizeMove() : attempted to check if window using name \"" << m_config_name
                           << "\" was initialized but the options do not appear to be registered in the OptionsDB.";
@@ -225,10 +238,10 @@ void CUIWnd::InitSizeMove(const GG::Pt& ul, const GG::Pt& lr) {
 
 CUIWnd::~CUIWnd() {
     try {
-        if (!m_config_name.empty() && GetOptionsDB().OptionExists("UI.windows." + m_config_name + ".initialized"))
-            GetOptionsDB().Remove("UI.windows."+m_config_name+".initialized");
+        if (!m_config_name.empty() && GetOptionsDB().OptionExists("ui." + m_config_name + ".initialized"))
+            GetOptionsDB().Remove("ui." + m_config_name + ".initialized");
     } catch (std::exception& e) { // catch std::runtime_error, boost::bad_any_cast
-        ErrorLogger() << "CUIWnd::~CUIWnd() : caught exception while removing \"UI.windows." << m_config_name
+        ErrorLogger() << "CUIWnd::~CUIWnd() : caught exception while removing \"ui." << m_config_name
                       << ".initialized\": " << e.what();
     }
     m_vertex_buffer.clear();
@@ -242,7 +255,7 @@ void CUIWnd::SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
     if (m_config_save) {    // can write position/size to OptionsDB
 
         GG::Pt available_size;
-        if (const GG::Wnd* parent = Parent()) {
+        if (const auto&& parent = Parent()) {
             // Keep this CUIWnd entirely inside its parent.
             available_size = parent->ClientSize();
         } else if (const HumanClientApp* app = HumanClientApp::GetApp()) {
@@ -255,8 +268,9 @@ void CUIWnd::SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
         }
 
         // Limit window size to be no larger than the containing window.
-        GG::Pt new_size(std::min(lr.x - ul.x, available_size.x),
-                        std::min(lr.y - ul.y, available_size.y));
+        GG::Pt new_size(std::max(std::min(lr.x - ul.x, available_size.x), MinimizedSize().x),
+                        std::max(std::min(lr.y - ul.y, available_size.y),
+                                 TopBorder() + INNER_BORDER_ANGLE_OFFSET + BORDER_BOTTOM + 50));
 
         // Clamp position of this window to keep its entire area visible in the
         // containing window.
@@ -359,7 +373,7 @@ void CUIWnd::LDrag(const GG::Pt& pt, const GG::Pt& move, GG::Flags<GG::ModKey> m
         GG::Pt requested_lr = pt - m_drag_offset;
 
         GG::Pt max_lr;
-        if (const GG::Wnd* parent = Parent()) {
+        if (const auto&& parent = Parent()) {
             max_lr = parent->ClientLowerRight();
         } else {
             max_lr.x = GG::GUI::GetGUI()->AppWidth();
@@ -437,32 +451,36 @@ void CUIWnd::InitButtons() {
 
     // create the close button
     if (m_closable) {
-        m_close_button = new CUIButton(
+        m_close_button = Wnd::Create<CUIButton>(
             GG::SubTexture(ClientUI::GetTexture(button_texture_dir / "close.png")),
             GG::SubTexture(ClientUI::GetTexture(button_texture_dir / "close_clicked.png")),
             GG::SubTexture(ClientUI::GetTexture(button_texture_dir / "close_mouseover.png")));
         m_close_button->SetColor(ClientUI::WndInnerBorderColor());
-        GG::Connect(m_close_button->LeftClickedSignal, &PlayCloseSound, -1);
+        m_close_button->LeftClickedSignal.connect(-1,
+            &PlayCloseSound);
         m_close_button->Resize(GG::Pt(GG::X(ClientUI::TitlePts()), GG::Y(ClientUI::TitlePts())));
-        GG::Connect(m_close_button->LeftClickedSignal, &CUIWnd::CloseClicked, this);
+        m_close_button->LeftClickedSignal.connect(
+            boost::bind(&CUIWnd::CloseClicked, this));
         AttachChild(m_close_button);
         m_close_button->NonClientChild(true);
     }
 
     // create the minimize button
     if (m_minimizable) {
-        m_minimize_button = new CUI_MinRestoreButton();
+        m_minimize_button = Wnd::Create<CUI_MinRestoreButton>();
         m_minimize_button->Resize(GG::Pt(GG::X(ClientUI::TitlePts()), GG::Y(ClientUI::TitlePts())));
-        GG::Connect(m_minimize_button->LeftClickedSignal, &CUIWnd::MinimizeClicked, this);
+        m_minimize_button->LeftClickedSignal.connect(
+            boost::bind(&CUIWnd::MinimizeClicked, this));
         AttachChild(m_minimize_button);
         m_minimize_button->NonClientChild(true);
     }
 
     // create the pin button
     if (m_pinable) {
-        m_pin_button = new CUI_PinButton();
+        m_pin_button = Wnd::Create<CUI_PinButton>();
         m_pin_button->Resize(GG::Pt(GG::X(ClientUI::TitlePts()), GG::Y(ClientUI::TitlePts())));
-        GG::Connect(m_pin_button->LeftClickedSignal, &CUIWnd::PinClicked, this);
+        m_pin_button->LeftClickedSignal.connect(
+            boost::bind(&CUIWnd::PinClicked, this));
         AttachChild(m_pin_button);
         m_pin_button->NonClientChild(true);
     }
@@ -491,10 +509,10 @@ int CUIWnd::InnerBorderAngleOffset() const
 
 void CUIWnd::CloseClicked() {
     m_done = true;
-    if (Parent())
-        Parent()->DetachChild(this);
+    if (auto&& parent = Parent())
+        parent->DetachChild(this);
     else
-        GG::GUI::GetGUI()->Remove(this);
+        GG::GUI::GetGUI()->Remove(shared_from_this());
 
     //m_minimized_buffer.clear();
     //m_outer_border_buffer.clear();
@@ -519,7 +537,7 @@ void CUIWnd::MinimizeClicked() {
 
         // hide all children, re-showing only position/size controls
         Hide();
-        Show(false);
+        Show();
         if (m_close_button)
             m_close_button->Show();
         if (m_minimize_button)
@@ -599,16 +617,16 @@ void CUIWnd::InitBuffers() {
 
     m_vertex_buffer.createServerBuffer();
 
-    //std::cout << "CUIWnd vertex buffer final size: " << previous_buffer_size << std::endl;
+    //TraceLogger() << "CUIWnd vertex buffer final size: " << previous_buffer_size << std::endl;
 }
 
-void CUIWnd::Hide(bool children) {
-    GG::Wnd::Hide(children);
+void CUIWnd::Hide() {
+    GG::Wnd::Hide();
     SaveOptions();
 }
 
-void CUIWnd::Show(bool children) {
-    GG::Wnd::Show(children);
+void CUIWnd::Show() {
+    GG::Wnd::Show();
     SaveOptions();
 }
 
@@ -621,17 +639,78 @@ void CUIWnd::ResetDefaultPosition() {
 GG::Rect CUIWnd::CalculatePosition() const
 { return GG::Rect(INVALID_X, INVALID_Y, INVALID_X, INVALID_Y); }
 
+void CUIWnd::SetDefaultedOptions() {
+    OptionsDB& db = GetOptionsDB();
+    std::set<std::string> window_options;
+    db.FindOptions(window_options, "ui." + m_config_name);
+    for (auto& option : window_options) {
+        if (db.IsDefaultValue(option))
+            m_defaulted_options.emplace(option);
+    }
+}
+
+void CUIWnd::SaveDefaultedOptions() {
+    OptionsDB& db = GetOptionsDB();
+    std::string config_prefix = "ui." + m_config_name;
+    std::string window_mode = db.Get<bool>("video.fullscreen.enabled") ?
+                              ".fullscreen" : ".windowed";
+    GG::Pt size;
+    if (m_minimized)
+        size = m_original_size;
+    else
+        size = Size();
+
+    std::string config_name = config_prefix + window_mode + ".left";
+    int int_value = Value(RelativeUpperLeft().x);
+    if (m_defaulted_options.count(config_name))
+        db.SetDefault<int>(config_name, int_value);
+
+    config_name = config_prefix + window_mode + ".top";
+    int_value = Value(RelativeUpperLeft().y);
+    if (m_defaulted_options.count(config_name))
+        db.SetDefault<int>(config_name, int_value);
+
+    config_name = config_prefix + window_mode + ".width";
+    int_value = Value(size.x);
+    if (m_defaulted_options.count(config_name))
+        db.SetDefault<int>(config_name, int_value);
+
+    config_name = config_prefix + window_mode + ".height";
+    int_value = Value(size.y);
+    if (m_defaulted_options.count(config_name))
+        db.SetDefault<int>(config_name, int_value);
+
+    if (!Modal()) {
+        config_name = config_prefix + ".visible";
+        bool bool_value = Visible();
+        if (m_defaulted_options.count(config_name))
+            db.SetDefault<bool>(config_name, bool_value);
+
+        config_name = config_prefix + ".pinned";
+        bool_value = m_pinned;
+        if (m_defaulted_options.count(config_name))
+            db.SetDefault<bool>(config_name, bool_value);
+
+        config_name = config_prefix + ".minimized";
+        bool_value = m_minimized;
+        if (m_defaulted_options.count(config_name))
+            db.SetDefault<bool>(config_name, bool_value);
+    }
+}
+
 void CUIWnd::SaveOptions() const {
     OptionsDB& db = GetOptionsDB();
 
     // The default empty string means 'do not save/load properties'
     // Also do not save while the window is being dragged.
-    if (m_config_name.empty() || !m_config_save || GG::GUI::GetGUI()->DragWnd(this, 0)) {
+    auto gui = GG::GUI::GetGUI();
+    std::string option_prefix = "ui." + m_config_name;
+    if (m_config_name.empty() || !m_config_save || !gui || gui->DragWnd(this, 0)) {
         return;
-    } else if (!db.OptionExists("UI.windows."+m_config_name+".initialized")) {
+    } else if (!db.OptionExists(option_prefix + ".initialized")) {
         ErrorLogger() << "CUIWnd::SaveOptions() : attempted to save window options using name \"" << m_config_name << "\" but the options do not appear to be registered in the OptionsDB.";
         return;
-    } else if (!db.Get<bool>("UI.windows."+m_config_name+".initialized")) {
+    } else if (!db.Get<bool>(option_prefix + ".initialized")) {
         // Don't save until the window has been given its proper default values
         return;
     }
@@ -642,19 +721,18 @@ void CUIWnd::SaveOptions() const {
     else
         size = Size();
 
-    std::string windowed = ""; // empty string in fullscreen mode, appends -windowed in windowed mode
-    if (!db.Get<bool>("fullscreen"))
-        windowed = "-windowed";
+    std::string window_mode = db.Get<bool>("video.fullscreen.enabled") ?
+                              ".fullscreen" : ".windowed";
 
-    db.Set<int>("UI.windows."+m_config_name+".left"+windowed,   Value(RelativeUpperLeft().x));
-    db.Set<int>("UI.windows."+m_config_name+".top"+windowed,    Value(RelativeUpperLeft().y));
-    db.Set<int>("UI.windows."+m_config_name+".width"+windowed,  Value(size.x));
-    db.Set<int>("UI.windows."+m_config_name+".height"+windowed, Value(size.y));
+    db.Set<int>(option_prefix + window_mode + ".left",     Value(RelativeUpperLeft().x));
+    db.Set<int>(option_prefix + window_mode + ".top",      Value(RelativeUpperLeft().y));
+    db.Set<int>(option_prefix + window_mode + ".width",         Value(size.x));
+    db.Set<int>(option_prefix + window_mode + ".height",        Value(size.y));
 
     if (!Modal()) {
-        db.Set<bool>("UI.windows."+m_config_name+".visible", Visible());
-        db.Set<bool>("UI.windows."+m_config_name+".pinned", m_pinned);
-        db.Set<bool>("UI.windows."+m_config_name+".minimized", m_minimized);
+        db.Set<bool>(option_prefix + ".visible", Visible());
+        db.Set<bool>(option_prefix + ".pinned", m_pinned);
+        db.Set<bool>(option_prefix + ".minimized", m_minimized);
     }
 
     db.Commit();
@@ -664,9 +742,10 @@ void CUIWnd::LoadOptions() {
     OptionsDB& db = GetOptionsDB();
 
     // The default empty string means 'do not save/load properties'
+    std::string option_prefix = "ui." + m_config_name;
     if (m_config_name.empty()) {
         return;
-    } else if (!db.OptionExists("UI.windows."+m_config_name+".initialized")) {
+    } else if (!db.OptionExists(option_prefix + ".initialized")) {
         ErrorLogger() << "CUIWnd::LoadOptions() : attempted to load window options using name \"" << m_config_name << "\" but the options do not appear to be registered in the OptionsDB.";
         return;
     }
@@ -674,14 +753,12 @@ void CUIWnd::LoadOptions() {
     // These functions are only called in certain circumstances, could pass in
     // things like the fullscreen/windowed mode instead of using global program
     // state like this?
-    std::string windowed = ""; // empty string in fullscreen mode, appends -windowed in windowed mode
-    if (!db.Get<bool>("fullscreen"))
-        windowed = "-windowed";
-
-    GG::Pt ul   = GG::Pt(GG::X(db.Get<int>("UI.windows."+m_config_name+".left"+windowed)),
-                         GG::Y(db.Get<int>("UI.windows."+m_config_name+".top"+windowed)));
-    GG::Pt size = GG::Pt(GG::X(db.Get<int>("UI.windows."+m_config_name+".width"+windowed)),
-                         GG::Y(db.Get<int>("UI.windows."+m_config_name+".height"+windowed)));
+    std::string window_mode = db.Get<bool>("video.fullscreen.enabled") ?
+                              ".fullscreen" : ".windowed";
+    GG::Pt ul   = GG::Pt(GG::X(db.Get<int>(option_prefix + window_mode + ".left")),
+                         GG::Y(db.Get<int>(option_prefix + window_mode + ".top")));
+    GG::Pt size = GG::Pt(GG::X(db.Get<int>(option_prefix + window_mode + ".width")),
+                         GG::Y(db.Get<int>(option_prefix + window_mode + ".height")));
 
     m_config_save = false;
 
@@ -702,17 +779,17 @@ void CUIWnd::LoadOptions() {
     }
 
     if (!Modal()) {
-        if (db.Get<bool>("UI.windows."+m_config_name+".visible")) {
+        if (db.Get<bool>(option_prefix + ".visible")) {
             Show();
         } else {
             Hide();
         }
 
-        if (db.Get<bool>("UI.windows."+m_config_name+".pinned") != m_pinned) {
+        if (db.Get<bool>(option_prefix + ".pinned") != m_pinned) {
             PinClicked();
         }
 
-        if (db.Get<bool>("UI.windows."+m_config_name+".minimized") != m_minimized) {
+        if (db.Get<bool>(option_prefix + ".minimized") != m_minimized) {
             MinimizeClicked();
         }
     }
@@ -728,49 +805,49 @@ const std::string CUIWnd::AddWindowOptions(const std::string& config_name,
     OptionsDB& db = GetOptionsDB();
     std::string new_name = "";
 
-    if (db.OptionExists("UI.windows."+config_name+".left")) {
+    if (db.OptionExists("ui." + config_name + ".fullscreen.left")) {
         // If the option has already been added, a window was previously created with this name...
         if (config_name.empty()) {
             // Should never happen, but just in case.
-            db.Remove("UI.windows..left");
-            db.Remove("UI.windows..top");
-            db.Remove("UI.windows..left-windowed");
-            db.Remove("UI.windows..top-windowed");
-            db.Remove("UI.windows..width");
-            db.Remove("UI.windows..height");
-            db.Remove("UI.windows..width-windowed");
-            db.Remove("UI.windows..height-windowed");
-            db.Remove("UI.windows..visible");
-            db.Remove("UI.windows..pinned");
-            db.Remove("UI.windows..minimized");
+            db.Remove("ui..fullscreen.left");
+            db.Remove("ui..fullscreen.top");
+            db.Remove("ui..windowed.left");
+            db.Remove("ui..windowed.top");
+            db.Remove("ui..fullscreen.width");
+            db.Remove("ui..fullscreen.height");
+            db.Remove("ui..windowed.width");
+            db.Remove("ui..windowed.height");
+            db.Remove("ui..visible");
+            db.Remove("ui..pinned");
+            db.Remove("ui..minimized");
             ErrorLogger() << "CUIWnd::AddWindowOptions() : Found window options with a blank name, removing those options.";
-        } else if (db.OptionExists("UI.windows."+config_name+".initialized")) {
+        } else if (db.OptionExists("ui." + config_name + ".initialized")) {
             // If the window's still there, shouldn't use the same name (but the window can still be created so don't throw)
             ErrorLogger() << "CUIWnd::AddWindowOptions() : Attempted to create a window with config_name = " << config_name << " but one already exists with that name.";
         } else {
             // Old window has been destroyed, use the properties it had.
-            db.Add<bool>("UI.windows."+config_name+".initialized",      UserStringNop("OPTIONS_DB_UI_WINDOWS_EXISTS"),          false,      Validator<bool>(),              false);
+            db.Add<bool>("ui." + config_name + ".initialized",      UserStringNop("OPTIONS_DB_UI_WINDOWS_EXISTS"),          false,      Validator<bool>(),              false);
             new_name = config_name;
         }
     } else if (!config_name.empty()) {
         const int max_width_plus_one = HumanClientApp::MaximumPossibleWidth() + 1;
         const int max_height_plus_one = HumanClientApp::MaximumPossibleHeight() + 1;
 
-        db.Add<bool>("UI.windows."+config_name+".initialized",      UserStringNop("OPTIONS_DB_UI_WINDOWS_EXISTS"),          false,      Validator<bool>(),              false);
+        db.Add<bool>("ui." + config_name + ".initialized",      UserStringNop("OPTIONS_DB_UI_WINDOWS_EXISTS"),          false,      Validator<bool>(),              false);
 
-        db.Add<int> ("UI.windows."+config_name+".left",             UserStringNop("OPTIONS_DB_UI_WINDOWS_LEFT"),            left,       OrValidator<int>(RangedValidator<int>(0, max_width_plus_one), DiscreteValidator<int>(INVALID_POS)));
-        db.Add<int> ("UI.windows."+config_name+".top",              UserStringNop("OPTIONS_DB_UI_WINDOWS_TOP"),             top,        OrValidator<int>(RangedValidator<int>(0, max_height_plus_one), DiscreteValidator<int>(INVALID_POS)));
-        db.Add<int> ("UI.windows."+config_name+".left-windowed",    UserStringNop("OPTIONS_DB_UI_WINDOWS_LEFT_WINDOWED"),   left,       OrValidator<int>(RangedValidator<int>(0, max_width_plus_one), DiscreteValidator<int>(INVALID_POS)));
-        db.Add<int> ("UI.windows."+config_name+".top-windowed",     UserStringNop("OPTIONS_DB_UI_WINDOWS_TOP_WINDOWED"),    top,        OrValidator<int>(RangedValidator<int>(0, max_height_plus_one), DiscreteValidator<int>(INVALID_POS)));
+        db.Add<int> ("ui." + config_name + ".fullscreen.left", UserStringNop("OPTIONS_DB_UI_WINDOWS_LEFT"), left, OrValidator<int>(RangedValidator<int>(0, max_width_plus_one), DiscreteValidator<int>(INVALID_POS)));
+        db.Add<int> ("ui." + config_name + ".fullscreen.top", UserStringNop("OPTIONS_DB_UI_WINDOWS_TOP"), top, OrValidator<int>(RangedValidator<int>(0, max_height_plus_one), DiscreteValidator<int>(INVALID_POS)));
+        db.Add<int> ("ui." + config_name + ".windowed.left", UserStringNop("OPTIONS_DB_UI_WINDOWS_LEFT_WINDOWED"), left, OrValidator<int>(RangedValidator<int>(0, max_width_plus_one), DiscreteValidator<int>(INVALID_POS)));
+        db.Add<int> ("ui." + config_name + ".windowed.top", UserStringNop("OPTIONS_DB_UI_WINDOWS_TOP_WINDOWED"), top, OrValidator<int>(RangedValidator<int>(0, max_height_plus_one), DiscreteValidator<int>(INVALID_POS)));
 
-        db.Add<int> ("UI.windows."+config_name+".width",            UserStringNop("OPTIONS_DB_UI_WINDOWS_WIDTH"),           width,      RangedValidator<int>(0, max_width_plus_one));
-        db.Add<int> ("UI.windows."+config_name+".height",           UserStringNop("OPTIONS_DB_UI_WINDOWS_HEIGHT"),          height,     RangedValidator<int>(0, max_height_plus_one));
-        db.Add<int> ("UI.windows."+config_name+".width-windowed",   UserStringNop("OPTIONS_DB_UI_WINDOWS_WIDTH_WINDOWED"),  width,      RangedValidator<int>(0, max_width_plus_one));
-        db.Add<int> ("UI.windows."+config_name+".height-windowed",  UserStringNop("OPTIONS_DB_UI_WINDOWS_HEIGHT_WINDOWED"), height,     RangedValidator<int>(0, max_height_plus_one));
+        db.Add<int> ("ui." + config_name + ".fullscreen.width", UserStringNop("OPTIONS_DB_UI_WINDOWS_WIDTH"), width, RangedValidator<int>(0, max_width_plus_one));
+        db.Add<int> ("ui." + config_name + ".fullscreen.height", UserStringNop("OPTIONS_DB_UI_WINDOWS_HEIGHT"), height, RangedValidator<int>(0, max_height_plus_one));
+        db.Add<int> ("ui." + config_name + ".windowed.width", UserStringNop("OPTIONS_DB_UI_WINDOWS_WIDTH_WINDOWED"), width, RangedValidator<int>(0, max_width_plus_one));
+        db.Add<int> ("ui." + config_name + ".windowed.height", UserStringNop("OPTIONS_DB_UI_WINDOWS_HEIGHT_WINDOWED"), height, RangedValidator<int>(0, max_height_plus_one));
 
-        db.Add<bool>("UI.windows."+config_name+".visible",          UserStringNop("OPTIONS_DB_UI_WINDOWS_VISIBLE"),         visible,    Validator<bool>());
-        db.Add<bool>("UI.windows."+config_name+".pinned",           UserStringNop("OPTIONS_DB_UI_WINDOWS_PINNED"),          pinned,     Validator<bool>());
-        db.Add<bool>("UI.windows."+config_name+".minimized",        UserStringNop("OPTIONS_DB_UI_WINDOWS_MINIMIZED"),       minimized,  Validator<bool>());
+        db.Add<bool>("ui." + config_name + ".visible",          UserStringNop("OPTIONS_DB_UI_WINDOWS_VISIBLE"),         visible,    Validator<bool>());
+        db.Add<bool>("ui." + config_name + ".pinned",           UserStringNop("OPTIONS_DB_UI_WINDOWS_PINNED"),          pinned,     Validator<bool>());
+        db.Add<bool>("ui." + config_name + ".minimized",        UserStringNop("OPTIONS_DB_UI_WINDOWS_MINIMIZED"),       minimized,  Validator<bool>());
 
         new_name = config_name;
     }
@@ -791,30 +868,28 @@ const std::string CUIWnd::AddWindowOptions(const std::string& config_name,
 
 void CUIWnd::InvalidateWindowOptions(const std::string& config_name) {
     OptionsDB& db = GetOptionsDB();
-    if (db.OptionExists("UI.windows."+config_name+".initialized")) {
+    std::string window_mode = db.Get<bool>("video.fullscreen.enabled") ? ".fullscreen" : ".windowed";
+    std::string edge_option_prefix = "ui." + config_name + window_mode;
+
+    if (db.OptionExists("ui." + config_name + ".initialized")) {
         // Should be removed in window dtor.
         ErrorLogger() << "CUIWnd::RemoveWindowOptions() : attempted to remove window options using name \"" << config_name << "\" but they appear to be in use by a window.";
         return;
-    } else if (!db.OptionExists("UI.windows."+config_name+".left")) {
+    } else if (!db.OptionExists(edge_option_prefix + ".left")) {
         ErrorLogger() << "CUIWnd::RemoveWindowOptions() : attempted to remove window options using name \"" << config_name << "\" but the options do not appear to be registered in the OptionsDB.";
         return;
     }
 
-    std::string windowed = ""; // empty string in fullscreen mode, appends -windowed in windowed mode
-    if (!db.Get<bool>("fullscreen"))
-        windowed = "-windowed";
-
-    db.Set<int>("UI.windows."+config_name+".left"+windowed, INVALID_POS);
-    db.Set<int>("UI.windows."+config_name+".top"+windowed,  INVALID_POS);
-    db.Set<bool>("UI.windows."+config_name+".visible", db.GetDefault<bool>("UI.windows."+config_name+".visible"));
-    db.Set<bool>("UI.windows."+config_name+".pinned", db.GetDefault<bool>("UI.windows."+config_name+".pinned"));
-    db.Set<bool>("UI.windows."+config_name+".minimized", db.GetDefault<bool>("UI.windows."+config_name+".minimized"));
+    db.Set<int>(edge_option_prefix + ".left", INVALID_POS);
+    db.Set<int>(edge_option_prefix + ".top",  INVALID_POS);
+    db.Set<bool>("ui." + config_name + ".visible",      db.GetDefault<bool>("ui." + config_name + ".visible"));
+    db.Set<bool>("ui." + config_name + ".pinned",       db.GetDefault<bool>("ui." + config_name + ".pinned"));
+    db.Set<bool>("ui." + config_name + ".minimized",    db.GetDefault<bool>("ui." + config_name + ".minimized"));
 }
 
 void CUIWnd::InvalidateUnusedOptions() {
     OptionsDB& db = GetOptionsDB();
-    std::string prefix("UI.windows.");
-    std::string suffix_used(".initialized"); // this is present if the options are being used by a window
+    std::string prefix("ui.");
     std::string suffix_exist(".left");
 
     // Remove unrecognized options from the DB so that their values aren't
@@ -826,24 +901,21 @@ void CUIWnd::InvalidateUnusedOptions() {
     std::set<std::string> window_options;
     db.FindOptions(window_options, prefix);
     for (const std::string& option : window_options) {
-        // If the ".left" option is registered, the rest are implied to be
-        // there.
-        if (option.rfind(suffix_exist) == option.length() - suffix_exist.length() &&
-            db.OptionExists(option))
-        {
-            std::string name = option.substr(prefix.length(), option.length() - prefix.length() - suffix_exist.length());
-            // If the ".initialized" option isn't present under this name,
-            // remove the options.
-            if (window_options.find(prefix + name + suffix_used) == window_options.end()) {
-                InvalidateWindowOptions(name);
-            }
+        if (!boost::algorithm::find_last(option, suffix_exist))
+            continue;
+        // If the ".left" option is registered, the rest are implied to be there.
+        if ((option.rfind(suffix_exist) == (option.length() - suffix_exist.length())) && db.OptionExists(option)) {
+            auto window_name = WindowNameFromOption(option);
+            // If the ".initialized" option isn't present under this name, remove the options.
+            if (!window_name.empty() && !window_options.count(prefix + window_name + ".initialized"))
+                InvalidateWindowOptions(window_name);
         }
     }
 
     db.Commit();
 }
 
-void CUIWnd::SetParent(GG::Wnd* wnd) {
+void CUIWnd::SetParent(const std::shared_ptr<GG::Wnd>& wnd) {
     GG::Wnd::SetParent(wnd);
     m_vertex_buffer.clear();    // force buffer re-init on next Render call, so background is properly positioned for new parent-relative position
 }
@@ -857,9 +929,14 @@ const int CUIEditWnd::CONTROL_MARGIN = 5;
 CUIEditWnd::CUIEditWnd(GG::X w, const std::string& prompt_text, const std::string& edit_text, GG::Flags<GG::WndFlag> flags/* = Wnd::MODAL*/) :
     CUIWnd(prompt_text, GG::X0, GG::Y0, w, GG::Y1, flags)
 {
-    m_edit = new CUIEdit(edit_text);
-    m_ok_bn = new CUIButton(UserString("OK"));
-    m_cancel_bn = new CUIButton(UserString("CANCEL"));
+    m_edit = GG::Wnd::Create<CUIEdit>(edit_text);
+}
+
+void CUIEditWnd::CompleteConstruction() {
+    CUIWnd::CompleteConstruction();
+
+    m_ok_bn = GG::Wnd::Create<CUIButton>(UserString("OK"));
+    m_cancel_bn = GG::Wnd::Create<CUIButton>(UserString("CANCEL"));
 
     m_edit->MoveTo(GG::Pt(LeftBorder() + 3, TopBorder() + 3));
     m_edit->Resize(GG::Pt(ClientWidth() - 2 * BUTTON_WIDTH - 2 * CONTROL_MARGIN - 6 - LeftBorder() - RightBorder(), m_edit->MinUsableSize().y));
@@ -872,15 +949,17 @@ CUIEditWnd::CUIEditWnd(GG::X w, const std::string& prompt_text, const std::strin
     m_cancel_bn->Resize(GG::Pt(BUTTON_WIDTH, m_cancel_bn->MinUsableSize().y));
     m_cancel_bn->OffsetMove(GG::Pt(GG::X0, (m_edit->Height() - m_ok_bn->Height()) / 2));
 
-    Resize(GG::Pt(w, std::max(m_edit->Bottom(), m_cancel_bn->Bottom()) + BottomBorder() + 3));
-    MoveTo(GG::Pt((GG::GUI::GetGUI()->AppWidth() - w) / 2, (GG::GUI::GetGUI()->AppHeight() - Height()) / 2));
+    Resize(GG::Pt(Width(), std::max(m_edit->Bottom(), m_cancel_bn->Bottom()) + BottomBorder() + 3));
+    MoveTo(GG::Pt((GG::GUI::GetGUI()->AppWidth() - Width()) / 2, (GG::GUI::GetGUI()->AppHeight() - Height()) / 2));
 
     AttachChild(m_edit);
     AttachChild(m_ok_bn);
     AttachChild(m_cancel_bn);
 
-    GG::Connect(m_ok_bn->LeftClickedSignal,     &CUIEditWnd::OkClicked, this);
-    GG::Connect(m_cancel_bn->LeftClickedSignal, &CUIWnd::CloseClicked, static_cast<CUIWnd*>(this));
+    m_ok_bn->LeftClickedSignal.connect(
+        boost::bind(&CUIEditWnd::OkClicked, this));
+    m_cancel_bn->LeftClickedSignal.connect(
+        boost::bind(&CUIWnd::CloseClicked, static_cast<CUIWnd*>(this)));
 
     m_edit->SelectAll();
 }
