@@ -9,8 +9,9 @@
 class CensusBrowseWnd : public GG::BrowseInfoWnd {
 public:
     CensusBrowseWnd(const std::string& title_text, const std::map<std::string, float>& population_counts,
-                    const std::map<std::string, float>& tag_counts);
+                    const std::map<std::string, float>& tag_counts, const std::vector<std::string>& census_order);
 
+    void CompleteConstruction() override;
     bool WndHasBrowseInfo(const Wnd* wnd, std::size_t mode) const override;
 
     void Render() override;
@@ -20,12 +21,15 @@ private:
 
     GG::GL2DVertexBuffer    m_buffer;
 
-    GG::Label*      m_title_text;
-    GG::Label*      m_species_text;
-    GG::ListBox*    m_list;
-    GG::Label*      m_tags_text;
-    GG::ListBox*    m_tags_list;
+    std::shared_ptr<GG::Label>      m_title_text;
+    std::shared_ptr<GG::Label>      m_species_text;
+    std::shared_ptr<GG::ListBox>    m_list;
+    std::shared_ptr<GG::Label>      m_tags_text;
+    std::shared_ptr<GG::ListBox>    m_tags_list;
     GG::Pt          m_offset;
+    std::map<std::string, float>    m_population_counts;
+    std::map<std::string, float>    m_tag_counts;
+    std::vector<std::string>        m_census_order;
 
     void InitRowSizes();
 };
