@@ -26,11 +26,8 @@ FieldIcon::FieldIcon(int field_id) :
     Refresh();
 }
 
-FieldIcon::~FieldIcon() {
-    delete m_selection_indicator;
-    delete m_mouseover_indicator;
-    delete m_name;
-}
+FieldIcon::~FieldIcon()
+{}
 
 int FieldIcon::FieldID() const
 { return m_field_id; }
@@ -97,8 +94,6 @@ void FieldIcon::RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) {
     if (!Disabled())
         RightClickedSignal(m_field_id);
 
-    GG::MenuItem menu_contents;
-
     std::shared_ptr<const Field> field = GetField(m_field_id);
     if (!field)
         return;
@@ -107,13 +102,10 @@ void FieldIcon::RClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) {
         return;
 
     std::string popup_label = boost::io::str(FlexibleFormat(UserString("ENC_LOOKUP")) % UserString(field_type_name));
-    menu_contents.next_level.push_back(GG::MenuItem(popup_label, 1, false, false));
-    CUIPopupMenu popup(pt.x, pt.y, menu_contents);
-
-    if (!popup.Run() || popup.MenuID() != 1)
-        return;
-
-    ClientUI::GetClientUI()->ZoomToFieldType(field_type_name);
+    auto popup = GG::Wnd::Create<CUIPopupMenu>(pt.x, pt.y);
+    auto pedia_lookup_field_type_action = [field_type_name]() { ClientUI::GetClientUI()->ZoomToFieldType(field_type_name); };
+    popup->AddMenuItem(GG::MenuItem(popup_label, false, false, pedia_lookup_field_type_action));
+    popup->Run();
 }
 
 void FieldIcon::LDoubleClick(const GG::Pt& pt, GG::Flags<GG::ModKey> mod_keys) {
