@@ -13,7 +13,11 @@ public:
     //@}
 
     //! \name Mutators //@{
-    void            AddSeries(const std::vector<std::pair<double, double> >& data, const GG::Clr& clr);
+    void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override;
+
+    void Render() override;
+
+    void            AddSeries(const std::vector<std::pair<double, double>>& data, const GG::Clr& clr);
     void            Clear();
 
     void            SetXMin(double x_min);
@@ -26,9 +30,6 @@ public:
     void            ShowPoints(bool show = true);
     void            ShowLines(bool show = true);
     void            ShowScale(bool show = true);
-
-    virtual void    SizeMove(const GG::Pt& ul, const GG::Pt& lr);
-    virtual void    Render();
     //@}
 
 private:
@@ -36,7 +37,7 @@ private:
 
     bool    m_show_points, m_show_lines, m_show_scale;
     double  m_x_min, m_x_max, m_y_min, m_y_max;
-    std::vector<std::pair<std::vector<std::pair<double, double> >,  GG::Clr> > m_data;
+    std::vector<std::pair<std::vector<std::pair<double, double>>, GG::Clr>> m_data;
     GG::GL2DVertexBuffer    m_vert_buf;
     GG::GLRGBAColorBuffer   m_colour_buf;
     std::map<int, double>   m_x_scale_ticks;
