@@ -18,9 +18,9 @@ public:
     void Render() override;
 
 private:
-    GG::StaticGraphic*                   m_icon;
-    GG::Label*                           m_title_text_label;
-    GG::Label*                           m_main_text_label;
+    std::shared_ptr<GG::StaticGraphic>                   m_icon;
+    std::shared_ptr<GG::Label>                           m_title_text_label;
+    std::shared_ptr<GG::Label>                           m_main_text_label;
     const std::shared_ptr<GG::Texture> m_texture;
     const std::string                    m_title_text;
     const std::string                    m_main_text;
