@@ -32,11 +32,11 @@ namespace {
     const GG::Y MAIN_MENU_HEIGHT(450);
 
     void Options(OptionsDB& db) {
-        db.AddFlag("force-external-server",             UserStringNop("OPTIONS_DB_FORCE_EXTERNAL_SERVER"),     false);
-        db.Add<std::string>("external-server-address",  UserStringNop("OPTIONS_DB_EXTERNAL_SERVER_ADDRESS"),   "localhost");
-        db.Add("UI.main-menu.x",                        UserStringNop("OPTIONS_DB_UI_MAIN_MENU_X"),            0.75,   RangedStepValidator<double>(0.01, 0.0, 1.0));
-        db.Add("UI.main-menu.y",                        UserStringNop("OPTIONS_DB_UI_MAIN_MENU_Y"),            0.5,    RangedStepValidator<double>(0.01, 0.0, 1.0));
-        db.Add("checked-gl-version",                    UserStringNop("OPTIONS_DB_CHECKED_GL_VERSION"),        false);
+        db.AddFlag("network.server.external.force",     UserStringNop("OPTIONS_DB_FORCE_EXTERNAL_SERVER"),      false);
+        db.Add<std::string>("network.server.uri",       UserStringNop("OPTIONS_DB_EXTERNAL_SERVER_ADDRESS"),    "localhost");
+        db.Add("ui.intro.menu.center.x",                UserStringNop("OPTIONS_DB_UI_MAIN_MENU_X"),             0.75,               RangedStepValidator<double>(0.01, 0.0, 1.0));
+        db.Add("ui.intro.menu.center.y",                UserStringNop("OPTIONS_DB_UI_MAIN_MENU_Y"),             0.5,                RangedStepValidator<double>(0.01, 0.0, 1.0));
+        db.Add("version.gl.check.done",                 UserStringNop("OPTIONS_DB_CHECKED_GL_VERSION"),         false);
     }
     bool foo_bool = RegisterOptions(&Options);
 }
@@ -63,9 +63,6 @@ public:
             OnExit();
     }
 
-    /** Handle app resizing by closing the credits window. */
-    void WindowResizedSlot(GG::X x, GG::Y y);
-
 private:
     void            DrawCredits(GG::X x1, GG::Y y1, GG::X x2, GG::Y y2, int transparency);
     void            OnExit();
@@ -95,8 +92,9 @@ CreditsWnd::CreditsWnd(GG::X x, GG::Y y, GG::X w, GG::Y h, const XMLElement &cre
 {
     m_font = ClientUI::GetFont(static_cast<int>(ClientUI::Pts()*1.3));
 
-    Connect(GG::GUI::GetGUI()->WindowResizedSignal,
-            boost::bind(&CreditsWnd::WindowResizedSlot, this, _1, _2));
+    /** Handle app resizing by closing the credits window. */
+    GG::GUI::GetGUI()->WindowResizedSignal.connect(
+        boost::bind(&CreditsWnd::OnExit, this));
 }
 
 CreditsWnd::~CreditsWnd() {
@@ -223,8 +221,6 @@ void CreditsWnd::Render() {
         OnExit();
 }
 
-void CreditsWnd::WindowResizedSlot(GG::X x, GG::Y y)
-{ OnExit(); }
 
 /////////////////////////////////
 // IntroScreen
@@ -245,15 +241,19 @@ IntroScreen::IntroScreen() :
     m_splash(nullptr),
     m_logo(nullptr),
     m_version(nullptr)
-{
-    m_menu = new CUIWnd(UserString("INTRO_WINDOW_TITLE"), GG::X1, GG::Y1,
-                        MAIN_MENU_WIDTH, MAIN_MENU_HEIGHT, GG::ONTOP | GG::INTERACTIVE);
+{}
 
-    m_splash = new GG::StaticGraphic(ClientUI::GetTexture(ClientUI::ArtDir() / "splash.png"), GG::GRAPHIC_FITGRAPHIC, GG::INTERACTIVE);
+void IntroScreen::CompleteConstruction() {
+    GG::Wnd::CompleteConstruction();
 
-    m_logo = new GG::StaticGraphic(ClientUI::GetTexture(ClientUI::ArtDir() / "logo.png"), GG::GRAPHIC_FITGRAPHIC | GG::GRAPHIC_PROPSCALE);
+    m_menu = GG::Wnd::Create<CUIWnd>(UserString("INTRO_WINDOW_TITLE"), GG::X1, GG::Y1,
+                                  MAIN_MENU_WIDTH, MAIN_MENU_HEIGHT, GG::ONTOP | GG::INTERACTIVE);
 
-    m_version = new CUILabel(FreeOrionVersionString(), GG::FORMAT_NOWRAP, GG::INTERACTIVE);
+    m_splash = GG::Wnd::Create<GG::StaticGraphic>(ClientUI::GetTexture(ClientUI::ArtDir() / "splash.png"), GG::GRAPHIC_FITGRAPHIC, GG::INTERACTIVE);
+
+    m_logo = GG::Wnd::Create<GG::StaticGraphic>(ClientUI::GetTexture(ClientUI::ArtDir() / "logo.png"), GG::GRAPHIC_FITGRAPHIC | GG::GRAPHIC_PROPSCALE);
+
+    m_version = GG::Wnd::Create<CUILabel>(FreeOrionVersionString(), GG::FORMAT_NOWRAP, GG::INTERACTIVE);
     m_version->MoveTo(GG::Pt(Width() - m_version->Width(), Height() - m_version->Height()));
 
     AttachChild(m_splash);
@@ -262,47 +262,50 @@ IntroScreen::IntroScreen() :
     m_splash->AttachChild(m_version);
 
     //create buttons
-    m_single_player = new CUIButton(UserString("INTRO_BTN_SINGLE_PLAYER"));
-    m_quick_start =   new CUIButton(UserString("INTRO_BTN_QUICK_START"));
-    m_multi_player =  new CUIButton(UserString("INTRO_BTN_MULTI_PLAYER"));
-    m_load_game =     new CUIButton(UserString("INTRO_BTN_LOAD_GAME"));
-    m_options =       new CUIButton(UserString("INTRO_BTN_OPTIONS"));
-    m_pedia =         new CUIButton(UserString("INTRO_BTN_PEDIA"));
-    m_about =         new CUIButton(UserString("INTRO_BTN_ABOUT"));
-    m_website =       new CUIButton(UserString("INTRO_BTN_WEBSITE"));
-    m_credits =       new CUIButton(UserString("INTRO_BTN_CREDITS"));
-    m_exit_game =     new CUIButton(UserString("INTRO_BTN_EXIT"));
-
-    //attach buttons
-    m_menu->AttachChild(m_single_player);
-    m_menu->AttachChild(m_quick_start);
-    m_menu->AttachChild(m_multi_player);
-    m_menu->AttachChild(m_load_game);
-    m_menu->AttachChild(m_options);
-    m_menu->AttachChild(m_pedia);
-    m_menu->AttachChild(m_about);
-    m_menu->AttachChild(m_website);
-    m_menu->AttachChild(m_credits);
-    m_menu->AttachChild(m_exit_game);
+    m_continue =      Wnd::Create<CUIButton>(UserString("INTRO_BTN_CONTINUE"));
+    m_single_player = Wnd::Create<CUIButton>(UserString("INTRO_BTN_SINGLE_PLAYER"));
+    m_quick_start =   Wnd::Create<CUIButton>(UserString("INTRO_BTN_QUICK_START"));
+    m_multi_player =  Wnd::Create<CUIButton>(UserString("INTRO_BTN_MULTI_PLAYER"));
+    m_load_game =     Wnd::Create<CUIButton>(UserString("INTRO_BTN_LOAD_GAME"));
+    m_options =       Wnd::Create<CUIButton>(UserString("INTRO_BTN_OPTIONS"));
+    m_pedia =         Wnd::Create<CUIButton>(UserString("INTRO_BTN_PEDIA"));
+    m_about =         Wnd::Create<CUIButton>(UserString("INTRO_BTN_ABOUT"));
+    m_website =       Wnd::Create<CUIButton>(UserString("INTRO_BTN_WEBSITE"));
+    m_credits =       Wnd::Create<CUIButton>(UserString("INTRO_BTN_CREDITS"));
+    m_exit_game =     Wnd::Create<CUIButton>(UserString("INTRO_BTN_EXIT"));
 
     //connect signals and slots
-    GG::Connect(m_single_player->LeftClickedSignal, &IntroScreen::OnSinglePlayer,   this);
-    GG::Connect(m_quick_start->LeftClickedSignal,   &IntroScreen::OnQuickStart,     this);
-    GG::Connect(m_multi_player->LeftClickedSignal,  &IntroScreen::OnMultiPlayer,    this);
-    GG::Connect(m_load_game->LeftClickedSignal,     &IntroScreen::OnLoadGame,       this);
-    GG::Connect(m_options->LeftClickedSignal,       &IntroScreen::OnOptions,        this);
-    GG::Connect(m_pedia->LeftClickedSignal,         &IntroScreen::OnPedia,          this);
-    GG::Connect(m_about->LeftClickedSignal,         &IntroScreen::OnAbout,          this);
-    GG::Connect(m_website->LeftClickedSignal,       &IntroScreen::OnWebsite,        this);
-    GG::Connect(m_credits->LeftClickedSignal,       &IntroScreen::OnCredits,        this);
-    GG::Connect(m_exit_game->LeftClickedSignal,     &IntroScreen::OnExitGame,       this);
+    m_continue->LeftClickedSignal.connect(
+        boost::bind(&IntroScreen::OnContinue, this));
+    m_single_player->LeftClickedSignal.connect(
+        boost::bind(&IntroScreen::OnSinglePlayer, this));
+    m_quick_start->LeftClickedSignal.connect(
+        boost::bind(&IntroScreen::OnQuickStart, this));
+    m_multi_player->LeftClickedSignal.connect(
+        boost::bind(&IntroScreen::OnMultiPlayer, this));
+    m_load_game->LeftClickedSignal.connect(
+        boost::bind(&IntroScreen::OnLoadGame, this));
+    m_options->LeftClickedSignal.connect(
+        boost::bind(&IntroScreen::OnOptions, this));
+    m_pedia->LeftClickedSignal.connect(
+        boost::bind(&IntroScreen::OnPedia, this));
+    m_about->LeftClickedSignal.connect(
+        boost::bind(&IntroScreen::OnAbout, this));
+    m_website->LeftClickedSignal.connect(
+        boost::bind(&IntroScreen::OnWebsite, this));
+    m_credits->LeftClickedSignal.connect(
+        boost::bind(&IntroScreen::OnCredits, this));
+    m_exit_game->LeftClickedSignal.connect(
+        boost::bind(&IntroScreen::OnExitGame, this));
 
-    DoLayout();
+    RequirePreRender();
 }
 
-IntroScreen::~IntroScreen() {
-    delete m_splash;
-    // m_menu, m_version, m_logo were childs of m_splash, so don't need to be deleted here
+IntroScreen::~IntroScreen()
+{}
+
+void IntroScreen::OnContinue() {
+    HumanClientApp::GetApp()->ContinueSinglePlayerGame();
 }
 
 void IntroScreen::OnSinglePlayer() {
@@ -322,27 +325,29 @@ void IntroScreen::OnLoadGame() {
 }
 
 void IntroScreen::OnOptions() {
-    OptionsWnd options_wnd;
-    options_wnd.Run();
+    auto options_wnd = GG::Wnd::Create<OptionsWnd>(false);
+    options_wnd->Run();
 }
 
 void IntroScreen::OnPedia() {
-    static const std::string INTRO_PEDIA_WND_NAME = "introscreen.pedia";
-    EncyclopediaDetailPanel enc_panel(GG::MODAL | GG::INTERACTIVE | GG::DRAGABLE |
-                                      GG::RESIZABLE | CLOSABLE | PINABLE, INTRO_PEDIA_WND_NAME);
-    enc_panel.SizeMove(GG::Pt(GG::X(100), GG::Y(100)), Size() - GG::Pt(GG::X(100), GG::Y(100)));
-    enc_panel.ClearItems();
-    enc_panel.SetIndex();
-    enc_panel.ValidatePosition();
+    static const std::string INTRO_PEDIA_WND_NAME = "intro.pedia";
+    auto enc_panel = GG::Wnd::Create<EncyclopediaDetailPanel>(
+        GG::MODAL | GG::INTERACTIVE | GG::DRAGABLE |
+        GG::RESIZABLE | CLOSABLE | PINABLE, INTRO_PEDIA_WND_NAME);
+    enc_panel->InitSizeMove(GG::Pt(GG::X(100), GG::Y(100)), Size() - GG::Pt(GG::X(100), GG::Y(100)));
+    enc_panel->ClearItems();
+    enc_panel->SetIndex();
+    enc_panel->ValidatePosition();
 
-    GG::Connect(enc_panel.ClosingSignal, &EncyclopediaDetailPanel::EndRun, &enc_panel);
+    enc_panel->ClosingSignal.connect(
+        boost::bind(&EncyclopediaDetailPanel::EndRun, enc_panel));
 
-    enc_panel.Run();
+    enc_panel->Run();
 }
 
 void IntroScreen::OnAbout() {
-    About about_wnd;
-    about_wnd.Run();
+    auto about_wnd = GG::Wnd::Create<About>();
+    about_wnd->Run();
 }
 
 void IntroScreen::OnWebsite()
@@ -366,16 +371,17 @@ void IntroScreen::OnCredits() {
 
     int credit_side_pad(30);
 
-    CreditsWnd credits_wnd(GG::X0, nUpperLine, GG::GUI::GetGUI()->AppWidth(), nLowerLine-nUpperLine,
-                           credits,
-                           credit_side_pad, 0, Value(m_menu->Left()) - credit_side_pad,
-                           Value(nLowerLine-nUpperLine), Value((nLowerLine-nUpperLine))/2);
+    auto credits_wnd = GG::Wnd::Create<CreditsWnd>(
+        GG::X0, nUpperLine, GG::GUI::GetGUI()->AppWidth(), nLowerLine-nUpperLine,
+        credits,
+        credit_side_pad, 0, Value(m_menu->Left()) - credit_side_pad,
+        Value(nLowerLine-nUpperLine), Value((nLowerLine-nUpperLine))/2);
 
-    credits_wnd.Run();
+    credits_wnd->Run();
 }
 
 void IntroScreen::OnExitGame() {
-    GG::GUI::GetGUI()->Exit(0);
+    GG::GUI::GetGUI()->ExitApp(0);
 }
 
 void IntroScreen::KeyPress(GG::Key key, std::uint32_t key_code_point, GG::Flags<GG::ModKey> mod_keys) {
@@ -386,10 +392,21 @@ void IntroScreen::KeyPress(GG::Key key, std::uint32_t key_code_point, GG::Flags<
 void IntroScreen::Close()
 { OnExitGame(); }
 
+void IntroScreen::SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
+    GG::Pt old_size = GG::Wnd::Size();
+
+    GG::Wnd::SizeMove(ul, lr);
+
+    if (old_size != GG::Wnd::Size())
+        RequirePreRender();
+}
+
 void IntroScreen::Render()
 {}
 
-void IntroScreen::DoLayout() {
+void IntroScreen::PreRender() {
+    GG::Wnd::PreRender();
+
     m_splash->Resize(this->Size());
     m_logo->Resize(GG::Pt(this->Width(), this->Height() / 10));
     m_version->MoveTo(GG::Pt(this->Width() - m_version->Width(), this->Height() - m_version->Height()));
@@ -405,6 +422,7 @@ void IntroScreen::DoLayout() {
     GG::Y mainmenu_height(0);           //height of the mainmenu
 
     //calculate necessary button width
+    button_width = std::max(button_width, m_continue->MinUsableSize().x);
     button_width = std::max(button_width, m_single_player->MinUsableSize().x);
     button_width = std::max(button_width, m_quick_start->MinUsableSize().x);
     button_width = std::max(button_width, m_multi_player->MinUsableSize().x);
@@ -419,9 +437,9 @@ void IntroScreen::DoLayout() {
 
     //calculate  necessary button height
     button_cell_height = std::max(MIN_BUTTON_HEIGHT, m_exit_game->MinUsableSize().y);
-    //culate window width and height
+    // calculate window width and height
     mainmenu_width  =         button_width  + H_MAINMENU_MARGIN;
-    mainmenu_height = 10.75 * button_cell_height + V_MAINMENU_MARGIN; // 10 rows + 0.75 before exit button
+    mainmenu_height = 1.75 * button_cell_height + V_MAINMENU_MARGIN; // 1.75 for the exit button
 
     // place buttons
     GG::Pt button_ul(GG::X(15), GG::Y(12));
@@ -429,40 +447,48 @@ void IntroScreen::DoLayout() {
 
     button_lr += button_ul;
 
-    m_single_player->SizeMove(button_ul, button_lr);
-    button_ul.y += GG::Y(button_cell_height);
-    button_lr.y += GG::Y(button_cell_height);
-    m_quick_start->SizeMove(button_ul, button_lr);
-    button_ul.y += GG::Y(button_cell_height);
-    button_lr.y += GG::Y(button_cell_height);
-    m_multi_player->SizeMove(button_ul, button_lr);
-    button_ul.y += GG::Y(button_cell_height);
-    button_lr.y += GG::Y(button_cell_height);
-    m_load_game->SizeMove(button_ul, button_lr);
-    button_ul.y += GG::Y(button_cell_height);
-    button_lr.y += GG::Y(button_cell_height);
-    m_options->SizeMove(button_ul, button_lr);
-    button_ul.y += GG::Y(button_cell_height);
-    button_lr.y += GG::Y(button_cell_height);
-    m_pedia->SizeMove(button_ul, button_lr);
-    button_ul.y += GG::Y(button_cell_height);
-    button_lr.y += GG::Y(button_cell_height);
-    m_about->SizeMove(button_ul, button_lr);
-    button_ul.y += GG::Y(button_cell_height);
-    button_lr.y += GG::Y(button_cell_height);
-    m_website->SizeMove(button_ul, button_lr);
-    button_ul.y += GG::Y(button_cell_height);
-    button_lr.y += GG::Y(button_cell_height);
-    m_credits->SizeMove(button_ul, button_lr);
-    button_ul.y += GG::Y(button_cell_height) * 1.75;
-    button_lr.y += GG::Y(button_cell_height) * 1.75;
+    const auto place_button =
+        [&button_ul, &button_lr, &button_cell_height, &mainmenu_height]
+        (CUIWnd* menu, std::shared_ptr<GG::Button> button)
+        {
+            button->SizeMove(button_ul, button_lr);
+            menu->AttachChild(std::move(button));
+            button_ul.y += GG::Y(button_cell_height);
+            button_lr.y += GG::Y(button_cell_height);
+            mainmenu_height += button_cell_height;
+        };
+
+    const auto unplace_button = [](CUIWnd* menu, const std::shared_ptr<GG::Button>& button) {
+        menu->DetachChild(button);
+    };
+
+    if (HumanClientApp::GetApp()->IsLoadGameAvailable())
+        place_button(m_menu.get(), m_continue);
+    else
+        unplace_button(m_menu.get(), m_continue);
+    place_button(m_menu.get(), m_single_player);
+    place_button(m_menu.get(), m_quick_start);
+    place_button(m_menu.get(), m_multi_player);
+    if (HumanClientApp::GetApp()->IsLoadGameAvailable())
+        place_button(m_menu.get(), m_load_game);
+    else
+        unplace_button(m_menu.get(), m_load_game);
+    place_button(m_menu.get(), m_options);
+    place_button(m_menu.get(), m_pedia);
+    place_button(m_menu.get(), m_about);
+    place_button(m_menu.get(), m_website);
+    place_button(m_menu.get(), m_credits);
+
+    button_ul.y += GG::Y(button_cell_height) * 0.75;
+    button_lr.y += GG::Y(button_cell_height) * 0.75;
+    m_menu->AttachChild(m_exit_game);
     m_exit_game->SizeMove(button_ul, button_lr);
 
     // position menu window
-    GG::Pt ul(Width()  * GetOptionsDB().Get<double>("UI.main-menu.x") - mainmenu_width/2,
-              Height() * GetOptionsDB().Get<double>("UI.main-menu.y") - mainmenu_height/2);
-    GG::Pt lr(Width()  * GetOptionsDB().Get<double>("UI.main-menu.x") + mainmenu_width/2,
-              Height() * GetOptionsDB().Get<double>("UI.main-menu.y") + mainmenu_height/2);
+    GG::Pt ul(Width()  * GetOptionsDB().Get<double>("ui.intro.menu.center.x") - mainmenu_width/2,
+              Height() * GetOptionsDB().Get<double>("ui.intro.menu.center.y") - mainmenu_height/2);
+    GG::Pt lr(Width()  * GetOptionsDB().Get<double>("ui.intro.menu.center.x") + mainmenu_width/2,
+              Height() * GetOptionsDB().Get<double>("ui.intro.menu.center.y") + mainmenu_height/2);
 
-    m_menu->SizeMove(ul, lr);
+    m_menu->InitSizeMove(ul, lr);
 }
