@@ -9,6 +9,7 @@
 #include "../Empire/Empire.h"
 #include "../universe/Enums.h"
 #include "TextBrowseWnd.h"
+
 #include <GG/Button.h>
 
 #include <iterator>
@@ -42,80 +43,88 @@ ModeratorActionsWnd::ModeratorActionsWnd(const std::string& config_name) :
     m_add_starlane_button(nullptr),
     m_remove_starlane_button(nullptr)
 {
+}
+
+void ModeratorActionsWnd::CompleteConstruction() {
     ClientUI* ui = ClientUI::GetClientUI();
     GG::Flags<GG::GraphicStyle> style = GG::GRAPHIC_CENTER | GG::GRAPHIC_VCENTER | GG::GRAPHIC_FITGRAPHIC | GG::GRAPHIC_PROPSCALE;
 
     boost::filesystem::path button_texture_dir = ClientUI::ArtDir() / "icons" / "buttons";
 
     // button for no action
-    m_no_action_button = new CUIButton(
+    m_no_action_button = Wnd::Create<CUIButton>(
         GG::SubTexture(ClientUI::GetTexture(button_texture_dir / "nomoderatoraction.png")),
         GG::SubTexture(ClientUI::GetTexture(button_texture_dir / "nomoderatoraction_clicked.png")),
         GG::SubTexture(ClientUI::GetTexture(button_texture_dir / "nomoderatoraction_mouseover.png")));
 
-    m_no_action_button->SetBrowseModeTime(GetOptionsDB().Get<int>("UI.tooltip-delay"));
-    m_no_action_button->SetBrowseInfoWnd(std::make_shared<TextBrowseWnd>(
+    m_no_action_button->SetBrowseModeTime(GetOptionsDB().Get<int>("ui.tooltip.delay"));
+    m_no_action_button->SetBrowseInfoWnd(GG::Wnd::Create<TextBrowseWnd>(
         UserString("MOD_NONE"), UserString("MOD_NONE")));
     AttachChild(m_no_action_button);
-    GG::Connect(m_no_action_button->LeftClickedSignal,  &ModeratorActionsWnd::NoActionClicked,      this);
+    m_no_action_button->LeftClickedSignal.connect(
+        boost::bind(&ModeratorActionsWnd::NoAction, this));
 
     // button for create system and droplist to select system type to create
-    m_create_system_button = new CUIButton(
+    m_create_system_button = Wnd::Create<CUIButton>(
         GG::SubTexture(ClientUI::GetTexture(button_texture_dir / "addstar.png")),
         GG::SubTexture(ClientUI::GetTexture(button_texture_dir / "addstar_clicked.png")),
         GG::SubTexture(ClientUI::GetTexture(button_texture_dir / "addstar_mouseover.png")));
 
-    m_create_system_button->SetBrowseModeTime(GetOptionsDB().Get<int>("UI.tooltip-delay"));
-    m_create_system_button->SetBrowseInfoWnd(std::make_shared<TextBrowseWnd>(
+    m_create_system_button->SetBrowseModeTime(GetOptionsDB().Get<int>("ui.tooltip.delay"));
+    m_create_system_button->SetBrowseInfoWnd(GG::Wnd::Create<TextBrowseWnd>(
         UserString("MOD_CREATE_SYSTEM"), UserString("MOD_CREATE_SYSTEM")));
     AttachChild(m_create_system_button);
 
-    GG::Connect(m_create_system_button->LeftClickedSignal,  &ModeratorActionsWnd::CreateSystemClicked,  this);
-    m_star_type_drop = new CUIDropDownList(6);
+    m_create_system_button->LeftClickedSignal.connect(
+        boost::bind(&ModeratorActionsWnd::CreateSystem, this));
+    m_star_type_drop = GG::Wnd::Create<CUIDropDownList>(6);
     m_star_type_drop->Resize(GG::Pt(DROP_WIDTH, CONTROL_HEIGHT));
     for (StarType star_type = STAR_BLUE; star_type != NUM_STAR_TYPES; star_type = StarType(star_type + 1)) {
         std::shared_ptr<GG::Texture> disc_texture = ui->GetModuloTexture(
             ClientUI::ArtDir() / "stars", ClientUI::StarTypeFilePrefixes()[star_type], 0);
-        GG::DropDownList::Row* row = new GG::DropDownList::Row();
-        GG::StaticGraphic* icon = new GG::StaticGraphic(disc_texture, style);
+        auto row = GG::Wnd::Create<GG::DropDownList::Row>();
+        auto icon = GG::Wnd::Create<GG::StaticGraphic>(disc_texture, style);
         icon->Resize(GG::Pt(CONTROL_WIDTH, CONTROL_HEIGHT));
         row->push_back(icon);
         m_star_type_drop->Insert(row);
     }
     m_star_type_drop->Select(m_star_type_drop->begin());        // default select first type
-    GG::Connect(m_star_type_drop->SelChangedSignal,     &ModeratorActionsWnd::StarTypeSelected,     this);
+    m_star_type_drop->SelChangedSignal.connect(
+        boost::bind(&ModeratorActionsWnd::CreateSystem, this));
 
     // button for create planet and droplists to select planet type and size
-    m_create_planet_button = new CUIButton(
+    m_create_planet_button = Wnd::Create<CUIButton>(
         GG::SubTexture(ClientUI::GetTexture(button_texture_dir / "addplanet.png")),
         GG::SubTexture(ClientUI::GetTexture(button_texture_dir / "addplanet_clicked.png")),
         GG::SubTexture(ClientUI::GetTexture(button_texture_dir / "addplanet_mouseover.png")));
 
-    m_create_planet_button->SetBrowseModeTime(GetOptionsDB().Get<int>("UI.tooltip-delay"));
-    m_create_planet_button->SetBrowseInfoWnd(std::make_shared<TextBrowseWnd>(
+    m_create_planet_button->SetBrowseModeTime(GetOptionsDB().Get<int>("ui.tooltip.delay"));
+    m_create_planet_button->SetBrowseInfoWnd(GG::Wnd::Create<TextBrowseWnd>(
         UserString("MOD_CREATE_PLANET"), UserString("MOD_CREATE_PLANET")));
     AttachChild(m_create_planet_button);
-    GG::Connect(m_create_planet_button->LeftClickedSignal,  &ModeratorActionsWnd::CreatePlanetClicked,  this);
+    m_create_planet_button->LeftClickedSignal.connect(
+        boost::bind(&ModeratorActionsWnd::CreatePlanet, this));
 
-    m_planet_type_drop = new CUIDropDownList(6);
+    m_planet_type_drop = GG::Wnd::Create<CUIDropDownList>(6);
     m_planet_type_drop->Resize(GG::Pt(DROP_WIDTH, CONTROL_HEIGHT));
     for (PlanetType planet_type = PT_SWAMP; planet_type != NUM_PLANET_TYPES; planet_type = PlanetType(planet_type + 1)) {
         std::shared_ptr<GG::Texture> texture = ClientUI::PlanetIcon(planet_type);
-        GG::DropDownList::Row* row = new GG::DropDownList::Row();
-        GG::StaticGraphic* icon = new GG::StaticGraphic(texture, style);
+        auto row = GG::Wnd::Create<GG::DropDownList::Row>();
+        auto icon = GG::Wnd::Create<GG::StaticGraphic>(texture, style);
         icon->Resize(GG::Pt(CONTROL_WIDTH, CONTROL_HEIGHT));
         row->push_back(icon);
         m_planet_type_drop->Insert(row);
     }
     m_planet_type_drop->Select(m_planet_type_drop->begin());    // default select first type
-    GG::Connect(m_planet_type_drop->SelChangedSignal,   &ModeratorActionsWnd::PlanetTypeSelected,   this);
+    m_planet_type_drop->SelChangedSignal.connect(
+        boost::bind(&ModeratorActionsWnd::CreatePlanet, this));
 
-    m_planet_size_drop = new CUIDropDownList(6);
+    m_planet_size_drop = GG::Wnd::Create<CUIDropDownList>(6);
     m_planet_size_drop->Resize(GG::Pt(DROP_WIDTH, CONTROL_HEIGHT));
     for (PlanetSize planet_size = SZ_TINY; planet_size != NUM_PLANET_SIZES; planet_size = PlanetSize(planet_size + 1)) {
         std::shared_ptr<GG::Texture> texture = ClientUI::PlanetSizeIcon(planet_size);
-        GG::DropDownList::Row* row = new GG::DropDownList::Row();
-        GG::StaticGraphic* icon = new GG::StaticGraphic(texture, style);
+        auto row = GG::Wnd::Create<GG::DropDownList::Row>();
+        auto icon = GG::Wnd::Create<GG::StaticGraphic>(texture, style);
         icon->Resize(GG::Pt(CONTROL_WIDTH, CONTROL_HEIGHT));
         row->push_back(icon);
         m_planet_size_drop->Insert(row);
@@ -123,72 +132,77 @@ ModeratorActionsWnd::ModeratorActionsWnd(const std::string& config_name) :
     GG::DropDownList::iterator it = m_planet_size_drop->begin();
     std::advance(it, 2);
     m_planet_size_drop->Select(it); // default select 3rd size (should be medium?)
-    GG::Connect(m_planet_size_drop->SelChangedSignal,   &ModeratorActionsWnd::PlanetSizeSelected,   this);
+    m_planet_size_drop->SelChangedSignal.connect(
+        boost::bind(&ModeratorActionsWnd::CreatePlanet, this));
 
     // button for destroying object
-    m_delete_object_button = new CUIButton(
+    m_delete_object_button = Wnd::Create<CUIButton>(
         GG::SubTexture(ClientUI::GetTexture(button_texture_dir / "delete.png")),
         GG::SubTexture(ClientUI::GetTexture(button_texture_dir / "delete_clicked.png")),
         GG::SubTexture(ClientUI::GetTexture(button_texture_dir / "delete_mouseover.png")));
 
-    m_delete_object_button->SetBrowseModeTime(GetOptionsDB().Get<int>("UI.tooltip-delay"));
-    m_delete_object_button->SetBrowseInfoWnd(std::make_shared<TextBrowseWnd>(
+    m_delete_object_button->SetBrowseModeTime(GetOptionsDB().Get<int>("ui.tooltip.delay"));
+    m_delete_object_button->SetBrowseInfoWnd(GG::Wnd::Create<TextBrowseWnd>(
         UserString("MOD_DESTROY"), UserString("MOD_DESTROY")));
     AttachChild(m_delete_object_button);
-    GG::Connect(m_delete_object_button->LeftClickedSignal,  &ModeratorActionsWnd::DeleteObjectClicked,  this);
+    m_delete_object_button->LeftClickedSignal.connect(
+        boost::bind(&ModeratorActionsWnd::DeleteObject, this));
 
     // button for setting owner
-    m_set_owner_button = new CUIButton(
+    m_set_owner_button = Wnd::Create<CUIButton>(
         GG::SubTexture(ClientUI::GetTexture(button_texture_dir / "setowner.png")),
         GG::SubTexture(ClientUI::GetTexture(button_texture_dir / "setowner_clicked.png")),
         GG::SubTexture(ClientUI::GetTexture(button_texture_dir / "setowner_mouseover.png")));
 
-    m_set_owner_button->SetBrowseModeTime(GetOptionsDB().Get<int>("UI.tooltip-delay"));
-    m_set_owner_button->SetBrowseInfoWnd(std::make_shared<TextBrowseWnd>(
+    m_set_owner_button->SetBrowseModeTime(GetOptionsDB().Get<int>("ui.tooltip.delay"));
+    m_set_owner_button->SetBrowseInfoWnd(GG::Wnd::Create<TextBrowseWnd>(
         UserString("MOD_SET_OWNER"), UserString("MOD_SET_OWNER")));
     AttachChild(m_set_owner_button);
 
-    GG::Connect(m_set_owner_button->LeftClickedSignal,      &ModeratorActionsWnd::SetOwnerClicked,      this);
-    m_empire_drop = new CUIDropDownList(6);
+    m_set_owner_button->LeftClickedSignal.connect(
+        boost::bind(&ModeratorActionsWnd::SetOwner, this));
+    m_empire_drop = GG::Wnd::Create<CUIDropDownList>(6);
     m_empire_drop->SetStyle(GG::LIST_NOSORT);
     // empires added later when gamestate info available
-    GG::Connect(m_empire_drop->SelChangedSignal,        &ModeratorActionsWnd::EmpireSelected,       this);
+    m_empire_drop->SelChangedSignal.connect(
+        boost::bind(&ModeratorActionsWnd::SetOwner, this));
 
     // button for creating starlane
-    m_add_starlane_button = new CUIButton(
+    m_add_starlane_button = Wnd::Create<CUIButton>(
         GG::SubTexture(ClientUI::GetTexture(button_texture_dir / "addstarlane.png")),
         GG::SubTexture(ClientUI::GetTexture(button_texture_dir / "addstarlane_clicked.png")),
         GG::SubTexture(ClientUI::GetTexture(button_texture_dir / "addstarlane_mouseover.png")));
 
-    m_add_starlane_button->SetBrowseModeTime(GetOptionsDB().Get<int>("UI.tooltip-delay"));
-    m_add_starlane_button->SetBrowseInfoWnd(std::make_shared<TextBrowseWnd>(
+    m_add_starlane_button->SetBrowseModeTime(GetOptionsDB().Get<int>("ui.tooltip.delay"));
+    m_add_starlane_button->SetBrowseInfoWnd(GG::Wnd::Create<TextBrowseWnd>(
         UserString("MOD_ADD_STARLANE"), UserString("MOD_ADD_STARLANE")));
     AttachChild(m_add_starlane_button);
-    GG::Connect(m_add_starlane_button->LeftClickedSignal,&ModeratorActionsWnd::AddStarlane,         this);
+    m_add_starlane_button->LeftClickedSignal.connect(
+        boost::bind(&ModeratorActionsWnd::AddStarlane, this));
 
     // button for removing starlane
-    m_remove_starlane_button = new CUIButton(
+    m_remove_starlane_button = Wnd::Create<CUIButton>(
         GG::SubTexture(ClientUI::GetTexture(button_texture_dir / "removestarlane.png")),
         GG::SubTexture(ClientUI::GetTexture(button_texture_dir / "removestarlane_clicked.png")),
         GG::SubTexture(ClientUI::GetTexture(button_texture_dir / "removestarlane_mouseover.png")));
 
-    m_remove_starlane_button->SetBrowseModeTime(GetOptionsDB().Get<int>("UI.tooltip-delay"));
-    m_remove_starlane_button->SetBrowseInfoWnd(std::make_shared<TextBrowseWnd>(
+    m_remove_starlane_button->SetBrowseModeTime(GetOptionsDB().Get<int>("ui.tooltip.delay"));
+    m_remove_starlane_button->SetBrowseInfoWnd(GG::Wnd::Create<TextBrowseWnd>(
         UserString("MOD_REMOVE_STARLANE"), UserString("MOD_REMOVE_STARLANE")));
     AttachChild(m_remove_starlane_button);
-    GG::Connect(m_remove_starlane_button->LeftClickedSignal,&ModeratorActionsWnd::RemoveStarlane,   this);
+    m_remove_starlane_button->LeftClickedSignal.connect(
+        boost::bind(&ModeratorActionsWnd::RemoveStarlane, this));
+
+    CUIWnd::CompleteConstruction();
 
     DoLayout();
+    SaveDefaultedOptions();
 }
 
-ModeratorActionsWnd::~ModeratorActionsWnd() {
-    delete m_star_type_drop;
-    delete m_planet_type_drop;
-    delete m_planet_size_drop;
-    delete m_empire_drop;
-}
+ModeratorActionsWnd::~ModeratorActionsWnd()
+{}
 
-void ModeratorActionsWnd::NoActionClicked() {
+void ModeratorActionsWnd::NoAction() {
     m_selected_action = MAS_NoAction;
     NoActionSelectedSignal();
     DetachChild(m_star_type_drop);
@@ -197,7 +211,7 @@ void ModeratorActionsWnd::NoActionClicked() {
     DetachChild(m_empire_drop);
 }
 
-void ModeratorActionsWnd::CreateSystemClicked() {
+void ModeratorActionsWnd::CreateSystem() {
     m_selected_action = MAS_CreateSystem;
     CreateSystemActionSelectedSignal(SelectedStarType());
     AttachChild(m_star_type_drop);
@@ -206,10 +220,7 @@ void ModeratorActionsWnd::CreateSystemClicked() {
     DetachChild(m_empire_drop);
 }
 
-void ModeratorActionsWnd::StarTypeSelected(GG::DropDownList::iterator it)
-{ CreateSystemClicked(); }
-
-void ModeratorActionsWnd::CreatePlanetClicked() {
+void ModeratorActionsWnd::CreatePlanet() {
     m_selected_action = MAS_CreatePlanet;
     CreatePlanetActionSelectedSignal(SelectedPlanetType());
     DetachChild(m_star_type_drop);
@@ -218,13 +229,7 @@ void ModeratorActionsWnd::CreatePlanetClicked() {
     DetachChild(m_empire_drop);
 }
 
-void ModeratorActionsWnd::PlanetTypeSelected(GG::DropDownList::iterator it)
-{ CreatePlanetClicked(); }
-
-void ModeratorActionsWnd::PlanetSizeSelected(GG::DropDownList::iterator it)
-{ CreatePlanetClicked(); }
-
-void ModeratorActionsWnd::DeleteObjectClicked() {
+void ModeratorActionsWnd::DeleteObject() {
     m_selected_action = MAS_Destroy;
     DeleteObjectActionSelectedSignal();
     DetachChild(m_star_type_drop);
@@ -233,7 +238,7 @@ void ModeratorActionsWnd::DeleteObjectClicked() {
     DetachChild(m_empire_drop);
 }
 
-void ModeratorActionsWnd::SetOwnerClicked() {
+void ModeratorActionsWnd::SetOwner() {
     m_selected_action = MAS_SetOwner;
     SetOwnerActionSelectedSignal(SelectedEmpire());
     DetachChild(m_star_type_drop);
@@ -242,9 +247,6 @@ void ModeratorActionsWnd::SetOwnerClicked() {
     AttachChild(m_empire_drop);
 }
 
-void ModeratorActionsWnd::EmpireSelected(GG::DropDownList::iterator it)
-{ SetOwnerClicked(); }
-
 void ModeratorActionsWnd::AddStarlane() {
     m_selected_action = MAS_AddStarlane;
     AddStarlaneActionSelectedSignal();
@@ -328,18 +330,18 @@ void ModeratorActionsWnd::Refresh() {
     // todo: get currently selected empire, if any, reselect after refresh
 
     m_empire_drop->Clear();
-    for (const std::map<int, Empire*>::value_type& entry : Empires()) {
+    for (const auto& entry : Empires()) {
         const Empire* empire = entry.second;
-        GG::DropDownList::Row* row = new GG::DropDownList::Row();
-        GG::Label* label = new CUILabel(empire->Name(), GG::FORMAT_NOWRAP);
+        auto row = GG::Wnd::Create<GG::DropDownList::Row>();
+        auto label = GG::Wnd::Create<CUILabel>(empire->Name(), GG::FORMAT_NOWRAP);
         label->SetTextColor(empire->Color());
         row->push_back(label);
         m_empire_drop->Insert(row);
     }
 
     // no empire / monsters
-    GG::DropDownList::Row* row = new GG::DropDownList::Row();
-    GG::Label* label = new CUILabel(UserString("UNOWNED"), GG::FORMAT_NOWRAP);
+    auto row = GG::Wnd::Create<GG::DropDownList::Row>();
+    auto label = GG::Wnd::Create<CUILabel>(UserString("UNOWNED"), GG::FORMAT_NOWRAP);
     label->SetTextColor(GG::CLR_RED);
     row->push_back(label);
     m_empire_drop->Insert(row);
