@@ -28,8 +28,8 @@ namespace {
 
     void AddOptions(OptionsDB& db) {
         // queue width used also on production screen. prevent double-adding...
-        if (!db.OptionExists("UI.queue-width"))
-            db.Add("UI.queue-width",    UserStringNop("OPTIONS_DB_UI_QUEUE_WIDTH"), 300,    RangedValidator<int>(200, 500));
+        if (!db.OptionExists("ui.queue.width"))
+            db.Add("ui.queue.width", UserStringNop("OPTIONS_DB_UI_QUEUE_WIDTH"), 300, RangedValidator<int>(200, 500));
     }
     bool temp_bool = RegisterOptions(&AddOptions);
 
@@ -41,6 +41,7 @@ namespace {
         QueueTechPanel(GG::X x, GG::Y y, GG::X w, const std::string& tech_name, double allocated_rp,
                        int turns_left, double turns_completed, int empire_id, bool paused = false);
 
+        void CompleteConstruction() override;
         void Render() override;
 
         void SizeMove(const GG::Pt& ul, const GG::Pt& lr) override;
@@ -51,11 +52,11 @@ namespace {
         void Draw(GG::Clr clr, bool fill);
 
         const std::string&      m_tech_name;
-        GG::Label*              m_name_text;
-        GG::Label*              m_RPs_and_turns_text;
-        GG::Label*              m_turns_remaining_text;
-        GG::StaticGraphic*      m_icon;
-        MultiTurnProgressBar*   m_progress_bar;
+        std::shared_ptr<GG::Label>              m_name_text;
+        std::shared_ptr<GG::Label>              m_RPs_and_turns_text;
+        std::shared_ptr<GG::Label>              m_turns_remaining_text;
+        std::shared_ptr<GG::StaticGraphic>      m_icon;
+        std::shared_ptr<MultiTurnProgressBar>   m_progress_bar;
         bool                    m_in_progress;
         int                     m_total_turns;
         int                     m_empire_id;
@@ -86,16 +87,16 @@ namespace {
             else if (empire)
                 progress = empire->ResearchProgress(elem.name);
 
-            panel = new QueueTechPanel(GG::X(GetLayout()->BorderMargin()), GG::Y(GetLayout()->BorderMargin()),
-                                       ClientWidth(), elem.name, elem.allocated_rp,
-                                       elem.turns_left, progress / per_turn_cost,
-                                       elem.empire_id);
+            panel = GG::Wnd::Create<QueueTechPanel>(GG::X(GetLayout()->BorderMargin()), GG::Y(GetLayout()->BorderMargin()),
+                                                    ClientWidth(), elem.name, elem.allocated_rp,
+                                                    elem.turns_left, progress / per_turn_cost,
+                                                    elem.empire_id);
             push_back(panel);
 
             SetDragDropDataType("RESEARCH_QUEUE_ROW");
 
-            SetBrowseModeTime(GetOptionsDB().Get<int>("UI.tooltip-delay"));
-            SetBrowseInfoWnd(TechPanelRowBrowseWnd(elem.name, elem.empire_id));
+            SetBrowseModeTime(GetOptionsDB().Get<int>("ui.tooltip.delay"));
+            SetBrowseInfoWnd(TechRowBrowseWnd(elem.name, elem.empire_id));
         }
 
         void PreRender() override {
@@ -118,7 +119,7 @@ namespace {
         }
 
         const ResearchQueue::Element elem;
-        GG::Control* panel;
+        std::shared_ptr<GG::Control> panel;
     };
 
     //////////////////////////////////////////////////
@@ -158,13 +159,13 @@ namespace {
         GG::Y top(MARGIN);
         GG::X left(MARGIN);
 
-        m_icon = new GG::StaticGraphic(ClientUI::TechIcon(m_tech_name), GG::GRAPHIC_FITGRAPHIC);
+        m_icon = GG::Wnd::Create<GG::StaticGraphic>(ClientUI::TechIcon(m_tech_name), GG::GRAPHIC_FITGRAPHIC);
         m_icon->MoveTo(GG::Pt(left, top));
         m_icon->Resize(GG::Pt(GG::X(GRAPHIC_SIZE), GG::Y(GRAPHIC_SIZE)));
         m_icon->SetColor(tech ? ClientUI::CategoryColor(tech->Category()) : GG::Clr());
         left += m_icon->Width() + MARGIN;
 
-        m_name_text = new CUILabel(m_paused ? UserString("PAUSED") : UserString(m_tech_name), GG::FORMAT_TOP | GG::FORMAT_LEFT);
+        m_name_text = GG::Wnd::Create<CUILabel>(m_paused ? UserString("PAUSED") : UserString(m_tech_name), GG::FORMAT_TOP | GG::FORMAT_LEFT);
         m_name_text->MoveTo(GG::Pt(left, top));
         m_name_text->Resize(GG::Pt(NAME_WIDTH, GG::Y(FONT_PTS + 2*MARGIN)));
         m_name_text->SetTextColor(clr);
@@ -185,12 +186,12 @@ namespace {
         if (m_in_progress)
             outline_color = GG::LightColor(outline_color);
 
-        m_progress_bar = new MultiTurnProgressBar(total_time,
-                                                  perc_complete,
-                                                  next_progress,
-                                                  GG::LightColor(ClientUI::TechWndProgressBarBackgroundColor()),
-                                                  ClientUI::TechWndProgressBarColor(),
-                                                  outline_color);
+        m_progress_bar = GG::Wnd::Create<MultiTurnProgressBar>(total_time,
+                                                               perc_complete,
+                                                               next_progress,
+                                                               GG::LightColor(ClientUI::TechWndProgressBarBackgroundColor()),
+                                                               ClientUI::TechWndProgressBarColor(),
+                                                               outline_color);
 
         m_progress_bar->MoveTo(GG::Pt(left, top));
         m_progress_bar->Resize(GG::Pt(METER_WIDTH, METER_HEIGHT));
@@ -202,7 +203,7 @@ namespace {
         std::string turns_cost_text = str(FlexibleFormat(UserString("TECH_TURN_COST_STR"))
             % DoubleToString(turn_spending, 3, false)
             % DoubleToString(max_spending_per_turn, 3, false));
-        m_RPs_and_turns_text = new CUILabel(turns_cost_text, GG::FORMAT_LEFT);
+        m_RPs_and_turns_text = GG::Wnd::Create<CUILabel>(turns_cost_text, GG::FORMAT_LEFT);
         m_RPs_and_turns_text->MoveTo(GG::Pt(left, top));
         m_RPs_and_turns_text->Resize(GG::Pt(TURNS_AND_COST_WIDTH, GG::Y(FONT_PTS + MARGIN)));
         m_RPs_and_turns_text->SetTextColor(clr);
@@ -213,13 +214,16 @@ namespace {
 
         std::string turns_left_text = turns_left < 0 ? UserString("TECH_TURNS_LEFT_NEVER")
                                                      : str(FlexibleFormat(UserString("TECH_TURNS_LEFT_STR")) % turns_left);
-        m_turns_remaining_text = new CUILabel(turns_left_text, GG::FORMAT_RIGHT);
+        m_turns_remaining_text = GG::Wnd::Create<CUILabel>(turns_left_text, GG::FORMAT_RIGHT);
         m_turns_remaining_text->MoveTo(GG::Pt(left, top));
         m_turns_remaining_text->Resize(GG::Pt(TURNS_AND_COST_WIDTH, GG::Y(FONT_PTS + MARGIN)));
         m_turns_remaining_text->SetTextColor(clr);
         m_turns_remaining_text->ClipText(true);
         m_turns_remaining_text->SetChildClippingMode(ClipToClient);
+    }
 
+    void QueueTechPanel::CompleteConstruction() {
+        GG::Control::CompleteConstruction();
         AttachChild(m_name_text);
         AttachChild(m_RPs_and_turns_text);
         AttachChild(m_turns_remaining_text);
@@ -273,8 +277,6 @@ namespace {
     }
 
     GG::Y QueueTechPanel::DefaultHeight() {
-        const int MARGIN = 2;
-
         const int FONT_PTS = ClientUI::Pts();
         const GG::Y METER_HEIGHT(FONT_PTS);
 
@@ -289,32 +291,42 @@ namespace {
 //////////////////////////////////////////////////
 class ResearchQueueListBox : public QueueListBox {
 public:
-    ResearchQueueListBox(const std::string& drop_type_str, const std::string& prompt_str) :
+    ResearchQueueListBox(const boost::optional<std::string>& drop_type_str, const std::string& prompt_str) :
         QueueListBox(drop_type_str, prompt_str)
     {}
 
+    void CompleteConstruction() override
+    { QueueListBox::CompleteConstruction(); }
+
     boost::signals2::signal<void ()>                            ShowPediaSignal;
     boost::signals2::signal<void (GG::ListBox::iterator, bool)> QueueItemPausedSignal;
 
 protected:
     void ItemRightClickedImpl(GG::ListBox::iterator it, const GG::Pt& pt, const GG::Flags<GG::ModKey>& modkeys) override {
         // mostly duplicated equivalent in QueueListBox, but with an extra command...
+        auto pedia_action = [&it, this, pt, modkeys]() {
+            ShowPediaSignal();
+            this->LeftClickedRowSignal(it, pt, modkeys);
+        };
+        auto resume_action = [&it, this]() { this->QueueItemPausedSignal(it, false); };
+        auto pause_action = [&it, this]() { this->QueueItemPausedSignal(it, true); };
 
-        GG::MenuItem menu_contents;
-        menu_contents.next_level.push_back(GG::MenuItem(UserString("MOVE_UP_QUEUE_ITEM"),   1, false, false));
-        menu_contents.next_level.push_back(GG::MenuItem(UserString("MOVE_DOWN_QUEUE_ITEM"), 2, false, false));
-        menu_contents.next_level.push_back(GG::MenuItem(UserString("DELETE_QUEUE_ITEM"),    3, false, false));
+        auto popup = GG::Wnd::Create<CUIPopupMenu>(pt.x, pt.y);
 
-        GG::ListBox::Row* row = *it;
-        QueueRow* queue_row = row ? dynamic_cast<QueueRow*>(row) : nullptr;
+        popup->AddMenuItem(GG::MenuItem(UserString("MOVE_UP_QUEUE_ITEM"),   false, false, MoveToTopAction(it)));
+        popup->AddMenuItem(GG::MenuItem(UserString("MOVE_DOWN_QUEUE_ITEM"), false, false, MoveToBottomAction(it)));
+        popup->AddMenuItem(GG::MenuItem(UserString("DELETE_QUEUE_ITEM"),    false, false, DeleteAction(it)));
+
+        auto& row = *it;
+        QueueRow* queue_row = row ? dynamic_cast<QueueRow*>(row.get()) : nullptr;
         if (!queue_row)
             return;
 
         // pause / resume commands
         if (queue_row->elem.paused) {
-            menu_contents.next_level.push_back(GG::MenuItem(UserString("RESUME"),           7, false, false));
+            popup->AddMenuItem(GG::MenuItem(UserString("RESUME"),           false, false, resume_action));
         } else {
-            menu_contents.next_level.push_back(GG::MenuItem(UserString("PAUSE"),            8, false, false));
+            popup->AddMenuItem(GG::MenuItem(UserString("PAUSE"),            false, false, pause_action));
         }
 
         // pedia lookup
@@ -328,46 +340,9 @@ protected:
             tech_name = UserString(queue_row->elem.name);
 
         std::string popup_label = boost::io::str(FlexibleFormat(UserString("ENC_LOOKUP")) % tech_name);
-        menu_contents.next_level.push_back(GG::MenuItem(popup_label, 4, false, false));
+        popup->AddMenuItem(GG::MenuItem(popup_label, false, false, pedia_action));
 
-        CUIPopupMenu popup(pt.x, pt.y, menu_contents);
-
-        if (popup.Run()) {
-            switch (popup.MenuID()) {
-            case 1: { // move item to top
-                if (GG::ListBox::Row* row = *it)
-                    this->QueueItemMovedSignal(row, 0);
-                break;
-            }
-            case 2: { // move item to bottom
-                if (GG::ListBox::Row* row = *it)
-                    this->QueueItemMovedSignal(row, NumRows());
-                break;
-            }
-            case 3: { // delete item
-                this->QueueItemDeletedSignal(it);
-                break;
-            }
-            case 4: { // pedia lookup
-                this->LeftClickedSignal(it, pt, modkeys);
-                ShowPediaSignal();
-                break;
-            }
-
-
-            case 7: { // resume
-                this->QueueItemPausedSignal(it, false);
-                break;
-            }
-            case 8: { // pause
-                this->QueueItemPausedSignal(it, true);
-                break;
-            }
-
-            default:
-                break;
-            }
-        }
+        popup->Run();
     }
 };
 
@@ -379,10 +354,24 @@ public:
     /** \name Structors */ //@{
     ResearchQueueWnd(GG::X x, GG::Y y, GG::X w, GG::Y h) :
         CUIWnd("", x, y, w, h, GG::INTERACTIVE | GG::RESIZABLE | GG::DRAGABLE | GG::ONTOP | PINABLE,
-               "research.ResearchQueueWnd"),
+               "research.queue"),
         m_queue_lb(nullptr)
-    {
-        Init(HumanClientApp::GetApp()->EmpireID());
+    {}
+
+    void CompleteConstruction() override {
+        m_queue_lb = GG::Wnd::Create<ResearchQueueListBox>(std::string("RESEARCH_QUEUE_ROW"), UserString("RESEARCH_QUEUE_PROMPT"));
+        m_queue_lb->SetStyle(GG::LIST_NOSORT | GG::LIST_NOSEL | GG::LIST_USERDELETE);
+        m_queue_lb->SetName("ResearchQueue ListBox");
+
+        SetEmpire(HumanClientApp::GetApp()->EmpireID());
+
+        AttachChild(m_queue_lb);
+
+        CUIWnd::CompleteConstruction();
+
+        DoLayout();
+        SaveDefaultedOptions();
+        SaveOptions();
     }
     //@}
 
@@ -394,7 +383,7 @@ public:
             DoLayout();
     }
 
-    ResearchQueueListBox*   GetQueueListBox() { return m_queue_lb; }
+    ResearchQueueListBox*   GetQueueListBox() { return m_queue_lb.get(); }
 
     void                SetEmpire(int id) {
         if (const Empire* empire = GetEmpire(id))
@@ -410,18 +399,7 @@ private:
                                 GG::Pt(ClientWidth(), ClientHeight() - GG::Y(CUIWnd::INNER_BORDER_ANGLE_OFFSET)));
     }
 
-    void Init(int empire_id) {
-        m_queue_lb = new ResearchQueueListBox("RESEARCH_QUEUE_ROW", UserString("RESEARCH_QUEUE_PROMPT"));
-        m_queue_lb->SetStyle(GG::LIST_NOSORT | GG::LIST_NOSEL | GG::LIST_USERDELETE);
-        m_queue_lb->SetName("ResearchQueue ListBox");
-
-        SetEmpire(empire_id);
-
-        AttachChild(m_queue_lb);
-        DoLayout();
-    }
-
-    ResearchQueueListBox*   m_queue_lb;
+    std::shared_ptr<ResearchQueueListBox>   m_queue_lb;
 };
 
 
@@ -436,21 +414,33 @@ ResearchWnd::ResearchWnd(GG::X w, GG::Y h, bool initially_hidden /*= true*/) :
     m_enabled(false),
     m_empire_shown_id(ALL_EMPIRES)
 {
-    GG::X queue_width(GetOptionsDB().Get<int>("UI.queue-width"));
-    GG::Pt tech_tree_wnd_size = ClientSize() - GG::Pt(GG::X(GetOptionsDB().Get<int>("UI.queue-width")), GG::Y0);
+    GG::X queue_width(GetOptionsDB().Get<int>("ui.queue.width"));
+    GG::Pt tech_tree_wnd_size = ClientSize() - GG::Pt(GG::X(GetOptionsDB().Get<int>("ui.queue.width")), GG::Y0);
 
-    m_research_info_panel = new ProductionInfoPanel(UserString("RESEARCH_WND_TITLE"), UserString("RESEARCH_INFO_RP"),
-                                                    GG::X0, GG::Y0, GG::X(queue_width), GG::Y(100), "research.InfoPanel");
-    m_queue_wnd = new ResearchQueueWnd(GG::X0, GG::Y(100), queue_width, GG::Y(ClientSize().y - 100));
-    m_tech_tree_wnd = new TechTreeWnd(tech_tree_wnd_size.x, tech_tree_wnd_size.y, initially_hidden);
+    m_research_info_panel = GG::Wnd::Create<ResourceInfoPanel>(
+        UserString("RESEARCH_WND_TITLE"), UserString("RESEARCH_INFO_RP"),
+        GG::X0, GG::Y0, GG::X(queue_width), GG::Y(100), "research.info");
+    m_queue_wnd = GG::Wnd::Create<ResearchQueueWnd>(GG::X0, GG::Y(100), queue_width, GG::Y(ClientSize().y - 100));
+    m_tech_tree_wnd = GG::Wnd::Create<TechTreeWnd>(tech_tree_wnd_size.x, tech_tree_wnd_size.y, initially_hidden);
 
-    GG::Connect(m_queue_wnd->GetQueueListBox()->QueueItemMovedSignal,   &ResearchWnd::QueueItemMoved,               this);
-    GG::Connect(m_queue_wnd->GetQueueListBox()->QueueItemDeletedSignal, &ResearchWnd::DeleteQueueItem,              this);
-    GG::Connect(m_queue_wnd->GetQueueListBox()->LeftClickedSignal,      &ResearchWnd::QueueItemClickedSlot,         this);
-    GG::Connect(m_queue_wnd->GetQueueListBox()->DoubleClickedSignal,    &ResearchWnd::QueueItemDoubleClickedSlot,   this);
-    GG::Connect(m_queue_wnd->GetQueueListBox()->ShowPediaSignal,        &ResearchWnd::ShowPedia,                    this);
-    GG::Connect(m_queue_wnd->GetQueueListBox()->QueueItemPausedSignal,  &ResearchWnd::QueueItemPaused,              this);
-    GG::Connect(m_tech_tree_wnd->AddTechsToQueueSignal,                 &ResearchWnd::AddTechsToQueueSlot,          this);
+    m_queue_wnd->GetQueueListBox()->MovedRowSignal.connect(
+        boost::bind(&ResearchWnd::QueueItemMoved, this, _1, _2));
+    m_queue_wnd->GetQueueListBox()->QueueItemDeletedSignal.connect(
+        boost::bind(&ResearchWnd::DeleteQueueItem, this, _1));
+    m_queue_wnd->GetQueueListBox()->LeftClickedRowSignal.connect(
+        boost::bind(&ResearchWnd::QueueItemClickedSlot, this, _1, _2, _3));
+    m_queue_wnd->GetQueueListBox()->DoubleClickedRowSignal.connect(
+        boost::bind(&ResearchWnd::QueueItemDoubleClickedSlot, this, _1, _2, _3));
+    m_queue_wnd->GetQueueListBox()->ShowPediaSignal.connect(
+        boost::bind(&ResearchWnd::ShowPedia, this));
+    m_queue_wnd->GetQueueListBox()->QueueItemPausedSignal.connect(
+        boost::bind(&ResearchWnd::QueueItemPaused, this, _1, _2));
+    m_tech_tree_wnd->AddTechsToQueueSignal.connect(
+        boost::bind(&ResearchWnd::AddTechsToQueueSlot, this, _1, _2));
+}
+
+void ResearchWnd::CompleteConstruction() {
+    GG::Wnd::CompleteConstruction();
 
     AttachChild(m_research_info_panel);
     AttachChild(m_queue_wnd);
@@ -458,7 +448,7 @@ ResearchWnd::ResearchWnd(GG::X w, GG::Y h, bool initially_hidden /*= true*/) :
 
     SetChildClippingMode(ClipToClient);
 
-    DoLayout();
+    DoLayout(true);
 }
 
 ResearchWnd::~ResearchWnd()
@@ -471,14 +461,25 @@ void ResearchWnd::SizeMove(const GG::Pt& ul, const GG::Pt& lr) {
         DoLayout();
 }
 
-void ResearchWnd::DoLayout() {
+void ResearchWnd::DoLayout(bool init) {
     m_research_info_panel->MoveTo(GG::Pt(GG::X0, GG::Y0));
-    m_research_info_panel->Resize(GG::Pt(GG::X(GetOptionsDB().Get<int>("UI.queue-width")),
-                                         m_research_info_panel->MinUsableSize().y));
+    GG::X queue_width = GG::X(init ? GetOptionsDB().GetDefault<int>("ui.queue.width") :
+                                     GetOptionsDB().Get<int>("ui.queue.width"));
+    if (init) {
+        GG::Pt info_ul = m_research_info_panel->UpperLeft();
+        GG::Pt info_lr(info_ul.x + queue_width, info_ul.y + m_research_info_panel->MinUsableSize().y);
+        m_research_info_panel->InitSizeMove(info_ul, info_lr);
+    } else {
+        m_research_info_panel->Resize(GG::Pt(queue_width, m_research_info_panel->MinUsableSize().y));
+    }
+
     GG::Pt queue_ul = GG::Pt(GG::X(2), m_research_info_panel->Height());
     GG::Pt queue_size = GG::Pt(m_research_info_panel->Width() - 4,
                                ClientSize().y - 4 - m_research_info_panel->Height());
-    m_queue_wnd->SizeMove(queue_ul, queue_ul + queue_size);
+    if (init)
+        m_queue_wnd->InitSizeMove(queue_ul, queue_ul + queue_size);
+    else
+        m_queue_wnd->SizeMove(queue_ul, queue_ul + queue_size);
 
     GG::Pt tech_tree_wnd_size = ClientSize() - GG::Pt(m_research_info_panel->Width(), GG::Y0);
     GG::Pt tech_tree_wnd_ul = GG::Pt(m_research_info_panel->Width(), GG::Y0);
@@ -492,8 +493,8 @@ void ResearchWnd::Refresh() {
     m_empire_connection.disconnect();
 
     if (Empire* empire = GetEmpire(HumanClientApp::GetApp()->EmpireID()))
-        m_empire_connection = GG::Connect(empire->GetResearchQueue().ResearchQueueChangedSignal,
-                                          &ResearchWnd::ResearchQueueChangedSlot, this);
+        m_empire_connection = empire->GetResearchQueue().ResearchQueueChangedSignal.connect(
+                                boost::bind(&ResearchWnd::ResearchQueueChangedSlot, this));
     Update();
 }
 
@@ -513,10 +514,12 @@ void ResearchWnd::Update() {
 void ResearchWnd::CenterOnTech(const std::string& tech_name)
 { m_tech_tree_wnd->CenterOnTech(tech_name); }
 
-void ResearchWnd::ShowTech(const std::string& tech_name) {
-    m_tech_tree_wnd->CenterOnTech(tech_name);
+void ResearchWnd::ShowTech(const std::string& tech_name, bool force) {
     m_tech_tree_wnd->SetEncyclopediaTech(tech_name);
-    m_tech_tree_wnd->SelectTech(tech_name);
+    if (force || m_tech_tree_wnd->TechIsVisible(tech_name)) {
+        m_tech_tree_wnd->CenterOnTech(tech_name);
+        m_tech_tree_wnd->SelectTech(tech_name);
+    }
 }
 
 void ResearchWnd::ShowPedia()
@@ -531,11 +534,18 @@ void ResearchWnd::TogglePedia()
 bool ResearchWnd::PediaVisible()
 { return m_tech_tree_wnd->PediaVisible(); }
 
-void ResearchWnd::QueueItemMoved(GG::ListBox::Row* row, std::size_t position) {
-    if (QueueRow* queue_row = boost::polymorphic_downcast<QueueRow*>(row)) {
+void ResearchWnd::QueueItemMoved(const GG::ListBox::iterator& row_it, const GG::ListBox::iterator& original_position_it) {
+    if (QueueRow* queue_row = boost::polymorphic_downcast<QueueRow*>(row_it->get())) {
         int empire_id = HumanClientApp::GetApp()->EmpireID();
+
+        // This precorrects the position for a factor in Empire::PlaceTechInQueue
+        auto position = std::distance(m_queue_wnd->GetQueueListBox()->begin(), row_it);
+        auto original_position = std::distance(m_queue_wnd->GetQueueListBox()->begin(), original_position_it);
+        auto orginal_greater = original_position > position;
+        auto corrected_position = position + (orginal_greater ? 0 : 1);
+
         HumanClientApp::GetApp()->Orders().IssueOrder(
-            OrderPtr(new ResearchQueueOrder(empire_id, queue_row->elem.name, static_cast<int>(position))));
+            std::make_shared<ResearchQueueOrder>(empire_id, queue_row->elem.name, static_cast<int>(corrected_position)));
         if (Empire* empire = GetEmpire(empire_id))
             empire->UpdateResearchQueue();
     }
@@ -575,7 +585,7 @@ void ResearchWnd::UpdateQueue() {
         return;
 
     for (const ResearchQueue::Element& elem : empire->GetResearchQueue()) {
-        QueueRow* row = new QueueRow(queue_lb->RowWidth(), elem);
+        auto row = GG::Wnd::Create<QueueRow>(queue_lb->RowWidth(), elem);
         queue_lb->Insert(row);
     }
 
@@ -633,10 +643,10 @@ void ResearchWnd::AddTechsToQueueSlot(const std::vector<std::string>& tech_vec,
         // or that we skipped because it happened to already be in the right spot.
         if (pos == -1) {
             if (!queue.InQueue(tech_name)) {
-                orders.IssueOrder(OrderPtr(new ResearchQueueOrder(empire_id, tech_name, pos)));
+                orders.IssueOrder(std::make_shared<ResearchQueueOrder>(empire_id, tech_name, pos));
             }
         } else if (!queue.InQueue(tech_name) || ((queue.find(tech_name) - queue.begin()) > pos)) {
-            orders.IssueOrder(OrderPtr(new ResearchQueueOrder(empire_id, tech_name, pos)));
+            orders.IssueOrder(std::make_shared<ResearchQueueOrder>(empire_id, tech_name, pos));
             pos += 1;
         } else {
             if ((queue.find(tech_name) - queue.begin()) == pos)
@@ -651,24 +661,32 @@ void ResearchWnd::DeleteQueueItem(GG::ListBox::iterator it) {
         return;
     int empire_id = HumanClientApp::GetApp()->EmpireID();
     OrderSet& orders = HumanClientApp::GetApp()->Orders();
-    if (QueueRow* queue_row = boost::polymorphic_downcast<QueueRow*>(*it))
-        orders.IssueOrder(OrderPtr(new ResearchQueueOrder(empire_id, queue_row->elem.name)));
+    if (QueueRow* queue_row = boost::polymorphic_downcast<QueueRow*>(it->get()))
+        orders.IssueOrder(std::make_shared<ResearchQueueOrder>(empire_id, queue_row->elem.name));
     if (Empire* empire = GetEmpire(empire_id))
         empire->UpdateResearchQueue();
 }
 
 void ResearchWnd::QueueItemClickedSlot(GG::ListBox::iterator it, const GG::Pt& pt, const GG::Flags<GG::ModKey>& modkeys) {
     if (m_queue_wnd->GetQueueListBox()->DisplayingValidQueueItems()) {
-        QueueRow* queue_row = boost::polymorphic_downcast<QueueRow*>(*it);
-        if (!queue_row)
-            return;
-        ShowTech(queue_row->elem.name);
+        if (modkeys & GG::MOD_KEY_CTRL) {
+            DeleteQueueItem(it);
+        } else {
+            auto queue_row = boost::polymorphic_downcast<QueueRow*>(it->get());
+            if (!queue_row)
+                return;
+            ShowTech(queue_row->elem.name, false);
+        }
     }
 }
 
 void ResearchWnd::QueueItemDoubleClickedSlot(GG::ListBox::iterator it, const GG::Pt& pt, const GG::Flags<GG::ModKey>& modkeys) {
-    if (m_queue_wnd->GetQueueListBox()->DisplayingValidQueueItems())
-        DeleteQueueItem(it);
+    if (m_queue_wnd->GetQueueListBox()->DisplayingValidQueueItems()) {
+        QueueRow* queue_row = boost::polymorphic_downcast<QueueRow*>(it->get());
+        if (!queue_row)
+            return;
+        ShowTech(queue_row->elem.name);
+    }
 }
 
 void ResearchWnd::QueueItemPaused(GG::ListBox::iterator it, bool pause) {
@@ -681,9 +699,9 @@ void ResearchWnd::QueueItemPaused(GG::ListBox::iterator it, bool pause) {
 
     // todo: reject action if shown queue is not this client's empire's queue
 
-    if (QueueRow* queue_row = boost::polymorphic_downcast<QueueRow*>(*it))
+    if (QueueRow* queue_row = boost::polymorphic_downcast<QueueRow*>(it->get()))
         HumanClientApp::GetApp()->Orders().IssueOrder(
-            OrderPtr(new ResearchQueueOrder(client_empire_id, queue_row->elem.name, pause, -1.0f)));
+            std::make_shared<ResearchQueueOrder>(client_empire_id, queue_row->elem.name, pause, -1.0f));
 
     empire->UpdateResearchQueue();
 }
