@@ -216,7 +216,7 @@ Sound::Impl::Impl() :
     m_temporary_disable_count(0),
     m_initialized(false)
 {
-    if (!GetOptionsDB().Get<bool>("UI.sound.enabled") && !GetOptionsDB().Get<bool>("UI.sound.music-enabled"))
+    if (!GetOptionsDB().Get<bool>("audio.effects.enabled") && !GetOptionsDB().Get<bool>("audio.music.enabled"))
         return;
 
     try {
@@ -370,7 +370,7 @@ void Sound::Impl::ShutdownOpenAL() {
         alDeleteSources(NUM_SOURCES, m_sources); // Automatically stops currently playing sources
 
         alDeleteBuffers(NUM_MUSIC_BUFFERS, m_music_buffers);
-        for (std::map<std::string, ALuint>::value_type& buffer : m_buffers)
+        for (auto& buffer : m_buffers)
             alDeleteBuffers(1, &(buffer.second));
 
         ALCdevice* device = alcGetContextsDevice(context);
@@ -390,9 +390,9 @@ void Sound::Impl::PlayMusic(const boost::filesystem::path& path, int loops /* =
         return;
 
     ALenum m_openal_error;
-    std::string filename = PathString(path);
+    std::string filename = PathToString(path);
     FILE* m_f = nullptr;
-    vorbis_info* vorbis_info;
+    vorbis_info* vorbis_info_ptr;
     m_music_loops = 0;
 
 #ifdef FREEORION_WIN32
@@ -424,12 +424,12 @@ void Sound::Impl::PlayMusic(const boost::filesystem::path& path, int loops /* =
             {
                 ov_test_open(&m_ogg_file); // it is, now fully open the file
                 /* now we need to take some info we will need later */
-                vorbis_info = ov_info(&m_ogg_file, -1);
-                if (vorbis_info->channels == 1)
+                vorbis_info_ptr = ov_info(&m_ogg_file, -1);
+                if (vorbis_info_ptr->channels == 1)
                     m_ogg_format = AL_FORMAT_MONO16;
                 else
                     m_ogg_format = AL_FORMAT_STEREO16;
-                m_ogg_freq = vorbis_info->rate;
+                m_ogg_freq = vorbis_info_ptr->rate;
                 m_music_loops = loops;
                 /* fill up the buffers and queue them up for the first time */
                 if (!RefillBuffer(&m_ogg_file, m_ogg_format, m_ogg_freq, m_music_buffers[0], BUFFER_SIZE, m_music_loops))
@@ -500,10 +500,10 @@ void Sound::Impl::StopMusic() {
 }
 
 void Sound::Impl::PlaySound(const boost::filesystem::path& path, bool is_ui_sound/* = false*/) {
-    if (!m_initialized || !GetOptionsDB().Get<bool>("UI.sound.enabled") || (is_ui_sound && UISoundsTemporarilyDisabled()))
+    if (!m_initialized || !GetOptionsDB().Get<bool>("audio.effects.enabled") || (is_ui_sound && UISoundsTemporarilyDisabled()))
         return;
 
-    std::string filename = PathString(path);
+    std::string filename = PathToString(path);
     ALuint current_buffer;
     ALenum source_state;
     ALsizei ogg_freq;
@@ -535,7 +535,7 @@ void Sound::Impl::PlaySound(const boost::filesystem::path& path, bool is_ui_soun
         } else {
             if ((file = fopen(filename.c_str(), "rb")) != nullptr) { // make sure we CAN open it
                 OggVorbis_File ogg_file;
-                vorbis_info *vorbis_info;
+                vorbis_info * vorbis_info_ptr;
                 ALenum ogg_format;
 #ifdef FREEORION_WIN32
                 if (!(ov_test_callbacks(file, &ogg_file, nullptr, 0, callbacks))) // check if it's a proper ogg
@@ -545,13 +545,13 @@ void Sound::Impl::PlaySound(const boost::filesystem::path& path, bool is_ui_soun
                 {
                     ov_test_open(&ogg_file); // it is, now fully open the file
                     /* now we need to take some info we will need later */
-                    vorbis_info = ov_info(&ogg_file, -1);
-                    if (vorbis_info->channels == 1)
+                    vorbis_info_ptr = ov_info(&ogg_file, -1);
+                    if (vorbis_info_ptr->channels == 1)
                         ogg_format = AL_FORMAT_MONO16;
                     else
                         ogg_format = AL_FORMAT_STEREO16;
-                    ogg_freq = vorbis_info->rate;
-                    ogg_int64_t byte_size = ov_pcm_total(&ogg_file, -1) * vorbis_info->channels * 2;
+                    ogg_freq = vorbis_info_ptr->rate;
+                    ogg_int64_t byte_size = ov_pcm_total(&ogg_file, -1) * vorbis_info_ptr->channels * 2;
                     if (byte_size <= 1024 * 1024 * 1024) {
                         /* fill up the buffers and queue them up for the first time */
                         ALuint sound_handle;
@@ -563,7 +563,7 @@ void Sound::Impl::PlaySound(const boost::filesystem::path& path, bool is_ui_soun
 
                         current_buffer = sound_handle;
                         found_buffer = true;
-                        m_buffers.insert(std::make_pair(filename, sound_handle));
+                        m_buffers.insert({filename, sound_handle});
                     }
                     else
                     {
@@ -604,7 +604,7 @@ void Sound::Impl::FreeSound(const boost::filesystem::path& path) {
         return;
 
     ALenum m_openal_error;
-    std::string filename = PathString(path);
+    std::string filename = PathToString(path);
     std::map<std::string, ALuint>::iterator it = m_buffers.find(filename);
 
     if (it != m_buffers.end()) {
@@ -648,7 +648,7 @@ void Sound::Impl::SetMusicVolume(int vol) {
 
     /* normalize value, then apply to all sound sources */
     vol = std::max(0, std::min(vol, 255));
-    GetOptionsDB().Set<int>("UI.sound.music-volume", vol);
+    GetOptionsDB().Set<int>("audio.music.volume", vol);
     if (alcGetCurrentContext())
     {
         alSourcef(m_sources[0], AL_GAIN, ((ALfloat) vol)/255.0);
@@ -667,7 +667,7 @@ void Sound::Impl::SetUISoundsVolume(int vol) {
 
     /* normalize value, then apply to all sound sources */
     vol = std::max(0, std::min(vol, 255));
-    GetOptionsDB().Set<int>("UI.sound.volume", vol);
+    GetOptionsDB().Set<int>("audio.effects.volume", vol);
     if (alcGetCurrentContext()) {
         for (int it = 1; it < NUM_SOURCES; ++it)
             alSourcef(m_sources[it], AL_GAIN, ((ALfloat) vol)/255.0);
