@@ -2,7 +2,8 @@
 #define _Sound_h_
 
 #include <boost/filesystem/path.hpp>
-#include <boost/scoped_ptr.hpp>
+
+#include <memory>
 
 
 class Sound
@@ -31,6 +32,12 @@ public:
         loops + 1 times otherwise. */
     void PlayMusic(const boost::filesystem::path& path, int loops = 0);
 
+    /** Pauses music play, to be continued from the same position */
+    void PauseMusic();
+
+    /** Resumes music play */
+    void ResumeMusic();
+
     /** Stops playing music. */
     void StopMusic();
 
@@ -59,12 +66,13 @@ public:
     void Disable();
 
 private:
-    class SoundImpl;
-    // TODO use C++11 unique_ptr
-    boost::scoped_ptr<SoundImpl> const pimpl;
-    Sound();  ///< ctor.
-    ~Sound(); ///< dotr.
+    class Impl;
 
+    std::unique_ptr<Impl> const m_impl;
+
+    Sound();
+
+    ~Sound();
 };
 
 #endif // _Sound_h_
