@@ -13,7 +13,10 @@ namespace {
     /** Returns how much of specified \a resource_type is being consumed by the
       * empire with id \a empire_id at the location of the specified
       * object \a obj. */
-    double ObjectResourceConsumption(std::shared_ptr<const UniverseObject> obj, ResourceType resource_type, int empire_id = ALL_EMPIRES) {
+    double ObjectResourceConsumption(std::shared_ptr<const UniverseObject> obj,
+                                     ResourceType resource_type,
+                                     int empire_id = ALL_EMPIRES)
+    {
         if (!obj) {
             ErrorLogger() << "ObjectResourceConsumption passed a null object";
             return 0.0;
@@ -50,13 +53,13 @@ namespace {
             // PP (equal to mineral and industry) cost of objects on production queue at this object's location
             if (empire) {
                 // add allocated PP for all production items at this location for this empire
-                for (const ProductionQueue::Element& elem : empire->GetProductionQueue())
+                for (const auto& elem : empire->GetProductionQueue())
                     if (elem.location == obj->ID())
                         prod_queue_allocation_sum += elem.allocated_pp;
 
             } else {
                 // add allocated PP for all production items at this location for all empires
-                for (std::map<int, Empire*>::value_type& entry : Empires()) {
+                for (auto& entry : Empires()) {
                     empire = entry.second;
                     for (const ProductionQueue::Element& elem : empire->GetProductionQueue())
                         if (elem.location == obj->ID())
@@ -67,11 +70,9 @@ namespace {
             break;
 
         case RE_TRADE:
-            return 0.0;
-            break;
-
         case RE_RESEARCH:
-            // research isn't consumed at a particular location, so none is consumed at any location
+        case RE_STOCKPILE:
+            // research/stockpile aren't consumed at a particular location, so none is consumed at any location
         default:
             // for INVALID_RESOURCE_TYPE just return 0.0.  Could throw an exception, I suppose...
             break;
@@ -87,7 +88,8 @@ namespace {
     { return GG::X(ClientUI::Pts()*4); }
 }
 
-SystemResourceSummaryBrowseWnd::SystemResourceSummaryBrowseWnd(ResourceType resource_type, int system_id, int empire_id) :
+SystemResourceSummaryBrowseWnd::SystemResourceSummaryBrowseWnd(ResourceType resource_type,
+                                                               int system_id, int empire_id) :
     GG::BrowseInfoWnd(GG::X0, GG::Y0, LabelWidth() + ValueWidth(), GG::Y1),
     m_resource_type(resource_type),
     m_system_id(system_id),
@@ -136,7 +138,7 @@ void SystemResourceSummaryBrowseWnd::Initialize() {
 
 
     production_label_top = top;
-    m_production_label = new CUILabel("", GG::FORMAT_RIGHT);
+    m_production_label = GG::Wnd::Create<CUILabel>("", GG::FORMAT_RIGHT);
     m_production_label->MoveTo(GG::Pt(GG::X0, production_label_top));
     m_production_label->Resize(GG::Pt(TOTAL_WIDTH - EDGE_PAD, row_height));
     m_production_label->SetFont(ClientUI::GetBoldFont());
@@ -146,7 +148,7 @@ void SystemResourceSummaryBrowseWnd::Initialize() {
 
 
     allocation_label_top = top;
-    m_allocation_label = new CUILabel("", GG::FORMAT_RIGHT);
+    m_allocation_label = GG::Wnd::Create<CUILabel>("", GG::FORMAT_RIGHT);
     m_allocation_label->MoveTo(GG::Pt(GG::X0, allocation_label_top));
     m_allocation_label->Resize(GG::Pt(TOTAL_WIDTH - EDGE_PAD, row_height));
     m_allocation_label->SetFont(ClientUI::GetBoldFont());
@@ -156,7 +158,7 @@ void SystemResourceSummaryBrowseWnd::Initialize() {
 
 
     import_export_label_top = top;
-    m_import_export_label = new CUILabel("", GG::FORMAT_RIGHT);
+    m_import_export_label = GG::Wnd::Create<CUILabel>("", GG::FORMAT_RIGHT);
     m_import_export_label->MoveTo(GG::Pt(GG::X0, import_export_label_top));
     m_import_export_label->Resize(GG::Pt(TOTAL_WIDTH - EDGE_PAD, row_height));
     m_import_export_label->SetFont(ClientUI::GetBoldFont());
@@ -171,13 +173,13 @@ void SystemResourceSummaryBrowseWnd::Initialize() {
 void SystemResourceSummaryBrowseWnd::UpdateProduction(GG::Y& top) {
     // adds pairs of labels for ResourceCenter name and production of resource starting at vertical position \a top
     // and updates \a top to the vertical position after the last entry
-    for (const std::pair<GG::Label*, GG::Label*>& label_pair : m_production_labels_and_amounts) {
-        DeleteChild(label_pair.first);
-        DeleteChild(label_pair.second);
+    for (const auto& label_pair : m_production_labels_and_amounts) {
+        DetachChild(label_pair.first);
+        DetachChild(label_pair.second);
     }
     m_production_labels_and_amounts.clear();
 
-    std::shared_ptr<const System> system = GetSystem(m_system_id);
+    auto system = GetSystem(m_system_id);
     if (!system || m_resource_type == INVALID_RESOURCE_TYPE)
         return;
 
@@ -186,15 +188,14 @@ void SystemResourceSummaryBrowseWnd::UpdateProduction(GG::Y& top) {
 
 
     // add label-value pair for each resource-producing object in system to indicate amount of resource produced
-    std::vector<std::shared_ptr<const UniverseObject>> objects =
-        Objects().FindObjects<const UniverseObject>(system->ContainedObjectIDs());
+    auto objects = Objects().FindObjects<const UniverseObject>(system->ContainedObjectIDs());
 
-    for (std::shared_ptr<const UniverseObject> obj : objects) {
+    for (auto& obj : objects) {
         // display information only for the requested player
         if (m_empire_id != ALL_EMPIRES && !obj->OwnedBy(m_empire_id))
             continue;   // if m_empire_id == -1, display resource production for all empires.  otherwise, skip this resource production if it's not owned by the requested player
 
-        std::shared_ptr<const ResourceCenter> rc = std::dynamic_pointer_cast<const ResourceCenter>(obj);
+        auto rc = std::dynamic_pointer_cast<const ResourceCenter>(obj);
         if (!rc) continue;
 
         std::string name = obj->Name();
@@ -204,17 +205,17 @@ void SystemResourceSummaryBrowseWnd::UpdateProduction(GG::Y& top) {
         std::string amount_text = DoubleToString(production, 3, false);
 
 
-        GG::Label* label = new CUILabel(name, GG::FORMAT_RIGHT);
+        auto label = GG::Wnd::Create<CUILabel>(name, GG::FORMAT_RIGHT);
         label->MoveTo(GG::Pt(GG::X0, top));
         label->Resize(GG::Pt(LabelWidth(), row_height));
         AttachChild(label);
 
-        GG::Label* value = new CUILabel(amount_text);
+        auto value = GG::Wnd::Create<CUILabel>(amount_text);
         value->MoveTo(GG::Pt(LabelWidth(), top));
         value->Resize(GG::Pt(ValueWidth(), row_height));
         AttachChild(value);
 
-        m_production_labels_and_amounts.push_back(std::pair<GG::Label*, GG::Label*>(label, value));
+        m_production_labels_and_amounts.push_back({label, value});
 
         top += row_height;
     }
@@ -222,17 +223,17 @@ void SystemResourceSummaryBrowseWnd::UpdateProduction(GG::Y& top) {
 
     if (m_production_labels_and_amounts.empty()) {
         // add "blank" line to indicate no production
-        GG::Label* label = new CUILabel(UserString("NOT_APPLICABLE"));
+        auto label = GG::Wnd::Create<CUILabel>(UserString("NOT_APPLICABLE"));
         label->MoveTo(GG::Pt(GG::X0, top));
         label->Resize(GG::Pt(LabelWidth(), row_height));
         AttachChild(label);
 
-        GG::Label* value = new CUILabel("");
+        auto value = GG::Wnd::Create<CUILabel>("");
         value->MoveTo(GG::Pt(LabelWidth(), top));
         value->Resize(GG::Pt(ValueWidth(), row_height));
         AttachChild(value);
 
-        m_production_labels_and_amounts.push_back(std::pair<GG::Label*, GG::Label*>(label, value));
+        m_production_labels_and_amounts.push_back({label, value});
 
         top += row_height;
     }
@@ -247,6 +248,8 @@ void SystemResourceSummaryBrowseWnd::UpdateProduction(GG::Y& top) {
         resource_text = UserString("RESEARCH_PRODUCTION");  break;
     case RE_TRADE:
         resource_text = UserString("TRADE_PRODUCTION");     break;
+    case RE_STOCKPILE:
+        resource_text = UserString("STOCKPILE_GENERATION"); break;
     default:
         resource_text = UserString("UNKNOWN_VALUE_SYMBOL"); break;
     }
@@ -261,13 +264,13 @@ void SystemResourceSummaryBrowseWnd::UpdateProduction(GG::Y& top) {
 void SystemResourceSummaryBrowseWnd::UpdateAllocation(GG::Y& top) {
     // adds pairs of labels for allocation of resources in system, starting at vertical position \a top and
     // updates \a top to be the vertical position after the last entry
-    for (const std::pair<GG::Label*, GG::Label*>& label_pair : m_allocation_labels_and_amounts) {
-        DeleteChild(label_pair.first);
-        DeleteChild(label_pair.second);
+    for (const auto& label_pair : m_allocation_labels_and_amounts) {
+        DetachChild(label_pair.first);
+        DetachChild(label_pair.second);
     }
     m_allocation_labels_and_amounts.clear();
 
-    std::shared_ptr<const System> system = GetSystem(m_system_id);
+    auto system = GetSystem(m_system_id);
     if (!system || m_resource_type == INVALID_RESOURCE_TYPE)
         return;
 
@@ -276,7 +279,7 @@ void SystemResourceSummaryBrowseWnd::UpdateAllocation(GG::Y& top) {
 
 
     // add label-value pair for each resource-consuming object in system to indicate amount of resource consumed
-    for (std::shared_ptr<const UniverseObject> obj : Objects().FindObjects<const UniverseObject>(system->ContainedObjectIDs())) {
+    for (auto& obj : Objects().FindObjects<const UniverseObject>(system->ContainedObjectIDs())) {
         // display information only for the requested player
         if (m_empire_id != ALL_EMPIRES && !obj->OwnedBy(m_empire_id))
             continue;   // if m_empire_id == ALL_EMPIRES, display resource production for all empires.  otherwise, skip this resource production if it's not owned by the requested player
@@ -301,18 +304,18 @@ void SystemResourceSummaryBrowseWnd::UpdateAllocation(GG::Y& top) {
         std::string amount_text = DoubleToString(allocation, 3, false);
 
 
-        GG::Label* label = new CUILabel(name, GG::FORMAT_RIGHT);
+        auto label = GG::Wnd::Create<CUILabel>(name, GG::FORMAT_RIGHT);
         label->MoveTo(GG::Pt(GG::X0, top));
         label->Resize(GG::Pt(LabelWidth(), row_height));
         AttachChild(label);
 
 
-        GG::Label* value = new CUILabel(amount_text);
+        auto value = GG::Wnd::Create<CUILabel>(amount_text);
         value->MoveTo(GG::Pt(LabelWidth(), top));
         value->Resize(GG::Pt(ValueWidth(), row_height));
         AttachChild(value);
 
-        m_allocation_labels_and_amounts.push_back(std::pair<GG::Label*, GG::Label*>(label, value));
+        m_allocation_labels_and_amounts.push_back({label, value});
 
         top += row_height;
     }
@@ -320,17 +323,17 @@ void SystemResourceSummaryBrowseWnd::UpdateAllocation(GG::Y& top) {
 
     if (m_allocation_labels_and_amounts.empty()) {
         // add "blank" line to indicate no allocation
-        GG::Label* label = new CUILabel(UserString("NOT_APPLICABLE"), GG::FORMAT_RIGHT);
+        auto label = GG::Wnd::Create<CUILabel>(UserString("NOT_APPLICABLE"), GG::FORMAT_RIGHT);
         label->MoveTo(GG::Pt(GG::X0, top));
         label->Resize(GG::Pt(LabelWidth(), row_height));
         AttachChild(label);
 
-        GG::Label* value = new CUILabel("");
+        auto value = GG::Wnd::Create<CUILabel>("");
         value->MoveTo(GG::Pt(LabelWidth(), top));
         value->Resize(GG::Pt(ValueWidth(), row_height));
         AttachChild(value);
 
-        m_allocation_labels_and_amounts.push_back(std::pair<GG::Label*, GG::Label*>(label, value));
+        m_allocation_labels_and_amounts.push_back({label, value});
 
         top += row_height;
     }
@@ -345,15 +348,19 @@ void SystemResourceSummaryBrowseWnd::UpdateAllocation(GG::Y& top) {
         resource_text = UserString("RESEARCH_CONSUMPTION"); break;
     case RE_TRADE:
         resource_text = UserString("TRADE_CONSUMPTION");    break;
+    case RE_STOCKPILE:
+        resource_text = UserString("STOCKPILE_USE");        break;
     default:
         resource_text = UserString("UNKNOWN_VALUE_SYMBOL"); break;
     }
 
     std::string system_allocation_text = DoubleToString(m_allocation, 3, false);
 
-    // for research only, local allocation makes no sense
+    // for research and stockpiling, local allocation makes no sense
     if (m_resource_type == RE_RESEARCH && m_allocation == 0.0)
         system_allocation_text = UserString("NOT_APPLICABLE");
+    if (m_resource_type == RE_STOCKPILE && m_allocation == 0.0)
+        system_allocation_text = UserString("NOT_APPLICABLE");
 
 
     m_allocation_label->SetText(boost::io::str(FlexibleFormat(UserString("RESOURCE_ALLOCATION_TOOLTIP")) %
@@ -370,7 +377,10 @@ void SystemResourceSummaryBrowseWnd::UpdateImportExport(GG::Y& top) {
 
     // check for early exit cases...
     bool abort = false;
-    if (m_empire_id == ALL_EMPIRES ||m_resource_type == RE_RESEARCH) {
+    if (m_empire_id == ALL_EMPIRES ||
+        m_resource_type == RE_RESEARCH ||
+        m_resource_type == RE_STOCKPILE)
+    {
         // multiple empires have complicated stockpiling which don't make sense to try to display.
         // Research use is nonlocalized, so importing / exporting doesn't make sense to display
         abort = true;
@@ -405,6 +415,7 @@ void SystemResourceSummaryBrowseWnd::UpdateImportExport(GG::Y& top) {
             }
             break;
         case RE_RESEARCH:
+        case RE_STOCKPILE:
         default:
             // show nothing
             abort = true;
@@ -420,41 +431,41 @@ void SystemResourceSummaryBrowseWnd::UpdateImportExport(GG::Y& top) {
 
 
     // add label and amount.  may be "NOT APPLIABLE" and nothing if aborted above
-    GG::Label* label = new CUILabel(label_text, GG::FORMAT_RIGHT);
+    auto label = GG::Wnd::Create<CUILabel>(label_text, GG::FORMAT_RIGHT);
     label->MoveTo(GG::Pt(GG::X0, top));
     label->Resize(GG::Pt(LabelWidth(), row_height));
     AttachChild(label);
 
-    GG::Label* value = new CUILabel(amount_text);
+    auto value = GG::Wnd::Create<CUILabel>(amount_text);
     value->MoveTo(GG::Pt(LabelWidth(), top));
     value->Resize(GG::Pt(ValueWidth(), row_height));
     AttachChild(value);
 
-    m_import_export_labels_and_amounts.push_back(std::pair<GG::Label*, GG::Label*>(label, value));
+    m_import_export_labels_and_amounts.push_back({label, value});
 
     top += row_height;
 }
 
 void SystemResourceSummaryBrowseWnd::Clear() {
-    DeleteChild(m_production_label);
-    DeleteChild(m_allocation_label);
-    DeleteChild(m_import_export_label);
+    DetachChildAndReset(m_production_label);
+    DetachChildAndReset(m_allocation_label);
+    DetachChildAndReset(m_import_export_label);
 
-    for (const std::pair<GG::Label*, GG::Label*>& label_pair : m_production_labels_and_amounts) {
-        DeleteChild(label_pair.first);
-        DeleteChild(label_pair.second);
+    for (const auto& label_pair : m_production_labels_and_amounts) {
+        DetachChild(label_pair.first);
+        DetachChild(label_pair.second);
     }
     m_production_labels_and_amounts.clear();
 
-    for (const std::pair<GG::Label*, GG::Label*>& label_pair : m_allocation_labels_and_amounts) {
-        DeleteChild(label_pair.first);
-        DeleteChild(label_pair.second);
+    for (const auto& label_pair : m_allocation_labels_and_amounts) {
+        DetachChild(label_pair.first);
+        DetachChild(label_pair.second);
     }
     m_allocation_labels_and_amounts.clear();
 
-    for (const std::pair<GG::Label*, GG::Label*>& label_pair : m_import_export_labels_and_amounts) {
-        DeleteChild(label_pair.first);
-        DeleteChild(label_pair.second);
+    for (const auto& label_pair : m_import_export_labels_and_amounts) {
+        DetachChild(label_pair.first);
+        DetachChild(label_pair.second);
     }
     m_import_export_labels_and_amounts.clear();
 }
