@@ -19,6 +19,7 @@ required_boost_libraries = [
     "boost_iostreams",
     "boost_locale",
     "boost_log",
+    "boost_log_setup",
     "boost_python",
     "boost_regex",
     "boost_serialization",
@@ -45,9 +46,15 @@ class Generator(object):
         if build_no == INVALID_BUILD_NO:
             print "WARNING: Can't determine git commit!"
 
-        if os.path.isfile(self.outfile) and not build_no == INVALID_BUILD_NO:
+        if os.path.isfile(self.outfile):
             with open(self.outfile) as check_file:
-                if build_no in check_file.read():
+                check_file_contents = check_file.read()
+                if build_no == INVALID_BUILD_NO:
+                    if version in check_file_contents:
+                        print "Version matches version in existing Version.cpp, skip regenerating it"
+                        return
+                elif build_no in check_file_contents:
+                    print "Build number matches build number in existing Version.cpp, skip regenerating it"
                     return
 
         try:
@@ -113,7 +120,7 @@ if system() == 'Windows':
 if system() == 'Darwin':
     generators.append(Generator('Xcode/Info.plist.in', 'Xcode/Info.plist'))
 
-version = "0.4.7.1"
+version = "0.4.8"
 branch = ""
 build_no = INVALID_BUILD_NO
 
