@@ -5,7 +5,7 @@ import freeOrionAIInterface as fo
 import FreeOrionAI as foAI
 from character.character_module import Aggression
 from character.character_strings_module import possible_greetings
-from freeorion_tools import UserStringList, chat_on_error
+from freeorion_tools import UserStringList
 
 
 def handle_pregame_chat(sender_player_id, message_txt):
@@ -22,7 +22,6 @@ class DiplomaticCorp(object):
     def __init__(self):
         self.be_chatty = True
 
-    @chat_on_error
     def handle_diplomatic_message(self, message):
         """Handle a diplomatic message update from the server,
         such as if another player declares war, accepts peace, or cancels a proposed peace treaty.
@@ -79,7 +78,6 @@ class DiplomaticCorp(object):
             greets = UserStringList("AI_FIRST_TURN_GREETING_BEGINNER")
         return random.choice(greets)
 
-    @chat_on_error
     def handle_diplomatic_status_update(self, status_update):
         """Handle an update about the diplomatic status between players, which may
         or may not include this player."""
@@ -113,8 +111,8 @@ class DiplomaticCorp(object):
 class BeginnerDiplomaticCorp(DiplomaticCorp):
     pass
 
+
 class ManiacalDiplomaticCorp(DiplomaticCorp):
     def __init__(self):
         DiplomaticCorp.__init__(self)
         self.be_chatty = False
-
