@@ -1,33 +1,31 @@
-import sys
+import math
+from logging import warn
 
 import freeOrionAIInterface as fo  # pylint: disable=import-error
 import FreeOrionAI as foAI
 from EnumsAI import MissionType, ShipRoleType
 import CombatRatingsAI
-from universe_object import Planet
+from universe_object import Planet, Fleet
 from ShipDesignAI import get_part_type
 from AIDependencies import INVALID_ID
 import AIDependencies
-__designStats = {}
+import universe_object
 
 
-def stats_meet_reqs(stats, reqs):
+def stats_meet_reqs(stats, requirements):
     """Check if (fleet) stats meet requirements.
 
     :param stats: Stats (of fleet)
     :type stats: dict
-    :param reqs: Requirements
-    :type reqs: dict
+    :param requirements: Requirements
+    :type requirements: dict
     :return: True if requirements are met.
     :rtype: bool
     """
-    try:
-        for key in reqs:
-            if stats.get(key, 0) < reqs[key]:
-                return False
-        return True
-    except:
-        return False
+    for key in requirements:
+        if stats.get(key, 0) < requirements[key]:
+            return False
+    return True
 
 
 def count_troops_in_fleet(fleet_id):
@@ -108,6 +106,7 @@ def get_fleets_for_mission(target_stats, min_stats, cur_stats, starting_system,
     # loop over systems in a breadth-first-search trying to find nearby suitable ships in fleet_pool_set
     while systems_enqueued and fleet_pool_set:
         this_system_id = systems_enqueued.pop(0)
+        this_system_obj = universe_object.System(this_system_id)
         systems_visited.append(this_system_id)
         accessible_fleets = foAI.foAIstate.systemStatus.get(this_system_id, {}).get('myFleetsAccessible', [])
         fleets_here = [fid for fid in accessible_fleets if fid in fleet_pool_set]
@@ -127,7 +126,8 @@ def get_fleets_for_mission(target_stats, min_stats, cur_stats, starting_system,
             if species:
                 for ship_id in fleet.shipIDs:
                     ship = universe.getShip(ship_id)
-                    if ship and foAI.foAIstate.get_ship_role(ship.design.id) in colonization_roles and species == ship.speciesName:
+                    if (ship and foAI.foAIstate.get_ship_role(ship.design.id) in colonization_roles and
+                            species == ship.speciesName):
                         break
                 else:  # no suitable species found
                     continue
@@ -137,6 +137,11 @@ def get_fleets_for_mission(target_stats, min_stats, cur_stats, starting_system,
                 troop_capacity = count_troops_in_fleet(fleet_id)
                 if troop_capacity <= 0:
                     continue
+                # providing move_path_func in this fashion rather than directly importing here, in order
+                # to avoid a circular import problem
+                if 'target_system' in target_stats and 'move_path_func' in target_stats:
+                    if not target_stats['move_path_func'](fleet_id, this_system_obj, target_stats['target_system']):
+                        continue
 
             # check if we need additional rating vs planets
             this_rating_vs_planets = 0
@@ -147,8 +152,13 @@ def get_fleets_for_mission(target_stats, min_stats, cur_stats, starting_system,
                     continue
 
             # all checks passed, add ship to selected fleets and update the stats
+            try:
+                fleet_pool_set.remove(fleet_id)
+            except KeyError:
+                warn("After having split a fleet, the original fleet apparently no longer exists.")
+                continue
             fleet_list.append(fleet_id)
-            fleet_pool_set.remove(fleet_id)
+
             this_rating = foAI.foAIstate.get_rating(fleet_id)
             cur_stats['rating'] = CombatRatingsAI.combine_ratings(cur_stats.get('rating', 0), this_rating)
             if 'ratingVsPlanets' in target_stats:
@@ -162,8 +172,7 @@ def get_fleets_for_mission(target_stats, min_stats, cur_stats, starting_system,
                 return fleet_list
 
         # finished system without meeting requirements. Add neighboring systems to search queue.
-        for neighbor_id in [el.key() for el in
-                            universe.getSystemNeighborsMap(this_system_id, fo.empireID())]:
+        for neighbor_id in universe.getImmediateNeighbors(this_system_id, fo.empireID()):
             if all((
                     neighbor_id not in systems_visited,
                     neighbor_id not in systems_enqueued,
@@ -203,7 +212,7 @@ def split_fleet(fleet_id):
         if new_fleet_id:
             new_fleet = universe.getFleet(new_fleet_id)
             if not new_fleet:
-                print >> sys.stderr, "Newly split fleet %d not available from universe" % new_fleet_id
+                warn("Newly split fleet %d not available from universe" % new_fleet_id)
             fo.issueRenameOrder(new_fleet_id, "Fleet %4d" % new_fleet_id)  # to ease review of debugging logs
             fo.issueAggressionOrder(new_fleet_id, True)
             foAI.foAIstate.update_fleet_rating(new_fleet_id)
@@ -211,9 +220,11 @@ def split_fleet(fleet_id):
             foAI.foAIstate.newlySplitFleets[new_fleet_id] = True
         else:
             if fleet.systemID == INVALID_ID:
-                print >> sys.stderr, "Tried to split ship id (%d) from fleet %d when fleet is in starlane" % (ship_id, fleet_id)
+                warn("Tried to split ship id (%d) from fleet %d when fleet is in starlane" % (
+                    ship_id, fleet_id))
             else:
-                print >> sys.stderr, "Got no fleet ID back after trying to split ship id (%d) from fleet %d" % (ship_id, fleet_id)
+                warn("Got no fleet ID back after trying to split ship id (%d) from fleet %d" % (
+                    ship_id, fleet_id))
     foAI.foAIstate.get_fleet_role(fleet_id, force_new=True)
     foAI.foAIstate.update_fleet_rating(fleet_id)
     if newfleets:
@@ -227,19 +238,14 @@ def merge_fleet_a_into_b(fleet_a_id, fleet_b_id, leave_rating=0, need_rating=0,
     fleet_b = universe.getFleet(fleet_b_id)
     if not fleet_a or not fleet_b:
         return 0
+    system_id = fleet_a.systemID
+    if fleet_b.systemID != system_id:
+        return 0
     remaining_rating = CombatRatingsAI.get_fleet_rating(fleet_a_id)
     transferred_rating = 0
-    b_has_monster = False
-    for ship_id in fleet_b.shipIDs:
-        this_ship = universe.getShip(ship_id)
-        if not this_ship:
-            continue
-        if this_ship.isMonster:
-            b_has_monster = True
-            break
     for ship_id in fleet_a.shipIDs:
         this_ship = universe.getShip(ship_id)
-        if not this_ship or this_ship.isMonster != b_has_monster:  # TODO Is there any reason for the monster check?
+        if not this_ship:
             continue
         this_rating = CombatRatingsAI.ShipCombatStats(ship_id).get_rating()
         remaining_rating = CombatRatingsAI.rating_needed(remaining_rating, this_rating)
@@ -250,7 +256,7 @@ def merge_fleet_a_into_b(fleet_a_id, fleet_b_id, leave_rating=0, need_rating=0,
             transferred_rating = CombatRatingsAI.combine_ratings(transferred_rating, this_rating)
         else:
             print "  *** transfer of ship %4d, formerly of fleet %4d, into fleet %4d failed; %s" % (
-                ship_id, fleet_a_id, fleet_b_id, [" context is %s" % context, ""][context == ""])
+                ship_id, fleet_a_id, fleet_b_id, (" context is %s" % context) if context else "")
         if need_rating != 0 and need_rating <= transferred_rating:
             break
     fleet_a = universe.getFleet(fleet_a_id)
@@ -426,7 +432,8 @@ def generate_fleet_orders_for_fleet_missions():
     print "Orbital Defense Fleets: %s" % get_empire_fleet_ids_by_role(MissionType.ORBITAL_DEFENSE)
     print "Outpost Base Fleets: %s" % get_empire_fleet_ids_by_role(MissionType.ORBITAL_OUTPOST)
     print "Invasion Base Fleets: %s" % get_empire_fleet_ids_by_role(MissionType.ORBITAL_INVASION)
-    print "Securing Fleets: %s  (currently FLEET_MISSION_MILITARY should be used instead of this Role)" % get_empire_fleet_ids_by_role(MissionType.SECURE)
+    print "Securing Fleets: %s  (currently FLEET_MISSION_MILITARY should be used instead of this Role)" % (
+        get_empire_fleet_ids_by_role(MissionType.SECURE))
 
     if fo.currentTurn() < 50:
         print
@@ -460,11 +467,12 @@ def generate_fleet_orders_for_fleet_missions():
     for outpost_fleet_mission in outpost_fleet_missions:
         print "    %s" % outpost_fleet_mission
 
-    outpost_base_fleet_missions = foAI.foAIstate.get_fleet_missions_with_any_mission_types([MissionType.ORBITAL_OUTPOST])
+    outpost_base_fleet_missions = foAI.foAIstate.get_fleet_missions_with_any_mission_types(
+        [MissionType.ORBITAL_OUTPOST])
     if outpost_base_fleet_missions:
         print "Outpost Base targets (must have been interrupted by combat): "
     else:
-        print "Outpost targets: None (as expected, due to expected timing of order submission and execution)"
+        print "Outpost Base targets: None (as expected, due to expected timing of order submission and execution)"
     for outpost_fleet_mission in outpost_base_fleet_missions:
         print "    %s" % outpost_fleet_mission
 
@@ -598,3 +606,60 @@ def get_max_fuel(fleet_id):
 def get_fleet_upkeep():
     # TODO: Use new upkeep calculation
     return 1 + AIDependencies.SHIP_UPKEEP * foAI.foAIstate.shipCount
+
+
+def calculate_estimated_time_of_arrival(fleet_id, target_system_id):
+    universe = fo.getUniverse()
+    fleet = universe.getFleet(fleet_id)
+    if not fleet or not fleet.speed:
+        return 99999
+    distance = universe.shortestPathDistance(fleet_id, target_system_id)
+    return math.ceil(float(distance) / fleet.speed)
+
+
+def get_fleet_system(fleet):
+    """Return the current fleet location or the target system if currently on starlane.
+
+    :param fleet:
+    :type fleet: universe_object.Fleet | int
+    :return: current system_id or target system_id if currently on starlane
+    :rtype: int
+    """
+    if isinstance(fleet, int):
+        fleet = fo.getUniverse().getFleet(fleet)
+    return fleet.systemID if fleet.systemID != INVALID_ID else fleet.nextSystemID
+
+
+def get_current_and_max_structure(fleet):
+    """Return a 2-tuple of the sums of structure and maxStructure meters of all ships in the fleet
+
+    :param fleet:
+    :type fleet: int | universe_object.Fleet | fo.Fleet
+    :return: tuple of sums of structure and maxStructure meters of all ships in the fleet
+    :rtype: (float, float)
+    """
+
+    universe = fo.getUniverse()
+    destroyed_ids = universe.destroyedObjectIDs(fo.empireID())
+    if isinstance(fleet, int):
+        fleet = universe.getFleet(fleet)
+    elif isinstance(fleet, Fleet):
+        fleet = fleet.get_object()
+    if not fleet:
+        return (0.0, 0.0)
+    ships_cur_health = 0
+    ships_max_health = 0
+    for ship_id in fleet.shipIDs:
+        # Check if we have see this ship get destroyed in a different fleet since the last time we saw the subject fleet
+        # this may be redundant with the new fleet assignment check made below, but for its limited scope it may be more
+        # reliable, in that it does not rely on any particular handling of post-destruction stats
+        if ship_id in destroyed_ids:
+            continue
+        this_ship = universe.getShip(ship_id)
+        # check that the ship has not been seen in a new fleet since this current fleet was last observed
+        if not (this_ship and this_ship.fleetID == fleet.id):
+            continue
+        ships_cur_health += this_ship.initialMeterValue(fo.meterType.structure)
+        ships_max_health += this_ship.initialMeterValue(fo.meterType.maxStructure)
+
+    return ships_cur_health, ships_max_health
