@@ -1,5 +1,5 @@
 import math
-import sys
+from logging import debug, info, warn
 
 from turn_state import state
 import freeOrionAIInterface as fo
@@ -8,20 +8,23 @@ from common.print_utils import Table, Text, Float
 import FreeOrionAI as foAI
 import AIstate
 import AIDependencies
+import EspionageAI
 import FleetUtilsAI
 import PlanetUtilsAI
 import universe_object
+from MoveUtilsAI import can_travel_to_system
 import ProductionAI
 import ColonisationAI
 import MilitaryAI
 from EnumsAI import MissionType, PriorityType
 import CombatRatingsAI
-from freeorion_tools import tech_is_complete, AITimer
+from freeorion_tools import tech_is_complete, AITimer, get_partial_visibility_turn
 from AIDependencies import INVALID_ID
 
 MAX_BASE_TROOPERS_GOOD_INVADERS = 20
 MAX_BASE_TROOPERS_POOR_INVADERS = 10
 _TROOPS_SAFETY_MARGIN = 1  # try to send this amount of additional troops to account for uncertainties in calculation
+MIN_INVASION_SCORE = 20
 
 invasion_timer = AITimer('get_invasion_fleets()', write_log=False)
 
@@ -32,18 +35,17 @@ def get_invasion_fleets():
     empire = fo.getEmpire()
     empire_id = fo.empireID()
 
-    all_invasion_fleet_ids = FleetUtilsAI.get_empire_fleet_ids_by_role(MissionType.INVASION)
-    AIstate.invasionFleetIDs = FleetUtilsAI.extract_fleet_ids_without_mission_types(all_invasion_fleet_ids)
-
     home_system_id = PlanetUtilsAI.get_capital_sys_id()
-    visible_system_ids = foAI.foAIstate.visInteriorSystemIDs.keys() + foAI.foAIstate. visBorderSystemIDs.keys()
+    visible_system_ids = list(foAI.foAIstate.visInteriorSystemIDs) + list(foAI.foAIstate.visBorderSystemIDs)
 
     if home_system_id != INVALID_ID:
-        accessible_system_ids = [sys_id for sys_id in visible_system_ids
-                                 if (sys_id != INVALID_ID) and universe.systemsConnected(sys_id, home_system_id, empire_id)]
+        accessible_system_ids = [sys_id for sys_id in visible_system_ids if
+                                 (sys_id != INVALID_ID) and universe.systemsConnected(sys_id, home_system_id,
+                                                                                      empire_id)]
     else:
         print "Warning: Empire has no identifiable homeworld; will treat all visible planets as accessible."
-        accessible_system_ids = visible_system_ids  # TODO: check if any troop ships owned, use their system as home system
+        # TODO: check if any troop ships owned, use their system as home system
+        accessible_system_ids = visible_system_ids
 
     acessible_planet_ids = PlanetUtilsAI.get_planets_in__systems_ids(accessible_system_ids)
     all_owned_planet_ids = PlanetUtilsAI.get_all_owned_planet_ids(acessible_planet_ids)  # includes unpopulated outposts
@@ -52,14 +54,15 @@ def get_invasion_fleets():
     invadable_planet_ids = set(all_owned_planet_ids).union(all_populated_planets) - set(empire_owned_planet_ids)
 
     invasion_targeted_planet_ids = get_invasion_targeted_planet_ids(universe.planetIDs, MissionType.INVASION)
-    invasion_targeted_planet_ids.extend(get_invasion_targeted_planet_ids(universe.planetIDs, MissionType.ORBITAL_INVASION))
+    invasion_targeted_planet_ids.extend(
+        get_invasion_targeted_planet_ids(universe.planetIDs, MissionType.ORBITAL_INVASION))
     all_invasion_targeted_system_ids = set(PlanetUtilsAI.get_systems(invasion_targeted_planet_ids))
 
     invasion_fleet_ids = FleetUtilsAI.get_empire_fleet_ids_by_role(MissionType.INVASION)
     num_invasion_fleets = len(FleetUtilsAI.extract_fleet_ids_without_mission_types(invasion_fleet_ids))
 
     print "Current Invasion Targeted SystemIDs: ", PlanetUtilsAI.sys_name_ids(AIstate.invasionTargetedSystemIDs)
-    print "Current Invasion Targeted PlanetIDs: ", PlanetUtilsAI.planet_name_ids(invasion_targeted_planet_ids)
+    print "Current Invasion Targeted PlanetIDs: ", PlanetUtilsAI.planet_string(invasion_targeted_planet_ids)
     print invasion_fleet_ids and "Invasion Fleet IDs: %s" % invasion_fleet_ids or "Available Invasion Fleets: 0"
     print "Invasion Fleets Without Missions: %s" % num_invasion_fleets
 
@@ -93,7 +96,8 @@ def get_invasion_fleets():
         # value to instead record how many base troopers have been queued, so that on later turns we can assess if the
         # process got delayed & perhaps more troopers need to be queued).
 
-        secure_ai_fleet_missions = foAI.foAIstate.get_fleet_missions_with_any_mission_types([MissionType.SECURE])
+        secure_ai_fleet_missions = foAI.foAIstate.get_fleet_missions_with_any_mission_types([MissionType.SECURE,
+                                                                                             MissionType.MILITARY])
 
         # Pass 1: identify qualifying base troop invasion targets
         for pid in invadable_planet_ids:  # TODO: reorganize
@@ -103,13 +107,13 @@ def get_invasion_fleets():
             if not planet:
                 continue
             sys_id = planet.systemID
-            sys_partial_vis_turn = universe.getVisibilityTurnsMap(planet.systemID, empire_id).get(fo.visibility.partial, -9999)
-            planet_partial_vis_turn = universe.getVisibilityTurnsMap(pid, empire_id).get(fo.visibility.partial, -9999)
+            sys_partial_vis_turn = get_partial_visibility_turn(sys_id)
+            planet_partial_vis_turn = get_partial_visibility_turn(pid)
             if planet_partial_vis_turn < sys_partial_vis_turn:
                 continue
             best_base_planet = INVALID_ID
             best_trooper_count = 0
-            for pid2 in state.get_empire_inhabited_planets_by_system().get(sys_id, []):
+            for pid2 in state.get_empire_planets_by_system(sys_id, include_outposts=False):
                 if available_pp.get(pid2, 0) < 2:  # TODO: improve troop base PP sufficiency determination
                     break
                 planet2 = universe.getPlanet(pid2)
@@ -146,7 +150,7 @@ def get_invasion_fleets():
                     all_invasion_targeted_system_ids.add(planet.systemID)
                 # TODO: evaluate changes to situation, any more troops needed, etc.
                 continue  # already building for here
-            _, planet_troops = evaluate_invasion_planet(pid, secure_ai_fleet_missions, False)
+            _, planet_troops = evaluate_invasion_planet(pid, secure_ai_fleet_missions, True)
             sys_id = planet.systemID
             this_sys_status = foAI.foAIstate.systemStatus.get(sys_id, {})
             troop_tally = 0
@@ -165,37 +169,38 @@ def get_invasion_fleets():
             best_base_trooper_here = ProductionAI.get_best_ship_info(PriorityType.PRODUCTION_ORBITAL_INVASION, loc)[1]
             loc_planet = universe.getPlanet(loc)
             if best_base_trooper_here is None:  # shouldn't be possible at this point, but just to be safe
-                print >> sys.stderr, "Could not find a suitable orbital invasion design at %s" % loc_planet
+                warn("Could not find a suitable orbital invasion design at %s" % loc_planet)
                 continue
             # TODO: have TroopShipDesigner give the expected number of troops including species effects directly
             troops_per_ship = best_base_trooper_here.troopCapacity
             species_troop_grade = CombatRatingsAI.get_species_troops_grade(loc_planet.speciesName)
             troops_per_ship = CombatRatingsAI.weight_attack_troops(troops_per_ship, species_troop_grade)
             if not troops_per_ship:
-                print >> sys.stderr, "The best orbital invasion design at %s seems not to have any troop capacity." % loc_planet
+                warn("The best orbital invasion design at %s seems not to have any troop capacity." % loc_planet)
                 continue
-            _, col_design, build_choices = ProductionAI.get_best_ship_info(PriorityType.PRODUCTION_ORBITAL_INVASION, loc)
+            _, col_design, build_choices = ProductionAI.get_best_ship_info(PriorityType.PRODUCTION_ORBITAL_INVASION,
+                                                                           loc)
             if not col_design:
                 continue
             if loc not in build_choices:
-                sys.stderr.write(
-                    'Best troop design %s can not be produces in at planet with id: %s\d' % (col_design, build_choices)
-                )
+                warn('Best troop design %s can not be produced at planet with id: %s\d' % (col_design, build_choices))
+                continue
             n_bases = math.ceil((planet_troops + 1) / troops_per_ship)  # TODO: reconsider this +1 safety factor
             # TODO: evaluate cost and time-to-build of best base trooper here versus cost and time-to-build-and-travel
             # for best regular trooper elsewhere
             # For now, we assume what building base troopers is best so long as either (1) we would need no more than
             # MAX_BASE_TROOPERS_POOR_INVADERS base troop ships, or (2) our base troopers have more than 1 trooper per
             # ship and we would need no more than MAX_BASE_TROOPERS_GOOD_INVADERS base troop ships
-            if (n_bases <= MAX_BASE_TROOPERS_POOR_INVADERS or
-                    (troops_per_ship > 1 and n_bases <= MAX_BASE_TROOPERS_GOOD_INVADERS)):
+            if (n_bases > MAX_BASE_TROOPERS_POOR_INVADERS or
+                    (troops_per_ship > 1 and n_bases > MAX_BASE_TROOPERS_GOOD_INVADERS)):
                 print "ruling out base invasion troopers for %s due to high number (%d) required." % (planet, n_bases)
                 del foAI.foAIstate.qualifyingTroopBaseTargets[pid]
                 continue
             print "Invasion base planning, need %d troops at %d pership, will build %d ships." % (
                 (planet_troops + 1), troops_per_ship, n_bases)
             retval = fo.issueEnqueueShipProductionOrder(col_design.id, loc)
-            print "Enqueueing %d Troop Bases at %s for %s" % (n_bases, PlanetUtilsAI.planet_name_ids([loc]), PlanetUtilsAI.planet_name_ids([pid]))
+            print "Enqueueing %d Troop Bases at %s for %s" % (n_bases, PlanetUtilsAI.planet_string(loc),
+                                                              PlanetUtilsAI.planet_string(pid))
             if retval != 0:
                 all_invasion_targeted_system_ids.add(planet.systemID)
                 reserved_troop_base_targets.append(pid)
@@ -205,7 +210,8 @@ def get_invasion_fleets():
 
     invasion_timer.start("evaluating target planets")
     # TODO: check if any invasion_targeted_planet_ids need more troops assigned
-    evaluated_planet_ids = list(set(invadable_planet_ids) - set(invasion_targeted_planet_ids) - set(reserved_troop_base_targets))
+    evaluated_planet_ids = list(
+        set(invadable_planet_ids) - set(invasion_targeted_planet_ids) - set(reserved_troop_base_targets))
     evaluated_planets = assign_invasion_values(evaluated_planet_ids)
 
     sorted_planets = [(pid, pscore % 10000, ptroops) for pid, (pscore, ptroops) in evaluated_planets.items()]
@@ -223,8 +229,7 @@ def get_invasion_fleets():
             planet and planet.speciesName or "unknown",
             ptroops
         ])
-    print
-    invasion_table.print_table()
+    info(invasion_table)
 
     sorted_planets = filter(lambda x: x[1] > 0, sorted_planets)
     # export opponent planets for other AI modules
@@ -265,7 +270,8 @@ def assign_invasion_values(planet_ids):
     neighbor_values = {}
     neighbor_val_ratio = .95
     universe = fo.getUniverse()
-    secure_missions = foAI.foAIstate.get_fleet_missions_with_any_mission_types([MissionType.SECURE])
+    secure_missions = foAI.foAIstate.get_fleet_missions_with_any_mission_types([MissionType.SECURE,
+                                                                                MissionType.MILITARY])
     for pid in planet_ids:
         planet_values[pid] = neighbor_values.setdefault(pid, evaluate_invasion_planet(pid, secure_missions))
         print "planet %d, values %s" % (pid, planet_values[pid])
@@ -282,20 +288,69 @@ def assign_invasion_values(planet_ids):
                 species_name2 = (planet2 and planet2.speciesName) or ""
                 species2 = fo.getSpecies(species_name2)
                 if species2 and species2.canProduceShips:
-                    planet_industries[pid2] = planet2.currentMeterValue(fo.meterType.industry) + 0.1  # to prevent divide-by-zero
+                    # to prevent divide-by-zero
+                    planet_industries[pid2] = planet2.initialMeterValue(fo.meterType.industry) + 0.1
             industry_ratio = planet_industries[pid] / max(planet_industries.values())
             for pid2 in system.planetIDs:
                 if pid2 == pid:
                     continue
                 planet2 = universe.getPlanet(pid2)
-                if planet2 and (planet2.owner != empire_id) and ((planet2.owner != -1) or (planet.currentMeterValue(fo.meterType.population) > 0)):  # TODO check for allies
-                    planet_values[pid][0] += industry_ratio * neighbor_val_ratio * (neighbor_values.setdefault(pid2, evaluate_invasion_planet(pid2, secure_missions))[0])
+                # TODO check for allies
+                if (planet2 and (planet2.owner != empire_id) and
+                        ((planet2.owner != -1) or (planet2.initialMeterValue(fo.meterType.population) > 0))):
+                    planet_values[pid][0] += (
+                        industry_ratio *
+                        neighbor_val_ratio *
+                        (neighbor_values.setdefault(pid2, evaluate_invasion_planet(pid2, secure_missions))[0])
+                    )
     return planet_values
 
 
 def evaluate_invasion_planet(planet_id, secure_fleet_missions, verbose=True):
     """Return the invasion value (score, troops) of a planet."""
+    universe = fo.getUniverse()
+    empire_id = fo.empireID()
     detail = []
+
+    planet = universe.getPlanet(planet_id)
+    if planet is None:
+        print "Invasion AI couldn't access any info for planet id %d" % planet_id
+        return [0, 0]
+
+    system_id = planet.systemID
+
+    # by using the following instead of simply relying on stealth meter reading, can (sometimes) plan ahead even if
+    # planet is temporarily shrouded by an ion storm
+    predicted_detectable = EspionageAI.colony_detectable_by_empire(planet_id, empire=fo.empireID(),
+                                                                   default_result=False)
+    if not predicted_detectable:
+        if get_partial_visibility_turn(planet_id) < fo.currentTurn():
+            debug("InvasionAI predicts planet id %d to be stealthed" % planet_id)
+            return [0, 0]
+        else:
+            debug("InvasionAI predicts planet id %d to be stealthed" % planet_id +
+                  ", but somehow have current visibity anyway, will still consider as target")
+
+    # Check if the target planet was extra-stealthed somehow its system was last viewed
+    # this test below may augment the tests above, but can be thrown off by temporary combat-related sighting
+    system_last_seen = get_partial_visibility_turn(planet_id)
+    planet_last_seen = get_partial_visibility_turn(system_id)
+    if planet_last_seen < system_last_seen:
+        # TODO: track detection strength, order new scouting when it goes up
+        debug("Invasion AI considering planet id %d (stealthed at last view), still proceeding." % planet_id)
+
+    # get a baseline evaluation of the planet as determined by ColonisationAI
+    species_name = planet.speciesName
+    species = fo.getSpecies(species_name)
+    if not species or AIDependencies.TAG_DESTROYED_ON_CONQUEST in species.tags:
+        # this call iterates over this Empire's available species with which it could colonize after an invasion
+        planet_eval = ColonisationAI.assign_colonisation_values([planet_id], MissionType.INVASION, None, detail)
+        colony_base_value = max(0.75 * planet_eval.get(planet_id, [0])[0],
+                                ColonisationAI.evaluate_planet(planet_id, MissionType.OUTPOST, None, detail))
+    else:
+        colony_base_value = ColonisationAI.evaluate_planet(planet_id, MissionType.INVASION, species_name, detail)
+
+    # Add extra score for all buildings on the planet
     building_values = {"BLD_IMPERIAL_PALACE": 1000,
                        "BLD_CULTURE_ARCHIVES": 1000,
                        "BLD_AUTO_HISTORY_ANALYSER": 100,
@@ -321,59 +376,32 @@ def evaluate_invasion_planet(planet_id, secure_fleet_missions, verbose=True):
                        "BLD_BIOTERROR_PROJECTOR": 1000,
                        "BLD_SHIPYARD_ENRG_COMP": 3000,
                        }
-    # TODO: add more factors, as used for colonization
-    universe = fo.getUniverse()
-    empire_id = fo.empireID()
-    max_jumps = 8
-    planet = universe.getPlanet(planet_id)
-    if planet is None:  # TODO: exclude planets with stealth higher than empireDetection
-        print "invasion AI couldn't access any info for planet id %d" % planet_id
-        return [0, 0]
-
-    sys_partial_vis_turn = universe.getVisibilityTurnsMap(planet.systemID, empire_id).get(fo.visibility.partial, -9999)
-    planet_partial_vis_turn = universe.getVisibilityTurnsMap(planet_id, empire_id).get(fo.visibility.partial, -9999)
-
-    if planet_partial_vis_turn < sys_partial_vis_turn:
-        print "invasion AI couldn't get current info on planet id %d (was stealthed at last sighting)" % planet_id
-        # TODO: track detection strength, order new scouting when it goes up
-        return [0, 0]  # last time we had partial vis of the system, the planet was stealthed to us
-
-    species_name = planet.speciesName
-    species = fo.getSpecies(species_name)
-    if not species or AIDependencies.TAG_DESTROYED_ON_CONQUEST in species.tags:
-        # this call iterates over this Empire's available species with which it could colonize after an invasion
-        planet_eval = ColonisationAI.assign_colonisation_values([planet_id], MissionType.INVASION, None, detail)
-        pop_val = max(0.75 * planet_eval.get(planet_id, [0])[0],
-                      ColonisationAI.evaluate_planet(planet_id, MissionType.OUTPOST, None, detail))
-    else:
-        pop_val = ColonisationAI.evaluate_planet(planet_id, MissionType.INVASION, species_name, detail)
-
     bld_tally = 0
     for bldType in [universe.getBuilding(bldg).buildingTypeName for bldg in planet.buildingIDs]:
         bval = building_values.get(bldType, 50)
         bld_tally += bval
         detail.append("%s: %d" % (bldType, bval))
 
+    # Add extra score for unlocked techs when we conquer the species
     tech_tally = 0
+    value_per_pp = 4
     for unlocked_tech in AIDependencies.SPECIES_TECH_UNLOCKS.get(species_name, []):
         if not tech_is_complete(unlocked_tech):
             rp_cost = fo.getTech(unlocked_tech).researchCost(empire_id)
-            tech_tally += rp_cost * 4
-            detail.append("%s: %d" % (unlocked_tech, rp_cost * 4))
+            tech_value = value_per_pp * rp_cost
+            tech_tally += tech_value
+            detail.append("%s: %d" % (unlocked_tech, tech_value))
 
-    p_sys_id = planet.systemID
+    max_jumps = 8
     capitol_id = PlanetUtilsAI.get_capital()
     least_jumps_path = []
     clear_path = True
     if capitol_id:
         homeworld = universe.getPlanet(capitol_id)
-        if homeworld:
-            home_system_id = homeworld.systemID
-            eval_system_id = planet.systemID
-            if (home_system_id != INVALID_ID) and (eval_system_id != INVALID_ID):
-                least_jumps_path = list(universe.leastJumpsPath(home_system_id, eval_system_id, empire_id))
-                max_jumps = len(least_jumps_path)
-    system_status = foAI.foAIstate.systemStatus.get(p_sys_id, {})
+        if homeworld and homeworld.systemID != INVALID_ID and system_id != INVALID_ID:
+            least_jumps_path = list(universe.leastJumpsPath(homeworld.systemID, system_id, empire_id))
+            max_jumps = len(least_jumps_path)
+    system_status = foAI.foAIstate.systemStatus.get(system_id, {})
     system_fleet_treat = system_status.get('fleetThreat', 1000)
     system_monster_threat = system_status.get('monsterThreat', 0)
     sys_total_threat = system_fleet_treat + system_monster_threat + system_status.get('planetThreat', 0)
@@ -390,21 +418,22 @@ def evaluate_invasion_planet(planet_id, secure_fleet_missions, verbose=True):
     pop = planet.currentMeterValue(fo.meterType.population)
     target_pop = planet.currentMeterValue(fo.meterType.targetPopulation)
     troops = planet.currentMeterValue(fo.meterType.troops)
+    troop_regen = planet.currentMeterValue(fo.meterType.troops) - planet.initialMeterValue(fo.meterType.troops)
     max_troops = planet.currentMeterValue(fo.meterType.maxTroops)
     # TODO: refactor troop determination into function for use in mid-mission updates and also consider defender techs
     max_troops += AIDependencies.TROOPS_PER_POP * (target_pop - pop)
 
-    this_system = universe.getSystem(p_sys_id)
-    secure_targets = [p_sys_id] + list(this_system.planetIDs)
+    this_system = universe.getSystem(system_id)
+    secure_targets = [system_id] + list(this_system.planetIDs)
     system_secured = False
     for mission in secure_fleet_missions:
         if system_secured:
             break
         secure_fleet_id = mission.fleet.id
         s_fleet = universe.getFleet(secure_fleet_id)
-        if not s_fleet or s_fleet.systemID != p_sys_id:
+        if not s_fleet or s_fleet.systemID != system_id:
             continue
-        if mission.type == MissionType.SECURE:
+        if mission.type in [MissionType.SECURE, MissionType.MILITARY]:
             target_obj = mission.target.get_object()
             if target_obj is not None and target_obj.id in secure_targets:
                 system_secured = True
@@ -418,31 +447,22 @@ def evaluate_invasion_planet(planet_id, secure_fleet_missions, verbose=True):
                " - sysMonsterThreat: %.1f") % (
             planet, planet.currentMeterValue(fo.meterType.maxShield), system_fleet_treat,
             system_monster_threat)
-    supply_val = 0
     enemy_val = 0
     if planet.owner != -1:  # value in taking this away from an enemy
-        enemy_val = 20 * (planet.currentMeterValue(fo.meterType.targetIndustry) + 2*planet.currentMeterValue(fo.meterType.targetResearch))
-    if p_sys_id in ColonisationAI.annexable_system_ids:  # TODO: extend to rings
-        supply_val = 100
-    elif p_sys_id in ColonisationAI.annexable_ring1:
-        supply_val = 200
-    elif p_sys_id in ColonisationAI.annexable_ring2:
-        supply_val = 300
-    elif p_sys_id in ColonisationAI.annexable_ring3:
-        supply_val = 400
-    if max_path_threat > 0.5 * mil_ship_rating:
-        if max_path_threat < 3 * mil_ship_rating:
-            supply_val *= 0.5
-        else:
-            supply_val *= 0.2
+        enemy_val = 20 * (planet.currentMeterValue(fo.meterType.targetIndustry) +
+                          2*planet.currentMeterValue(fo.meterType.targetResearch))
 
-    threat_factor = min(1, 0.2*MilitaryAI.get_tot_mil_rating()/(sys_total_threat+0.001))**2  # devalue invasions that would require too much military force
+    # devalue invasions that would require too much military force
+    preferred_max_portion = MilitaryAI.get_preferred_max_military_portion_for_single_battle()
+    total_max_mil_rating = MilitaryAI.get_concentrated_tot_mil_rating()
+    threat_exponent = 2  # TODO: make this a character trait; higher aggression with a lower exponent
+    threat_factor = min(1, preferred_max_portion * total_max_mil_rating/(sys_total_threat+0.001))**threat_exponent
 
     design_id, _, locs = ProductionAI.get_best_ship_info(PriorityType.PRODUCTION_INVASION)
     if not locs or not universe.getPlanet(locs[0]):
         # We are in trouble anyway, so just calculate whatever approximation...
         build_time = 4
-        planned_troops = troops if system_secured else min(troops + max_jumps + build_time, max_troops)
+        planned_troops = troops if system_secured else min(troops + troop_regen*(max_jumps + build_time), max_troops)
         planned_troops += .01  # we must attack with more troops than there are defenders
         troop_cost = math.ceil((planned_troops+_TROOPS_SAFETY_MARGIN) / 6.0) * 20 * FleetUtilsAI.get_fleet_upkeep()
     else:
@@ -453,31 +473,38 @@ def evaluate_invasion_planet(planet_id, secure_fleet_missions, verbose=True):
         build_time = design.productionTime(empire_id, loc)
         troops_per_ship = CombatRatingsAI.weight_attack_troops(design.troopCapacity,
                                                                CombatRatingsAI.get_species_troops_grade(species_here))
-        planned_troops = troops if system_secured else min(troops + max_jumps + build_time, max_troops)
+        planned_troops = troops if system_secured else min(troops + troop_regen*(max_jumps + build_time), max_troops)
         planned_troops += .01  # we must attack with more troops than there are defenders
         ships_needed = math.ceil((planned_troops+_TROOPS_SAFETY_MARGIN) / float(troops_per_ship))
         troop_cost = ships_needed * cost_per_ship  # fleet upkeep is already included in query from server
 
     # apply some bias to expensive operations
     normalized_cost = float(troop_cost) / max(fo.getEmpire().productionPoints, 1)
-    normalized_cost = max(1, normalized_cost)
+    normalized_cost = max(1., normalized_cost)
     cost_score = (normalized_cost**2 / 50.0) * troop_cost
 
-    base_score = pop_val + supply_val + bld_tally + tech_tally + enemy_val - cost_score
+    base_score = colony_base_value + bld_tally + tech_tally + enemy_val - cost_score
+    # If the AI does have enough total miltary to attack this target, and the target is more than minimally valuable,
+    # don't let the threat_factor discount the adjusted value below MIN_INVASION_SCORE +1, so that if there are no
+    # other targets the AI could still pursue this one.  Otherwise, scoring pressure from
+    # MilitaryAI.get_preferred_max_military_portion_for_single_battle might prevent the AI from attacking heavily
+    # defended but still defeatable targets even if it has no softer targets available.
+    if total_max_mil_rating > sys_total_threat and base_score > 2 * MIN_INVASION_SCORE:
+        threat_factor = max(threat_factor, (MIN_INVASION_SCORE + 1)/base_score)
     planet_score = retaliation_risk_factor(planet.owner) * threat_factor * max(0, base_score)
     if clear_path:
         planet_score *= 1.5
     if verbose:
         print (' - planet score: %.2f\n'
-               ' - troop score: %.2f\n'
+               ' - planned troops: %.2f\n'
                ' - projected troop cost: %.1f\n'
                ' - threat factor: %s\n'
                ' - planet detail: %s\n'
                ' - popval: %.1f\n'
-               ' - supplyval: %.1f\n'
                ' - bldval: %s\n'
                ' - enemyval: %s') % (planet_score, planned_troops, troop_cost,
-                                     threat_factor, detail, pop_val, supply_val, bld_tally, enemy_val)
+                                     threat_factor, detail, colony_base_value, bld_tally, enemy_val)
+        print (' - system secured: %s' % system_secured)
     return [planet_score, planned_troops]
 
 
@@ -490,6 +517,8 @@ def send_invasion_fleets(fleet_ids, evaluated_planets, mission_type):
     invasion_fleet_pool = set(fleet_ids)
 
     for planet_id, pscore, ptroops in evaluated_planets:
+        if pscore < MIN_INVASION_SCORE:
+            continue
         planet = universe.getPlanet(planet_id)
         if not planet:
             continue
@@ -497,7 +526,10 @@ def send_invasion_fleets(fleet_ids, evaluated_planets, mission_type):
         found_fleets = []
         found_stats = {}
         min_stats = {'rating': 0, 'troopCapacity': ptroops}
-        target_stats = {'rating': 10, 'troopCapacity': ptroops + _TROOPS_SAFETY_MARGIN}
+        target_stats = {'rating': 10,
+                        'troopCapacity': ptroops + _TROOPS_SAFETY_MARGIN,
+                        'target_system': universe_object.System(sys_id),
+                        'move_path_func': can_travel_to_system}
         these_fleets = FleetUtilsAI.get_fleets_for_mission(target_stats, min_stats, found_stats,
                                                            starting_system=sys_id, fleet_pool_set=invasion_fleet_pool,
                                                            fleet_list=found_fleets)
@@ -578,7 +610,8 @@ def assign_invasion_fleets_to_invade():
 
     assign_invasion_bases()
 
-    invasion_fleet_ids = AIstate.invasionFleetIDs
+    all_invasion_fleet_ids = FleetUtilsAI.get_empire_fleet_ids_by_role(MissionType.INVASION)
+    invasion_fleet_ids = FleetUtilsAI.extract_fleet_ids_without_mission_types(all_invasion_fleet_ids)
     send_invasion_fleets(invasion_fleet_ids, AIstate.invasionTargets, MissionType.INVASION)
     all_invasion_fleet_ids = FleetUtilsAI.get_empire_fleet_ids_by_role(MissionType.INVASION)
     for fid in FleetUtilsAI.extract_fleet_ids_without_mission_types(all_invasion_fleet_ids):
