@@ -1,6 +1,8 @@
 import math
 
 import freeOrionAIInterface as fo  # pylint: disable=import-error
+
+import AIDependencies
 import AIstate
 import ColonisationAI
 import ExplorationAI
@@ -11,15 +13,13 @@ import MilitaryAI
 import PlanetUtilsAI
 import ProductionAI
 import ResearchAI
-import AIDependencies
-from turn_state import state
-from EnumsAI import PriorityType, MissionType, EmpireProductionTypes, get_priority_production_types, ShipRoleType
-from freeorion_tools import AITimer, tech_is_complete
 from AIDependencies import INVALID_ID
+from EnumsAI import EmpireProductionTypes, MissionType, PriorityType, ShipRoleType, get_priority_production_types
+from freeorion_tools import AITimer, tech_is_complete
+from turn_state import state
 
 prioritiees_timer = AITimer('calculate_priorities()')
 
-allottedInvasionTargets = 0
 allottedColonyTargets = 0
 allotted_outpost_targets = 0
 unmetThreat = 0
@@ -37,11 +37,11 @@ def calculate_priorities():
 
     print "\n*** Updating Colonization Status ***\n"
     prioritiees_timer.start('Evaluating Colonization Status')
-    ColonisationAI.get_colony_fleets()  # sets foAI.foAIstate.colonisablePlanetIDs and foAI.foAIstate.outpostPlanetIDs and many other values used by other modules
+    ColonisationAI.get_colony_fleets()  # sets foAI.foAIstate.colonisablePlanetIDs and many other values used by other modules
 
     print "\n*** Updating Invasion Status ***\n"
     prioritiees_timer.start('Evaluating Invasion Status')
-    InvasionAI.get_invasion_fleets()  # sets AIstate.invasionFleetIDs, AIstate.opponentPlanetIDs, and AIstate.invasionTargetedPlanetIDs
+    InvasionAI.get_invasion_fleets()  # sets AIstate.opponentPlanetIDs, and AIstate.invasionTargetedPlanetIDs
 
     print "\n*** Updating Military Status ***\n"
     prioritiees_timer.start('Evaluating Military Status')
@@ -67,13 +67,6 @@ def calculate_priorities():
     prioritiees_timer.start('setting other priorities')
     foAI.foAIstate.set_priority(PriorityType.PRODUCTION_BUILDINGS, 25)
 
-    foAI.foAIstate.set_priority(PriorityType.RESEARCH_LEARNING, _calculate_learning_priority())
-    foAI.foAIstate.set_priority(PriorityType.RESEARCH_GROWTH, _calculate_growth_priority())
-    foAI.foAIstate.set_priority(PriorityType.RESEARCH_PRODUCTION, _calculate_techs_production_priority())
-    foAI.foAIstate.set_priority(PriorityType.RESEARCH_CONSTRUCTION, _calculate_construction_priority())
-    foAI.foAIstate.set_priority(PriorityType.RESEARCH_ECONOMICS, 0)
-    foAI.foAIstate.set_priority(PriorityType.RESEARCH_SHIPS, _calculate_ships_priority())
-    foAI.foAIstate.set_priority(PriorityType.RESEARCH_DEFENSE, 0)
     prioritiees_timer.stop_print_and_clear()
 
 
@@ -100,7 +93,6 @@ def _calculate_research_priority():
     """Calculates the AI empire's demand for research."""
     universe = fo.getUniverse()
     empire = fo.getEmpire()
-    empire_id = empire.empireID
     current_turn = fo.currentTurn()
     enemies_sighted = foAI.foAIstate.misc.get('enemies_sighted', {})
     recent_enemies = [x for x in enemies_sighted if x > current_turn - 8]
@@ -115,7 +107,7 @@ def _calculate_research_priority():
     mgrav_prod_tech = AIDependencies.PRO_MICROGRAV_MAN
     got_mgrav_prod = tech_is_complete(mgrav_prod_tech)
     # got_solar_gen = tech_is_complete(AIDependencies.PRO_SOL_ORB_GEN)
-    
+
     milestone_techs = ["PRO_SENTIENT_AUTOMATION", "LRN_DISTRIB_THOUGHT", "LRN_QUANT_NET", "SHP_WEAPON_2_4", "SHP_WEAPON_3_2", "SHP_WEAPON_4_2"]
     milestones_done = [mstone for mstone in milestone_techs if tech_is_complete(mstone)]
     print "Research Milestones accomplished at turn %d: %s" % (current_turn, milestones_done)
@@ -125,11 +117,11 @@ def _calculate_research_priority():
     industry_surge = (foAI.foAIstate.character.may_surge_industry(total_pp, total_rp) and
                       (((orb_gen_tech in research_queue_list[:2] or got_orb_gen) and state.have_gas_giant) or
                        ((mgrav_prod_tech in research_queue_list[:2] or got_mgrav_prod) and state.have_asteroids)) and
-                      (not (len(AIstate.popCtrIDs) >= 12)))
+                      (state.get_number_of_colonies() < 12))
     # get current industry production & Target
     owned_planet_ids = PlanetUtilsAI.get_owned_planets_by_empire(universe.planetIDs)
     planets = map(universe.getPlanet, owned_planet_ids)
-    target_rp = sum(map(lambda x: x.currentMeterValue(fo.meterType.targetResearch), planets))
+    target_rp = sum(map(lambda _x: _x.currentMeterValue(fo.meterType.targetResearch), planets))
     galaxy_is_sparse = ColonisationAI.galaxy_is_sparse()
     enemies_sighted = foAI.foAIstate.misc.get('enemies_sighted', {})
 
@@ -165,7 +157,7 @@ def _calculate_research_priority():
             research_priority *= 0.5
         else:
             research_priority *= 0.8
-                
+
     if ((tech_is_complete("SHP_WEAPON_2_4") or
          tech_is_complete("SHP_WEAPON_4_1")) and
             tech_is_complete(AIDependencies.PROD_AUTO_NAME)):
@@ -188,7 +180,7 @@ def _calculate_research_priority():
 def _calculate_exploration_priority():
     """Calculates the demand for scouts by unexplored systems."""
     empire = fo.getEmpire()
-    num_unexplored_systems = len(ExplorationAI.borderUnexploredSystemIDs)
+    num_unexplored_systems = len(ExplorationAI.border_unexplored_system_ids)
     num_scouts = sum([foAI.foAIstate.fleetStatus.get(fid, {}).get('nships', 0) for fid in FleetUtilsAI.get_empire_fleet_ids_by_role(
         MissionType.EXPLORATION)])
     production_queue = empire.productionQueue
@@ -225,13 +217,15 @@ def _calculate_colonisation_priority():
     enemies_sighted = foAI.foAIstate.misc.get('enemies_sighted', {})
     galaxy_is_sparse = ColonisationAI.galaxy_is_sparse()
     total_pp = fo.getEmpire().productionPoints
-    num_colonies = len(list(AIstate.popCtrIDs))
+    num_colonies = state.get_number_of_colonies()
     colony_growth_barrier = foAI.foAIstate.character.max_number_colonies()
+    if num_colonies > colony_growth_barrier:
+        return 0.0
     colony_cost = AIDependencies.COLONY_POD_COST * (1 + AIDependencies.COLONY_POD_UPKEEP * num_colonies)
     turns_to_build = 8  # TODO: check for susp anim pods, build time 10
     mil_prio = foAI.foAIstate.get_priority(PriorityType.PRODUCTION_MILITARY)
     allotted_portion = ([[[0.6, 0.8], [0.3, 0.4]], [[0.8, 0.9], [0.4, 0.6]]][galaxy_is_sparse]
-                       [any(enemies_sighted)])
+                        [any(enemies_sighted)])
     allotted_portion = foAI.foAIstate.character.preferred_colonization_portion(allotted_portion)
     # if ( foAI.foAIstate.get_priority(AIPriorityType.PRIORITY_PRODUCTION_COLONISATION)
     # > 2 * foAI.foAIstate.get_priority(AIPriorityType.PRIORITY_PRODUCTION_MILITARY)):
@@ -245,20 +239,31 @@ def _calculate_colonisation_priority():
     # allottedColonyTargets = 1+ int(fo.currentTurn()/50)
     allottedColonyTargets = 1 + int(total_pp * turns_to_build * allotted_portion / colony_cost)
     outpost_prio = foAI.foAIstate.get_priority(PriorityType.PRODUCTION_OUTPOST)
-    # if have any outposts to build, don't build colony ships TODO: make more complex assessment
-    if outpost_prio > 0 or num_colonies > colony_growth_barrier:
-        return 0.0
 
-    if num_colonies > colony_growth_barrier:
+    # if have no SP_SLY, and have any outposts to build, don't build colony ships TODO: make more complex assessment
+    colonizers = list(ColonisationAI.empire_colonizers)
+    if "SP_SLY" not in colonizers and outpost_prio > 0:
         return 0.0
-    num_colonisable_planet_ids = len([pid for (pid, (score, _)) in foAI.foAIstate.colonisablePlanetIDs.items()
-                                   if score > 60][:allottedColonyTargets + 2])
+    min_score = ColonisationAI.MINIMUM_COLONY_SCORE
+    minimal_top = min_score + 2  # one more than the conditional floor set by ColonisationAI.revise_threat_factor()
+    minimal_opportunities = [species_name for (_, (score, species_name)) in foAI.foAIstate.colonisablePlanetIDs.items()
+                             if min_score < score <= minimal_top]
+    decent_opportunities = [species_name for (_, (score, species_name)) in foAI.foAIstate.colonisablePlanetIDs.items()
+                            if score > minimal_top]
+    minimal_planet_factor = 0.2  # count them for something, but not much
+    num_colonisable_planet_ids = len(decent_opportunities) + minimal_planet_factor * len(minimal_opportunities)
     if num_colonisable_planet_ids == 0:
         return 1
 
     colony_ship_ids = FleetUtilsAI.get_empire_fleet_ids_by_role(MissionType.COLONISATION)
     num_colony_ships = len(FleetUtilsAI.extract_fleet_ids_without_mission_types(colony_ship_ids))
-    colonisation_priority = 60 * (1 + num_colonisable_planet_ids - num_colony_ships) / (num_colonisable_planet_ids + 1)
+    colonisation_priority = 60 * (1.0 + num_colonisable_planet_ids - num_colony_ships) / (num_colonisable_planet_ids + 1)
+
+    if colonizers == ["SP_SLY"]:
+        colonisation_priority *= 2
+    elif "SP_SLY" in colonizers:
+        colony_opportunities = minimal_opportunities + decent_opportunities
+        colonisation_priority *= (1.0 + colony_opportunities.count("SP_SLY")) / len(colony_opportunities)
 
     # print
     # print "Number of Colony Ships : " + str(num_colony_ships)
@@ -278,9 +283,8 @@ def _calculate_outpost_priority():
     enemies_sighted = foAI.foAIstate.misc.get('enemies_sighted', {})
     galaxy_is_sparse = ColonisationAI.galaxy_is_sparse()
     total_pp = fo.getEmpire().productionPoints
-    num_colonies = len(list(AIstate.popCtrIDs))
     colony_growth_barrier = foAI.foAIstate.character.max_number_colonies()
-    if num_colonies > colony_growth_barrier:
+    if state.get_number_of_colonies() > colony_growth_barrier:
         return 0.0
     mil_prio = foAI.foAIstate.get_priority(PriorityType.PRODUCTION_MILITARY)
 
@@ -300,13 +304,19 @@ def _calculate_outpost_priority():
     allotted_outpost_targets = 1 + int(total_pp * 3 * allotted_portion / base_outpost_cost)
 
     num_outpost_targets = len([pid for (pid, (score, specName)) in foAI.foAIstate.colonisableOutpostIDs.items()
-                               if score > 1.0 * base_outpost_cost / 3.0][:allotted_outpost_targets])
+                               if score > max(1.0 * base_outpost_cost / 3.0, ColonisationAI.MINIMUM_COLONY_SCORE)]
+                              [:allotted_outpost_targets])
     if num_outpost_targets == 0 or not tech_is_complete(AIDependencies.OUTPOSTING_TECH):
         return 0
 
     outpost_ship_ids = FleetUtilsAI.get_empire_fleet_ids_by_role(MissionType.OUTPOST)
     num_outpost_ships = len(FleetUtilsAI.extract_fleet_ids_without_mission_types(outpost_ship_ids))
-    outpost_priority = 50 * (num_outpost_targets - num_outpost_ships) / num_outpost_targets
+    outpost_priority = (50.0 * (num_outpost_targets - num_outpost_ships)) / num_outpost_targets
+
+    # discourage early outposting for SP_SLY, due to supply and stealth considerations they are best off
+    # using colony ships until they have other colonizers (and more established military)
+    if list(ColonisationAI.empire_colonizers) == ["SP_SLY"]:
+        outpost_priority /= 3.0
 
     # print
     # print "Number of Outpost Ships : " + str(num_outpost_ships)
@@ -320,28 +330,25 @@ def _calculate_outpost_priority():
 
 def _calculate_invasion_priority():
     """Calculates the demand for troop ships by opponent planets."""
-    
-    global allottedInvasionTargets
+
     if not foAI.foAIstate.character.may_invade():
         return 0
-    
+
     empire = fo.getEmpire()
     enemies_sighted = foAI.foAIstate.misc.get('enemies_sighted', {})
     multiplier = 1
-    num_colonies = len(list(AIstate.popCtrIDs))
     colony_growth_barrier = foAI.foAIstate.character.max_number_colonies()
-    if num_colonies > colony_growth_barrier:
+    if state.get_number_of_colonies() > colony_growth_barrier:
         return 0.0
-    
+
     if len(foAI.foAIstate.colonisablePlanetIDs) > 0:
         best_colony_score = max(2, foAI.foAIstate.colonisablePlanetIDs.items()[0][1][0])
     else:
         best_colony_score = 2
 
-    allottedInvasionTargets = 1 + int(fo.currentTurn() / 25)
     total_val = 0
     troops_needed = 0
-    for pid, pscore, trp in AIstate.invasionTargets[:allottedInvasionTargets]:
+    for pid, pscore, trp in AIstate.invasionTargets[:allotted_invasion_targets()]:
         if pscore > best_colony_score:
             multiplier += 1
             total_val += 2 * pscore
@@ -384,13 +391,17 @@ def _calculate_invasion_priority():
             invasion_priority *= 1.5
     if not enemies_sighted:
         invasion_priority *= 1.5
-        
+
     if invasion_priority < 0:
         return 0
 
     return invasion_priority * foAI.foAIstate.character.invasion_priority_scaling()
 
 
+def allotted_invasion_targets():
+    return 1 + int(fo.currentTurn() / 25)
+
+
 def _calculate_military_priority():
     """Calculates the demand for military ships by military targeted systems."""
     global unmetThreat
@@ -399,7 +410,7 @@ def _calculate_military_priority():
     capital_id = PlanetUtilsAI.get_capital()
     if capital_id is None or capital_id == INVALID_ID:
         return 0  # no capitol (not even a capitol-in-the-making), means can't produce any ships
-        
+
     have_l1_weaps = (tech_is_complete("SHP_WEAPON_1_4") or
                      (tech_is_complete("SHP_WEAPON_1_3") and tech_is_complete("SHP_MIL_ROBO_CONT")) or
                      tech_is_complete("SHP_WEAPON_2_1") or
@@ -407,11 +418,12 @@ def _calculate_military_priority():
     have_l2_weaps = (tech_is_complete("SHP_WEAPON_2_3") or
                      tech_is_complete("SHP_WEAPON_4_1"))
     enemies_sighted = foAI.foAIstate.misc.get('enemies_sighted', {})
-        
-    allotted_invasion_targets = 1 + int(fo.currentTurn()/25)
-    target_planet_ids = [pid for pid, pscore, trp in AIstate.invasionTargets[:allotted_invasion_targets]] + [pid for pid, pscore in foAI.foAIstate.colonisablePlanetIDs.items()[:allottedColonyTargets]] + [pid for pid, pscore in foAI.foAIstate.colonisableOutpostIDs.items()[:allottedColonyTargets]]
 
-    my_systems = set(AIstate.popCtrSystemIDs).union(AIstate.outpostSystemIDs)
+    target_planet_ids = ([pid for pid, pscore, trp in AIstate.invasionTargets[:allotted_invasion_targets()]] +
+                         [pid for pid, pscore in foAI.foAIstate.colonisablePlanetIDs.items()[:allottedColonyTargets]] +
+                         [pid for pid, pscore in foAI.foAIstate.colonisableOutpostIDs.items()[:allottedColonyTargets]])
+
+    my_systems = set(state.get_empire_planets_by_system())
     target_systems = set(PlanetUtilsAI.get_systems(target_planet_ids))
 
     cur_ship_rating = ProductionAI.cur_best_military_design_rating()
@@ -457,7 +469,7 @@ def _calculate_military_priority():
     fmt_string = "Calculating Military Priority:  min(t,40) + max(0,75 * ships_needed) \n\t  Priority: %d  \t ships_needed: %d \t defense_ships_needed: %d \t curShipRating: %.0f \t l1_weaps: %s \t enemies_sighted: %s"
     print fmt_string % (military_priority, ships_needed, defense_ships_needed, cur_ship_rating, have_l1_weaps, enemies_sighted)
     print "Source of milship demand: ", ships_needed_allocation
-    
+
     military_priority *= foAI.foAIstate.character.military_priority_scaling()
     return max(military_priority, 0)
 
@@ -476,48 +488,3 @@ def _calculate_top_production_queue_priority():
             top_production_queue_priority = evaluationPair[0]
 
     return top_production_queue_priority
-
-
-def _calculate_learning_priority():
-    """Calculates the demand for techs learning category."""
-    turn = fo.currentTurn()
-    if turn == 1:
-        return 100
-    elif turn > 1:
-        return 0
-
-
-def _calculate_growth_priority():
-    """Calculates the demand for techs growth category."""
-    production_priority = _calculate_top_production_queue_priority()
-    if production_priority == 8:
-        return 70
-    elif production_priority != 8:
-        return 0
-
-
-def _calculate_techs_production_priority():
-    """Calculates the demand for techs production category."""
-    production_priority = _calculate_top_production_queue_priority()
-    if production_priority == 7 or production_priority == 9:
-        return 60
-    elif production_priority != 7 or production_priority != 9:
-        return 0
-
-
-def _calculate_construction_priority():
-    """Calculates the demand for techs construction category."""
-    production_priority = _calculate_top_production_queue_priority()
-    if production_priority == 6 or production_priority == 11:
-        return 80
-    elif production_priority != 6 or production_priority != 11:
-        return 30
-
-
-def _calculate_ships_priority():
-    """Calculates the demand for techs ships category."""
-    production_priority = _calculate_top_production_queue_priority()
-    if production_priority == 10:
-        return 90
-    elif production_priority != 10:
-        return 0
