@@ -3,13 +3,10 @@ The TechsListAI module provides functions that describes dependencies between
 various technologies to help the AI decide which technologies should be
 researched next.
 """
-import sys
+from logging import warn
 
-from freeorion_tools import print_error
 import freeOrionAIInterface as fo  # pylint: disable=import-error
 
-EXOBOT_TECH_NAME = "PRO_EXOBOTS"
-
 
 def unusable_techs():
     """
@@ -85,12 +82,12 @@ class TechGroup(object):
                     # Do not display error message as those should be shown only once per game session
                     # by the initial test_tech_integrity() call.
                     msg = "Try to enqueue tech from empty list"
-                    print >> sys.stderr, msg
+                    warn(msg)
                     self._errors.append(msg)
                     continue
             if tech_name in self._tech_queue:
                 msg = "Tech is already in queue: %s" % tech_name
-                print >> sys.stderr, msg
+                warn(msg)
                 self._errors.append(msg)
             else:
                 self._tech_queue.append(tech_name)
@@ -109,10 +106,10 @@ class TechGroup1(TechGroup):
     def __init__(self):
         super(TechGroup1, self).__init__()
         self.economy.extend([
-            "GRO_PLANET_ECOL",
-            "GRO_SUBTER_HAB",
-            "LRN_ALGO_ELEGANCE",
             "LRN_PHYS_BRAIN",
+            "GRO_PLANET_ECOL",
+            "LRN_ALGO_ELEGANCE",
+            "GRO_SUBTER_HAB",
             "LRN_ARTIF_MINDS",
             "CON_ORBITAL_CON",  # not a economy tech in the strictest sense but bonus supply often equals more planets
             "PRO_ROBOTIC_PROD",
@@ -199,6 +196,7 @@ class TechGroup1SparseB(TechGroup1):
             "SHP_SPACE_FLUX_DRIVE"
         )
 
+
 class TechGroup1SparseC(TechGroup1):
     def __init__(self):
         super(TechGroup1SparseC, self).__init__()
@@ -645,10 +643,10 @@ def test_tech_integrity():
         techs = this_group.get_techs()
         for tech in techs:
             if not fo.getTech(tech):
-                print_error("In %s: Tech %s seems not to exist!" % (group.__name__, tech))
+                warn("In %s: Tech %s seems not to exist!" % (group.__name__, tech))
                 error_occured = True
         for err in this_group.get_errors():
-            print_error(err, location=group.__name__)
+            warn(err, exc_info=True)
             error_occured = True
         if not error_occured:
             print "Seems to be OK!"
