@@ -1,4 +1,4 @@
-import sys
+from logging import warn
 
 import freeOrionAIInterface as fo  # pylint: disable=import-error
 from AIDependencies import INVALID_ID
@@ -9,11 +9,12 @@ class UniverseObject(object):
     Stores information about AI target - its id and type.
     :type id: int
     """
+    object_name = 'universe_object'
 
     def __init__(self, target_id):
         self.id = target_id
         if not self:
-            print >> sys.stderr, "Target is invalid %s" % self
+            warn("Target is invalid %s" % self)
 
     def __cmp__(self, other):
         return type(self) == type(other) and cmp(self.id, other.id)
@@ -77,6 +78,17 @@ class System(UniverseObject):
 class Fleet(UniverseObject):
     object_name = 'fleet'
 
+    def get_current_system_id(self):
+        """
+        Get current systemID (or INVALID_ID if on a starlane).
+
+        :rtype: int
+        """
+        universe = fo.getUniverse()
+        fleet = universe.getFleet(self.id)
+        # will also return INVALID_ID if somehow the fleet cannot be retrieved
+        return fleet.systemID if fleet else INVALID_ID
+
     def get_system(self):
         """
         Get current fleet location or target system if currently on starlane.
@@ -90,4 +102,7 @@ class Fleet(UniverseObject):
         return System(system_id)
 
     def get_object(self):
+        """
+        :rtype fo.fleet:
+        """
         return fo.getUniverse().getFleet(self.id)
