@@ -72,9 +72,12 @@ def _create_default_config_file(path):
         for k, v in entries.iteritems():
             config.set(section, k, str(v))
     if path:
-        with open(unicode(path, 'utf-8'), 'w') as configfile:
-            config.write(configfile)
-        print "default config is dumped to %s" % path
+        try:
+            with open(unicode(path, 'utf-8'), 'w') as configfile:
+                config.write(configfile)
+            print "default config is dumped to %s" % path
+        except IOError:
+            sys.stderr.write("AI Config Error: could not write default config %s\n" % path)
     return config
 
 
@@ -99,6 +102,12 @@ def get_sectioned_option_dict():
 
 
 def parse_config(option_string, config_dir):
+
+    if option_string is not None and not isinstance(option_string, str):
+        # probably called by unit test
+        print >> sys.stderr, "Specified option string is not a string: ", option_string
+        return
+
     # get defaults; check if don't already exist and can write
     default_file = _get_option_file(config_dir)
     if os.path.exists(default_file):
