@@ -6,24 +6,29 @@
 #include <GG/Enum.h>
 
 #include <boost/shared_array.hpp>
+#include <boost/date_time/posix_time/posix_time_types.hpp>
+#include <boost/uuid/uuid.hpp>
 
 #if defined(_MSC_VER) && defined(int64_t)
 #undef int64_t
 #endif
 
 #include <array>
+#include <set>
 #include <map>
 #include <string>
 #include <vector>
 
 
+enum class LogLevel;
 class EmpireManager;
 class SupplyManager;
 class SpeciesManager;
-class CombatLog;
+struct CombatLog;
 class CombatLogManager;
 class Message;
 struct MultiplayerLobbyData;
+struct ChatHistoryEntity;
 class OrderSet;
 struct PlayerInfo;
 struct SaveGameUIData;
@@ -47,10 +52,12 @@ namespace Moderator {
   * misbehave as well.) */
 class FO_COMMON_API Message {
 public:
-    typedef std::array<int, 5> HeaderBuffer;
+    enum Parts : size_t {TYPE = 0, SIZE, Parts_Count};
+
+    typedef std::array<int, Parts::Parts_Count> HeaderBuffer;
 
     constexpr static size_t HeaderBufferSize =
-        std::tuple_size<HeaderBuffer>::value * sizeof(HeaderBuffer::value_type);
+        std::tuple_size<HeaderBuffer>::value* sizeof(HeaderBuffer::value_type);
 
     /** Represents the type of the message */
     GG_CLASS_ENUM(MessageType,
@@ -62,7 +69,6 @@ public:
         JOIN_GAME,              ///< sent when a client wishes to join a game being established at the server
         HOST_ID,                ///< sent to clients when the server changes the ID of the host player
         LOBBY_UPDATE,           ///< used to synchronize multiplayer lobby dialogs among different players, when a user changes a setting, or the server updates the state
-        LOBBY_CHAT,             ///< used to send chat messages in the multiplayer lobby
         LOBBY_EXIT,             ///< sent to server by clients when a player leaves the multiplayer lobby, or by server to clients when a player leaves the multiplayer lobby
         START_MP_GAME,          ///< sent to server (by the "host" client only) when the settings in the MP lobby are satisfactory and it is time to start the game
         SAVE_GAME_INITIATE,     ///< sent to server (by the "host" client only) when a game is to be saved
@@ -88,9 +94,16 @@ public:
         MODERATOR_ACTION,       ///< sent by client to server when a moderator edits the universe
         SHUT_DOWN_SERVER,       ///< sent by host client to server to kill the server process
         REQUEST_SAVE_PREVIEWS,  ///< sent by client to request previews of available savegames
-        DISPATCH_SAVE_PREVIEWS,  ///< sent by host to client to provide the savegame previews
-        REQUEST_COMBAT_LOGS,  ///< sent by client to request combat logs
-        DISPATCH_COMBAT_LOGS  ///< sent by host to client to provide combat logs
+        DISPATCH_SAVE_PREVIEWS, ///< sent by host to client to provide the savegame previews
+        REQUEST_COMBAT_LOGS,    ///< sent by client to request combat logs
+        DISPATCH_COMBAT_LOGS,   ///< sent by host to client to provide combat logs
+        LOGGER_CONFIG,          ///< sent by host to server and server to ais to configure logging
+        CHECKSUM,               ///< sent by host to clients to specify what the parsed content checksum values should be
+        AUTH_REQUEST,           ///< sent by server to client if choosed player_name require authentiation
+        AUTH_RESPONSE,          ///< sent by client to server to provide password or other credentials
+        CHAT_HISTORY,           ///< sent by server to client to show previous messages
+        SET_AUTH_ROLES,         ///< sent by server to client to set authorization roles
+        ELIMINATE_SELF          ///< sent by client to server if the player wants to resign
     )
 
     GG_CLASS_ENUM(TurnProgressPhase,
@@ -121,17 +134,11 @@ public:
     Message();
 
     Message(MessageType message_type,
-            int sending_player,
-            int receiving_player,
-            const std::string& text,
-            bool synchronous_response = false);
+            const std::string& text);
     //@}
 
     /** \name Accessors */ //@{
     MessageType Type() const;               ///< Returns the type of the message.
-    int         SendingPlayer() const;      ///< Returns the ID of the sending player.
-    int         ReceivingPlayer() const;    ///< Returns the ID of the receiving player.
-    bool        SynchronousResponse() const;///< Returns true if this message is in reponse to a synchronous message
     std::size_t Size() const;               ///< Returns the size of the underlying buffer.
     const char* Data() const;               ///< Returns the underlying buffer.
     std::string Text() const;               ///< Returns the underlying buffer as a std::string.
@@ -145,9 +152,6 @@ public:
 
 private:
     MessageType   m_type;
-    int           m_sending_player;
-    int           m_receiving_player;
-    bool          m_synchronous_response;
     int           m_message_size;
 
     boost::shared_array<char> m_message_text;
@@ -180,8 +184,8 @@ FO_COMMON_API std::ostream& operator<<(std::ostream& os, const Message& msg);
 ////////////////////////////////////////////////
 
 /** creates an ERROR_MSG message*/
-FO_COMMON_API Message ErrorMessage(const std::string& problem, bool fatal = true);
-FO_COMMON_API Message ErrorMessage(int player_id, const std::string& problem, bool fatal = true);
+FO_COMMON_API Message ErrorMessage(const std::string& problem, bool fatal = true,
+                                   int player_id = Networking::INVALID_PLAYER_ID);
 
 /** creates a HOST_SP_GAME message*/
 FO_COMMON_API Message HostSPGameMessage(const SinglePlayerSetupData& setup_data);
@@ -189,22 +193,24 @@ FO_COMMON_API Message HostSPGameMessage(const SinglePlayerSetupData& setup_data)
 /** creates a minimal HOST_MP_GAME message used to initiate multiplayer "lobby" setup*/
 FO_COMMON_API Message HostMPGameMessage(const std::string& host_player_name);
 
-/** creates a JOIN_GAME message.  The sender's player name and client type are sent in the message.*/
-FO_COMMON_API Message JoinGameMessage(const std::string& player_name, Networking::ClientType client_type);
+/** creates a JOIN_GAME message.  The sender's player name, client type, and cookie are sent in the message.*/
+FO_COMMON_API Message JoinGameMessage(const std::string& player_name,
+                                      Networking::ClientType client_type,
+                                      boost::uuids::uuid cookie);
 
 /** creates a HOST_ID message.  The player ID of the host is sent in the message. */
 FO_COMMON_API Message HostIDMessage(int host_player_id);
 
-/** creates a GAME_START message.  Contains the initial game state visible to player \a player_id.*/
-FO_COMMON_API Message GameStartMessage(int player_id, bool single_player_game, int empire_id, int current_turn,
+/** creates a GAME_START message.  Contains the initial game state visible to the player.*/
+FO_COMMON_API Message GameStartMessage(bool single_player_game, int empire_id, int current_turn,
                                        const EmpireManager& empires, const Universe& universe,
                                        const SpeciesManager& species, CombatLogManager& combat_logs,
                                        const SupplyManager& supply, const std::map<int, PlayerInfo>& players,
                                        const GalaxySetupData& galaxy_setup_data, bool use_binary_serialization);
 
 /** creates a GAME_START message.  Contains the initial game state visible to
-  * player \a player_id.  Also includes data loaded from a saved game. */
-FO_COMMON_API Message GameStartMessage(int player_id, bool single_player_game, int empire_id, int current_turn,
+  * the player.  Also includes data loaded from a saved game. */
+FO_COMMON_API Message GameStartMessage(bool single_player_game, int empire_id, int current_turn,
                                        const EmpireManager& empires, const Universe& universe,
                                        const SpeciesManager& species, CombatLogManager& combat_logs,
                                        const SupplyManager& supply,
@@ -213,8 +219,8 @@ FO_COMMON_API Message GameStartMessage(int player_id, bool single_player_game, i
                                        const GalaxySetupData& galaxy_setup_data, bool use_binary_serialization);
 
 /** creates a GAME_START message.  Contains the initial game state visible to
-  * player \a player_id.  Also includes state string loaded from a saved game. */
-FO_COMMON_API Message GameStartMessage(int player_id, bool single_player_game, int empire_id, int current_turn,
+  * the player.  Also includes state string loaded from a saved game. */
+FO_COMMON_API Message GameStartMessage(bool single_player_game, int empire_id, int current_turn,
                                        const EmpireManager& empires, const Universe& universe,
                                        const SpeciesManager& species, CombatLogManager& combat_logs,
                                        const SupplyManager& supply,
@@ -231,144 +237,153 @@ FO_COMMON_API Message HostSPAckMessage(int player_id);
 FO_COMMON_API Message HostMPAckMessage(int player_id);
 
 /** creates a JOIN_GAME acknowledgement message.  The \a player_id is the ID of
-  * the receiving player.  This message should only be sent by the server.*/
-FO_COMMON_API Message JoinAckMessage(int player_id);
+  * the receiving player and \a cookie is a token to quickly authenticate player.
+  * This message should only be sent by the server.*/
+FO_COMMON_API Message JoinAckMessage(int player_id, boost::uuids::uuid cookie);
 
 /** creates a TURN_ORDERS message. */
-FO_COMMON_API Message TurnOrdersMessage(int sender, const OrderSet& orders);
+FO_COMMON_API Message TurnOrdersMessage(const OrderSet& orders);
 
 /** creates a TURN_PROGRESS message. */
-FO_COMMON_API Message TurnProgressMessage(Message::TurnProgressPhase phase_id, int player_id = Networking::INVALID_PLAYER_ID);
+FO_COMMON_API Message TurnProgressMessage(Message::TurnProgressPhase phase_id);
 
 /** creates a PLAYER_STATUS message. */
-FO_COMMON_API Message PlayerStatusMessage(int player_id, int about_player_id, Message::PlayerStatus player_status);
+FO_COMMON_API Message PlayerStatusMessage(int about_player_id, Message::PlayerStatus player_status);
 
 /** creates a TURN_UPDATE message. */
-FO_COMMON_API Message TurnUpdateMessage(int player_id, int empire_id, int current_turn,
+FO_COMMON_API Message TurnUpdateMessage(int empire_id, int current_turn,
                                         const EmpireManager& empires, const Universe& universe,
                                         const SpeciesManager& species, CombatLogManager& combat_logs,
                                         const SupplyManager& supply,
                                         const std::map<int, PlayerInfo>& players, bool use_binary_serialization);
 
 /** create a TURN_PARTIAL_UPDATE message. */
-FO_COMMON_API Message TurnPartialUpdateMessage(int player_id, int empire_id, const Universe& universe,
+FO_COMMON_API Message TurnPartialUpdateMessage(int empire_id, const Universe& universe,
                                                bool use_binary_serialization);
 
 /** creates a CLIENT_SAVE_DATA message, including UI data but without a state string. */
-FO_COMMON_API Message ClientSaveDataMessage(int sender, const OrderSet& orders, const SaveGameUIData& ui_data);
+FO_COMMON_API Message ClientSaveDataMessage(const OrderSet& orders, const SaveGameUIData& ui_data);
 
 /** creates a CLIENT_SAVE_DATA message, without UI data but with a state string. */
-FO_COMMON_API Message ClientSaveDataMessage(int sender, const OrderSet& orders, const std::string& save_state_string);
+FO_COMMON_API Message ClientSaveDataMessage(const OrderSet& orders, const std::string& save_state_string);
 
 /** creates a CLIENT_SAVE_DATA message, without UI data and without a state string. */
-Message ClientSaveDataMessage(int sender, const OrderSet& orders);
-
-/** creates an REQUEST_NEW_OBJECT_ID message. This message is a synchronous
-    message, when sent it will wait for a reply from the server */
-FO_COMMON_API Message RequestNewObjectIDMessage(int sender);
-
-/** creates an DISPATCH_NEW_OBJECT_ID  message.  This message is sent to a
-  * client who is waiting for a new object ID */
-FO_COMMON_API Message DispatchObjectIDMessage(int player_id, int new_id);
-
-/** creates an REQUEST_NEW_DESIGN_ID message. This message is a synchronous
-    message, when sent it will wait for a reply from the server */
-FO_COMMON_API Message RequestNewDesignIDMessage(int sender);
-
-/** creates an DISPATCH_NEW_DESIGN_ID  message.  This message is sent to a
-  * client who is waiting for a new design ID */
-FO_COMMON_API Message DispatchDesignIDMessage(int player_id, int new_id);
+FO_COMMON_API Message ClientSaveDataMessage(const OrderSet& orders);
 
 /** creates a SAVE_GAME_INITIATE request message.  This message should only be sent by
   * the host player.*/
-FO_COMMON_API Message HostSaveGameInitiateMessage(int sender, const std::string& filename);
+FO_COMMON_API Message HostSaveGameInitiateMessage(const std::string& filename);
 
 /** creates a SAVE_GAME_DATA_REQUEST data request message.  This message should
     only be sent by the server to get game data from a client, or to respond to
     the host player requesting a save be initiated. */
-FO_COMMON_API Message ServerSaveGameDataRequestMessage(int receiver, bool synchronous_response);
+FO_COMMON_API Message ServerSaveGameDataRequestMessage();
 
 /** creates a SAVE_GAME_COMPLETE complete message.  This message should only be
     sent by the server to inform clients that the last initiated save has been
     completed successfully. */
 FO_COMMON_API Message ServerSaveGameCompleteMessage(const std::string& save_filename, int bytes_written);
 
-/** creates a PLAYER_CHAT, which is sent to the server, and then from the server
-  * to all players, including the originating player.*/
-FO_COMMON_API Message GlobalChatMessage(int sender, const std::string& msg);
-
-/** creates a PLAYER_CHAT message, which is sent to the server, and then from
-  * the server to a single recipient player */
-FO_COMMON_API Message SingleRecipientChatMessage(int sender, int receiver, const std::string& msg);
-
 /** creates a DIPLOMACY message, which is sent between players via the server to
   * declare, proposed, or accept / reject diplomatic arrangements or agreements. */
-FO_COMMON_API Message DiplomacyMessage(int sender, int receiver, const DiplomaticMessage& diplo_message);
+FO_COMMON_API Message DiplomacyMessage(const DiplomaticMessage& diplo_message);
 
 /** creates a DIPLOMATIC_STATUS message, which is sent to players by the server to
   * update them on diplomatic status changes between players. */
-FO_COMMON_API Message DiplomaticStatusMessage(int receiver, const DiplomaticStatusUpdateInfo& diplo_update);
+FO_COMMON_API Message DiplomaticStatusMessage(const DiplomaticStatusUpdateInfo& diplo_update);
 
 /** creates an END_GAME message used to terminate an active game. */
-FO_COMMON_API Message EndGameMessage(int receiver, Message::EndGameReason reason, const std::string& reason_player_name = "");
+FO_COMMON_API Message EndGameMessage(Message::EndGameReason reason, const std::string& reason_player_name = "");
 
 /** creates an AI_END_GAME_ACK message used to indicate that the AI has shutdown. */
-FO_COMMON_API Message AIEndGameAcknowledgeMessage(int sender);
+FO_COMMON_API Message AIEndGameAcknowledgeMessage();
 
 /** creates a MODERATOR_ACTION message used to implement moderator commands. */
-FO_COMMON_API Message ModeratorActionMessage(int sender, const Moderator::ModeratorAction& mod_action);
+FO_COMMON_API Message ModeratorActionMessage(const Moderator::ModeratorAction& mod_action);
 
 /** tells server to shut down. */
-FO_COMMON_API Message ShutdownServerMessage(int sender);
+FO_COMMON_API Message ShutdownServerMessage();
 
 /** requests previews of savefiles from server */
-FO_COMMON_API Message RequestSavePreviewsMessage(int sender, std::string directory);
+FO_COMMON_API Message RequestSavePreviewsMessage(std::string relative_directory);
 
 /** returns the savegame previews to the client */
-FO_COMMON_API Message DispatchSavePreviewsMessage(int receiver, const PreviewInformation& preview);
+FO_COMMON_API Message DispatchSavePreviewsMessage(const PreviewInformation& preview);
 
 /** requests combat logs from server */
-FO_COMMON_API Message RequestCombatLogsMessage(int sender, const std::vector<int>& ids);
+FO_COMMON_API Message RequestCombatLogsMessage(const std::vector<int>& ids);
 
 /** returns combat logs to the client */
-FO_COMMON_API Message DispatchCombatLogsMessage(int receiver, const std::vector<std::pair<int, const CombatLog>>& logs);
+FO_COMMON_API Message DispatchCombatLogsMessage(const std::vector<std::pair<int, const CombatLog>>& logs);
+
+/** Sends logger configuration details to server or ai process. */
+FO_COMMON_API Message LoggerConfigMessage(int sender, const std::set<std::tuple<std::string, std::string, LogLevel>>& options);
 
 ////////////////////////////////////////////////
 // Multiplayer Lobby Message named ctors
 ////////////////////////////////////////////////
 
-/** creates an LOBBY_UPDATE message containing changes to the lobby settings that need to propogate to the 
+/** creates an LOBBY_UPDATE message containing changes to the lobby settings that need to propogate to the
     server, then to other users.  Clients must send all such updates to the server directly; the server
     will send updates to the other clients as needed.*/
-FO_COMMON_API Message LobbyUpdateMessage(int sender, const MultiplayerLobbyData& lobby_data);
+FO_COMMON_API Message LobbyUpdateMessage(const MultiplayerLobbyData& lobby_data);
 
-/** creates an LOBBY_UPDATE message containing changes to the lobby settings that need to propogate to the users.  
+/** creates an LOBBY_UPDATE message containing changes to the lobby settings that need to propogate to the users.
     This message should only be sent by the server.*/
-FO_COMMON_API Message ServerLobbyUpdateMessage(int receiver, const MultiplayerLobbyData& lobby_data);
+FO_COMMON_API Message ServerLobbyUpdateMessage(const MultiplayerLobbyData& lobby_data);
 
-/** creates an LOBBY_CHAT message containing a chat string to be broadcast to player \a receiver, or all players if \a
+/** creates an CHAT_HISTORY message containing latest chat messages.
+    This message should only be sent by the server.*/
+FO_COMMON_API Message ChatHistoryMessage(const std::vector<std::reference_wrapper<const ChatHistoryEntity>>& chat_history);
+
+/** creates an PLAYER_CHAT message containing a chat string to be broadcast to player \a receiver, or all players if \a
     receiver is Networking::INVALID_PLAYER_ID. Note that the receiver of this message is always the server.*/
-FO_COMMON_API Message LobbyChatMessage(int sender, int receiver, const std::string& text);
+FO_COMMON_API Message PlayerChatMessage(const std::string& text, int receiver = Networking::INVALID_PLAYER_ID);
 
-/** creates an LOBBY_CHAT message containing a chat string from \a sender to be displayed in \a receiver's lobby dialog.
+/** creates an PLAYER_CHAT message containing a chat string from \a sender at \a timestamp to be displayed in chat.
     This message should only be sent by the server.*/
-FO_COMMON_API Message ServerLobbyChatMessage(int sender, int receiver, const std::string& text);
+FO_COMMON_API Message ServerPlayerChatMessage(int sender,
+                                              const boost::posix_time::ptime& timestamp,
+                                              const std::string& text);
 
 /** creates a START_MP_GAME used to finalize the multiplayer lobby setup.*/
-FO_COMMON_API Message StartMPGameMessage(int player_id);
+FO_COMMON_API Message StartMPGameMessage();
 
+/** creates a CHECKSUM message containing checksums of parsed content. */
+FO_COMMON_API Message ContentCheckSumMessage();
+
+/** creates a AUTH_REQUEST message containing \a player_name to login and \a auth additional authentication data. */
+FO_COMMON_API Message AuthRequestMessage(const std::string& player_name, const std::string& auth);
+
+/** creates a AUTH_RESPONSE message containing \a player_name to login and \a auth credentials. */
+FO_COMMON_API Message AuthResponseMessage(const std::string& player_name, const std::string& auth);
+
+/** notifies client about changes in his authorization \a roles. */
+FO_COMMON_API Message SetAuthorizationRolesMessage(const Networking::AuthRoles& roles);
+
+/** creates a ELIMINATE_SELF message to resign from the game. */
+FO_COMMON_API Message EliminateSelfMessage();
 
 ////////////////////////////////////////////////
 // Message data extractors
 ////////////////////////////////////////////////
 
-FO_COMMON_API void ExtractErrorMessageData(const Message& msg, std::string& problem, bool& fatal);
+FO_COMMON_API void ExtractErrorMessageData(const Message& msg, int& player_id, std::string& problem, bool& fatal);
 
 FO_COMMON_API void ExtractHostMPGameMessageData(const Message& msg, std::string& host_player_name,
                                                 std::string& client_version_string);
 
 FO_COMMON_API void ExtractLobbyUpdateMessageData(const Message& msg, MultiplayerLobbyData& lobby_data);
 
+FO_COMMON_API void ExtractChatHistoryMessage(const Message& msg, std::vector<ChatHistoryEntity>& chat_history);
+
+FO_COMMON_API void ExtractPlayerChatMessageData(const Message& msg, int& receiver, std::string& data);
+
+FO_COMMON_API void ExtractServerPlayerChatMessageData(const Message& msg,
+                                                      int& sender,
+                                                      boost::posix_time::ptime& timestamp,
+                                                      std::string& data);
+
 FO_COMMON_API void ExtractGameStartMessageData(const Message& msg, bool& single_player_game, int& empire_id,
                                                int& current_turn, EmpireManager& empires, Universe& universe,
                                                SpeciesManager& species, CombatLogManager& combat_logs,
@@ -380,7 +395,11 @@ FO_COMMON_API void ExtractGameStartMessageData(const Message& msg, bool& single_
 
 FO_COMMON_API void ExtractJoinGameMessageData(const Message& msg, std::string& player_name,
                                               Networking::ClientType& client_type,
-                                              std::string& version_string);
+                                              std::string& version_string,
+                                              boost::uuids::uuid& cookie);
+
+FO_COMMON_API void ExtractJoinAckMessageData(const Message& msg, int& player_id,
+                                             boost::uuids::uuid& cookie);
 
 FO_COMMON_API void ExtractTurnOrdersMessageData(const Message& msg, OrderSet& orders);
 
@@ -419,4 +438,14 @@ FO_COMMON_API void ExtractRequestCombatLogsMessageData(const Message& msg, std::
 
 FO_COMMON_API void ExtractDispatchCombatLogsMessageData(const Message& msg, std::vector<std::pair<int, CombatLog>>& logs);
 
+FO_COMMON_API void ExtractLoggerConfigMessageData(const Message& msg, std::set<std::tuple<std::string, std::string, LogLevel>>& options);
+
+FO_COMMON_API void ExtractContentCheckSumMessageData(const Message& msg, std::map<std::string, unsigned int>& checksums);
+
+FO_COMMON_API void ExtractAuthRequestMessageData(const Message& msg, std::string& player_name, std::string& auth);
+
+FO_COMMON_API void ExtractAuthResponseMessageData(const Message& msg, std::string& player_name, std::string& auth);
+
+FO_COMMON_API void ExtractSetAuthorizationRolesMessage(const Message &msg, Networking::AuthRoles& roles);
+
 #endif // _Message_h_
