@@ -8,6 +8,7 @@
 
 #include "Tokens.h"
 
+#include <unordered_map>
 
 /** \namespace parse \brief The namespace that encloses the script file lexer
     and parser. */
@@ -23,7 +24,6 @@ typedef boost::spirit::lex::lexertl::position_token<
         bool,
         int,
         double,
-        const char*,
         std::string
     >
 > token_type;
@@ -34,13 +34,16 @@ typedef boost::spirit::lex::lexertl::actor_lexer<token_type> spirit_lexer_base_t
 struct lexer :
     boost::spirit::lex::lexer<spirit_lexer_base_type>
 {
-    static const lexer& instance();
+    /** Ctor. */
+    lexer();
 
     /** \name Comment tokens */ ///@{
     boost::spirit::lex::token_def<boost::spirit::lex::omit> inline_comment;
     boost::spirit::lex::token_def<boost::spirit::lex::omit> end_of_line_comment;
     //@}
 
+    using string_token_def = boost::spirit::lex::token_def<std::string>;
+
     /** \name Tokens for common C++ types and builtins. */ ///@{
     boost::spirit::lex::token_def<bool> bool_;
     boost::spirit::lex::token_def<int> int_;
@@ -51,7 +54,7 @@ struct lexer :
     /** \name Keyword tokens.  These should be kept in lexicographically
         sorted order, so that finding, adding, and removing tokens is a bit
         easier.  See the note above the Enum tokens section. */ ///@{
-#define DECLARE_TOKEN(r, _, name) boost::spirit::lex::token_def<const char*> BOOST_PP_CAT(name, _);
+#define DECLARE_TOKEN(r, _, name) string_token_def BOOST_PP_CAT(name, _);
     BOOST_PP_SEQ_FOR_EACH(DECLARE_TOKEN, _, TOKEN_SEQ_1)
     BOOST_PP_SEQ_FOR_EACH(DECLARE_TOKEN, _, TOKEN_SEQ_2)
     BOOST_PP_SEQ_FOR_EACH(DECLARE_TOKEN, _, TOKEN_SEQ_3)
@@ -66,6 +69,9 @@ struct lexer :
     BOOST_PP_SEQ_FOR_EACH(DECLARE_TOKEN, _, TOKEN_SEQ_12)
     BOOST_PP_SEQ_FOR_EACH(DECLARE_TOKEN, _, TOKEN_SEQ_13)
     BOOST_PP_SEQ_FOR_EACH(DECLARE_TOKEN, _, TOKEN_SEQ_14)
+    BOOST_PP_SEQ_FOR_EACH(DECLARE_TOKEN, _, TOKEN_SEQ_15)
+    BOOST_PP_SEQ_FOR_EACH(DECLARE_TOKEN, _, TOKEN_SEQ_16)
+    BOOST_PP_SEQ_FOR_EACH(DECLARE_TOKEN, _, TOKEN_SEQ_17)
 #undef DECLARE_TOKEN
     //@}
 
@@ -73,19 +79,12 @@ struct lexer :
     boost::spirit::lex::token_def<boost::spirit::lex::omit> error_token;
     //@}
 
-    /** Returns the token_def<const char*> associated with \a name. */
-    const boost::spirit::lex::token_def<const char*>& name_token(const char* name) const;
-
     static const char* bool_regex;
     static const char* int_regex;
     static const char* double_regex;
     static const char* string_regex;
 
 private:
-    /** Ctor. */
-    lexer();
-
-    std::map<const char*, boost::spirit::lex::token_def<const char*>*> m_name_tokens;
 };
 
 /** The type of iterator passed to the script file parser by the script file
@@ -104,19 +103,6 @@ namespace boost { namespace spirit { namespace traits {
 
     // If you want to create a token with a custom value type, you must
     // declare the conversion handler here, and define it in the .cpp file.
-
-    // These template specializations are required by Spirit.Lex to automatically
-    // convert an iterator pair to an const char* in the lexer.
-    template <>
-    struct assign_to_attribute_from_iterators<const char*, parse::text_iterator, void>
-    { static void call(const parse::text_iterator& first, const parse::text_iterator& last, const char*& attr); };
-
-    // HACK! This is only necessary because of a bug in Spirit in Boost
-    // versions <= 1.45.
-    template <>
-    struct assign_to_attribute_from_iterators<bool, parse::text_iterator, void>
-    { static void call(const parse::text_iterator& first, const parse::text_iterator& last, bool& attr); };
-
 } } }
 
 #endif
