@@ -16,9 +16,8 @@
     (Affiliation)                               \
     (Age)                                       \
     (Aggressive)                                \
-    (Alignment)                                 \
-    (AlignmentEffects)                          \
     (All)                                       \
+    (Allowed)                                   \
     (AllyOf)                                    \
     (And)                                       \
     (AnyEmpire)                                 \
@@ -80,6 +79,7 @@
 #define TOKEN_SEQ_3                             \
     (Damage)                                    \
     (Data)                                      \
+    (Default)                                   \
     (Defense)                                   \
     (Described)                                 \
     (Description)                               \
@@ -94,6 +94,7 @@
     (Destroy)                                   \
     (Detection)                                 \
     (DirectDistanceBetween)                     \
+    (Disabled)                                  \
     (Distance)                                  \
     (DistanceFromOriginalType)                  \
     (Effects)                                   \
@@ -103,7 +104,9 @@
     (Empire)                                    \
     (EmpireMeter)                               \
     (EmpireMeterValue)                          \
+    (EmpireObjectVisibility)                    \
     (EmpireShipsDestroyed)                      \
+    (Enabled)                                   \
     (Endpoint)                                  \
     (EnemyOf)                                   \
     (Enqueued)                                  \
@@ -141,6 +144,7 @@
     (GalaxySpecialFrequency)                    \
     (GalaxyStarlaneFrequency)                   \
     (Gameplay_Description)                      \
+    (GameRule)                                  \
     (GasGiant)                                  \
     (General)
 
@@ -149,6 +153,7 @@
     (GiveEmpireTech)                            \
     (Good)                                      \
     (Graphic)                                   \
+    (HabitableSize)                             \
     (Happiness)                                 \
     (HasSpecial)                                \
     (HasSpecialCapacity)                        \
@@ -166,6 +171,7 @@
     (HullSpeed)                                 \
     (HullStealth)                               \
     (HullStructure)                             \
+    (HullType)                                  \
     (Human)                                     \
     (Icon)                                      \
     (ID)                                        \
@@ -174,6 +180,7 @@
     (Inferno)                                   \
     (InSystem)                                  \
     (Interceptor)                               \
+    (Integer)                                   \
     (Internal)                                  \
     (Invisible)                                 \
     (Item)                                      \
@@ -185,8 +192,11 @@
 #define TOKEN_SEQ_6                             \
     (Label)                                     \
     (Large)                                     \
-    (LastTurnBattleHere)                        \
     (LastTurnActiveInBattle)                    \
+    (LastTurnAttackedByShip)                    \
+    (LastTurnBattleHere)                        \
+    (LastTurnConquered)                         \
+    (LastTurnResupplied)                        \
     (LeastHappySpecies)                         \
     (LocalCandidate)                            \
     (Location)                                  \
@@ -202,6 +212,7 @@
     (MaxFuel)                                   \
     (MaximumNumberOf)                           \
     (MaxShield)                                 \
+    (MaxStockpile)                              \
     (MaxStructure)                              \
     (MaxSupply)                                 \
     (MaxTroops)                                 \
@@ -255,6 +266,8 @@
     (Object)                                    \
     (ObjectType)                                \
     (Ocean)                                     \
+    (Off)                                       \
+    (On)                                        \
     (OneOf)                                     \
     (Opinion)                                   \
     (Or)                                        \
@@ -281,6 +294,7 @@
     (Parts)                                     \
     (PartOfClassInShipDesign)                   \
     (PartsInShipDesign)                         \
+    (PartType)                                  \
     (Passive)                                   \
     (Planet)                                    \
     (Planetbound)                               \
@@ -321,6 +335,7 @@
     (RandomTransferrableTech)                   \
     (RandomNumber)                              \
     (Range)                                     \
+    (Real)                                      \
     (Reason)                                    \
     (RebelTroops)                               \
     (Red)                                       \
@@ -349,7 +364,7 @@
     (SetEmpireCapital)                          \
     (SetEmpireMeter)                            \
     (SetEmpireTechProgress)                     \
-    (SetEmpireTradeStockpile)                   \
+    (SetEmpireStockpile)                        \
     (SetFuel)                                   \
     (SetHappiness)                              \
     (SetIndustry)                               \
@@ -359,6 +374,7 @@
     (SetMaxFuel)                                \
     (SetMaxSecondaryStat)                       \
     (SetMaxShield)                              \
+    (SetMaxStockpile)                           \
     (SetMaxStructure)                           \
     (SetMaxSupply)                              \
     (SetMaxTroops)                              \
@@ -382,6 +398,7 @@
     (SetSpeed)                                  \
     (SetStarType)                               \
     (SetStealth)                                \
+    (SetStockpile)                              \
     (SetStructure)                              \
     (SetSupply)                                 \
     (SetTargetConstruction)                     \
@@ -393,9 +410,12 @@
     (SetTexture)                                \
     (SetTrade)                                  \
     (SetTroops)                                 \
-    (SetVisibility)                             \
+    (SetVisibility)
+
+#define TOKEN_SEQ_13                            \
     (Shield)                                    \
     (ShipDesign)                                \
+    (ShipDesignOrdering)                        \
     (ShipDesignsDestroyed)                      \
     (ShipDesignsLost)                           \
     (ShipDesignsOwned)                          \
@@ -415,7 +435,7 @@
     (Size)                                      \
     (SizeAsDouble)
 
-#define TOKEN_SEQ_13                            \
+#define TOKEN_SEQ_14                            \
     (Slot)                                      \
     (Slots)                                     \
     (SlotsInHull)                               \
@@ -429,11 +449,16 @@
     (SpawnLimit)                                \
     (SpawnRate)                                 \
     (Special)                                   \
+    (SpecialAddedOnTurn)                        \
+    (SpecialCapacity)                           \
     (Species)                                   \
     (SpeciesID)                                 \
+    (SpeciesCensusOrdering)                     \
     (SpeciesOpinion)                            \
     (SpeciesPlanetsBombed)                      \
-    (SpeciesColoniesOwned)                      \
+    (SpeciesColoniesOwned)
+
+#define TOKEN_SEQ_15                            \
     (SpeciesPlanetsDepoped)                     \
     (SpeciesPlanetsInvaded)                     \
     (SpeciesShipsDestroyed)                     \
@@ -450,6 +475,9 @@
     (Statistic)                                 \
     (StDev)                                     \
     (Stealth)                                   \
+    (Stockpile)                                 \
+    (String)                                    \
+    (StringList)                                \
     (Structure)                                 \
     (Sum)                                       \
     (Supply)                                    \
@@ -458,7 +486,7 @@
     (System)                                    \
     (SystemID)
 
-#define TOKEN_SEQ_14                            \
+#define TOKEN_SEQ_16                            \
     (Tag)                                       \
     (Tags)                                      \
     (Target)                                    \
@@ -481,16 +509,19 @@
     (ThisSpecies)                               \
     (ThisSpecial)                               \
     (Tiny)                                      \
+    (Toggle)                                    \
     (TopPriorityEnqueuedTech)                   \
     (TopPriorityResearchableTech)               \
     (TopPriorityTransferrableTech)              \
     (Toxic)                                     \
-    (Trade)                                     \
-    (TradeStockpile)                            \
+    (Trade)
+
+#define TOKEN_SEQ_17                            \
     (Troops)                                    \
     (Tundra)                                    \
     (Turn)                                      \
     (TurnsSinceFocusChange)                     \
+    (TurnTechResearched)                        \
     (Type)                                      \
     (Uninhabitable)                             \
     (UniverseCentreX)                           \
@@ -502,6 +533,7 @@
     (Unresearchable)                            \
     (UpgradeVisibility)                         \
     (UserString)                                \
+    (UUID)                                      \
     (Value)                                     \
     (Victory)                                   \
     (VisibleToEmpire)                           \
@@ -513,23 +545,4 @@
     (Y)                                         \
     (Yellow)
 
-#define DECLARE_TOKEN(r, _, elem) extern const char* BOOST_PP_CAT(elem, _token);
-BOOST_PP_SEQ_FOR_EACH(DECLARE_TOKEN, _, TOKEN_SEQ_1)
-BOOST_PP_SEQ_FOR_EACH(DECLARE_TOKEN, _, TOKEN_SEQ_2)
-BOOST_PP_SEQ_FOR_EACH(DECLARE_TOKEN, _, TOKEN_SEQ_3)
-BOOST_PP_SEQ_FOR_EACH(DECLARE_TOKEN, _, TOKEN_SEQ_4)
-BOOST_PP_SEQ_FOR_EACH(DECLARE_TOKEN, _, TOKEN_SEQ_5)
-BOOST_PP_SEQ_FOR_EACH(DECLARE_TOKEN, _, TOKEN_SEQ_6)
-BOOST_PP_SEQ_FOR_EACH(DECLARE_TOKEN, _, TOKEN_SEQ_7)
-BOOST_PP_SEQ_FOR_EACH(DECLARE_TOKEN, _, TOKEN_SEQ_8)
-BOOST_PP_SEQ_FOR_EACH(DECLARE_TOKEN, _, TOKEN_SEQ_9)
-BOOST_PP_SEQ_FOR_EACH(DECLARE_TOKEN, _, TOKEN_SEQ_10)
-BOOST_PP_SEQ_FOR_EACH(DECLARE_TOKEN, _, TOKEN_SEQ_11)
-BOOST_PP_SEQ_FOR_EACH(DECLARE_TOKEN, _, TOKEN_SEQ_12)
-BOOST_PP_SEQ_FOR_EACH(DECLARE_TOKEN, _, TOKEN_SEQ_13)
-BOOST_PP_SEQ_FOR_EACH(DECLARE_TOKEN, _, TOKEN_SEQ_14)
-
-#undef DECLARE_TOKEN
-
-
 #endif
