@@ -81,10 +81,10 @@ namespace {
     }
 
     boost::python::str GetUserConfigDirWrapper()
-    { return boost::python::str(PathString(GetUserConfigDir())); }
+    { return boost::python::str(PathToString(GetUserConfigDir())); }
 
     boost::python::str GetUserDataDirWrapper()
-    { return boost::python::str(PathString(GetUserDataDir())); }
+    { return boost::python::str(PathToString(GetUserDataDir())); }
 
 }
 
@@ -166,6 +166,8 @@ namespace FreeOrionPython {
         def("issueChangeProductionQuantityOrder",   AIInterface::IssueChangeProductionQuantityOrder);
         def("issueRequeueProductionOrder",          AIInterface::IssueRequeueProductionOrder, "Orders the item on the production queue at index oldQueueIndex (int) to be moved to index newQueueIndex (int). Returns 1 (int) on success or 0 (int) on failure if the old and new queue indices are equal, if either queue index is less than 0 or greater than the largest indexed item on the queue.");
         def("issueDequeueProductionOrder",          AIInterface::IssueDequeueProductionOrder, "Orders the item on the production queue at index queueIndex (int) to be removed form the production queue. Returns 1 (int) on success or 0 (int) on failure if the queue index is less than 0 or greater than the largest indexed item on the queue.");
+        def("issuePauseProductionOrder",            AIInterface::IssuePauseProductionOrder, "Orders the item on the production queue at index queueIndex (int) to be paused (or unpaused). Returns 1 (int) on success or 0 (int) on failure if the queue index is less than 0 or greater than the largest indexed item on the queue.");
+        def("issueAllowStockpileProductionOrder",   AIInterface::IssueAllowStockpileProductionOrder, "Orders the item on the production queue at index queueIndex (int) to be enabled (or disabled) to use the imperial stockpile. Returns 1 (int) on success or 0 (int) on failure if the queue index is less than 0 or greater than the largest indexed item on the queue.");
         def("issueCreateShipDesignOrder",           IssueCreateShipDesignOrderWrapper, "Orders the creation of a new ship design with the name (string), description (string), hull (string), parts vector partsVec (StringVec), graphic (string) and model (string). model should be left as an empty string as of this writing. There is currently no easy way to find the id of the new design, though the client's empire should have the new design after this order is issued successfully. Returns 1 (int) on success or 0 (int) on failure if any of the name, description, hull or graphic are empty strings, if the design is invalid (due to not following number and type of slot requirements for the hull) or if creating the design fails for some reason.");
 
         def("sendChatMessage",          AIInterface::SendPlayerChatMessage, "Sends the indicated message (string) to the player with the indicated recipientID (int) or to all players if recipientID is -1.");
