@@ -4,6 +4,9 @@
 #include "../util/Logger.h"
 #include "CommonWrappers.h"
 
+#include <boost/python/tuple.hpp>
+#include <boost/python/list.hpp>
+#include <boost/python/extract.hpp>
 #include <boost/python/docstring_options.hpp>
 
 using boost::python::object;
@@ -160,7 +163,7 @@ std::vector<std::string> PythonBase::ErrorReport() {
             return err_list;
         }
 
-        for(int i = 0; i < len(py_err_list); i++) {
+        for (int i = 0; i < len(py_err_list); i++) {
             err_list.push_back(extract<std::string>(py_err_list[i]));
         }
     }
