@@ -14,6 +14,8 @@ public:
 
     bool CreateUniverse(std::map<int, PlayerSetupData>& player_setup_data); // Wraps call to the main Python universe generator function
     bool ExecuteTurnEvents();    // Wraps call to the main Python turn events function
+    bool IsRequireAuthOrReturnRoles(const std::string& player_name, bool &result, Networking::AuthRoles& roles) const; // Wraps call to AuthProvider's method is_require_auth
+    bool IsSuccessAuthAndReturnRoles(const std::string& player_name, const std::string& auth, bool &result, Networking::AuthRoles& roles) const; // Wraps call to AuthProvider's method is_success_auth
 
 private:
     // reference to imported Python universe generator module
@@ -21,6 +23,9 @@ private:
 
     // reference to imported Python turn events module
     boost::python::object m_python_module_turn_events;
+
+    // reference to imported Python auth module
+    boost::python::object m_python_module_auth;
 };
 
 // Returns folder containing the Python universe generator scripts
@@ -29,5 +34,7 @@ const std::string GetPythonUniverseGeneratorDir();
 // Returns folder containing the Python turn events scripts
 const std::string GetPythonTurnEventsDir();
 
+// Returns folder containing the Python auth scripts
+const std::string GetPythonAuthDir();
 
 #endif /* defined(__FreeOrion__Python__ServerFramework__) */
