@@ -30,11 +30,8 @@
 ObjectMap::ObjectMap()
 {}
 
-ObjectMap::~ObjectMap() {
-    // Make sure to call ObjectMap::Clear() before destruction somewhere if
-    // this ObjectMap contains any unique pointers to UniverseObject objects.
-    // Otherwise, the pointed-to UniverseObjects will be leaked memory...
-}
+ObjectMap::~ObjectMap()
+{}
 
 void ObjectMap::Copy(const ObjectMap& copied_map, int empire_id/* = ALL_EMPIRES*/) {
     if (&copied_map == this)
@@ -67,7 +64,7 @@ void ObjectMap::CopyObject(std::shared_ptr<const UniverseObject> source, int emp
     if (std::shared_ptr<UniverseObject> destination = this->Object(source_id)) {
         destination->Copy(source, empire_id); // there already is a version of this object present in this ObjectMap, so just update it
     } else {
-        Insert(source->Clone(), empire_id); // this object is not yet present in this ObjectMap, so add a new UniverseObject object for it
+        InsertCore(std::shared_ptr<UniverseObject>(source->Clone()), empire_id); // this object is not yet present in this ObjectMap, so add a new UniverseObject object for it
     }
 }
 
@@ -92,7 +89,7 @@ std::shared_ptr<UniverseObject> ObjectMap::Object(int id)
 std::vector<std::shared_ptr<const UniverseObject>> ObjectMap::FindObjects(const std::vector<int>& object_ids) const {
     std::vector<std::shared_ptr<const UniverseObject>> result;
     for (int object_id : object_ids)
-        if (std::shared_ptr<const UniverseObject> obj = Object(object_id))
+        if (auto obj = Object(object_id))
             result.push_back(obj);
         else
             ErrorLogger() << "ObjectMap::FindObjects couldn't find object with id " << object_id;
@@ -102,7 +99,7 @@ std::vector<std::shared_ptr<const UniverseObject>> ObjectMap::FindObjects(const
 std::vector<std::shared_ptr<const UniverseObject>> ObjectMap::FindObjects(const std::set<int>& object_ids) const {
     std::vector<std::shared_ptr<const UniverseObject>> result;
     for (int object_id : object_ids)
-        if (std::shared_ptr<const UniverseObject> obj = Object(object_id))
+        if (auto obj = Object(object_id))
             result.push_back(obj);
         else
             ErrorLogger() << "ObjectMap::FindObjects couldn't find object with id " << object_id;
@@ -112,7 +109,7 @@ std::vector<std::shared_ptr<const UniverseObject>> ObjectMap::FindObjects(const
 std::vector<std::shared_ptr<UniverseObject>> ObjectMap::FindObjects(const std::vector<int>& object_ids) {
     std::vector<std::shared_ptr<UniverseObject>> result;
     for (int object_id : object_ids)
-        if (std::shared_ptr<UniverseObject> obj = Object(object_id))
+        if (auto obj = Object(object_id))
             result.push_back(obj);
         else
             ErrorLogger() << "ObjectMap::FindObjects couldn't find object with id " << object_id;
@@ -122,7 +119,7 @@ std::vector<std::shared_ptr<UniverseObject>> ObjectMap::FindObjects(const std::v
 std::vector<std::shared_ptr<UniverseObject>> ObjectMap::FindObjects(const std::set<int>& object_ids) {
     std::vector<std::shared_ptr<UniverseObject>> result;
     for (int object_id : object_ids)
-        if (std::shared_ptr<UniverseObject> obj = Object(object_id))
+        if (auto obj = Object(object_id))
             result.push_back(obj);
         else
             ErrorLogger() << "ObjectMap::FindObjects couldn't find object with id " << object_id;
@@ -131,8 +128,8 @@ std::vector<std::shared_ptr<UniverseObject>> ObjectMap::FindObjects(const std::s
 
 std::vector<std::shared_ptr<const UniverseObject>> ObjectMap::FindObjects(const UniverseObjectVisitor& visitor) const {
     std::vector<std::shared_ptr<const UniverseObject>> result;
-    for (const_iterator<> it = const_begin(); it != const_end(); ++it) {
-        if (std::shared_ptr<UniverseObject> obj = it->Accept(visitor))
+    for (auto it = const_begin(); it != const_end(); ++it) {
+        if (auto obj = it->Accept(visitor))
             result.push_back(Object(obj->ID()));
     }
     return result;
@@ -140,7 +137,7 @@ std::vector<std::shared_ptr<const UniverseObject>> ObjectMap::FindObjects(const
 
 std::vector<std::shared_ptr<UniverseObject>> ObjectMap::FindObjects(const UniverseObjectVisitor& visitor) {
     std::vector<std::shared_ptr<UniverseObject>> result;
-    for (std::shared_ptr<UniverseObject> obj : *this) {
+    for (const auto& obj : *this) {
         if (std::shared_ptr<UniverseObject> match = obj->Accept(visitor))
             result.push_back(Object(match->ID()));
     }
@@ -149,16 +146,15 @@ std::vector<std::shared_ptr<UniverseObject>> ObjectMap::FindObjects(const Univer
 
 std::vector<int> ObjectMap::FindObjectIDs(const UniverseObjectVisitor& visitor) const {
     std::vector<int> result;
-    for (const_iterator<> it = const_begin(); it != const_end(); ++it) {
+    for (auto it = const_begin(); it != const_end(); ++it) {
         if (it->Accept(visitor))
             result.push_back(it->ID());
     }
     return result;
 }
 
-std::vector<int> ObjectMap::FindObjectIDs() const {
-    return FindObjectIDs<UniverseObject>();
-}
+std::vector<int> ObjectMap::FindObjectIDs() const
+{ return FindObjectIDs<UniverseObject>(); }
 
 ObjectMap::iterator<> ObjectMap::begin()
 { return begin<UniverseObject>(); }
@@ -172,13 +168,12 @@ ObjectMap::const_iterator<> ObjectMap::const_begin() const
 ObjectMap::const_iterator<> ObjectMap::const_end() const
 { return const_end<UniverseObject>(); }
 
-void ObjectMap::Insert(std::shared_ptr<UniverseObject> item, int empire_id/* = ALL_EMPIRES*/) {
+void ObjectMap::InsertCore(std::shared_ptr<UniverseObject> item, int empire_id/* = ALL_EMPIRES*/) {
     FOR_EACH_MAP(TryInsertIntoMap, item);
     if (item &&
-        GetUniverse().EmpireKnownDestroyedObjectIDs(empire_id).find(item->ID()) ==
-            GetUniverse().EmpireKnownDestroyedObjectIDs(empire_id).end())
+        !GetUniverse().EmpireKnownDestroyedObjectIDs(empire_id).count(item->ID()))
     {
-        std::shared_ptr<UniverseObject> this_item = this->Object(item->ID());
+        auto this_item = this->Object(item->ID());
         m_existing_objects[item->ID()] = this_item;
         switch (item->ObjectType()) {
             case OBJ_BUILDING:
@@ -215,12 +210,12 @@ void ObjectMap::Insert(std::shared_ptr<UniverseObject> item, int empire_id/* = A
 
 std::shared_ptr<UniverseObject> ObjectMap::Remove(int id) {
     // search for object in objects map
-    std::map<int, std::shared_ptr<UniverseObject>>::iterator it = m_objects.find(id);
+    auto it = m_objects.find(id);
     if (it == m_objects.end())
         return nullptr;
     //DebugLogger() << "Object was removed: " << it->second->Dump();
     // object found, so store pointer for later...
-    std::shared_ptr<UniverseObject> result = it->second;
+    auto result = it->second;
     // and erase from pointer maps
     m_objects.erase(it);
     FOR_EACH_SPECIALIZED_MAP(EraseFromMap, id);
@@ -236,17 +231,15 @@ std::shared_ptr<UniverseObject> ObjectMap::Remove(int id) {
     return result;
 }
 
-void ObjectMap::Clear() {
-    FOR_EACH_MAP(ClearMap);
-}
+void ObjectMap::Clear()
+{ FOR_EACH_MAP(ClearMap); }
 
-void ObjectMap::swap(ObjectMap& rhs) {
-    FOR_EACH_MAP(SwapMap, rhs);
-}
+void ObjectMap::swap(ObjectMap& rhs)
+{ FOR_EACH_MAP(SwapMap, rhs); }
 
 std::vector<int> ObjectMap::FindExistingObjectIDs() const {
     std::vector<int> result;
-    for (const std::map<int, std::shared_ptr<UniverseObject>>::value_type& entry : m_existing_objects)
+    for (const auto& entry : m_existing_objects)
     { result.push_back(entry.first); }
     return result;
 }
@@ -261,12 +254,12 @@ void ObjectMap::UpdateCurrentDestroyedObjects(const std::set<int>& destroyed_obj
     m_existing_pop_centers.clear();
     m_existing_resource_centers.clear();
     m_existing_systems.clear();
-    for (const std::map<int, std::shared_ptr<UniverseObject>>::value_type& entry : m_objects) {
+    for (const auto& entry : m_objects) {
         if (!entry.second)
             continue;
-        if (destroyed_object_ids.find(entry.first) != destroyed_object_ids.end())
+        if (destroyed_object_ids.count(entry.first))
             continue;
-        std::shared_ptr<UniverseObject> this_item = this->Object(entry.first);
+        auto this_item = this->Object(entry.first);
         m_existing_objects[entry.first] = this_item;
         switch (entry.second->ObjectType()) {
             case OBJ_BUILDING:
@@ -310,8 +303,8 @@ void ObjectMap::AuditContainment(const std::set<int>& destroyed_object_ids) {
     std::map<int, std::set<int>>    contained_ships;
     std::map<int, std::set<int>>    contained_fields;
 
-    for (std::shared_ptr<const UniverseObject> contained : *this) {
-        if (destroyed_object_ids.find(contained->ID()) != destroyed_object_ids.end())
+    for (const auto& contained : *this) {
+        if (destroyed_object_ids.count(contained->ID()))
             continue;
 
         int contained_id = contained->ID();
@@ -347,7 +340,7 @@ void ObjectMap::AuditContainment(const std::set<int>& destroyed_object_ids) {
     }
 
     // set contained objects of all possible containers
-    for (std::shared_ptr<UniverseObject> obj : *this) {
+    for (const auto& obj : *this) {
         if (obj->ObjectType() == OBJ_SYSTEM) {
             std::shared_ptr<System> sys = std::dynamic_pointer_cast<System>(obj);
             if (!sys)
@@ -374,22 +367,22 @@ void ObjectMap::AuditContainment(const std::set<int>& destroyed_object_ids) {
 
 void ObjectMap::CopyObjectsToSpecializedMaps() {
     FOR_EACH_SPECIALIZED_MAP(ClearMap);
-    for (std::map<int, std::shared_ptr<UniverseObject>>::iterator it = Map<UniverseObject>().begin();
+    for (auto it = Map<UniverseObject>().begin();
          it != Map<UniverseObject>().end(); ++it)
     { FOR_EACH_SPECIALIZED_MAP(TryInsertIntoMap, it->second); }
 }
 
-std::string ObjectMap::Dump() const {
+std::string ObjectMap::Dump(unsigned short ntabs) const {
     std::ostringstream dump_stream;
     dump_stream << "ObjectMap contains UniverseObjects: " << std::endl;
     for (const_iterator<> it = const_begin(); it != const_end(); ++it)
-        dump_stream << it->Dump() << std::endl;
+        dump_stream << it->Dump(ntabs) << std::endl;
     dump_stream << std::endl;
     return dump_stream.str();
 }
 
 std::shared_ptr<UniverseObject> ObjectMap::ExistingObject(int id) {
-    std::map<int, std::shared_ptr<UniverseObject>>::iterator it = m_existing_objects.find(id);
+    auto it = m_existing_objects.find(id);
     if (it != m_existing_objects.end())
         return it->second;
     return nullptr;
@@ -410,7 +403,9 @@ void ObjectMap::SwapMap(std::map<int, std::shared_ptr<T>>& map, ObjectMap& rhs)
 { map.swap(rhs.Map<T>()); }
 
 template <class T>
-void ObjectMap::TryInsertIntoMap(std::map<int, std::shared_ptr<T>>& map, std::shared_ptr<UniverseObject> item) {
+void ObjectMap::TryInsertIntoMap(std::map<int, std::shared_ptr<T>>& map,
+                                 std::shared_ptr<UniverseObject> item)
+{
     if (dynamic_cast<T*>(item.get()))
         map[item->ID()] = std::dynamic_pointer_cast<T, UniverseObject>(item);
 }
