@@ -7,13 +7,16 @@
 #include "UniverseObject.h"
 #include "ValueRef.h"
 #include "Enums.h"
-#include "../parse/Parse.h"
 #include "../util/OptionsDB.h"
 #include "../util/Logger.h"
 #include "../util/Random.h"
 #include "../util/AppInterface.h"
+#include "../util/CheckSums.h"
+#include "../util/ScopedTimer.h"
 
 #include <boost/filesystem/fstream.hpp>
+//TODO: replace with std::make_unique when transitioning to C++14
+#include <boost/smart_ptr/make_unique.hpp>
 
 #include <iterator>
 
@@ -22,24 +25,34 @@
 // FocusType                                   //
 /////////////////////////////////////////////////
 FocusType::FocusType(const std::string& name, const std::string& description,
-                     const Condition::ConditionBase* location, const std::string& graphic) :
+                     std::unique_ptr<Condition::ConditionBase>&& location, const std::string& graphic) :
     m_name(name),
     m_description(description),
-    m_location(location),
+    m_location(std::move(location)),
     m_graphic(graphic)
 {}
 
-std::string FocusType::Dump() const {
-    std::string retval = DumpIndent() + "FocusType\n";
-    ++g_indent;
-    retval += DumpIndent() + "name = \"" + m_name + "\"\n";
-    retval += DumpIndent() + "description = \"" + m_description + "\"\n";
-    retval += DumpIndent() + "location = \n";
-    ++g_indent;
-    retval += m_location->Dump();
-    --g_indent;
-    retval += DumpIndent() + "graphic = \"" + m_graphic + "\"\n";
-    --g_indent;
+FocusType::~FocusType()
+{}
+
+std::string FocusType::Dump(unsigned short ntabs) const {
+    std::string retval = DumpIndent(ntabs) + "FocusType\n";
+    retval += DumpIndent(ntabs+1) + "name = \"" + m_name + "\"\n";
+    retval += DumpIndent(ntabs+1) + "description = \"" + m_description + "\"\n";
+    retval += DumpIndent(ntabs+1) + "location = \n";
+    retval += m_location->Dump(ntabs+2);
+    retval += DumpIndent(ntabs+1) + "graphic = \"" + m_graphic + "\"\n";
+    return retval;
+}
+
+unsigned int FocusType::GetCheckSum() const {
+    unsigned int retval{0};
+
+    CheckSums::CheckSumCombine(retval, m_name);
+    CheckSums::CheckSumCombine(retval, m_description);
+    CheckSums::CheckSumCombine(retval, m_location);
+    CheckSums::CheckSumCombine(retval, m_graphic);
+
     return retval;
 }
 
@@ -75,77 +88,96 @@ namespace {
     }
 }
 
+Species::Species(const SpeciesStrings& strings,
+                 const std::vector<FocusType>& foci,
+                 const std::string& preferred_focus,
+                 const std::map<PlanetType, PlanetEnvironment>& planet_environments,
+                 std::vector<std::unique_ptr<Effect::EffectsGroup>>&& effects,
+                 const SpeciesParams& params,
+                 const std::set<std::string>& tags,
+                 const std::string& graphic) :
+    m_name(strings.name),
+    m_description(strings.desc),
+    m_gameplay_description(strings.gameplay_desc),
+    m_foci(foci),
+    m_preferred_focus(preferred_focus),
+    m_planet_environments(planet_environments),
+    m_effects(),
+    m_location(),
+    m_playable(params.playable),
+    m_native(params.native),
+    m_can_colonize(params.can_colonize),
+    m_can_produce_ships(params.can_produce_ships),
+    m_tags(),
+    m_graphic(graphic)
+{
+    for (auto&& effect : effects)
+        m_effects.emplace_back(std::move(effect));
+
+    Init();
+    for (const std::string& tag : tags)
+        m_tags.insert(boost::to_upper_copy<std::string>(tag));
+}
+
 Species::~Species()
-{ delete m_location; }
+{}
 
 void Species::Init() {
     if (m_location)
         m_location->SetTopLevelContent(this->m_name);
-    for (std::shared_ptr<Effect::EffectsGroup> effect : m_effects) {
+    for (auto& effect : m_effects) {
         effect->SetTopLevelContent(m_name);
     }
+
+    TraceLogger() << "Species::Init: " << Dump();
 }
 
-std::string Species::Dump() const {
-    std::string retval = DumpIndent() + "Species\n";
-    ++g_indent;
-    retval += DumpIndent() + "name = \"" + m_name + "\"\n";
-    retval += DumpIndent() + "description = \"" + m_description + "\"\n";
-    retval += DumpIndent() + "gameplay_description = \"" + m_gameplay_description + "\"\n";
+std::string Species::Dump(unsigned short ntabs) const {
+    std::string retval = DumpIndent(ntabs) + "Species\n";
+    retval += DumpIndent(ntabs+1) + "name = \"" + m_name + "\"\n";
+    retval += DumpIndent(ntabs+1) + "description = \"" + m_description + "\"\n";
+    retval += DumpIndent(ntabs+1) + "gameplay_description = \"" + m_gameplay_description + "\"\n";
     if (m_playable)
-        retval += DumpIndent() + "Playable\n";
+        retval += DumpIndent(ntabs+1) + "Playable\n";
     if (m_native)
-        retval += DumpIndent() + "Native\n";
+        retval += DumpIndent(ntabs+1) + "Native\n";
     if (m_can_produce_ships)
-        retval += DumpIndent() + "CanProduceShips\n";
+        retval += DumpIndent(ntabs+1) + "CanProduceShips\n";
     if (m_can_colonize)
-        retval += DumpIndent() + "CanColonize\n";
+        retval += DumpIndent(ntabs+1) + "CanColonize\n";
     if (m_foci.size() == 1) {
-        retval += DumpIndent() + "foci =\n";
-        m_foci.begin()->Dump();
+        retval += DumpIndent(ntabs+1) + "foci =\n";
+        m_foci.begin()->Dump(ntabs+1);
     } else {
-        retval += DumpIndent() + "foci = [\n";
-        ++g_indent;
-        for (const FocusType& focus : m_foci) {
-            retval += focus.Dump();
-        }
-        --g_indent;
-        retval += DumpIndent() + "]\n";
+        retval += DumpIndent(ntabs+1) + "foci = [\n";
+        for (const FocusType& focus : m_foci)
+            retval += focus.Dump(ntabs+2);
+        retval += DumpIndent(ntabs+1) + "]\n";
     }
     if (m_effects.size() == 1) {
-        retval += DumpIndent() + "effectsgroups =\n";
-        ++g_indent;
-        retval += m_effects[0]->Dump();
-        --g_indent;
+        retval += DumpIndent(ntabs+1) + "effectsgroups =\n";
+        retval += m_effects[0]->Dump(ntabs+2);
     } else {
-        retval += DumpIndent() + "effectsgroups = [\n";
-        ++g_indent;
-        for (std::shared_ptr<Effect::EffectsGroup> effect : m_effects) {
-            retval += effect->Dump();
-        }
-        --g_indent;
-        retval += DumpIndent() + "]\n";
+        retval += DumpIndent(ntabs+1) + "effectsgroups = [\n";
+        for (auto& effect : m_effects)
+            retval += effect->Dump(ntabs+2);
+        retval += DumpIndent(ntabs+1) + "]\n";
     }
     if (m_planet_environments.size() == 1) {
-        retval += DumpIndent() + "environments =\n";
-        ++g_indent;
-        retval += DumpIndent() + "type = " + PlanetTypeToString(m_planet_environments.begin()->first)
-                               + " environment = " + PlanetEnvironmentToString(m_planet_environments.begin()->second)
-                               + "\n";
-        --g_indent;
+        retval += DumpIndent(ntabs+1) + "environments =\n";
+        retval += DumpIndent(ntabs+2) + "type = " + PlanetTypeToString(m_planet_environments.begin()->first)
+            + " environment = " + PlanetEnvironmentToString(m_planet_environments.begin()->second)
+            + "\n";
     } else {
-        retval += DumpIndent() + "environments = [\n";
-        ++g_indent;
-        for (const std::map<PlanetType, PlanetEnvironment>::value_type& entry : m_planet_environments) {
-            retval += DumpIndent() + "type = " + PlanetTypeToString(entry.first)
-                                   + " environment = " + PlanetEnvironmentToString(entry.second)
-                                   + "\n";
+        retval += DumpIndent(ntabs+1) + "environments = [\n";
+        for (const auto& entry : m_planet_environments) {
+            retval += DumpIndent(ntabs+2) + "type = " + PlanetTypeToString(entry.first)
+                + " environment = " + PlanetEnvironmentToString(entry.second)
+                + "\n";
         }
-        --g_indent;
-        retval += DumpIndent() + "]\n";
+        retval += DumpIndent(ntabs+1) + "]\n";
     }
-    retval += DumpIndent() + "graphic = \"" + m_graphic + "\"\n";
-    --g_indent;
+    retval += DumpIndent(ntabs+1) + "graphic = \"" + m_graphic + "\"\n";
     return retval;
 }
 
@@ -156,7 +188,7 @@ std::string Species::GameplayDescription() const {
 
     bool requires_separator = true;
 
-    for (std::shared_ptr<Effect::EffectsGroup> effect : m_effects) {
+    for (auto& effect : m_effects) {
         const std::string& description = effect->GetDescription();
         if (description.empty())
             continue;
@@ -175,24 +207,31 @@ std::string Species::GameplayDescription() const {
 const Condition::ConditionBase* Species::Location() const {
     if (!m_location) {
         // set up a Condition structure to match popcenters that have (not uninhabitable) environment for this species
-        std::vector<ValueRef::ValueRefBase< ::PlanetEnvironment>*> environments_vec;
-        environments_vec.push_back(new ValueRef::Constant<PlanetEnvironment>( ::PE_UNINHABITABLE));
-        ValueRef::Constant<std::string>* this_species_name_ref = new ValueRef::Constant<std::string>(m_name);  // m_name specifies this species
-        Condition::ConditionBase* enviro_cond = new Condition::Not(new Condition::PlanetEnvironment(environments_vec, this_species_name_ref));
+        std::vector<std::unique_ptr<ValueRef::ValueRefBase< ::PlanetEnvironment>>> environments_vec;
+        environments_vec.push_back(
+            boost::make_unique<ValueRef::Constant<PlanetEnvironment>>( ::PE_UNINHABITABLE));
+        auto this_species_name_ref =
+            boost::make_unique<ValueRef::Constant<std::string>>(m_name);  // m_name specifies this species
+        auto enviro_cond = std::unique_ptr<Condition::ConditionBase>(
+            boost::make_unique<Condition::Not>(
+                std::unique_ptr<Condition::ConditionBase>(
+                    boost::make_unique<Condition::PlanetEnvironment>(
+                        std::move(environments_vec), std::move(this_species_name_ref)))));
 
-        Condition::ConditionBase* type_cond = new Condition::Type(new ValueRef::Constant<UniverseObjectType>( ::OBJ_POP_CENTER));
+        auto type_cond = std::unique_ptr<Condition::ConditionBase>(boost::make_unique<Condition::Type>(
+            boost::make_unique<ValueRef::Constant<UniverseObjectType>>( ::OBJ_POP_CENTER)));
 
-        std::vector<Condition::ConditionBase*> operands;
-        operands.push_back(enviro_cond);
-        operands.push_back(type_cond);
+        std::vector<std::unique_ptr<Condition::ConditionBase>> operands;
+        operands.push_back(std::move(enviro_cond));
+        operands.push_back(std::move(type_cond));
 
-        m_location = new Condition::And(operands);
+        m_location = std::unique_ptr<Condition::ConditionBase>(boost::make_unique<Condition::And>(std::move(operands)));
     }
-    return m_location;
+    return m_location.get();
 }
 
 PlanetEnvironment Species::GetPlanetEnvironment(PlanetType planet_type) const {
-    std::map<PlanetType, PlanetEnvironment>::const_iterator it = m_planet_environments.find(planet_type);
+    auto it = m_planet_environments.find(planet_type);
     if (it == m_planet_environments.end())
         return PE_UNINHABITABLE;
     else
@@ -232,7 +271,7 @@ PlanetType Species::NextBetterPlanetType(PlanetType initial_planet_type) const
     // determine which environment rating is the best available for this species
     PlanetEnvironment best_environment = PE_UNINHABITABLE;
     //std::set<PlanetType> best_types;
-    for (const std::map<PlanetType, PlanetEnvironment>::value_type& entry : m_planet_environments) {
+    for (const auto& entry : m_planet_environments) {
         if (entry.second == best_environment) {
             //best_types.insert(entry.first);
         } else if (entry.second > best_environment) {
@@ -269,14 +308,14 @@ PlanetType Species::NextBetterPlanetType(PlanetType initial_planet_type) const
 void Species::AddHomeworld(int homeworld_id) {
     if (!GetUniverseObject(homeworld_id))
         DebugLogger() << "Species asked to add homeworld id " << homeworld_id << " but there is no such object in the Universe";
-    if (m_homeworlds.find(homeworld_id) != m_homeworlds.end())
+    if (m_homeworlds.count(homeworld_id))
         return;
     m_homeworlds.insert(homeworld_id);
     // TODO if needed: StateChangedSignal();
 }
 
 void Species::RemoveHomeworld(int homeworld_id) {
-    if (m_homeworlds.find(homeworld_id) == m_homeworlds.end()) {
+    if (!m_homeworlds.count(homeworld_id)) {
         DebugLogger() << "Species asked to remove homeworld id " << homeworld_id << " but doesn't have that id as a homeworld";
         return;
     }
@@ -303,6 +342,27 @@ void Species::SetOtherSpeciesOpinions(const std::map<std::string, double>& opini
 void Species::SetOtherSpeciesOpinion(const std::string& species_name, double opinion)
 {}
 
+unsigned int Species::GetCheckSum() const {
+    unsigned int retval{0};
+
+    CheckSums::CheckSumCombine(retval, m_name);
+    CheckSums::CheckSumCombine(retval, m_description);
+    CheckSums::CheckSumCombine(retval, m_gameplay_description);
+    // opinions and homeworlds are per-game specific, so not included in checksum
+    CheckSums::CheckSumCombine(retval, m_foci);
+    CheckSums::CheckSumCombine(retval, m_preferred_focus);
+    CheckSums::CheckSumCombine(retval, m_planet_environments);
+    CheckSums::CheckSumCombine(retval, m_effects);
+    CheckSums::CheckSumCombine(retval, m_location);
+    CheckSums::CheckSumCombine(retval, m_playable);
+    CheckSums::CheckSumCombine(retval, m_native);
+    CheckSums::CheckSumCombine(retval, m_can_colonize);
+    CheckSums::CheckSumCombine(retval, m_can_produce_ships);
+    CheckSums::CheckSumCombine(retval, m_tags);
+    CheckSums::CheckSumCombine(retval, m_graphic);
+
+    return retval;
+}
 
 /////////////////////////////////////////////////
 // SpeciesManager                              //
@@ -311,55 +371,36 @@ void Species::SetOtherSpeciesOpinion(const std::string& species_name, double opi
 SpeciesManager* SpeciesManager::s_instance = nullptr;
 
 bool SpeciesManager::PlayableSpecies::operator()(
-    const std::map<std::string, Species*>::value_type& species_map_iterator) const
-{ return species_map_iterator.second->Playable(); }
+    const std::map<std::string, std::unique_ptr<Species>>::value_type& species_entry) const
+{ return species_entry.second->Playable(); }
 
 bool SpeciesManager::NativeSpecies::operator()(
-    const std::map<std::string, Species*>::value_type& species_map_iterator) const
-{ return species_map_iterator.second->Native(); }
+    const std::map<std::string, std::unique_ptr<Species>>::value_type& species_entry) const
+{ return species_entry.second->Native(); }
 
 SpeciesManager::SpeciesManager() {
     if (s_instance)
         throw std::runtime_error("Attempted to create more than one SpeciesManager.");
+
+    // Only update the global pointer on sucessful construction.
     s_instance = this;
-
-    try {
-        parse::species(m_species);
-    } catch (const std::exception& e) {
-        ErrorLogger() << "Failed parsing species: error: " << e.what();
-        throw e;
-    }
-
-    if (GetOptionsDB().Get<bool>("verbose-logging")) {
-        DebugLogger() << "Species:";
-        for (const std::map<std::string, Species*>::value_type& entry : m_species) {
-            const Species* s = entry.second;
-            DebugLogger() << " ... " << s->Name() << "  \t" <<
-                (s->Playable() ?        "Playable " : "         ") <<
-                (s->Native() ?          "Native " : "       ") <<
-                (s->CanProduceShips() ? "CanProduceShips " : "                ") <<
-                (s->CanColonize() ?     "CanColonize " : "            ");
-        }
-    }
-}
-
-SpeciesManager::~SpeciesManager() {
-    for (std::map<std::string, Species*>::value_type& entry : m_species)
-        delete entry.second;
 }
 
 const Species* SpeciesManager::GetSpecies(const std::string& name) const {
-    std::map<std::string, Species*>::const_iterator it = m_species.find(name);
-    return it != m_species.end() ? it->second : nullptr;
+    CheckPendingSpeciesTypes();
+    auto it = m_species.find(name);
+    return it != m_species.end() ? it->second.get() : nullptr;
 }
 
 Species* SpeciesManager::GetSpecies(const std::string& name) {
-    std::map<std::string, Species*>::iterator it = m_species.find(name);
-    return it != m_species.end() ? it->second : nullptr;
+    CheckPendingSpeciesTypes();
+    auto it = m_species.find(name);
+    return it != m_species.end() ? it->second.get() : nullptr;
 }
 
 int SpeciesManager::GetSpeciesID(const std::string& name) const {
-    iterator it = m_species.find(name);
+    CheckPendingSpeciesTypes();
+    auto it = m_species.find(name);
     if (it == m_species.end())
         return -1;
     return std::distance(m_species.begin(), it);
@@ -370,29 +411,60 @@ SpeciesManager& SpeciesManager::GetSpeciesManager() {
     return manager;
 }
 
-SpeciesManager::iterator SpeciesManager::begin() const
-{ return m_species.begin(); }
+void SpeciesManager::SetSpeciesTypes(Pending::Pending<std::pair<SpeciesTypeMap, CensusOrder>>&& future)
+{ m_pending_types = std::move(future); }
 
-SpeciesManager::iterator SpeciesManager::end() const
-{ return m_species.end(); }
+void SpeciesManager::CheckPendingSpeciesTypes() const {
+    if (!m_pending_types) {
+        if (m_species.empty())
+            throw;
+        return;
+    }
+
+    auto container = std::make_pair(std::move(m_species), m_census_order);
+
+    Pending::SwapPending(m_pending_types, container);
+
+    m_species = std::move(container.first);
+    m_census_order = std::move(container.second);
+}
+
+SpeciesManager::iterator SpeciesManager::begin() const {
+    CheckPendingSpeciesTypes();
+    return m_species.begin();
+}
+
+SpeciesManager::iterator SpeciesManager::end() const {
+    CheckPendingSpeciesTypes();
+    return m_species.end();
+}
 
 SpeciesManager::playable_iterator SpeciesManager::playable_begin() const
-{ return playable_iterator(PlayableSpecies(), m_species.begin(), m_species.end()); }
+{ return playable_iterator(PlayableSpecies(), begin(), end()); }
 
 SpeciesManager::playable_iterator SpeciesManager::playable_end() const
-{ return playable_iterator(PlayableSpecies(), m_species.end(), m_species.end()); }
+{ return playable_iterator(PlayableSpecies(), end(), end()); }
 
 SpeciesManager::native_iterator SpeciesManager::native_begin() const
-{ return native_iterator(NativeSpecies(), m_species.begin(), m_species.end()); }
+{ return native_iterator(NativeSpecies(), begin(), end()); }
 
 SpeciesManager::native_iterator SpeciesManager::native_end() const
-{ return native_iterator(NativeSpecies(), m_species.end(), m_species.end()); }
+{ return native_iterator(NativeSpecies(), end(), end()); }
 
-bool SpeciesManager::empty() const
-{ return m_species.empty(); }
+const SpeciesManager::CensusOrder& SpeciesManager::census_order() const {
+    CheckPendingSpeciesTypes();
+    return m_census_order;
+}
 
-int SpeciesManager::NumSpecies() const
-{ return m_species.size(); }
+bool SpeciesManager::empty() const {
+    CheckPendingSpeciesTypes();
+    return m_species.empty();
+}
+
+int SpeciesManager::NumSpecies() const {
+    CheckPendingSpeciesTypes();
+    return m_species.size();
+}
 
 int SpeciesManager::NumPlayableSpecies() const
 { return std::distance(playable_begin(), playable_end()); }
@@ -405,11 +477,12 @@ namespace {
 }
 
 const std::string& SpeciesManager::RandomSpeciesName() const {
+    CheckPendingSpeciesTypes();
     if (m_species.empty())
         return EMPTY_STRING;
 
     int species_idx = RandSmallInt(0, static_cast<int>(m_species.size()) - 1);
-    return std::next(m_species.begin(), species_idx)->first;
+    return std::next(begin(), species_idx)->first;
 }
 
 const std::string& SpeciesManager::RandomPlayableSpeciesName() const {
@@ -430,20 +503,22 @@ const std::string& SpeciesManager::SequentialPlayableSpeciesName(int id) const {
 }
 
 void SpeciesManager::ClearSpeciesHomeworlds() {
-    for (std::map<std::string, Species*>::value_type& entry : m_species)
+    CheckPendingSpeciesTypes();
+    for (auto& entry : m_species)
         entry.second->SetHomeworlds(std::set<int>());
 }
 
 void SpeciesManager::SetSpeciesHomeworlds(const std::map<std::string, std::set<int>>& species_homeworld_ids) {
+    CheckPendingSpeciesTypes();
     ClearSpeciesHomeworlds();
-    for (const std::map<std::string, std::set<int>>::value_type& entry : species_homeworld_ids) {
+    for (auto& entry : species_homeworld_ids) {
         const std::string& species_name = entry.first;
         const std::set<int>& homeworlds = entry.second;
 
         Species* species = nullptr;
-        std::map<std::string, Species*>::iterator species_it = m_species.find(species_name);
-        if (species_it != m_species.end())
-            species = species_it->second;
+        auto species_it = m_species.find(species_name);
+        if (species_it != end())
+            species = species_it->second.get();
 
         if (species) {
             species->SetHomeworlds(homeworlds);
@@ -466,10 +541,11 @@ void SpeciesManager::SetSpeciesSpeciesOpinion(const std::string& opinionated_spe
 { m_species_species_opinions[opinionated_species][rated_species] = opinion; }
 
 std::map<std::string, std::set<int>> SpeciesManager::GetSpeciesHomeworldsMap(int encoding_empire/* = ALL_EMPIRES*/) const {
+    CheckPendingSpeciesTypes();
     std::map<std::string, std::set<int>> retval;
-    for (const std::map<std::string, Species*>::value_type& entry : m_species) {
+    for (const auto& entry : m_species) {
         const std::string species_name = entry.first;
-        const Species* species = entry.second;
+        const Species* species = entry.second.get();
         if (!species) {
             ErrorLogger() << "SpeciesManager::GetSpeciesHomeworldsMap found a null species pointer in SpeciesManager?!";
             continue;
@@ -487,22 +563,22 @@ const std::map<std::string, std::map<std::string, float>>& SpeciesManager::GetSp
 { return m_species_species_opinions; }
 
 float SpeciesManager::SpeciesEmpireOpinion(const std::string& species_name, int empire_id) const {
-    std::map<std::string, std::map<int, float>>::const_iterator sp_it = m_species_empire_opinions.find(species_name);
+    auto sp_it = m_species_empire_opinions.find(species_name);
     if (sp_it == m_species_empire_opinions.end())
         return 0.0f;
     const std::map<int, float>& emp_map = sp_it->second;
-    std::map<int, float>::const_iterator emp_it = emp_map.find(empire_id);
+    auto emp_it = emp_map.find(empire_id);
     if (emp_it == emp_map.end())
         return 0.0f;
     return emp_it->second;
 }
 
 float SpeciesManager::SpeciesSpeciesOpinion(const std::string& opinionated_species_name, const std::string& rated_species_name) const {
-    std::map<std::string, std::map<std::string, float>>::const_iterator sp_it = m_species_species_opinions.find(opinionated_species_name);
+    auto sp_it = m_species_species_opinions.find(opinionated_species_name);
     if (sp_it == m_species_species_opinions.end())
         return 0.0f;
-    const std::map<std::string, float>& ra_sp_map = sp_it->second;
-    std::map<std::string, float>::const_iterator ra_sp_it = ra_sp_map.find(rated_species_name);
+    const auto& ra_sp_map = sp_it->second;
+    auto ra_sp_it = ra_sp_map.find(rated_species_name);
     if (ra_sp_it == ra_sp_map.end())
         return 0.0f;
     return ra_sp_it->second;
@@ -516,12 +592,12 @@ void SpeciesManager::ClearSpeciesOpinions() {
 void SpeciesManager::UpdatePopulationCounter() {
     // ships of each species and design
     m_species_object_populations.clear();
-    for (const std::map<int, std::shared_ptr<UniverseObject>>::value_type& entry : Objects().ExistingObjects()) {
-        std::shared_ptr<UniverseObject> obj = entry.second;
+    for (const auto& entry : Objects().ExistingObjects()) {
+        auto obj = entry.second;
         if (obj->ObjectType() != OBJ_PLANET && obj->ObjectType() != OBJ_POP_CENTER)
             continue;
 
-        std::shared_ptr<PopCenter> pop_center = std::dynamic_pointer_cast<PopCenter>(obj);
+        auto pop_center = std::dynamic_pointer_cast<PopCenter>(obj);
         if (!pop_center)
             continue;
 
@@ -543,6 +619,16 @@ std::map<std::string, std::map<int, float>>& SpeciesManager::SpeciesObjectPopula
 std::map<std::string, std::map<std::string, int>>& SpeciesManager::SpeciesShipsDestroyed(int encoding_empire)
 { return m_species_species_ships_destroyed; }
 
+unsigned int SpeciesManager::GetCheckSum() const {
+    CheckPendingSpeciesTypes();
+    unsigned int retval{0};
+    for (auto const& name_type_pair : m_species)
+        CheckSums::CheckSumCombine(retval, name_type_pair);
+    CheckSums::CheckSumCombine(retval, m_species.size());
+
+    DebugLogger() << "SpeciesManager checksum: " << retval;
+    return retval;
+}
 
 ///////////////////////////////////////////////////////////
 // Free Functions                                        //
