@@ -3,6 +3,7 @@
 #include "OptionsDB.h"
 
 #include <GG/utf8/checked.h>
+#include "../universe/Enums.h"
 
 #include <boost/filesystem/convenience.hpp>
 #include <boost/filesystem/operations.hpp>
@@ -72,12 +73,12 @@ void InitDirs(const std::string& argv0) {
     bundle_path = fs::path(bundle_dir);
 
     // search bundle_path for a directory named "FreeOrion.app", exiting if not found, else constructing a path to application bundle contents
-    fs::path::iterator appiter =   std::find(bundle_path.begin(), bundle_path.end(), "FreeOrion.app");
+    auto appiter = std::find(bundle_path.begin(), bundle_path.end(), "FreeOrion.app");
     if (appiter == bundle_path.end()) {
         std::cerr << "Error: Application bundle must be named 'FreeOrion.app' and executables must not be called from outside of it." << std::endl;
         exit(-1);
     } else {
-        for (fs::path::iterator piter = bundle_path.begin(); piter != appiter; ++piter) {
+        for (auto piter = bundle_path.begin(); piter != appiter; ++piter) {
             app_path /= *piter;
         }
         app_path /= "FreeOrion.app/Contents";
@@ -97,6 +98,10 @@ void InitDirs(const std::string& argv0) {
     if (!exists(p))
         fs::create_directories(p);
 
+    // Intentionally do not create the server save dir.
+    // The server save dir is publically accessible and should not be
+    // automatically created for the user.
+
     g_initialized = true;
 }
 
@@ -130,7 +135,7 @@ const fs::path GetPythonHome() {
     return s_python_home;
 }
 
-#elif defined(FREEORION_LINUX) || defined(FREEORION_FREEBSD)
+#elif defined(FREEORION_LINUX) || defined(FREEORION_FREEBSD) || defined(FREEORION_OPENBSD)
 #include "binreloc.h"
 #include <unistd.h>
 #include <boost/filesystem/fstream.hpp>
@@ -174,7 +179,7 @@ namespace {
             << old_path << std::endl << std::endl
             << "Configuration were files copied to:" << std::endl << config_path << std::endl << std::endl
             << "Data Files were copied to:" << std::endl << data_path << std::endl << std::endl
-            << "If your save-dir option in persistent_config.xml was ~/.config, then you need to update it."
+            << "If your save.path option in persistent_config.xml was ~/.config, then you need to update it."
             << std::endl;
 
         try {
@@ -203,7 +208,7 @@ namespace {
                 }
             }
 
-            //Start update of save-dir in config file and complete it in CompleteXDGMigration()
+            //Start update of save.path in config file and complete it in CompleteXDGMigration()
             fs::path sentinel = GetUserDataDir() / "MIGRATION_TO_XDG_IN_PROGRESS";
             if (!exists(sentinel)) {
                 fs::ofstream touchfile(sentinel);
@@ -260,6 +265,10 @@ void InitDirs(const std::string& argv0) {
         fs::create_directories(p);
     }
 
+    // Intentionally do not create the server save dir.
+    // The server save dir is publically accessible and should not be
+    // automatically created for the user.
+
     InitBinDir(argv0);
 
     g_initialized = true;
@@ -281,7 +290,7 @@ const fs::path GetUserDataDir() {
 
 const fs::path GetRootDataDir() {
     if (!g_initialized) InitDirs("");
-    char* dir_name = br_find_data_dir("/usr/local/share");
+    char* dir_name = br_find_data_dir(SHAREPATH);
     fs::path p(dir_name);
     std::free(dir_name);
     p /= "freeorion";
@@ -312,6 +321,14 @@ void InitBinDir(const std::string& argv0) {
         mib[3] = -1;
         size_t buf_size = sizeof(buf);
         sysctl(mib, 4, buf, &buf_size, 0, 0);
+#elif defined(__OpenBSD__)
+        // OpenBSD does not have executable path's retrieval feature
+        std::string argpath(argv0);
+        boost::erase_all(argpath, "\"");
+        if (argpath[0] != '/')
+            problem = (nullptr == realpath(argpath.c_str(), buf));
+        else
+            strncpy(buf, argpath.c_str(), sizeof(buf));
 #else
         problem = (-1 == readlink("/proc/self/exe", buf, sizeof(buf) - 1));
 #endif
@@ -323,9 +340,9 @@ void InitBinDir(const std::string& argv0) {
             fs::path binary_file = fs::system_complete(fs::path(path_text));
             bin_dir = binary_file.branch_path();
 
-            // check that a "freeorion" file (hopefully the freeorion binary) exists in the found directory
+            // check that a "freeoriond" file (hopefully the freeorion server binary) exists in the found directory
             fs::path p(bin_dir);
-            p /= "freeorion";
+            p /= "freeoriond";
             if (!exists(p))
                 problem = true;
         }
@@ -336,7 +353,7 @@ void InitBinDir(const std::string& argv0) {
 
     if (problem) {
         // failed trying to parse the call path, so try hard-coded standard location...
-        char* dir_name = br_find_bin_dir("/usr/local/bin");
+        char* dir_name = br_find_bin_dir(BINPATH);
         fs::path p(dir_name);
         std::free(dir_name);
 
@@ -365,6 +382,10 @@ void InitDirs(const std::string& argv0) {
     if (!exists(p))
         fs::create_directories(p);
 
+    // Intentionally do not create the server save dir.
+    // The server save dir is publically accessible and should not be
+    // automatically created for the user.
+
     InitBinDir(argv0);
 
     g_initialized = true;
@@ -401,7 +422,7 @@ void InitBinDir(const std::string& argv0) {
 }
 
 #else
-#  error Neither FREEORION_LINUX, FREEORION_FREEBSD nor FREEORION_WIN32 set
+#  error Neither FREEORION_LINUX, FREEORION_FREEBSD, FREEORION_OPENBSD nor FREEORION_WIN32 set
 #endif
 
 void CompleteXDGMigration() {
@@ -409,24 +430,24 @@ void CompleteXDGMigration() {
     if (exists(sentinel)) {
         fs::remove(sentinel);
         // Update data dir in config file
-        const std::string options_save_dir = GetOptionsDB().Get<std::string>("save-dir");
+        const std::string options_save_dir = GetOptionsDB().Get<std::string>("save.path");
         const fs::path old_path = fs::path(getenv("HOME")) / ".freeorion";
         if (fs::path(options_save_dir) == old_path)
-            GetOptionsDB().Set<std::string>("save-dir", GetUserDataDir().string());
+            GetOptionsDB().Set<std::string>("save.path", GetUserDataDir().string());
     }
 }
 
 const fs::path GetResourceDir() {
     // if resource dir option has been set, use specified location. otherwise,
     // use default location
-    std::string options_resource_dir = GetOptionsDB().Get<std::string>("resource-dir");
+    std::string options_resource_dir = GetOptionsDB().Get<std::string>("resource.path");
     fs::path dir = FilenameToPath(options_resource_dir);
     if (fs::exists(dir) && fs::is_directory(dir))
         return dir;
 
-    dir = GetOptionsDB().GetDefault<std::string>("resource-dir");
+    dir = GetOptionsDB().GetDefault<std::string>("resource.path");
     if (!fs::is_directory(dir) || !fs::exists(dir))
-        dir = FilenameToPath(GetOptionsDB().GetDefault<std::string>("resource-dir"));
+        dir = FilenameToPath(GetOptionsDB().GetDefault<std::string>("resource.path"));
 
     return dir;
 }
@@ -444,9 +465,18 @@ const fs::path GetPersistentConfigPath() {
 const fs::path GetSaveDir() {
     // if save dir option has been set, use specified location.  otherwise,
     // use default location
-    std::string options_save_dir = GetOptionsDB().Get<std::string>("save-dir");
+    std::string options_save_dir = GetOptionsDB().Get<std::string>("save.path");
     if (options_save_dir.empty())
-        options_save_dir = GetOptionsDB().GetDefault<std::string>("save-dir");
+        options_save_dir = GetOptionsDB().GetDefault<std::string>("save.path");
+    return FilenameToPath(options_save_dir);
+}
+
+const fs::path GetServerSaveDir() {
+    // if server save dir option has been set, use specified location.  otherwise,
+    // use default location
+    std::string options_save_dir = GetOptionsDB().Get<std::string>("save.server.path");
+    if (options_save_dir.empty())
+        options_save_dir = GetOptionsDB().GetDefault<std::string>("save.server.path");
     return FilenameToPath(options_save_dir);
 }
 
@@ -454,10 +484,10 @@ fs::path RelativePath(const fs::path& from, const fs::path& to) {
     fs::path retval;
     fs::path from_abs = fs::absolute(from);
     fs::path to_abs = fs::absolute(to);
-    fs::path::iterator from_it = from_abs.begin();
-    fs::path::iterator end_from_it = from_abs.end();
-    fs::path::iterator to_it = to_abs.begin();
-    fs::path::iterator end_to_it = to_abs.end();
+    auto from_it = from_abs.begin();
+    auto end_from_it = from_abs.end();
+    auto to_it = to_abs.begin();
+    auto end_to_it = to_abs.end();
     while (from_it != end_from_it && to_it != end_to_it && *from_it == *to_it) {
         ++from_it;
         ++to_it;
@@ -469,28 +499,36 @@ fs::path RelativePath(const fs::path& from, const fs::path& to) {
     return retval;
 }
 
-std::string PathString(const fs::path& path) {
-#ifndef FREEORION_WIN32
-    return path.string();
-#else
-    fs::path::string_type native_string = path.native();
+#if defined(FREEORION_WIN32)
+
+std::string PathToString(const fs::path& path) {
+    fs::path::string_type native_string = path.generic_wstring();
     std::string retval;
     utf8::utf16to8(native_string.begin(), native_string.end(), std::back_inserter(retval));
     return retval;
-#endif
 }
 
-const fs::path FilenameToPath(const std::string& path_str) {
-#if defined(FREEORION_WIN32)
+fs::path FilenameToPath(const std::string& path_str) {
     // convert UTF-8 directory string to UTF-16
     boost::filesystem::path::string_type directory_native;
     utf8::utf8to16(path_str.begin(), path_str.end(), std::back_inserter(directory_native));
-    return fs::path(directory_native);
+#if (BOOST_VERSION >= 106300)
+    return fs::path(directory_native).generic_path();
 #else
-    return fs::path(path_str);
+    return fs::path(directory_native);
 #endif
 }
 
+#else // defined(FREEORION_WIN32)
+
+std::string PathToString(const fs::path& path)
+{ return path.string(); }
+
+fs::path FilenameToPath(const std::string& path_str)
+{ return fs::path(path_str); }
+
+#endif // defined(FREEORION_WIN32)
+
 std::string FilenameTimestamp() {
     boost::posix_time::time_facet* facet = new boost::posix_time::time_facet("%Y%m%d_%H%M%S");
     std::stringstream date_stream;
@@ -500,7 +538,7 @@ std::string FilenameTimestamp() {
 }
 
 /**  \brief Return a vector of absolute paths to files in the given path
- * 
+ *
  * @param[in] path relative or absolute directory (searched recursively)
  * @return Any regular files in
  * @return  if absolute directory: path
@@ -510,7 +548,7 @@ std::vector<fs::path> ListDir(const fs::path& path) {
     std::vector<fs::path> retval;
     bool is_rel = path.is_relative();
     if (!is_rel && (fs::is_empty(path) || !fs::is_directory(path))) {
-        DebugLogger() << "ListDir: File " << PathString(path) << " was not included as it is empty or not a directoy";
+        DebugLogger() << "ListDir: File " << PathToString(path) << " was not included as it is empty or not a directoy";
     } else {
         const fs::path& default_path = is_rel ? GetResourceDir() / path : path;
 
@@ -520,7 +558,7 @@ std::vector<fs::path> ListDir(const fs::path& path) {
             if (fs::is_regular_file(dir_it->status())) {
                 retval.push_back(dir_it->path());
             } else if (!fs::is_directory(dir_it->status())) {
-                TraceLogger() << "Parse: Unknown file not included: " << PathString(dir_it->path());
+                TraceLogger() << "Parse: Unknown file not included: " << PathToString(dir_it->path());
             }
         }
     }
@@ -534,3 +572,134 @@ std::vector<fs::path> ListDir(const fs::path& path) {
 
 bool IsValidUTF8(const std::string& in)
 { return utf8::is_valid(in.begin(), in.end()); }
+
+bool IsInDir(const fs::path& dir, const fs::path& test_dir) {
+    if (!fs::exists(dir) || !fs::is_directory(dir))
+        return false;
+
+    if (fs::exists(test_dir) && !fs::is_directory(test_dir))
+        return false;
+
+    // Resolve any symbolic links, dots or dot-dots
+    auto canon_dir = fs::canonical(dir);
+    // Don't resolve path if directory doesn't exist
+    // TODO: Change to fs::weakly_canonical after bump boost version above 1.60
+    auto canon_path = test_dir;
+    if (fs::exists(test_dir))
+        canon_path = fs::canonical(test_dir);
+
+    // Paths shorter than dir are not in dir
+    auto dir_length = std::distance(canon_dir.begin(), canon_dir.end());
+    auto path_length = std::distance(canon_path.begin(), canon_path.end());
+    if (path_length < dir_length)
+        return false;
+
+    // Check that the whole dir path matches the test path
+    // Extra portions of path are contained in dir
+    return std::equal(canon_dir.begin(), canon_dir.end(), canon_path.begin());
+}
+
+fs::path GetPath(PathType path_type) {
+    switch (path_type) {
+    case PATH_BINARY:
+        return GetBinDir();
+    case PATH_RESOURCE:
+        return GetResourceDir();
+    case PATH_DATA_ROOT:
+        return GetRootDataDir();
+    case PATH_DATA_USER:
+        return GetUserDataDir();
+    case PATH_CONFIG:
+        return GetUserConfigDir();
+    case PATH_SAVE:
+        return GetSaveDir();
+    case PATH_TEMP:
+        return fs::temp_directory_path();
+    case PATH_PYTHON:
+#if defined(FREEORION_MACOSX) || defined(FREEORION_WIN32)
+        return GetPythonHome();
+#endif
+    case PATH_INVALID:
+    default:
+        ErrorLogger() << "Invalid path type " << path_type;
+        return fs::temp_directory_path();
+    }
+}
+
+fs::path GetPath(const std::string& path_string) {
+    if (path_string.empty()) {
+        ErrorLogger() << "GetPath called with empty argument";
+        return fs::temp_directory_path();
+    }
+
+    PathType path_type;
+    try {
+        path_type = boost::lexical_cast<PathType>(path_string);
+    } catch (const boost::bad_lexical_cast& ec) {
+        // try partial match
+        std::string retval = path_string;
+        for (const auto& path_type_str : PathTypeStrings()) {
+            std::string path_type_string = PathToString(GetPath(path_type_str));
+            boost::replace_all(retval, path_type_str, path_type_string);
+        }
+        if (path_string != retval) {
+            return FilenameToPath(retval);
+        } else {
+            ErrorLogger() << "Invalid cast for PathType from string " << path_string;
+            return fs::temp_directory_path();
+        }
+    }
+    return GetPath(path_type);
+}
+
+bool IsExistingFile(const fs::path& path) {
+    try {
+        auto stat = fs::status(path);
+        return fs::exists(stat) && fs::is_regular_file(stat);
+    } catch(boost::filesystem::filesystem_error& ec) {
+        ErrorLogger() << "Filesystem error during stat of " << PathToString(path) << " : " << ec.what();
+    }
+
+    return false;
+}
+
+std::vector<fs::path> PathsInDir(const boost::filesystem::path& abs_dir_path,
+                                 std::function<bool (const fs::path&)> pred,
+                                 bool recursive_search)
+{
+    std::vector<fs::path> retval;
+    if (abs_dir_path.is_relative()) {
+        ErrorLogger() << "Passed relative path for fileysstem operation " << PathToString(abs_dir_path);
+        return retval;
+    }
+
+    try {
+        auto dir_stat = fs::status(abs_dir_path);
+        if (!fs::exists(dir_stat) || !fs::is_directory(dir_stat)) {
+            ErrorLogger() << "Path is not an existing directory " << PathToString(abs_dir_path);
+            return retval;
+        }
+
+        if (recursive_search) {
+            using dir_it_type = boost::filesystem::recursive_directory_iterator;
+            for (dir_it_type node_it(abs_dir_path); node_it != dir_it_type(); ++node_it) {
+                auto obj_path = node_it->path();
+                if (pred(obj_path))
+                    retval.push_back(obj_path);
+            }
+        } else {
+            using dir_it_type = boost::filesystem::directory_iterator;
+            for (dir_it_type node_it(abs_dir_path); node_it != dir_it_type(); ++node_it) {
+                auto obj_path = node_it->path();
+                if (pred(obj_path))
+                    retval.push_back(obj_path);
+            }
+        }
+    } catch(const fs::filesystem_error& ec) {
+        ErrorLogger() << "Filesystem error during directory traversal " << PathToString(abs_dir_path)
+                      << " : " << ec.what();
+        return {};
+    }
+
+    return retval;
+}
