@@ -5,6 +5,7 @@
 #include <string>
 
 #include "Export.h"
+#include "../universe/EnumsFwd.h"
 
 /** This function must be called before any Get*Dir function is called. It
   * stores the current working directory as well as creating local
@@ -12,7 +13,7 @@
 FO_COMMON_API void CompleteXDGMigration();
 
 /** This function completes the migration of directories to the XDG
-  * specified location by updating the save-dir option to the new location
+  * specified location by updating the save.path option to the new location
   * after the option is loaded from XML files.  It only updates the option
   * if it is set to the old default option. */
 FO_COMMON_API void InitDirs(const std::string& argv0);
@@ -40,7 +41,7 @@ FO_COMMON_API const boost::filesystem::path GetUserDataDir();
 
 /** Converts UTF-8 string into a path, doing any required wide-character
   * conversions as determined by the operating system / filesystem. */
-FO_COMMON_API const boost::filesystem::path FilenameToPath(const std::string& path_str);
+FO_COMMON_API boost::filesystem::path FilenameToPath(const std::string& path_str);
 
 /** Returns the directory that contains all game content files, such as string
   * table files, in-game tech, building, special, etc. definition files, and
@@ -63,7 +64,7 @@ FO_COMMON_API const boost::filesystem::path GetBinDir();
 
 #if defined(FREEORION_MACOSX) || defined(FREEORION_WIN32)
 /** This function returns the Python home directory from where it is embedded
-  * On OSX: within the application bundle 
+  * On OSX: within the application bundle
   * On Windows: same directory where the binaries are located */
 FO_COMMON_API const boost::filesystem::path GetPythonHome();
 #endif
@@ -78,8 +79,12 @@ FO_COMMON_API const boost::filesystem::path GetPersistentConfigPath();
   * the directory "save" within the user directory. */
 FO_COMMON_API const boost::filesystem::path GetSaveDir();
 
-/** Returns a canonical utf-8 string from the given filesystem path. */
-FO_COMMON_API std::string PathString(const boost::filesystem::path& path);
+/** Returns the directory where server save files are located.  This is typically
+  * the directory "save" within the user directory. */
+FO_COMMON_API const boost::filesystem::path GetServerSaveDir();
+
+/** Returns a utf-8 string from the given filesystem path. */
+FO_COMMON_API std::string PathToString(const boost::filesystem::path& path);
 
 /** Returns current timestamp in a form that can be used in file names */
 FO_COMMON_API std::string FilenameTimestamp();
@@ -93,5 +98,28 @@ FO_COMMON_API std::vector<boost::filesystem::path> ListDir(const boost::filesyst
 /** Returns true iff the string \a in is valid UTF-8. */
 FO_COMMON_API bool IsValidUTF8(const std::string& in);
 
+/** Returns true iff the \p test_dir is in \p dir and \p dir
+    is existing directory. */
+FO_COMMON_API bool IsInDir(const boost::filesystem::path& dir, const boost::filesystem::path& test_dir);
+
+/** Returns path currently defined for @p path_type */
+FO_COMMON_API boost::filesystem::path GetPath(PathType path_type);
+
+/** Returns path for path type cast from @p path_string */
+FO_COMMON_API boost::filesystem::path GetPath(const std::string& path_string);
+
+/** Returns if path exists and is a regular file */
+FO_COMMON_API bool IsExistingFile(const boost::filesystem::path& path);
+
+/** All paths contained in a directory filtered by a functor
+ *
+ * @param[in] abs_dir_path Absolute path to directory
+ * @param[in] pred Predicate functor accepting a boost::filesystem::path constant reference
+ * @param[in] recursive_search If true, recurses into sub-directories
+ *
+ * @return vector List of filesytem objects found in @p abs_dir_path which satisfy @p pred */
+FO_COMMON_API std::vector<boost::filesystem::path> PathsInDir(const boost::filesystem::path& abs_dir_path,
+                                                              std::function<bool (const boost::filesystem::path&)> pred,
+                                                              bool recursive_search);
 
 #endif
