@@ -6,9 +6,11 @@
 #include "OptionValidators.h"
 
 #include <boost/any.hpp>
+#include <boost/filesystem/fstream.hpp>
 #include <boost/signals2/signal.hpp>
 
 #include <map>
+#include <unordered_set>
 
 
 class OptionsDB;
@@ -81,8 +83,8 @@ FO_COMMON_API OptionsDB& GetOptionsDB();
   * options.  All flag command-line options (specified with AddFlag()) are
   * assumed to have false as their default value.  This means that their mere
   * presence on the command line means that they indicate a value of true;
-  * they need no argument.  For example, specifying "--help" on the command
-  * line sets the option "help" in the DB to true, and leaving it out sets the
+  * they need no argument.  For example, specifying "--version" on the command
+  * line sets the option "version" in the DB to true, and leaving it out sets the
   * option to false.
   * <br><br>Long-form names should be preceded with "--", and the
   * single-character version should be preceded with "-".  An exception to this
@@ -97,8 +99,7 @@ FO_COMMON_API OptionsDB& GetOptionsDB();
   * including string-to-type, type-to-string or type-to-type as in the case of
   * Get() calls with the wrong tempate parameter.
   * \see RegisterOptions (for static-time options specification) */
-class FO_COMMON_API OptionsDB
-{
+class FO_COMMON_API OptionsDB {
 public:
     /** \name Signal Types */ //@{
     /** emitted when an option has changed */
@@ -112,12 +113,18 @@ public:
     /** indicates whether an option with name \a name has been added to this
         OptionsDB. */
     bool        OptionExists(const std::string& name) const
-    { return m_options.find(name) != m_options.end() && m_options.at(name).recognized; }
+    { return m_options.count(name) && m_options.at(name).recognized; }
 
     /** write back the optionDB's state to the XML config file
         if it has changed since it was last saved. */
     void        Commit();
 
+    /** Write any options that are not at default value to persistent config, replacing any existing file
+     *
+     *  @returns bool If file was successfully written
+     */
+    bool        CommitPersistent();
+
     /** validates a value for an option. throws std::runtime_error if no option
       * \a name exists.  throws bad_lexical_cast if \a value cannot be
       * converted to the type of the option \a name. */
@@ -129,7 +136,7 @@ public:
     template <class T>
     T           Get(const std::string& name) const
     {
-        std::map<std::string, Option>::const_iterator it = m_options.find(name);
+        auto it = m_options.find(name);
         if (!OptionExists(it))
             throw std::runtime_error("OptionsDB::Get<>() : Attempted to get nonexistent option \"" + name + "\".");
         return boost::any_cast<T>(it->second.value);
@@ -141,14 +148,14 @@ public:
     template <class T>
     T           GetDefault(const std::string& name) const
     {
-        std::map<std::string, Option>::const_iterator it = m_options.find(name);
+        auto it = m_options.find(name);
         if (!OptionExists(it))
             throw std::runtime_error("OptionsDB::GetDefault<>() : Attempted to get nonexistent option \"" + name + "\".");
         return boost::any_cast<T>(it->second.default_value);
     }
 
     bool        IsDefaultValue(const std::string& name) const {
-        std::map<std::string, Option>::const_iterator it = m_options.find(name);
+        auto it = m_options.find(name);
         if (!OptionExists(it))
             throw std::runtime_error("OptionsDB::IsDefaultValue<>() : Attempted to get nonexistent option \"" + name + "\".");
         return IsDefaultValue(it);
@@ -170,21 +177,23 @@ public:
     std::shared_ptr<const ValidatorBase> GetValidator(const std::string& option_name) const;
 
     /** writes a usage message to \a os */
-    void        GetUsage(std::ostream& os, const std::string& command_line = "") const;
+    void        GetUsage(std::ostream& os, const std::string& command_line = "", bool allow_unrecognized = false) const;
 
     /** @brief  Saves the contents of the options DB to the @p doc XMLDoc.
      *
      * @param[in,out] doc  The document this OptionsDB should be written to.
      *      This resets the given @p doc.
+     * @param[in] non_default_only Do not include options which are set to their
+     *      default value, is unrecognized, or is "version.string"
      */
-    void GetXML(XMLDoc& doc) const;
+    void        GetXML(XMLDoc& doc, bool non_default_only = false) const;
 
     /** find all registered Options that begin with \a prefix and store them in
-      * \a ret. */
-    void        FindOptions(std::set<std::string>& ret, const std::string& prefix) const;
+      * \a ret. If \p allow_unrecognized then include unrecognized options. */
+    void        FindOptions(std::set<std::string>& ret, const std::string& prefix, bool allow_unrecognized = false) const;
 
     /** the option changed signal object for the given option */
-    OptionChangedSignalType&    OptionChangedSignal(const std::string& option);
+    OptionChangedSignalType&        OptionChangedSignal(const std::string& option);
 
     mutable OptionAddedSignalType   OptionAddedSignal;   ///< the option added signal object for this DB
     mutable OptionRemovedSignalType OptionRemovedSignal; ///< the change removed signal object for this DB
@@ -192,9 +201,10 @@ public:
     /** adds an Option, optionally with a custom validator */
     template <class T>
     void        Add(const std::string& name, const std::string& description, T default_value,
-                    const ValidatorBase& validator = Validator<T>(), bool storable = true)
+                    const ValidatorBase& validator = Validator<T>(), bool storable = true,
+                    const std::string& section = std::string())
     {
-        std::map<std::string, Option>::iterator it = m_options.find(name);
+        auto it = m_options.find(name);
         boost::any value = default_value;
         // Check that this option hasn't already been registered and apply any value that was specified on the command line or from a config file.
         if (it != m_options.end()) {
@@ -211,7 +221,8 @@ public:
                 }
             }
         }
-        m_options[name] = Option(static_cast<char>(0), name, value, default_value, description, validator.Clone(), storable, false, true);
+        m_options[name] = Option(static_cast<char>(0), name, value, default_value,
+                                 description, validator.Clone(), storable, false, true, section);
         m_dirty = true;
         OptionAddedSignal(name);
     }
@@ -220,9 +231,10 @@ public:
       * optionally with a custom validator */
     template <class T>
     void        Add(char short_name, const std::string& name, const std::string& description, T default_value,
-                    const ValidatorBase& validator = Validator<T>(), bool storable = true)
+                    const ValidatorBase& validator = Validator<T>(), bool storable = true,
+                    const std::string& section = std::string())
     {
-        std::map<std::string, Option>::iterator it = m_options.find(name);
+        auto it = m_options.find(name);
         boost::any value = default_value;
         // Check that this option hasn't already been registered and apply any value that was specified on the command line or from a config file.
         if (it != m_options.end()) {
@@ -239,7 +251,8 @@ public:
                 }
             }
         }
-        m_options[name] = Option(short_name, name, value, default_value, description, validator.Clone(), storable, false, true);
+        m_options[name] = Option(short_name, name, value, default_value, description,
+                                 validator.Clone(), storable, false, true, section);
         m_dirty = true;
         OptionAddedSignal(name);
     }
@@ -248,9 +261,9 @@ public:
       * of false.  Using the flag on the command line at all indicates that its
       * value it set to true. */
     void        AddFlag(const std::string& name, const std::string& description,
-                        bool storable = true)
+                        bool storable = true, const std::string& section = std::string())
     {
-        std::map<std::string, Option>::iterator it = m_options.find(name);
+        auto it = m_options.find(name);
         bool value = false;
         // Check that this option hasn't already been registered and apply any value that was specified on the command line or from a config file.
         if (it != m_options.end()) {
@@ -260,8 +273,9 @@ public:
                 ErrorLogger() << "OptionsDB::AddFlag<>() : Option " << name << " was specified with the value \"" << it->second.ValueToString() << "\", but flags should not have values assigned to them.";
             value = true; // if the flag is present at all its value is true
         }
-        m_options[name] = Option(static_cast<char>(0), name, value, boost::lexical_cast<std::string>(false),
-                                 description, nullptr, storable, true, true);
+        m_options[name] = Option(static_cast<char>(0), name, value,
+                                 boost::lexical_cast<std::string>(false),
+                                 description, nullptr, storable, true, true, section);
         m_dirty = true;
         OptionAddedSignal(name);
     }
@@ -270,9 +284,10 @@ public:
       * is treated as a boolean value with a default of false.  Using the flag
       * on the command line at all indicates that its value it set to true. */
     void        AddFlag(char short_name, const std::string& name,
-                        const std::string& description, bool storable = true)
+                        const std::string& description, bool storable = true,
+                        const std::string& section = std::string())
     {
-        std::map<std::string, Option>::iterator it = m_options.find(name);
+        auto it = m_options.find(name);
         bool value = false;
         // Check that this option hasn't already been registered and apply any value that was specified on the command line or from a config file.
         if (it != m_options.end()) {
@@ -282,8 +297,9 @@ public:
                 ErrorLogger() << "OptionsDB::AddFlag<>() : Option " << name << " was specified with the value \"" << it->second.ValueToString() << "\", but flags should not have values assigned to them.";
             value = true; // if the flag is present at all its value is true
         }
-        m_options[name] = Option(short_name, name, value, boost::lexical_cast<std::string>(false),
-                                 description, nullptr, storable, true, true);
+        m_options[name] = Option(short_name, name, value,
+                                 boost::lexical_cast<std::string>(false),
+                                 description, nullptr, storable, true, true, section);
         m_dirty = true;
         OptionAddedSignal(name);
     }
@@ -299,12 +315,29 @@ public:
     template <class T>
     void        Set(const std::string& name, const T& value)
     {
-        std::map<std::string, Option>::iterator it = m_options.find(name);
+        auto it = m_options.find(name);
         if (!OptionExists(it))
             throw std::runtime_error("OptionsDB::Set<>() : Attempted to set nonexistent option \"" + name + "\".");
         m_dirty |= it->second.SetFromValue(value);
     }
 
+    /** Set the default value of option @p name to @p value */
+    template <class T>
+    void        SetDefault(const std::string& name, const T& value) {
+        std::map<std::string, Option>::iterator it = m_options.find(name);
+        if (!OptionExists(it))
+            throw std::runtime_error("Attempted to set default value of nonexistent option \"" + name + "\".");
+        if (it->second.default_value.type() != typeid(T))
+            throw boost::bad_any_cast();
+        it->second.default_value = value;
+    }
+
+    /** if an xml file exists at \a file_path and has the same version tag as \a version, fill the
+      * DB options contained in that file (read the file using XMLDoc, then fill the DB using SetFromXML)
+      * if the \a version string is empty, bypass that check */
+    void        SetFromFile(const boost::filesystem::path& file_path,
+                            const std::string& version = "");
+
     /** fills some or all of the options of the DB from values passed in from
       * the command line */
     void        SetFromCommandLine(const std::vector<std::string>& args);
@@ -313,18 +346,20 @@ public:
       * XMLDoc \a doc */
     void        SetFromXML(const XMLDoc& doc);
 
-private:
     struct FO_COMMON_API Option {
         Option();
         Option(char short_name_, const std::string& name_, const boost::any& value_,
                const boost::any& default_value_, const std::string& description_,
-               const ValidatorBase *validator_, bool storable_, bool flag_, bool recognized_);
+               const ValidatorBase *validator_, bool storable_, bool flag_, bool recognized_,
+               const std::string& section = std::string());
 
         // SetFromValue returns true if this->value is successfully changed
         template <typename T>
         bool            SetFromValue(const T& value_);
         // SetFromString returns true if this->value is successfully changed
         bool            SetFromString(const std::string& str);
+        // SetToDefault returns true if this->value is successfully changed
+        bool            SetToDefault();
         std::string     ValueToString() const;
         std::string     DefaultValueToString() const;
 
@@ -333,6 +368,7 @@ private:
         boost::any      value;          ///< the value of the option
         boost::any      default_value;  ///< the default value of the option
         std::string     description;    ///< a desription of the option
+        std::unordered_set<std::string> sections; ///< sections this option should display under
 
         /** A validator for the option.  Flags have no validators; lexical_cast
             boolean conversions are done for them. */
@@ -347,6 +383,25 @@ private:
         static std::map<char, std::string> short_names;   ///< the master list of abbreviated option names, and their corresponding long-form names
     };
 
+    struct FO_COMMON_API OptionSection {
+        OptionSection();
+        OptionSection(const std::string& name_, const std::string& description_,
+                      std::function<bool (const std::string&)> option_predicate_);
+
+        std::string name;
+        std::string description;
+        std::function<bool (const std::string&)> option_predicate = nullptr;
+    };
+
+    /** Defines an option section with a description and optionally a option predicate.
+     *  @param name Name of section, typically in the form of a left side subset of an option name.
+     *  @param description Stringtable key used for local description
+     *  @param option_predicate Functor accepting a option name in the form of a std::string const ref and
+     *                          returning a bool. Options which return true are displayed in the section for @p name */
+    void AddSection(const std::string& name, const std::string& description,
+                    std::function<bool (const std::string&)> option_predicate = nullptr);
+
+private:
     /** indicates whether the option referenced by \a it has been added to this
         OptionsDB.  Overloaded for convenient use within other OptionsDB
         functions */
@@ -362,7 +417,12 @@ private:
 
     void        SetFromXMLRecursive(const XMLElement& elem, const std::string& section_name);
 
+    /** Determine known option sections and which options each contains
+     *  A special "root" section is added for determined top-level sections */
+    std::unordered_map<std::string, std::set<std::string>> OptionsBySection(bool allow_unrecognized = false) const;
+
     std::map<std::string, Option>   m_options;
+    std::unordered_map<std::string, OptionSection> m_sections;
     static OptionsDB*               s_options_db;
     bool                            m_dirty; //< has OptionsDB changed since last Commit()
 
@@ -377,7 +437,7 @@ bool OptionsDB::Option::SetFromValue(const T& value_) {
     bool changed = false;
 
     if (!flag) {
-        changed =  validator->String(value) != validator->String(value_);
+        changed = validator->String(value) != validator->String(value_);
     } else {
         changed = (boost::lexical_cast<std::string>(boost::any_cast<bool>(value))
                    != boost::lexical_cast<std::string>(boost::any_cast<bool>(value_)));
