@@ -65,7 +65,7 @@ private:
 
         profiled_code();
 
-        for() {
+        for () {
             timer.EnterSection("for loop 1 section");
 
             more_code();
@@ -103,4 +103,4 @@ private:
     std::unique_ptr<Impl> const m_impl;
 };
 
-#endif // _MultiplayerCommon_h_
+#endif // _ScopedTimer_h_
