@@ -14,7 +14,6 @@ class OrderSet;
 class PathingEngine;
 class Universe;
 class UniverseObject;
-template <class T> class TemporaryPtr;
 
 typedef boost::archive::binary_iarchive freeorion_bin_iarchive;
 typedef boost::archive::binary_oarchive freeorion_bin_oarchive;
@@ -30,7 +29,7 @@ FO_COMMON_API void Serialize(Archive& oa, const Universe& universe);
 
 /** Serializes \a object_map to output archive \a oa. */
 template <class Archive>
-void Serialize(Archive& oa, const std::map<int, TemporaryPtr<UniverseObject> >& objects);
+void Serialize(Archive& oa, const std::map<int, std::shared_ptr<UniverseObject>>& objects);
 
 /** Serializes \a order_set to output archive \a oa. */
 template <class Archive>
@@ -46,7 +45,7 @@ FO_COMMON_API void Deserialize(Archive& ia, Universe& universe);
 
 /** Serializes \a object_map from input archive \a ia. */
 template <class Archive>
-void Deserialize(Archive& ia, std::map<int, TemporaryPtr<UniverseObject> >& objects);
+void Deserialize(Archive& ia, std::map<int, std::shared_ptr<UniverseObject>>& objects);
 
 /** Deserializes \a order_set from input archive \a ia. */
 template <class Archive>
