@@ -3,6 +3,7 @@
 #include "Logger.h"
 #include "Serialize.ipp"
 
+#include "../universe/IDAllocator.h"
 #include "../universe/Building.h"
 #include "../universe/Fleet.h"
 #include "../universe/Ship.h"
@@ -13,15 +14,22 @@
 #include "../universe/Field.h"
 #include "../universe/Universe.h"
 
+#include <boost/lexical_cast.hpp>
+#include <boost/uuid/nil_generator.hpp>
+
 BOOST_CLASS_EXPORT(System)
 BOOST_CLASS_EXPORT(Field)
 BOOST_CLASS_EXPORT(Planet)
+BOOST_CLASS_VERSION(Planet, 1)
 BOOST_CLASS_EXPORT(Building)
 BOOST_CLASS_EXPORT(Fleet)
+BOOST_CLASS_VERSION(Fleet, 3)
 BOOST_CLASS_EXPORT(Ship)
-BOOST_CLASS_VERSION(Ship, 1)
-//BOOST_CLASS_EXPORT(ShipDesign)
-//BOOST_CLASS_VERSION(ShipDesign, 1)
+BOOST_CLASS_VERSION(Ship, 2)
+BOOST_CLASS_EXPORT(ShipDesign)
+BOOST_CLASS_VERSION(ShipDesign, 2)
+BOOST_CLASS_EXPORT(Universe)
+BOOST_CLASS_VERSION(Universe, 1)
 
 template <class Archive>
 void ObjectMap::serialize(Archive& ar, const unsigned int version)
@@ -57,6 +65,7 @@ void Universe::serialize(Archive& ar, const unsigned int version)
         GetEmpireKnownObjectsToSerialize(   empire_latest_known_objects,        m_encoding_empire);
         GetEmpireObjectVisibilityMap(       empire_object_visibility,           m_encoding_empire);
         GetEmpireObjectVisibilityTurnMap(   empire_object_visibility_turns,     m_encoding_empire);
+        //GetEffectSpecifiedVisibilities(     effect_specified_visibilities,      m_encoding_empire);
         GetEmpireKnownDestroyedObjects(     empire_known_destroyed_object_ids,  m_encoding_empire);
         GetEmpireStaleKnowledgeObjects(     empire_stale_knowledge_object_ids,  m_encoding_empire);
         GetShipDesignsToSerialize(          ship_designs,                       m_encoding_empire);
@@ -78,16 +87,19 @@ void Universe::serialize(Archive& ar, const unsigned int version)
 
     ar  & BOOST_SERIALIZATION_NVP(empire_object_visibility);
     ar  & BOOST_SERIALIZATION_NVP(empire_object_visibility_turns);
+    //ar  & BOOST_SERIALIZATION_NVP(effect_specified_visibilities);
     ar  & BOOST_SERIALIZATION_NVP(empire_known_destroyed_object_ids);
     ar  & BOOST_SERIALIZATION_NVP(empire_stale_knowledge_object_ids);
     DebugLogger() << "Universe::serialize : " << serializing_label
                   << " empire object visibility for " << empire_object_visibility.size() << ", "
                   << empire_object_visibility_turns.size() << ", "
+                  //<< effect_specified_visibilities.size() << ", "
                   << empire_known_destroyed_object_ids.size() << ", "
                   << empire_stale_knowledge_object_ids.size() <<  " empires";
     if (Archive::is_loading::value) {
         m_empire_object_visibility.swap(empire_object_visibility);
         m_empire_object_visibility_turns.swap(empire_object_visibility_turns);
+        //m_effect_specified_empire_object_visibilities.swap(effect_specified_visibilities);
         m_empire_known_destroyed_object_ids.swap(empire_known_destroyed_object_ids);
         m_empire_stale_knowledge_object_ids.swap(empire_stale_knowledge_object_ids);
     }
@@ -111,9 +123,31 @@ void Universe::serialize(Archive& ar, const unsigned int version)
         m_empire_latest_known_objects.swap(empire_latest_known_objects);
     }
 
-    DebugLogger() << "Universe::serialize : " << serializing_label << " last allocated ids";
-    ar  & BOOST_SERIALIZATION_NVP(m_last_allocated_object_id);
-    ar  & BOOST_SERIALIZATION_NVP(m_last_allocated_design_id);
+    if (version >= 1) {
+        DebugLogger() << "Universe::serialize : " << serializing_label << " id allocator version = " << version;
+        m_object_id_allocator->SerializeForEmpire(ar, version, m_encoding_empire);
+        m_design_id_allocator->SerializeForEmpire(ar, version, m_encoding_empire);
+    } else {
+        if (Archive::is_loading::value) {
+            int dummy_last_allocated_object_id;
+            int dummy_last_allocated_design_id;
+            DebugLogger() << "Universe::serialize : " << serializing_label << " legacy last allocated ids version = " << version;
+            ar  & boost::serialization::make_nvp("m_last_allocated_object_id", dummy_last_allocated_object_id);
+            DebugLogger() << "Universe::serialize : " << serializing_label << " legacy last allocated ids2";
+            ar  & boost::serialization::make_nvp("m_last_allocated_design_id", dummy_last_allocated_design_id);
+
+            DebugLogger() << "Universe::serialize : " << serializing_label << " legacy id allocator";
+            // For legacy loads pre-dating the use of the IDAllocator the server
+            // allocators need to be initialized with a list of the empires.
+            std::vector<int> allocating_empire_ids(m_empire_latest_known_objects.size());
+            std::transform(m_empire_latest_known_objects.begin(), m_empire_latest_known_objects.end(),
+                           allocating_empire_ids.begin(),
+                           [](const std::pair<int, ObjectMap> ii) { return ii.first; });
+
+            ResetAllIDAllocation(allocating_empire_ids);
+        }
+    }
+
     ar  & BOOST_SERIALIZATION_NVP(m_stat_records);
     DebugLogger() << "Universe::serialize : " << serializing_label << " " << m_stat_records.size() << " types of statistic";
 
@@ -123,16 +157,15 @@ void Universe::serialize(Archive& ar, const unsigned int version)
         DebugLogger() << "Universe::serialize : Cleaning up temporary data";
         // clean up temporary objects in temporary ObjectMaps
         objects.Clear();
-        for (std::map<int, ObjectMap>::value_type& elko : empire_latest_known_objects)
+        for (auto& elko : empire_latest_known_objects)
         { elko.second.Clear(); }
     }
 
     if (Archive::is_loading::value) {
         DebugLogger() << "Universe::serialize : updating empires' latest known object destruction states";
         // update known destroyed objects state in each empire's latest known objects
-        for (std::map<int, ObjectMap>::value_type& elko : m_empire_latest_known_objects) {
-            std::map<int, std::set<int>>::iterator destroyed_ids_it =
-                m_empire_known_destroyed_object_ids.find(elko.first);
+        for (auto& elko : m_empire_latest_known_objects) {
+            auto destroyed_ids_it = m_empire_known_destroyed_object_ids.find(elko.first);
             if (destroyed_ids_it != m_empire_known_destroyed_object_ids.end())
                 elko.second.UpdateCurrentDestroyedObjects(destroyed_ids_it->second);
         }
@@ -180,7 +213,7 @@ void Field::serialize(Archive& ar, const unsigned int version)
 template <class Archive>
 void Planet::serialize(Archive& ar, const unsigned int version)
 {
-   ar  & BOOST_SERIALIZATION_BASE_OBJECT_NVP(UniverseObject)
+   ar   & BOOST_SERIALIZATION_BASE_OBJECT_NVP(UniverseObject)
         & BOOST_SERIALIZATION_BASE_OBJECT_NVP(PopCenter)
         & BOOST_SERIALIZATION_BASE_OBJECT_NVP(ResourceCenter)
         & BOOST_SERIALIZATION_NVP(m_type)
@@ -190,9 +223,14 @@ void Planet::serialize(Archive& ar, const unsigned int version)
         & BOOST_SERIALIZATION_NVP(m_initial_orbital_position)
         & BOOST_SERIALIZATION_NVP(m_rotational_period)
         & BOOST_SERIALIZATION_NVP(m_axial_tilt)
-        & BOOST_SERIALIZATION_NVP(m_buildings)
-        & BOOST_SERIALIZATION_NVP(m_just_conquered)
-        & BOOST_SERIALIZATION_NVP(m_is_about_to_be_colonized)
+        & BOOST_SERIALIZATION_NVP(m_buildings);
+    if (version < 1) {
+        bool dummy = false;
+        ar   & boost::serialization::make_nvp("m_just_conquered", dummy);
+    } else {
+        ar   & BOOST_SERIALIZATION_NVP(m_turn_last_conquered);
+    }
+    ar  & BOOST_SERIALIZATION_NVP(m_is_about_to_be_colonized)
         & BOOST_SERIALIZATION_NVP(m_is_about_to_be_invaded)
         & BOOST_SERIALIZATION_NVP(m_is_about_to_be_bombarded)
         & BOOST_SERIALIZATION_NVP(m_ordered_given_to_empire_id)
@@ -218,9 +256,12 @@ void Fleet::serialize(Archive& ar, const unsigned int version)
         & BOOST_SERIALIZATION_NVP(m_next_system)
         & BOOST_SERIALIZATION_NVP(m_aggressive)
         & BOOST_SERIALIZATION_NVP(m_ordered_given_to_empire_id)
-        & BOOST_SERIALIZATION_NVP(m_travel_route)
-        & BOOST_SERIALIZATION_NVP(m_travel_distance)
-        & BOOST_SERIALIZATION_NVP(m_arrived_this_turn)
+        & BOOST_SERIALIZATION_NVP(m_travel_route);
+    if (version < 3) {
+        double dummy_travel_distance;
+        ar & boost::serialization::make_nvp("m_travel_distance", dummy_travel_distance);
+    }
+    ar  & BOOST_SERIALIZATION_NVP(m_arrived_this_turn)
         & BOOST_SERIALIZATION_NVP(m_arrival_starlane);
 }
 
@@ -240,6 +281,9 @@ void Ship::serialize(Archive& ar, const unsigned int version)
         & BOOST_SERIALIZATION_NVP(m_arrived_on_turn);
     if (version >= 1) {
         ar  & BOOST_SERIALIZATION_NVP(m_last_turn_active_in_combat);
+        if (version >= 2) {
+            ar  & BOOST_SERIALIZATION_NVP(m_last_resupplied_on_turn);
+        }
     }
 }
 
@@ -247,17 +291,44 @@ template <class Archive>
 void ShipDesign::serialize(Archive& ar, const unsigned int version)
 {
     ar  & BOOST_SERIALIZATION_NVP(m_id)
-        & BOOST_SERIALIZATION_NVP(m_name)
-        & BOOST_SERIALIZATION_NVP(m_description)
-        & BOOST_SERIALIZATION_NVP(m_designed_on_turn)
-        & BOOST_SERIALIZATION_NVP(m_hull)
+        & BOOST_SERIALIZATION_NVP(m_name);
+
+    TraceLogger() << "ship design serialize version: " << version << " : " << (Archive::is_saving::value ? "saving" : "loading");
+
+    if (version >= 1) {
+        // UUID serialization as a primitive doesn't work as expected from the documentation
+        // ar & BOOST_SERIALIZATION_NVP(m_uuid);
+        // This workaround instead serializes a string representation.
+        if (Archive::is_saving::value) {
+            auto string_uuid = boost::uuids::to_string(m_uuid);
+            ar & BOOST_SERIALIZATION_NVP(string_uuid);
+        } else {
+            std::string string_uuid;
+            ar & BOOST_SERIALIZATION_NVP(string_uuid);
+            try {
+                m_uuid = boost::lexical_cast<boost::uuids::uuid>(string_uuid);
+            } catch (const boost::bad_lexical_cast&) {
+                m_uuid = boost::uuids::nil_generator()();
+            }
+        }
+    } else if (Archive::is_loading::value) {
+        m_uuid = boost::uuids::nil_generator()();
+    }
+
+    ar  & BOOST_SERIALIZATION_NVP(m_description)
+        & BOOST_SERIALIZATION_NVP(m_designed_on_turn);
+    if (version >= 2)
+        ar  & BOOST_SERIALIZATION_NVP(m_designed_by_empire);
+    ar  & BOOST_SERIALIZATION_NVP(m_hull)
         & BOOST_SERIALIZATION_NVP(m_parts)
         & BOOST_SERIALIZATION_NVP(m_is_monster)
         & BOOST_SERIALIZATION_NVP(m_icon)
         & BOOST_SERIALIZATION_NVP(m_3D_model)
         & BOOST_SERIALIZATION_NVP(m_name_desc_in_stringtable);
-    if (Archive::is_loading::value)
+    if (Archive::is_loading::value) {
+        ForceValidDesignOrThrow(boost::none, true);
         BuildStatCaches();
+    }
 }
 
 template
@@ -273,7 +344,7 @@ template <class Archive>
 void SpeciesManager::serialize(Archive& ar, const unsigned int version)
 {
     // Don't need to send all the data about species, as this is derived from
-    // content data files in species.txt that should be available to any
+    // content data files in scripting/species that should be available to any
     // client or server.  Instead, just need to send the gamestate portion of
     // species: their homeworlds in the current game, and their opinions of
     // empires and eachother
