@@ -13,11 +13,12 @@
 SitRepEntry::SitRepEntry() :
     VarText(),
     m_turn(INVALID_GAME_TURN),
-    m_icon("/icons/sitrep/generic.png"),
-    m_label()
+    m_icon("/icons/sitrep/generic.png")
 {}
 
-SitRepEntry::SitRepEntry(const std::string& template_string, int turn, const std::string& icon, const std::string label, bool stringtable_lookup) :
+SitRepEntry::SitRepEntry(const std::string& template_string, int turn,
+                         const std::string& icon, const std::string label,
+                         bool stringtable_lookup) :
     VarText(template_string, stringtable_lookup),
     m_turn(turn),
     m_icon(icon.empty() ? "/icons/sitrep/generic.png" : icon),
@@ -25,7 +26,7 @@ SitRepEntry::SitRepEntry(const std::string& template_string, int turn, const std
 {}
 
 int SitRepEntry::GetDataIDNumber(const std::string& tag) const {
-    std::map<std::string, std::string>::const_iterator elem = m_variables.find(tag);
+    auto elem = m_variables.find(tag);
     try {
         if (elem != m_variables.end())
             return boost::lexical_cast<int>(elem->second);
@@ -37,7 +38,7 @@ int SitRepEntry::GetDataIDNumber(const std::string& tag) const {
 
 const std::string& SitRepEntry::GetDataString(const std::string& tag) const {
     static const std::string EMPTY_STRING;
-    std::map<std::string, std::string>::const_iterator elem = m_variables.find(tag);
+    auto elem = m_variables.find(tag);
     if (elem == m_variables.end())
         return EMPTY_STRING;
     return elem->second;
@@ -45,7 +46,7 @@ const std::string& SitRepEntry::GetDataString(const std::string& tag) const {
 
 std::string SitRepEntry::Dump() const {
     std::string retval = "SitRep template_string = \"" + m_template_string + "\"";
-    for (const std::map<std::string, std::string>::value_type& variable : m_variables)
+    for (const auto& variable : m_variables)
         retval += " " + variable.first + " = " + variable.second;
     retval += " turn = " + std::to_string(m_turn);
     retval += " icon = " + m_icon;
@@ -203,13 +204,13 @@ namespace {
 }
 
 SitRepEntry CreateCombatDamagedObjectSitRep(int object_id, int combat_system_id, int empire_id) {
-    std::shared_ptr<const UniverseObject> obj = GetUniverseObject(object_id);
+    auto obj = GetUniverseObject(object_id);
     if (!obj)
         return GenericCombatDamagedObjectSitrep(combat_system_id);
 
     SitRepEntry sitrep;
 
-    if (std::shared_ptr<const Ship> ship = std::dynamic_pointer_cast<const Ship>(obj)) {
+    if (auto ship = std::dynamic_pointer_cast<const Ship>(obj)) {
         if (ship->Unowned())
             sitrep = SitRepEntry(
                 UserStringNop("SITREP_UNOWNED_SHIP_DAMAGED_AT_SYSTEM"),
@@ -225,19 +226,19 @@ SitRepEntry CreateCombatDamagedObjectSitRep(int object_id, int combat_system_id,
         sitrep.AddVariable(VarText::SHIP_ID_TAG,       std::to_string(object_id));
         sitrep.AddVariable(VarText::DESIGN_ID_TAG,     std::to_string(ship->DesignID()));
 
-    } else if (std::shared_ptr<const Planet> planet = std::dynamic_pointer_cast<const Planet>(obj)) {
+    } else if (auto planet = std::dynamic_pointer_cast<const Planet>(obj)) {
         if (planet->Unowned())
             sitrep = SitRepEntry(
-                UserStringNop("SITREP_UNOWNED_PLANET_BOMBARDED_AT_SYSTEM"),
+                UserStringNop("SITREP_UNOWNED_PLANET_ATTACKED_AT_SYSTEM"),
                 CurrentTurn() + 1,
                 "icons/sitrep/colony_bombarded.png",
-                UserStringNop("SITREP_UNOWNED_PLANET_BOMBARDED_AT_SYSTEM_LABEL"), true);
+                UserStringNop("SITREP_UNOWNED_PLANET_ATTACKED_AT_SYSTEM_LABEL"), true);
         else
             sitrep = SitRepEntry(
-                UserStringNop("SITREP_PLANET_BOMBARDED_AT_SYSTEM"),
+                UserStringNop("SITREP_PLANET_ATTACKED_AT_SYSTEM"),
                 CurrentTurn() + 1,
                 "icons/sitrep/colony_bombarded.png",
-                UserStringNop("SITREP_PLANET_BOMBARDED_AT_SYSTEM_LABEL"), true);
+                UserStringNop("SITREP_PLANET_ATTACKED_AT_SYSTEM_LABEL"), true);
         sitrep.AddVariable(VarText::PLANET_ID_TAG,     std::to_string(object_id));
 
     } else {
@@ -251,7 +252,7 @@ SitRepEntry CreateCombatDamagedObjectSitRep(int object_id, int combat_system_id,
 }
 
 SitRepEntry CreateCombatDestroyedObjectSitRep(int object_id, int combat_system_id, int empire_id) {
-    std::shared_ptr<const UniverseObject> obj = GetEmpireKnownObject(object_id, empire_id);
+    auto obj = GetEmpireKnownObject(object_id, empire_id);
     if (!obj) {
         DebugLogger() << "Object " << object_id << " does not exist!!!";
         return GenericCombatDestroyedObjectSitrep(combat_system_id);
@@ -259,7 +260,7 @@ SitRepEntry CreateCombatDestroyedObjectSitRep(int object_id, int combat_system_i
 
     SitRepEntry sitrep;
 
-    if (std::shared_ptr<const Ship> ship = std::dynamic_pointer_cast<const Ship>(obj)) {
+    if (auto ship = std::dynamic_pointer_cast<const Ship>(obj)) {
         if (ship->Unowned())
             sitrep = SitRepEntry(
                 UserStringNop("SITREP_UNOWNED_SHIP_DESTROYED_AT_SYSTEM"),
@@ -281,7 +282,7 @@ SitRepEntry CreateCombatDestroyedObjectSitRep(int object_id, int combat_system_i
         sitrep.AddVariable(VarText::SHIP_ID_TAG,       std::to_string(object_id));
         sitrep.AddVariable(VarText::DESIGN_ID_TAG,     std::to_string(ship->DesignID()));
 
-    } else if (std::shared_ptr<const Fleet> fleet = std::dynamic_pointer_cast<const Fleet>(obj)) {
+    } else if (auto fleet = std::dynamic_pointer_cast<const Fleet>(obj)) {
         if (fleet->Unowned())
             sitrep = SitRepEntry(
                 UserStringNop("SITREP_UNOWNED_FLEET_DESTROYED_AT_SYSTEM"),
@@ -296,7 +297,7 @@ SitRepEntry CreateCombatDestroyedObjectSitRep(int object_id, int combat_system_i
                 UserStringNop("SITREP_FLEET_DESTROYED_AT_SYSTEM_LABEL"), true);
         sitrep.AddVariable(VarText::FLEET_ID_TAG,      std::to_string(object_id));
 
-    } else if (std::shared_ptr<const Planet> planet = std::dynamic_pointer_cast<const Planet>(obj)) {
+    } else if (auto planet = std::dynamic_pointer_cast<const Planet>(obj)) {
         if (planet->Unowned())
             sitrep = SitRepEntry(
                 UserStringNop("SITREP_UNOWNED_PLANET_DESTROYED_AT_SYSTEM"),
@@ -311,7 +312,7 @@ SitRepEntry CreateCombatDestroyedObjectSitRep(int object_id, int combat_system_i
                 UserStringNop("SITREP_PLANET_DESTROYED_AT_SYSTEM_LABEL"), true);
         sitrep.AddVariable(VarText::PLANET_ID_TAG,     std::to_string(object_id));
 
-    } else if (std::shared_ptr<const Building> building = std::dynamic_pointer_cast<const Building>(obj)) {
+    } else if (auto building = std::dynamic_pointer_cast<const Building>(obj)) {
         if (building->Unowned())
             sitrep = SitRepEntry(
                 UserStringNop("SITREP_UNOWNED_BUILDING_DESTROYED_ON_PLANET_AT_SYSTEM"),
@@ -368,7 +369,7 @@ SitRepEntry CreatePlanetOutpostedSitRep(int planet_id) {
 }
 
 SitRepEntry CreateFleetArrivedAtDestinationSitRep(int system_id, int fleet_id, int recipient_empire_id) {
-    std::shared_ptr<const Fleet> fleet = GetFleet(fleet_id);
+    auto fleet = GetFleet(fleet_id);
 
     //bool system_contains_recipient_empire_planets = false;
     //if (const System* system = GetSystem(system_id)) {
@@ -407,7 +408,7 @@ SitRepEntry CreateFleetArrivedAtDestinationSitRep(int system_id, int fleet_id, i
             sitrep.AddVariable(VarText::FLEET_ID_TAG,      std::to_string(fleet_id));
             int ship_id = *fleet->ShipIDs().begin();
             sitrep.AddVariable(VarText::SHIP_ID_TAG,       std::to_string(ship_id));
-            if (std::shared_ptr<const Ship> ship = GetShip(ship_id))
+            if (auto ship = GetShip(ship_id))
                 sitrep.AddVariable(VarText::DESIGN_ID_TAG, std::to_string(ship->DesignID()));
             return sitrep;
         } else {
@@ -443,7 +444,7 @@ SitRepEntry CreateFleetArrivedAtDestinationSitRep(int system_id, int fleet_id, i
             sitrep.AddVariable(VarText::EMPIRE_ID_TAG,     std::to_string(fleet->Owner()));
             int ship_id = *fleet->ShipIDs().begin();
             sitrep.AddVariable(VarText::SHIP_ID_TAG,       std::to_string(ship_id));
-            if (std::shared_ptr<const Ship> ship = GetShip(ship_id))
+            if (auto ship = GetShip(ship_id))
                 sitrep.AddVariable(VarText::DESIGN_ID_TAG, std::to_string(ship->DesignID()));
             return sitrep;
         } else {
@@ -470,7 +471,7 @@ SitRepEntry CreateFleetArrivedAtDestinationSitRep(int system_id, int fleet_id, i
             sitrep.AddVariable(VarText::EMPIRE_ID_TAG,     std::to_string(fleet->Owner()));
             int ship_id = *fleet->ShipIDs().begin();
             sitrep.AddVariable(VarText::SHIP_ID_TAG,       std::to_string(ship_id));
-            if (std::shared_ptr<const Ship> ship = GetShip(ship_id))
+            if (auto ship = GetShip(ship_id))
                 sitrep.AddVariable(VarText::DESIGN_ID_TAG, std::to_string(ship->DesignID()));
             return sitrep;
         } else {
