@@ -90,11 +90,6 @@ namespace {
 ////////////////////////////////////////////////
 // XMLElement
 ////////////////////////////////////////////////
-XMLElement::XMLElement(const std::string& tag, bool root) :
-    m_tag(tag),
-    m_root(root)
-{}
-
 const std::string& XMLElement::Tag() const
 { return m_tag; }
 
@@ -130,7 +125,7 @@ std::ostream& XMLElement::WriteElement(std::ostream& os, int indent/* = 0*/, boo
     if (whitespace)
         os << std::string(indent * 2, ' ');
     os << '<' << m_tag;
-    for (const std::map<std::string, std::string>::value_type& attribute : attributes)
+    for (const auto& attribute : attributes)
         os << ' ' << attribute.first << "=\"" << attribute.second << "\"";
     if (children.empty() && m_text.empty() && !m_root) {
         os << "/>";
